<?php
	if (!defined("ENV")) exit();
	$action = isset($action)?$action:"";
	
	// Figure out the current timezone set in the settings table for form selection
	date_default_timezone_set(APP_SYSTEM_TIMEZONE);
	$offset = date('O');
	$timezone_offset = substr($offset, 0, 3).':'.substr($offset, 3);
	
	
	if ($action != '')
	{
		$nonce = isset($nonce) ? $nonce : '';
		$nonce_valid = verify_nonce($nonce, 'localization_'.$action);
		if (!$nonce_valid)
		{
			invalid_nonce_message();
			exit;
		}
	}
	
	$currency_list = array(
		"USD"=>array("US dollar", 2, "$", ""),
		"EUR"=>array("Euro", 2, "&euro;", ""),
		"JPY"=>array("Japanese yen", 0, "&yen;", ""),
		"CYP"=>array("Cyprus pound", 2, "", "&pound;"),
		"CZK"=>array("Czech koruna", 2, "", "Kr"),
		"DKK"=>array("Danish krone", 2, "", "Kr"),
		"EEK"=>array("Estonian kroon", 2, "", "Kr"),
		"GBP"=>array("Pound sterling", 2, "", "&pound;"),
		"HUF"=>array("Hungarian forint", 2, "", "Fr"),
		"LTL"=>array("Lithuanian litas", 2, "", "Li"),
		"LVL"=>array("Latvian lats", 2, "", "LVL"),
		"MTL"=>array("Maltese lira", 2, "", "MTL"),
		"PLN"=>array("Polish zloty", 2, "", "Zl"),
		"SEK"=>array("Swedish krona", 2, "", "Kr"),
		"SIT"=>array("Slovenian tolar", 2, "", "SIT"),
		"SKK"=>array("Slovak koruna", 2, "", "Kr"),
		"CHF"=>array("Swiss franc", 2, "", "Fr"),
		"ISK"=>array("Icelandic krona", 2, "", "Kr"),
		"NOK"=>array("Norwegian krone", 2, "", "Kr"),
		"BGN"=>array("Bulgarian lev", 2, "", "BGN"),
		"HRK"=>array("Croatian kuna", 2, "", "HRK"),
		"RON"=>array("New Romanian leu", 2, "", "RON"),
		"RUB"=>array("Russian rouble", 2, "", "Rub"),
		"TRY"=>array("New Turkish lira", 2, "", "&#20A4;"),
		"AUD"=>array("Australian dollar", 2, "$", ""),
		"CAD"=>array("Canadian dollar", 2, "$", ""),
		"CNY"=>array("Chinese yuan renminbi", 2, "", "&#5143;"),
		"HKD"=>array("Hong Kong dollar", 2, "", "&#5713;"),
		"IDR"=>array("Indonesian rupiah", 2, "", "&#20A8;"),
		"KRW"=>array("South Korean won", 2, "", "&#20A9;"),
		"MYR"=>array("Malaysian ringgit", 2, "", "MYR"),
		"NZD"=>array("New Zealand dollar", 2, "", ""),
		"PHP"=>array("Philippine peso", 2, "", "&#20B1;"),
		"SGD"=>array("Singapore dollar", 2, "$", ""),
		"THB"=>array("Thai baht", 2, "", "&#3647;"),
		"ZAR"=>array("South African rand", 2, "", "R")
	);
	
	require_once("content/classes/class.currencies.php");
	require_once("content/classes/class.languages.php");
	$currencies = new Currencies($db);
	$lang = new Language($db);
	$lang->loadTemplate();
	
	$message = false;
	
	switch($action)
	{
		case "save_settings" : 
		{
			if (!DEMO_LOCK)
			{
				foreach ($form as $key=>$value)
				{
					$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($value)."' WHERE name='".$key."'");
					$settings[$key] = $value;
				}
			}
			$message = array("class"=>"ic-info", "text"=>"Local settings have been saved", "url"=>"", "visible"=>true);
			break;
		}
		
		case "add_currency" : 
		{
			if (!DEMO_LOCK)
			{
				$_POST["exchange_rate"] = normalizeNumber($_POST["exchange_rate"]);
				$currencies->addCurrency($_POST);
			}
			$message = array("class"=>"ic-info", "text"=>"New currency has been added", "url"=>"", "visible"=>true);
			break;
		}
		
		case "add_standard_currency" : 
		{
			$_POST["code"] = $code;
			$_POST["title"] = $currency_list[$code][0];
			$_POST["exchange_rate"] = normalizeNumber($_POST["exchange_rate"]);
			$_POST["decimal_places"] =  $currency_list[$code][1];
			$_POST["symbol_left"] = $currency_list[$code][2];
			$_POST["symbol_right"] =  $currency_list[$code][3];
			
			if (!DEMO_LOCK)
			{
				$currencies->addCurrency($_POST);
			}
			$message = array("class"=>"ic-info", "text"=>"New currency has been added", "url"=>"", "visible"=>true);

			break;
		}
		
		case "update_currencies" : 
		{
			if (!DEMO_LOCK)
			{
				$currencies->updateCurrencies($_POST);
			}
			$message = array("class"=>"ic-info", "text"=>"Currencies have been updated", "url"=>"", "visible"=>true);
			break;
		}
		
		case "delete_currency" : 
		{
			if (!DEMO_LOCK)
			{
				$currencies->deleteCurrency($currency_id);
			}
			$message = array("class"=>"ic-info", "text"=>"Currency has been removed", "url"=>"", "visible"=>true);
			break;
		}
		
		case "update_exchange_rates" : 
		{
			if (!DEMO_LOCK)
			{
				$currencies->updateExchangeRates();
			}
			$message = array("class"=>"ic-info", "text"=>"Exchange rates have been updated", "url"=>"", "visible"=>true);
			break;
		}
		
		case "set_default_language" : 
		{
			if (!DEMO_LOCK)
			{
				$default_language_id = isset($default_language_id)?intval($default_language_id):0;
				$lang->setDefaultLanguage($default_language_id);
				//$db->query("UPDATE ".DB_PREFIX."settings SET value='".$ActiveLanguage."' WHERE name='ActiveLanguage'");
				//$settings["ActiveLanguage"] = $ActiveLanguage;
			}
			break;
		}
		
		case "del_language" : 
		{
			if (!DEMO_LOCK)
			{
				$language_id = isset($language_id)?trim($language_id):"";
				$lang->deleteLanguage($language_id);
			}
			break;
		}
	}
	
	setApplicationDefaultTimezone($db, $settings);
	
	$_SESSION["admin_currency"] = $admin_currency = $currencies->getDefaultCurrency();

	$last_active_tab = isset($last_active_tab)?trim($last_active_tab):"settings";
	pageHeader("Site Local Settings", "", "icb-currency-sign-yen");
	
	if (count($currencies->errors) > 0)
	{
		$messages = array();
		foreach ($currencies->errors as $key=>$value)
		{
			$messages[] = array("class"=>"ic-cross", "text"=>"<b>".$value."</b>", "url"=>"", "visible"=>true);
		}
		pageNote($messages);
	}
	elseif ($message)
	{
		pageNote(array($message));
	}
	else
	{
		echo "&nbsp;";
	}

	$localizationTimezone = (trim($settings['LocalizationDefaultTimezone']) == '') ? gs(APP_SYSTEM_TIMEZONE) : gs($settings['LocalizationDefaultTimezone']);
	$localizationTimezoneCountry = '';
	if (strpos($localizationTimezone, '/'))
	{
		$parts = explode('/', $localizationTimezone);
		$localizationTimezoneCountry = $parts[0];
	}
?>
<style type="text/css">
	.region-option-group { display: none; }
	.region-option-group.active { display: inline; }
</style>
<div class="admin-tabs admin-tabs-framed" id="tabs-localization">
<?php 
	pageTabs(array(
		array("title"=>"Local Settings", "id"=>"settings"),
		array("title"=>"Currency", "id"=>"currency"),
		array("title"=>"Language", "id"=>"language"),
	), "tabs-localization", $last_active_tab);
?>
	<div id="admin-tab-sheet-settings" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<form id="localization-form" method="POST" action="admin.php?p=localization" style="padding:0px;spacing:0px;margin:0px;">
				<input type="hidden" name="last_active_tab" value="settings">
				<input type="hidden" name="action" value="save_settings">
				<input type="hidden" name="nonce" value="<?=create_nonce('localization_save_settings')?>" />
				<div class="admin-form-header first"><span class="icon ic-icon"></span>Date, Weight and Currency Formats</div>
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Local Time/Date Format</td>
						<td class="formItemControl">
							<input type="text" name="form[LocalizationDateTimeFormat]" value="<?=gs($settings["LocalizationDateTimeFormat"])?>">
							<div class="formHelp">
								Date/time format on the site. %Y-Year(yyyy), %m-Month(mm),%d-Day(dd), %r-Time(hh:mm:ss AM/PM);
								%T-Time(hh:mm:ss, 24 hours). Default value is %m/%d/%Y - %r (used MySQL notation)
							</div>
						</td>
					</tr>
					<tr valign="top" class="system_timezone">
						<td class="formItemCaption">Use System Timezone</td>
						<td class="formItemControl">
							<select id="use_system_timezone" name="form[LocalizationUseSystemTimezone]">
								<option value="Yes">Yes</option>
								<option value="No" <?=$settings["LocalizationUseSystemTimezone"]=="No"?"selected":""?>>No</option>
							</select>
							<div class="formHelp">
								All dates will be stored in the database using the system's current timezone which is <?=gs('('.$timezone_offset.') '.APP_SYSTEM_TIMEZONE)?>
							</div>
						</td>
					</tr>
					<tr valign="top" class="user_timezone">
						<td class="formItemCaption">Use This Timezone</td>
						<td class="formItemControl">
						
							<input id="default_timezone" type="hidden" name="form[LocalizationDefaultTimezone]" value="<?php echo gs($localizationTimezone); ?>" style="display:none;" />
						
							<select id="timezone_region" name="timezone_region">
								<option value="Africa" class="option-group-Africa"<?php echo ($localizationTimezoneCountry == 'Africa' ? ' selected="selected"' : ''); ?>>Africa</option>
								<option value="America" class="option-group-America"<?php echo ($localizationTimezoneCountry == 'America' ? ' selected="selected"' : ''); ?>>America</option>
								<option value="Antarctica" class="option-group-Antarctica"<?php echo ($localizationTimezoneCountry == 'Antarctica' ? ' selected="selected"' : ''); ?>>Antarctica</option>
								<option value="Asia" class="option-group-Asia"<?php echo ($localizationTimezoneCountry == 'Asia' ? ' selected="selected"' : ''); ?>>Asia</option>
								<option value="Atlantic" class="option-group-Atlantic"<?php echo ($localizationTimezoneCountry == 'Atlantic' ? ' selected="selected"' : ''); ?>>Atlantic</option>
								<option value="Europe" class="option-group-Europe"<?php echo ($localizationTimezoneCountry == 'Europe' ? ' selected="selected"' : ''); ?>>Europe</option>
								<option value="Etc" class="option-group-Etc"<?php echo ($localizationTimezoneCountry == 'Etc' ? ' selected="selected"' : ''); ?>>GMT Offset</option>
								<option value="Indian" class="option-group-Indian"<?php echo ($localizationTimezoneCountry == 'Indian' ? ' selected="selected"' : ''); ?>>Indian</option>
							</select>
							
							<select class="region-option-group Africa" name="LocalizationRegionsAfrica">
							    <option value="Africa/Abidjan">(GMT +00:00) Africa/Abidjan</option><option value="Africa/Accra">(GMT +00:00) Africa/Accra</option><option value="Africa/Addis_Ababa">(GMT +03:00) Africa/Addis Ababa</option><option value="Africa/Algiers">(GMT +01:00) Africa/Algiers</option><option value="Africa/Asmara">(GMT +03:00) Africa/Asmara</option><option value="Africa/Asmera">(GMT +03:00) Africa/Asmera</option><option value="Africa/Bamako">(GMT +00:00) Africa/Bamako</option><option value="Africa/Bangui">(GMT +01:00) Africa/Bangui</option><option value="Africa/Banjul">(GMT +00:00) Africa/Banjul</option><option value="Africa/Bissau">(GMT +00:00) Africa/Bissau</option><option value="Africa/Blantyre">(GMT +02:00) Africa/Blantyre</option><option value="Africa/Brazzaville">(GMT +01:00) Africa/Brazzaville</option><option value="Africa/Bujumbura">(GMT +02:00) Africa/Bujumbura</option><option value="Africa/Cairo">(GMT +03:00) Africa/Cairo</option><option value="Africa/Casablanca">(GMT +00:00) Africa/Casablanca</option><option value="Africa/Ceuta">(GMT +02:00) Africa/Ceuta</option><option value="Africa/Conakry">(GMT +00:00) Africa/Conakry</option><option value="Africa/Dakar">(GMT +00:00) Africa/Dakar</option><option value="Africa/Dar_es_Salaam">(GMT +03:00) Africa/Dar es Salaam</option><option value="Africa/Djibouti">(GMT +03:00) Africa/Djibouti</option><option value="Africa/Douala">(GMT +01:00) Africa/Douala</option><option value="Africa/El_Aaiun">(GMT +00:00) Africa/El Aaiun</option><option value="Africa/Freetown">(GMT +00:00) Africa/Freetown</option><option value="Africa/Gaborone">(GMT +02:00) Africa/Gaborone</option><option value="Africa/Harare">(GMT +02:00) Africa/Harare</option><option value="Africa/Johannesburg">(GMT +02:00) Africa/Johannesburg</option><option value="Africa/Kampala">(GMT +03:00) Africa/Kampala</option><option value="Africa/Khartoum">(GMT +03:00) Africa/Khartoum</option><option value="Africa/Kigali">(GMT +02:00) Africa/Kigali</option><option value="Africa/Kinshasa">(GMT +01:00) Africa/Kinshasa</option><option value="Africa/Lagos">(GMT +01:00) Africa/Lagos</option><option value="Africa/Libreville">(GMT +01:00) Africa/Libreville</option><option value="Africa/Lome">(GMT +00:00) Africa/Lome</option><option value="Africa/Luanda">(GMT +01:00) Africa/Luanda</option><option value="Africa/Lubumbashi">(GMT +02:00) Africa/Lubumbashi</option><option value="Africa/Lusaka">(GMT +02:00) Africa/Lusaka</option><option value="Africa/Malabo">(GMT +01:00) Africa/Malabo</option><option value="Africa/Maputo">(GMT +02:00) Africa/Maputo</option><option value="Africa/Maseru">(GMT +02:00) Africa/Maseru</option><option value="Africa/Mbabane">(GMT +02:00) Africa/Mbabane</option><option value="Africa/Mogadishu">(GMT +03:00) Africa/Mogadishu</option><option value="Africa/Monrovia">(GMT +00:00) Africa/Monrovia</option><option value="Africa/Nairobi">(GMT +03:00) Africa/Nairobi</option><option value="Africa/Ndjamena">(GMT +01:00) Africa/Ndjamena</option><option value="Africa/Niamey">(GMT +01:00) Africa/Niamey</option><option value="Africa/Nouakchott">(GMT +00:00) Africa/Nouakchott</option><option value="Africa/Ouagadougou">(GMT +00:00) Africa/Ouagadougou</option><option value="Africa/Porto-Novo">(GMT +01:00) Africa/Porto-Novo</option><option value="Africa/Sao_Tome">(GMT +00:00) Africa/Sao Tome</option><option value="Africa/Timbuktu">(GMT +00:00) Africa/Timbuktu</option><option value="Africa/Tripoli">(GMT +02:00) Africa/Tripoli</option><option value="Africa/Tunis">(GMT +02:00) Africa/Tunis</option><option value="Africa/Windhoek">(GMT +01:00) Africa/Windhoek</option>
							</select>
							
							<select class="region-option-group America" name="LocalizationRegionsAmerica">
								<option value="America/Adak">(GMT -09:00) America/Adak</option><option value="America/Anchorage">(GMT -08:00) America/Anchorage</option><option value="America/Anguilla">(GMT -04:00) America/Anguilla</option><option value="America/Antigua">(GMT -04:00) America/Antigua</option><option value="America/Araguaina">(GMT -03:00) America/Araguaina</option><option value="America/Argentina/Buenos_Aires">(GMT -03:00) America/Argentina/Buenos Aires</option><option value="America/Argentina/Catamarca">(GMT -03:00) America/Argentina/Catamarca</option><option value="America/Argentina/ComodRivadavia">(GMT -03:00) America/Argentina/ComodRivadavia</option><option value="America/Argentina/Cordoba">(GMT -03:00) America/Argentina/Cordoba</option><option value="America/Argentina/Jujuy">(GMT -03:00) America/Argentina/Jujuy</option><option value="America/Argentina/La_Rioja">(GMT -03:00) America/Argentina/La Rioja</option><option value="America/Argentina/Mendoza">(GMT -03:00) America/Argentina/Mendoza</option><option value="America/Argentina/Rio_Gallegos">(GMT -03:00) America/Argentina/Rio Gallegos</option><option value="America/Argentina/Salta">(GMT -03:00) America/Argentina/Salta</option><option value="America/Argentina/San_Juan">(GMT -03:00) America/Argentina/San Juan</option><option value="America/Argentina/San_Luis">(GMT -04:00) America/Argentina/San Luis</option><option value="America/Argentina/Tucuman">(GMT -03:00) America/Argentina/Tucuman</option><option value="America/Argentina/Ushuaia">(GMT -03:00) America/Argentina/Ushuaia</option><option value="America/Aruba">(GMT -04:00) America/Aruba</option><option value="America/Asuncion">(GMT -04:00) America/Asuncion</option><option value="America/Atikokan">(GMT -05:00) America/Atikokan</option><option value="America/Atka">(GMT -09:00) America/Atka</option><option value="America/Bahia">(GMT -03:00) America/Bahia</option><option value="America/Barbados">(GMT -04:00) America/Barbados</option><option value="America/Belem">(GMT -03:00) America/Belem</option><option value="America/Belize">(GMT -06:00) America/Belize</option><option value="America/Blanc-Sablon">(GMT -04:00) America/Blanc-Sablon</option><option value="America/Boa_Vista">(GMT -04:00) America/Boa Vista</option><option value="America/Bogota">(GMT -05:00) America/Bogota</option><option value="America/Boise">(GMT -06:00) America/Boise</option><option value="America/Buenos_Aires">(GMT -03:00) America/Buenos Aires</option><option value="America/Cambridge_Bay">(GMT -06:00) America/Cambridge Bay</option><option value="America/Campo_Grande">(GMT -04:00) America/Campo Grande</option><option value="America/Cancun">(GMT -05:00) America/Cancun</option><option value="America/Caracas">(GMT -04:30) America/Caracas</option><option value="America/Catamarca">(GMT -03:00) America/Catamarca</option><option value="America/Cayenne">(GMT -03:00) America/Cayenne</option><option value="America/Cayman">(GMT -05:00) America/Cayman</option><option value="America/Chicago">(GMT -05:00) America/Chicago</option><option value="America/Chihuahua">(GMT -06:00) America/Chihuahua</option><option value="America/Coral_Harbour">(GMT -05:00) America/Coral Harbour</option><option value="America/Cordoba">(GMT -03:00) America/Cordoba</option><option value="America/Costa_Rica">(GMT -06:00) America/Costa Rica</option><option value="America/Cuiaba">(GMT -04:00) America/Cuiaba</option><option value="America/Curacao">(GMT -04:00) America/Curacao</option><option value="America/Danmarkshavn">(GMT +00:00) America/Danmarkshavn</option><option value="America/Dawson">(GMT -07:00) America/Dawson</option><option value="America/Dawson_Creek">(GMT -07:00) America/Dawson Creek</option><option value="America/Denver">(GMT -06:00) America/Denver</option><option value="America/Detroit">(GMT -04:00) America/Detroit</option><option value="America/Dominica">(GMT -04:00) America/Dominica</option><option value="America/Edmonton">(GMT -06:00) America/Edmonton</option><option value="America/Eirunepe">(GMT -04:00) America/Eirunepe</option><option value="America/El_Salvador">(GMT -06:00) America/El Salvador</option><option value="America/Ensenada">(GMT -07:00) America/Ensenada</option><option value="America/Fort_Wayne">(GMT -04:00) America/Fort Wayne</option><option value="America/Fortaleza">(GMT -03:00) America/Fortaleza</option><option value="America/Glace_Bay">(GMT -03:00) America/Glace Bay</option><option value="America/Godthab">(GMT -02:00) America/Godthab</option><option value="America/Goose_Bay">(GMT -03:00) America/Goose Bay</option><option value="America/Grand_Turk">(GMT -04:00) America/Grand Turk</option><option value="America/Grenada">(GMT -04:00) America/Grenada</option><option value="America/Guadeloupe">(GMT -04:00) America/Guadeloupe</option><option value="America/Guatemala">(GMT -06:00) America/Guatemala</option><option value="America/Guayaquil">(GMT -05:00) America/Guayaquil</option><option value="America/Guyana">(GMT -04:00) America/Guyana</option><option value="America/Halifax">(GMT -03:00) America/Halifax</option><option value="America/Havana">(GMT -04:00) America/Havana</option><option value="America/Hermosillo">(GMT -07:00) America/Hermosillo</option><option value="America/Indiana/Indianapolis">(GMT -04:00) America/Indiana/Indianapolis</option><option value="America/Indiana/Knox">(GMT -05:00) America/Indiana/Knox</option><option value="America/Indiana/Marengo">(GMT -04:00) America/Indiana/Marengo</option><option value="America/Indiana/Petersburg">(GMT -04:00) America/Indiana/Petersburg</option><option value="America/Indiana/Tell_City">(GMT -05:00) America/Indiana/Tell City</option><option value="America/Indiana/Vevay">(GMT -04:00) America/Indiana/Vevay</option><option value="America/Indiana/Vincennes">(GMT -04:00) America/Indiana/Vincennes</option><option value="America/Indiana/Winamac">(GMT -04:00) America/Indiana/Winamac</option><option value="America/Indianapolis">(GMT -04:00) America/Indianapolis</option><option value="America/Inuvik">(GMT -06:00) America/Inuvik</option><option value="America/Iqaluit">(GMT -04:00) America/Iqaluit</option><option value="America/Jamaica">(GMT -05:00) America/Jamaica</option><option value="America/Jujuy">(GMT -03:00) America/Jujuy</option><option value="America/Juneau">(GMT -08:00) America/Juneau</option><option value="America/Kentucky/Louisville">(GMT -04:00) America/Kentucky/Louisville</option><option value="America/Kentucky/Monticello">(GMT -04:00) America/Kentucky/Monticello</option><option value="America/Knox_IN">(GMT -05:00) America/Knox IN</option><option value="America/La_Paz">(GMT -04:00) America/La Paz</option><option value="America/Lima">(GMT -05:00) America/Lima</option><option value="America/Los_Angeles">(GMT -07:00) America/Los Angeles</option><option value="America/Louisville">(GMT -04:00) America/Louisville</option><option value="America/Maceio">(GMT -03:00) America/Maceio</option><option value="America/Managua">(GMT -06:00) America/Managua</option><option value="America/Manaus">(GMT -04:00) America/Manaus</option><option value="America/Marigot">(GMT -04:00) America/Marigot</option><option value="America/Martinique">(GMT -04:00) America/Martinique</option><option value="America/Matamoros">(GMT -05:00) America/Matamoros</option><option value="America/Mazatlan">(GMT -06:00) America/Mazatlan</option><option value="America/Mendoza">(GMT -03:00) America/Mendoza</option><option value="America/Menominee">(GMT -05:00) America/Menominee</option><option value="America/Merida">(GMT -05:00) America/Merida</option><option value="America/Mexico_City">(GMT -05:00) America/Mexico City</option><option value="America/Miquelon">(GMT -02:00) America/Miquelon</option><option value="America/Moncton">(GMT -03:00) America/Moncton</option><option value="America/Monterrey">(GMT -05:00) America/Monterrey</option><option value="America/Montevideo">(GMT -03:00) America/Montevideo</option><option value="America/Montreal">(GMT -04:00) America/Montreal</option><option value="America/Montserrat">(GMT -04:00) America/Montserrat</option><option value="America/Nassau">(GMT -04:00) America/Nassau</option><option value="America/New_York">(GMT -04:00) America/New York</option><option value="America/Nipigon">(GMT -04:00) America/Nipigon</option><option value="America/Nome">(GMT -08:00) America/Nome</option><option value="America/Noronha">(GMT -02:00) America/Noronha</option><option value="America/North_Dakota/Center">(GMT -05:00) America/North Dakota/Center</option><option value="America/North_Dakota/New_Salem">(GMT -05:00) America/North_Dakota/New Salem</option><option value="America/Ojinaga">(GMT -06:00) America/Ojinaga</option><option value="America/Panama">(GMT -05:00) America/Panama</option><option value="America/Pangnirtung">(GMT -04:00) America/Pangnirtung</option><option value="America/Paramaribo">(GMT -03:00) America/Paramaribo</option><option value="America/Phoenix">(GMT -07:00) America/Phoenix</option><option value="America/Port-au-Prince">(GMT -05:00) America/Port-au-Prince</option><option value="America/Port_of_Spain">(GMT -04:00) America/Port of Spain</option><option value="America/Porto_Acre">(GMT -04:00) America/Porto Acre</option><option value="America/Porto_Velho">(GMT -04:00) America/Porto Velho</option><option value="America/Puerto_Rico">(GMT -04:00) America/Puerto Rico</option><option value="America/Rainy_River">(GMT -05:00) America/Rainy River</option><option value="America/Rankin_Inlet">(GMT -05:00) America/Rankin Inlet</option><option value="America/Recife">(GMT -03:00) America/Recife</option><option value="America/Regina">(GMT -06:00) America/Regina</option><option value="America/Resolute">(GMT -05:00) America/Resolute</option><option value="America/Rio_Branco">(GMT -04:00) America/Rio Branco</option><option value="America/Rosario">(GMT -03:00) America/Rosario</option><option value="America/Santa_Isabel">(GMT -07:00) America/Santa Isabel</option><option value="America/Santarem">(GMT -03:00) America/Santarem</option><option value="America/Santiago">(GMT -04:00) America/Santiago</option><option value="America/Santo_Domingo">(GMT -04:00) America/Santo Domingo</option><option value="America/Sao_Paulo">(GMT -03:00) America/Sao Paulo</option><option value="America/Scoresbysund">(GMT +00:00) America/Scoresbysund</option><option value="America/Shiprock">(GMT -06:00) America/Shiprock</option><option value="America/St_Barthelemy">(GMT -04:00) America/St Barthelemy</option><option value="America/St_Johns">(GMT -02:30) America/St Johns</option><option value="America/St_Kitts">(GMT -04:00) America/St Kitts</option><option value="America/St_Lucia">(GMT -04:00) America/St Lucia</option><option value="America/St_Thomas">(GMT -04:00) America/St Thomas</option><option value="America/St_Vincent">(GMT -04:00) America/St Vincent</option><option value="America/Swift_Current">(GMT -06:00) America/Swift Current</option><option value="America/Tegucigalpa">(GMT -06:00) America/Tegucigalpa</option><option value="America/Thule">(GMT -03:00) America/Thule</option><option value="America/Thunder_Bay">(GMT -04:00) America/Thunder Bay</option><option value="America/Tijuana">(GMT -07:00) America/Tijuana</option><option value="America/Toronto">(GMT -04:00) America/Toronto</option><option value="America/Tortola">(GMT -04:00) America/Tortola</option><option value="America/Vancouver">(GMT -07:00) America/Vancouver</option><option value="America/Virgin">(GMT -04:00) America/Virgin</option><option value="America/Whitehorse">(GMT -07:00) America/Whitehorse</option><option value="America/Winnipeg">(GMT -05:00) America/Winnipeg</option><option value="America/Yakutat">(GMT -08:00) America/Yakutat</option><option value="America/Yellowknife">(GMT -06:00) America/Yellowknife</option>
							</select>
							<select class="region-option-group Antarctica" name="LocalizationRegionsAntarctica">
								<option value="Antarctica/Casey">(GMT +11:00) Antarctica/Casey</option><option value="Antarctica/Davis">(GMT +05:00) Antarctica/Davis</option><option value="Antarctica/DumontDUrville">(GMT +10:00) Antarctica/DumontDUrville</option><option value="Antarctica/Mawson">(GMT +05:00) Antarctica/Mawson</option><option value="Antarctica/McMurdo">(GMT +12:00) Antarctica/McMurdo</option><option value="Antarctica/Palmer">(GMT -04:00) Antarctica/Palmer</option><option value="Antarctica/Rothera">(GMT -03:00) Antarctica/Rothera</option><option value="Antarctica/South_Pole">(GMT +12:00) Antarctica/South Pole</option><option value="Antarctica/Syowa">(GMT +03:00) Antarctica/Syowa</option><option value="Antarctica/Vostok">(GMT +06:00) Antarctica/Vostok</option>
							</select>
							<select class="region-option-group Asia" name="LocalizationRegionsAsia">
								<option value="Asia/Aden">(GMT +03:00) Asia/Aden</option><option value="Asia/Almaty">(GMT +06:00) Asia/Almaty</option><option value="Asia/Amman">(GMT +03:00) Asia/Amman</option><option value="Asia/Anadyr">(GMT +13:00) Asia/Anadyr</option><option value="Asia/Aqtau">(GMT +05:00) Asia/Aqtau</option><option value="Asia/Aqtobe">(GMT +05:00) Asia/Aqtobe</option><option value="Asia/Ashgabat">(GMT +05:00) Asia/Ashgabat</option><option value="Asia/Ashkhabad">(GMT +05:00) Asia/Ashkhabad</option><option value="Asia/Baghdad">(GMT +03:00) Asia/Baghdad</option><option value="Asia/Bahrain">(GMT +03:00) Asia/Bahrain</option><option value="Asia/Baku">(GMT +05:00) Asia/Baku</option><option value="Asia/Bangkok">(GMT +07:00) Asia/Bangkok</option><option value="Asia/Beirut">(GMT +03:00) Asia/Beirut</option><option value="Asia/Bishkek">(GMT +06:00) Asia/Bishkek</option><option value="Asia/Brunei">(GMT +08:00) Asia/Brunei</option><option value="Asia/Calcutta">(GMT +05:30) Asia/Calcutta</option><option value="Asia/Choibalsan">(GMT +08:00) Asia/Choibalsan</option><option value="Asia/Chongqing">(GMT +08:00) Asia/Chongqing</option><option value="Asia/Chungking">(GMT +08:00) Asia/Chungking</option><option value="Asia/Colombo">(GMT +05:30) Asia/Colombo</option><option value="Asia/Dacca">(GMT +07:00) Asia/Dacca</option><option value="Asia/Damascus">(GMT +03:00) Asia/Damascus</option><option value="Asia/Dhaka">(GMT +07:00) Asia/Dhaka</option><option value="Asia/Dili">(GMT +09:00) Asia/Dili</option><option value="Asia/Dubai">(GMT +04:00) Asia/Dubai</option><option value="Asia/Dushanbe">(GMT +05:00) Asia/Dushanbe</option><option value="Asia/Gaza">(GMT +03:00) Asia/Gaza</option><option value="Asia/Harbin">(GMT +08:00) Asia/Harbin</option><option value="Asia/Ho_Chi_Minh">(GMT +07:00) Asia/Ho Chi Minh</option><option value="Asia/Hong_Kong">(GMT +08:00) Asia/Hong Kong</option><option value="Asia/Hovd">(GMT +07:00) Asia/Hovd</option><option value="Asia/Irkutsk">(GMT +09:00) Asia/Irkutsk</option><option value="Asia/Istanbul">(GMT +03:00) Asia/Istanbul</option><option value="Asia/Jakarta">(GMT +07:00) Asia/Jakarta</option><option value="Asia/Jayapura">(GMT +09:00) Asia/Jayapura</option><option value="Asia/Jerusalem">(GMT +03:00) Asia/Jerusalem</option><option value="Asia/Kabul">(GMT +04:30) Asia/Kabul</option><option value="Asia/Kamchatka">(GMT +13:00) Asia/Kamchatka</option><option value="Asia/Karachi">(GMT +06:00) Asia/Karachi</option><option value="Asia/Kashgar">(GMT +08:00) Asia/Kashgar</option><option value="Asia/Kathmandu">(GMT +05:45) Asia/Kathmandu</option><option value="Asia/Katmandu">(GMT +05:45) Asia/Katmandu</option><option value="Asia/Kolkata">(GMT +05:30) Asia/Kolkata</option><option value="Asia/Krasnoyarsk">(GMT +08:00) Asia/Krasnoyarsk</option><option value="Asia/Kuala_Lumpur">(GMT +08:00) Asia/Kuala Lumpur</option><option value="Asia/Kuching">(GMT +08:00) Asia/Kuching</option><option value="Asia/Kuwait">(GMT +03:00) Asia/Kuwait</option><option value="Asia/Macao">(GMT +08:00) Asia/Macao</option><option value="Asia/Macau">(GMT +08:00) Asia/Macau</option><option value="Asia/Magadan">(GMT +12:00) Asia/Magadan</option><option value="Asia/Makassar">(GMT +08:00) Asia/Makassar</option><option value="Asia/Manila">(GMT +08:00) Asia/Manila</option><option value="Asia/Muscat">(GMT +04:00) Asia/Muscat</option><option value="Asia/Nicosia">(GMT +03:00) Asia/Nicosia</option><option value="Asia/Novokuznetsk">(GMT +07:00) Asia/Novokuznetsk</option><option value="Asia/Novosibirsk">(GMT +07:00) Asia/Novosibirsk</option><option value="Asia/Omsk">(GMT +07:00) Asia/Omsk</option><option value="Asia/Oral">(GMT +05:00) Asia/Oral</option><option value="Asia/Phnom_Penh">(GMT +07:00) Asia/Phnom Penh</option><option value="Asia/Pontianak">(GMT +07:00) Asia/Pontianak</option><option value="Asia/Pyongyang">(GMT +09:00) Asia/Pyongyang</option><option value="Asia/Qatar">(GMT +03:00) Asia/Qatar</option><option value="Asia/Qyzylorda">(GMT +06:00) Asia/Qyzylorda</option><option value="Asia/Rangoon">(GMT +06:30) Asia/Rangoon</option><option value="Asia/Riyadh">(GMT +03:00) Asia/Riyadh</option><option value="Asia/Saigon">(GMT +07:00) Asia/Saigon</option><option value="Asia/Sakhalin">(GMT +11:00) Asia/Sakhalin</option><option value="Asia/Samarkand">(GMT +05:00) Asia/Samarkand</option><option value="Asia/Seoul">(GMT +09:00) Asia/Seoul</option><option value="Asia/Shanghai">(GMT +08:00) Asia/Shanghai</option><option value="Asia/Singapore">(GMT +08:00) Asia/Singapore</option><option value="Asia/Taipei">(GMT +08:00) Asia/Taipei</option><option value="Asia/Tashkent">(GMT +05:00) Asia/Tashkent</option><option value="Asia/Tbilisi">(GMT +04:00) Asia/Tbilisi</option><option value="Asia/Tehran">(GMT +04:30) Asia/Tehran</option><option value="Asia/Tel_Aviv">(GMT +03:00) Asia/Tel Aviv</option><option value="Asia/Thimbu">(GMT +06:00) Asia/Thimbu</option><option value="Asia/Thimphu">(GMT +06:00) Asia/Thimphu</option><option value="Asia/Tokyo">(GMT +09:00) Asia/Tokyo</option><option value="Asia/Ujung_Pandang">(GMT +08:00) Asia/Ujung Pandang</option><option value="Asia/Ulaanbaatar">(GMT +08:00) Asia/Ulaanbaatar</option><option value="Asia/Ulan_Bator">(GMT +08:00) Asia/Ulan Bator</option><option value="Asia/Urumqi">(GMT +08:00) Asia/Urumqi</option><option value="Asia/Vientiane">(GMT +07:00) Asia/Vientiane</option><option value="Asia/Vladivostok">(GMT +11:00) Asia/Vladivostok</option><option value="Asia/Yakutsk">(GMT +10:00) Asia/Yakutsk</option><option value="Asia/Yekaterinburg">(GMT +06:00) Asia/Yekaterinburg</option><option value="Asia/Yerevan">(GMT +05:00) Asia/Yerevan</option>
							</select>
							
							<select class="region-option-group Atlantic" name="LocalizationRegionsAtlantic">
							    <option value="Atlantic/Azores">(GMT +00:00) Atlantic/Azores</option><option value="Atlantic/Bermuda">(GMT -03:00) Atlantic/Bermuda</option><option value="Atlantic/Canary">(GMT +01:00) Atlantic/Canary</option><option value="Atlantic/Cape_Verde">(GMT -01:00) Atlantic/Cape Verde</option><option value="Atlantic/Faeroe">(GMT +01:00) Atlantic/Faeroe</option><option value="Atlantic/Faroe">(GMT +01:00) Atlantic/Faroe</option><option value="Atlantic/Jan_Mayen">(GMT +02:00) Atlantic/Jan Mayen</option><option value="Atlantic/Madeira">(GMT +01:00) Atlantic/Madeira</option><option value="Atlantic/Reykjavik">(GMT +00:00) Atlantic/Reykjavik</option><option value="Atlantic/South_Georgia">(GMT -02:00) Atlantic/South Georgia</option><option value="Atlantic/St_Helena">(GMT +00:00) Atlantic/St Helena</option><option value="Atlantic/Stanley">(GMT -04:00) Atlantic/Stanley</option>
							</select>
							
							<select class="region-option-group Europe" name="LocalizationRegionsEurope">
								<option value="Europe/Amsterdam">(GMT +02:00) Europe/Amsterdam</option><option value="Europe/Andorra">(GMT +02:00) Europe/Andorra</option><option value="Europe/Athens">(GMT +03:00) Europe/Athens</option><option value="Europe/Belfast">(GMT +01:00) Europe/Belfast</option><option value="Europe/Belgrade">(GMT +02:00) Europe/Belgrade</option><option value="Europe/Berlin">(GMT +02:00) Europe/Berlin</option><option value="Europe/Bratislava">(GMT +02:00) Europe/Bratislava</option><option value="Europe/Brussels">(GMT +02:00) Europe/Brussels</option><option value="Europe/Bucharest">(GMT +03:00) Europe/Bucharest</option><option value="Europe/Budapest">(GMT +02:00) Europe/Budapest</option><option value="Europe/Chisinau">(GMT +03:00) Europe/Chisinau</option><option value="Europe/Copenhagen">(GMT +02:00) Europe/Copenhagen</option><option value="Europe/Dublin">(GMT +01:00) Europe/Dublin</option><option value="Europe/Gibraltar">(GMT +02:00) Europe/Gibraltar</option><option value="Europe/Guernsey">(GMT +01:00) Europe/Guernsey</option><option value="Europe/Helsinki">(GMT +03:00) Europe/Helsinki</option><option value="Europe/Isle_of_Man">(GMT +01:00) Europe/Isle of Man</option><option value="Europe/Istanbul">(GMT +03:00) Europe/Istanbul</option><option value="Europe/Jersey">(GMT +01:00) Europe/Jersey</option><option value="Europe/Kaliningrad">(GMT +03:00) Europe/Kaliningrad</option><option value="Europe/Kiev">(GMT +03:00) Europe/Kiev</option><option value="Europe/Lisbon">(GMT +01:00) Europe/Lisbon</option><option value="Europe/Ljubljana">(GMT +02:00) Europe/Ljubljana</option><option value="Europe/London">(GMT +01:00) Europe/London</option><option value="Europe/Luxembourg">(GMT +02:00) Europe/Luxembourg</option><option value="Europe/Madrid">(GMT +02:00) Europe/Madrid</option><option value="Europe/Malta">(GMT +02:00) Europe/Malta</option><option value="Europe/Mariehamn">(GMT +03:00) Europe/Mariehamn</option><option value="Europe/Minsk">(GMT +03:00) Europe/Minsk</option><option value="Europe/Monaco">(GMT +02:00) Europe/Monaco</option><option value="Europe/Moscow">(GMT +04:00) Europe/Moscow</option><option value="Europe/Nicosia">(GMT +03:00) Europe/Nicosia</option><option value="Europe/Oslo">(GMT +02:00) Europe/Oslo</option><option value="Europe/Paris">(GMT +02:00) Europe/Paris</option><option value="Europe/Podgorica">(GMT +02:00) Europe/Podgorica</option><option value="Europe/Prague">(GMT +02:00) Europe/Prague</option><option value="Europe/Riga">(GMT +03:00) Europe/Riga</option><option value="Europe/Rome">(GMT +02:00) Europe/Rome</option><option value="Europe/Samara">(GMT +05:00) Europe/Samara</option><option value="Europe/San_Marino">(GMT +02:00) Europe/San Marino</option><option value="Europe/Sarajevo">(GMT +02:00) Europe/Sarajevo</option><option value="Europe/Simferopol">(GMT +03:00) Europe/Simferopol</option><option value="Europe/Skopje">(GMT +02:00) Europe/Skopje</option><option value="Europe/Sofia">(GMT +03:00) Europe/Sofia</option><option value="Europe/Stockholm">(GMT +02:00) Europe/Stockholm</option><option value="Europe/Tallinn">(GMT +03:00) Europe/Tallinn</option><option value="Europe/Tirane">(GMT +02:00) Europe/Tirane</option><option value="Europe/Tiraspol">(GMT +03:00) Europe/Tiraspol</option><option value="Europe/Uzhgorod">(GMT +03:00) Europe/Uzhgorod</option><option value="Europe/Vaduz">(GMT +02:00) Europe/Vaduz</option><option value="Europe/Vatican">(GMT +02:00) Europe/Vatican</option><option value="Europe/Vienna">(GMT +02:00) Europe/Vienna</option><option value="Europe/Vilnius">(GMT +03:00) Europe/Vilnius</option><option value="Europe/Volgograd">(GMT +04:00) Europe/Volgograd</option><option value="Europe/Warsaw">(GMT +02:00) Europe/Warsaw</option><option value="Europe/Zagreb">(GMT +02:00) Europe/Zagreb</option><option value="Europe/Zaporozhye">(GMT +03:00) Europe/Zaporozhye</option><option value="Europe/Zurich">(GMT +02:00) Europe/Zurich</option>
							</select>
							
							<select class="region-option-group Etc" name="LocalizationRegionsEtc">
								<option value="Etc/GMT-12">(GMT +12:00) Etc/GMT-12</option><option value="Etc/GMT-11">(GMT +11:00) Etc/GMT-11</option><option value="Etc/GMT-10">(GMT +10:00) Etc/GMT-10</option><option value="Etc/GMT-9">(GMT +09:00) Etc/GMT-9</option><option value="Etc/GMT-8">(GMT +08:00) Etc/GMT-8</option><option value="Etc/GMT-7">(GMT +07:00) Etc/GMT-7</option><option value="Etc/GMT-6">(GMT +06:00) Etc/GMT-6</option><option value="Etc/GMT-5">(GMT +05:00) Etc/GMT-5</option><option value="Etc/GMT-4">(GMT +04:00) Etc/GMT-4</option><option value="Etc/GMT-3">(GMT +03:00) Etc/GMT-3</option><option value="Etc/GMT-2">(GMT +02:00) Etc/GMT-2</option><option value="Etc/GMT-1">(GMT +01:00) Etc/GMT-1</option><option value="Etc/GMT-0">(GMT +00:00) Etc/GMT-0</option><option value="Etc/GMT+0">(GMT +00:00) Etc/GMT+0</option><option value="Etc/GMT+1">(GMT -01:00) Etc/GMT+1</option><option value="Etc/GMT+2">(GMT -02:00) Etc/GMT+2</option><option value="Etc/GMT+3">(GMT -03:00) Etc/GMT+3</option><option value="Etc/GMT+4">(GMT -04:00) Etc/GMT+4</option><option value="Etc/GMT+5">(GMT -05:00) Etc/GMT+5</option><option value="Etc/GMT+6">(GMT -06:00) Etc/GMT+6</option><option value="Etc/GMT+7">(GMT -07:00) Etc/GMT+7</option><option value="Etc/GMT+8">(GMT -08:00) Etc/GMT+8</option><option value="Etc/GMT+9">(GMT -09:00) Etc/GMT+9</option><option value="Etc/GMT+10">(GMT -10:00) Etc/GMT+10</option><option value="Etc/GMT+11">(GMT -11:00) Etc/GMT+11</option><option value="Etc/GMT+12">(GMT -12:00) Etc/GMT+12</option>
							</select>
							
							<select class="region-option-group Indian" name="LocalizationRegionsIndian">
								<option value="Indian/Antananarivo">(GMT +03:00) Indian/Antananarivo</option><option value="Indian/Chagos">(GMT +06:00) Indian/Chagos</option><option value="Indian/Christmas">(GMT +07:00) Indian/Christmas</option><option value="Indian/Cocos">(GMT +06:30) Indian/Cocos</option><option value="Indian/Comoro">(GMT +03:00) Indian/Comoro</option><option value="Indian/Kerguelen">(GMT +05:00) Indian/Kerguelen</option><option value="Indian/Mahe">(GMT +04:00) Indian/Mahe</option><option value="Indian/Maldives">(GMT +05:00) Indian/Maldives</option><option value="Indian/Mauritius">(GMT +04:00) Indian/Mauritius</option><option value="Indian/Mayotte">(GMT +03:00) Indian/Mayotte</option><option value="Indian/Reunion">(GMT +04:00) Indian/Reunion</option>
							</select>
							<div class="formHelp">
								Use this timezone for all dates stored in the database if "Use System Timezone" has been set to "No". Note: This change is not retroactive.
							</div>
						</td>
					</tr>
					<tr>
						<td class="formItemCaption">Weight Unit</td>
						<td class="formItemControl">
							<select name="form[LocalizationWeightUnits]">
								<option value="lbs">lbs</option>
								<option value="kg" <?=$settings["LocalizationWeightUnits"]=="kg"?"selected":""?>>kg</option>
							</select>
						</td>
					</tr>
					<tr>
						<td class="formItemCaption">Length Unit</td>
						<td class="formItemControl">
							<select name="form[LocalizationLengthUnits]">
								<option value="inches">Inches</option>
								<option value="feet" <?=$settings["LocalizationLengthUnits"]=="feet"?"selected":""?>>Feet</option>
								<option value="cm" <?=$settings["LocalizationLengthUnits"]=="cm"?"selected":""?>>Centimeters</option>
							</select>
						</td>
					</tr>
					<tr>
						<td class="formItemCaption">Currency Decimal Symbol</td>
						<td class="formItemControl">
							<input type="text" name="form[LocalizationCurrencyDecimalSymbol]" value="<?=gs($settings["LocalizationCurrencyDecimalSymbol"])?>">
						</td>
					</tr>
					<tr>
						<td class="formItemCaption">Currency Thousands Separating Symbol</td>
						<td class="formItemControl"><input type="text" name="form[LocalizationCurrencySeparatingSymbol]" value="<?=gs($settings["LocalizationCurrencySeparatingSymbol"])?>"></td>
					</tr>
				</table>
				<div class="admin-form-buttons localization">
					<input type="submit" value="Save changes" class="submit" />
					<input type="reset" value="Reset form" class="reset"/>
				</div>
			</form>
		</div>
	</div>
	<div id="admin-tab-sheet-currency" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-tabs admin-tabs-framed" id="tabs-currencies">
<?php 
	pageTabs(array(
		array("title"=>"Currencies In Use", "id"=>"currencies-in-use"),
		array("title"=>"New Currency", "id"=>"new-currency"),
	), "tabs-currencies", false);
?>
				<div id="admin-tab-sheet-currencies-in-use" class="admin-tab-sheet hidden">
					<div class="admin-tab-sheet-wrap">
						<div class="admin-form-header first"><span class="icon ic-icon"></span>Currencies in Use</div>
						<form action="admin.php?p=localization" name="frmUpdateCurrency" method="POST">
							<input type="hidden" name="last_active_tab" value="currency">
							<input type="hidden" name="action" value="update_currencies">
							<input type="hidden" name="nonce" value="<?=create_nonce('localization_update_currencies')?>" />
							<table cellpadding="0" cellspacing="0" border="0" width="100%" class="admin-list-table">
								<thead>
									<tr>
										<th class="icon">&nbsp;</th>
										<th style="width:145px;">Code</th>
										<th style="width:145px;">Title</th>
										<th style="width:150px;">Exch. Rate</th>
										<th style="width:100px;">Decimal</th>
										<th style="width:100px;">Left</th>
										<th style="width:100px;">Right</th>	
										<th>&nbsp;</th>
									</tr>
								</thead>
								<tbody>
<?php
	$db->query("SELECT * FROM ".DB_PREFIX."currencies ORDER BY is_default, title");
	if ($db->numRows() > 0)
	{
		while ($db->moveNext())
		{
?>
									<tr>
										<td class="icon"><input type="radio" name="is_default" value="<?=$db->col["currency_id"]?>" <?=$db->col["is_default"] == "Yes" ? "checked" : ""?>></td>
										<td style="width:145px;"><input type="text" name="code[<?=$db->col["currency_id"]?>]" style="width:95%;" value="<?=gs($db->col["code"])?>"></td>
										<td style="width:145px;"><input type="text" name="title[<?=$db->col["currency_id"]?>]" style="width:95%;" value="<?=gs($db->col["title"])?>"></td>
										<td style="width:150px;"><input type="text" name="exchange_rate[<?=$db->col["currency_id"]?>]" style="width:95%;" value="<?=gs($db->col["exchange_rate"])?>"></td>
										<td style="width:100px;">
											<select name="decimal_places[<?=$db->col["currency_id"]?>]" style="width:95%;">
												<option value="2">2</option>
												<option value="1" <?=$db->col["decimal_places"]==1?"selected":""?>>1</option>
												<option value="0" <?=$db->col["decimal_places"]==0?"selected":""?>>0</option>
											</select>
										</td>
										<td style="width:100px;"><input type="text" name="symbol_left[<?=$db->col["currency_id"]?>]" style="width:95%;" value="<?=gs($db->col["symbol_left"])?>"></td>
										<td style="width:100px;"><input type="text" name="symbol_right[<?=$db->col["currency_id"]?>]" style="width:95%;" value="<?=gs($db->col["symbol_right"])?>"></td>
										<td>
<?php
			if ($db->col["is_default"] == "No")
			{
?>
											<a href="javascript:if(confirm('Do you really want to delete this currency?'))document.location='admin.php?p=localization&action=delete_currency&last_active_tab=currency&nonce=<?=create_nonce('localization_delete')?>&currency_id=<?=$db->col["currency_id"]?>';"><img src="images/icons/a/delete.png" border="0" alt="Delete Currency"></a>
<?php
			}
			else
			{
				echo "&nbsp;";
			}
?>
										</td>
									</tr>
<?php
		}
?>									
								</tbody>
							</table>
							<div class="admin-form-buttons">
								<input type="submit" value="Save changes"/>
								<input type="reset" value="Reset form"/>
							</div>
						</form>
						<div style="padding-left:26px;padding-bottom:15px;">
							<b><a href="admin.php?p=localization&action=update_exchange_rates&last_active_tab=currency&nonce=<?=create_nonce('localization_update_exchange_rates')?>">Update Exchange Rates</a></b><br>
							Please note: Currency exchange rates uses European Central Bank rates. For more information please visit 
							<a target="_blank" href="http://www.ecb.int/stats/exchange/eurofxref/html/index.en.html">http://www.ecb.int/stats/exchange/eurofxref/html/index.en.html</a>.<br>
							Remember to update exchange rates after the default currency has changed or after a new currency has been added. 
						</div>
<?php
		}
?>
					</div>
				</div>
				<div id="admin-tab-sheet-new-currency" class="admin-tab-sheet hidden">
					<div class="admin-tab-sheet-wrap">
						<div class="admin-form-header first"><span class="icon ic-icon"></span>Add New Currency</div>
						<form action="admin.php?p=localization" name="frmNewCurrency" method="POST">
							<input type="hidden" name="last_active_tab" value="currency">
							<input type="hidden" name="action" value="add_standard_currency">
							<input type="hidden" name="nonce" value="<?=create_nonce('localization_add_standard_currency')?>" />
				
							<table width="100%" cellpadding="0" cellspacing="0" border="0" class="admin-list-table">
								<tr>
									<td class="icon">&nbsp;</td>
									<td colspan="3">
										Use the drop-down to select the currency you want to add to your cart.
										If you want to add a custom currency, use the form below.
									</td>
								</tr>
								<tr>
									<td class="icon">&nbsp;</td>
									<td style="width:300px;">
										Select currency:<br/>
										<select name="code" style="width:100%;">
<?php
	foreach ($currency_list as $code=>$data)
	{
		echo '<option value="'.$code.'">'.$code." - ".$data[0].'</option>';
	}
?>
										</select>
									</td>
									<td style="width:150px;">
										Exchange rate:
										<input type="text" name="exchange_rate" style="width:100%;" value="1.0000000000">
									</td>
									<td>&nbsp;<br/><input type="submit" value=" Add "/></td>
								</tr>
							</table>
						</form>
						&nbsp;
						<form action="admin.php?p=localization" name="frmNewCurrency" method="POST" onsubmit="return checkAddCurrency(this)">
							<input type="hidden" name="last_active_tab" value="currency">
							<input type="hidden" name="action" value="add_currency">
							<input type="hidden" name="nonce" value="<?=create_nonce('localization_add_currency')?>" />
							<div class="admin-form-header"><span class="icon ic-icon"></span>Add Custom Currency</div>
							<table width="100%" cellpadding="0" cellspacing="0" border="0" class="admin-list-table">
								<tr>
									<td class="icon">&nbsp;</td>
									<td style="width:145px;">Code<br/><input type="text" name="code" style="width:95%;"></td>
									<td style="width:145px;">Title<br/><input type="text" name="title" style="width:95%;"></td>
									<td style="width:150px;">Exch. Rate<br/><input type="text" name="exchange_rate" style="width:95%;" value="1.0000000000"></td>
									<td style="width:100px;">
										Decimal<br/>
										<select name="decimal_places" style="width:95%;">
											<option value="2">2</option>
											<option value="1">1</option>
											<option value="0">0</option>
										</select>
									</td>
									<td style="width:100px;">Left<br/><input type="text" name="symbol_left" style="width:95%;"></td>
									<td style="width:100px;">Right<br/><input type="text" name="symbol_right" style="width:95%;"></td>
									<td>&nbsp;<br/><input type="submit" value=" Add " class="small"></td>
								</tr>
							</table>
						</form>
						&nbsp;
					</div>
				</div>
			</div>
		</div>
	</div>
	<div id="admin-tab-sheet-language" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon"></span>Site Language Settings</div>
<?php
	pageNote(
		array(
			array("class"=>"ic-info","url"=>"","text"=>"Please select which language template you would like to use on your site.<br/>If you would like to create a new template, click \"Add a New Language Template\" below.<br/>If you would like to edit or change text in an existing template, click the edit icon.", "visible"=>true),
			array("class"=>"ic-info","url"=>"","text"=>"Additional language templates can be found at <a href='http://cartmanual.com/languages/languages.html'>http://cartmanual.com/languages/languages.html</a>", "visible"=>true),
			array("class"=>"ic-add","url"=>"admin.php?p=language&mode=insert","text"=>"<b>Add a New Language Template</b>", "visible"=>true)
		)
	);
	loadJS("languages.js");
	$lang->getLanguages();
	
?>
			<form method="post" action="admin.php?p=localization">
				<input type="hidden" name="last_active_tab" value="language">
				<input type="hidden" name="action" value="set_default_language">
				<input type="hidden" name="nonce" value="<?=create_nonce('localization_set_default_language')?>" />
				<table width="100%" cellpadding="0" cellspacing="0" border="0" class="admin-list-table">
					<thead>
						<tr> 
							<th class="icon">&nbsp;</th>
							<th style="width:200px;">Language</th>
							<th style="width:130px;">Enabled</th>
							<th>&nbsp;</th>
						</tr>
					</thead>
					<tbody>
<?php
	foreach ($lang->languages as $key=>$lang_data)
	{
?>
						<tr valign="top">
							<td class="icon">
								<input type="radio" name="default_language_id" value="<?=$lang_data["language_id"]?>" <?=$lang_data["is_default"]=="Yes"?"checked":""?>/>
							</td>
							<td>
								<a href="admin.php?p=language&mode=update&language_id=<?=$lang_data["language_id"]?>"><?=htmlspecialchars($lang_data["name"])?></a>
							</td>
							<td><?=$lang_data["is_active"]?></td>
							<td>
								<a href="admin.php?p=language&mode=update&language_id=<?=$lang_data["language_id"]?>"><img src="images/icons/a/pencil.png" border="0" title="Edit Language"></a>
								<?if($lang_data["is_default"]!="Yes"){?>
								<input type="hidden" id="language-delete-<?=$lang_data["language_id"]?>" value="<?=create_nonce('language_del_language')?>" />
								<a href="javascript:ConfirmDeleteLanguage('<?=$lang_data["name"]?>', '<?=$lang_data["language_id"]?>', <?=$lang_data["language_id"]==$settings["ActiveLanguage"]?"true":"false"?>)"><img src="images/icons/a/delete.png" border="0" title="Delete Language"></a>
								<?}?>
							</td>
						</tr>				
<?php
	}
?>
					</tbody>
				</table>
				<div class="admin-form-buttons">
					<input type="submit" value="Save changes"/>
					<input type="reset" value="Reset form"/>
				</div>
			</form>
		</div>
	</div>
</div>

<script type="text/javascript">
	$(document).ready(function(){
		// toggle regions on initial page load
		toggleRegionGroup();
		toggleTimezoneFields();
		
		// change event
		$('#use_system_timezone').change(function(){
			toggleTimezoneFields();
		});
		
		// event handler on form submission
		$('#localization-form').submit(function(){
			if ($('#use_system_timezone').val() == 'No')
			{
				$('#default_timezone').val($('.region-option-group.active').val());
			}
		});
		
		// event handler on region dropdown change
		$('#timezone_region').change(function(){
			toggleRegionGroup();
		});
	});
	
	function toggleTimezoneFields()
	{
		if ($('#use_system_timezone').val() == 'No')
		{
			$('tr.user_timezone').show();
		}
		else
		{
			$('tr.user_timezone').hide();
		}
	}
	
	function toggleRegionGroup()
	{
		$('select.region-option-group').removeClass('active');
		var currentRegion = $('#timezone_region').val();
		$('select.region-option-group.'+currentRegion).addClass('active');
		
		var selectedIndex = $('select.region-option-group.active option[value="'+$('#default_timezone').val()+'"]');
		if (selectedIndex.length)
		{
			selectedIndex.attr('selected', 'selected');
		}
	}
</script>