<?php
class AddressValidator
{
	protected static $lastError = null;
	protected static $addressVariants = null;

	public static function setLastError($error)
	{
		self::$lastError = $error;
	}

	public static function getLastError()
	{
		return self::$lastError;
	}

	public static function setAddressVariants($addressVariants)
	{
		self::$addressVariants = $addressVariants;
	}

	public static function getAddressVariants()
	{
		return self::$addressVariants;
	}

	/**
	 * @param DB $db
	 * @param $addressToValidate
	 * @param $msg
	 *
	 * @return bool
	 */
	public static function validate(&$db, &$addressToValidate, $msg)
	{
		if (!is_null($addressToValidate))
		{
			self::$addressVariants = null;
			self::$lastError = null;

			$addressToValidate['name'] = isset($addressToValidate['name']) ? $addressToValidate['name'] : $addressToValidate['fname'].' '.$addressToValidate['lname'];
			$addressToValidate['country'] = isset($addressToValidate['country_id']) ? $addressToValidate['country_id'] : $addressToValidate['country'];
			$addressToValidate['state'] = isset($addressToValidate['state_id']) ? $addressToValidate['state_id'] : (isset($addressToValidate['state']) ? $addressToValidate['state'] : '0');
			$useUserAddress = isset($addressToValidate['use_user_address']) && $addressToValidate['use_user_address'] == '1';

			// validate only for US addresses and when user override is not active
			if ($addressToValidate['country'] == 1 && !$useUserAddress)
			{
				global $settings;
				$addressValidator = new Shipping_DialAZip('400540', '400540', 'HOCKEYSTAR', $settings);

				// select state code
				$stateCodeResult = $db->query('SELECT short_name FROM '.DB_PREFIX.'states WHERE coid="1" AND stid="'.intval($addressToValidate['state']).'"');
				if ($db->moveNext($stateCodeResult, MYSQL_ASSOC))
				{
					$stateCode = $db->col['short_name'];

					// validate address
					$validationResult = $addressValidator->validateAddress(
						$addressToValidate['name'],
						$addressToValidate['address1'],
						$addressToValidate['address2'],
						$addressToValidate['city'],
						$stateCode,
						$addressToValidate['zip']
					);

					switch ($validationResult)
					{
						case 10 : self::setLastError($msg['shipping']['validation_error_invalid_dual_address']); return false; break;
						case 11 : self::setLastError($msg['shipping']['validation_error_invalid_zip_code']); return false; break;
						case 12 : self::setLastError($msg['shipping']['validation_error_invalid_state']); return false; break;
						case 13 : self::setLastError($msg['shipping']['validation_error_invalid_city']); return false; break;
						case 21 : self::setLastError($msg['shipping']['validation_error_address_not_found']); return false; break;
						case 25 : self::setLastError($msg['shipping']['validation_error_street_error']); return false; break;
						case 22 : 
						{
							// preselect state ids
							$db->query('SELECT stid, short_name, name FROM '.DB_PREFIX.'states WHERE coid=1');
							$stateIds = array();
							while ($db->moveNext())
							{
								$stateIds[strtoupper($db->col['short_name'])] = array('id' => $db->col['stid'], 'name' => $db->col['name']);
							}
							
							// build array for address variants
							$addressVariants = $addressValidator->getAddressVariants(
								$addressToValidate['name'], 
								$addressToValidate['address1'], 
								$addressToValidate['address2'], 
								$addressToValidate['city'], 
								$stateCode, 
								$addressToValidate['zip'], 
								$stateIds
							);
							
							if ($addressVariants)
							{
								self::setAddressVariants($addressVariants);
							}

							// technically validation did not pass so return false
							return false;
							break;
						}
						
						case 31 :
						case 32 :
						{
							$validationResponse = $addressValidator->getLastResponse();

							if (is_string($validationResponse['RDI']) && $validationResponse['RDI'] != '')
							{
								$addressToValidate['address_type'] =  $validationResponse['RDI'] == 'B' ? 'Business' : 'Residential';
							}

							$addressToValidate['address1'] = $validationResponse['AddrLine1'];
							$addressToValidate['address2'] = is_string($validationResponse['AddrLine2']) && trim($validationResponse['AddrLine2']) != '' ? $validationResponse['AddrLine2'] : $addressToValidate['address2'];
							$addressToValidate['city'] = $validationResponse['City'];
							$addressToValidate['zip'] = $validationResponse['ZIP5'];

							if (strtoupper($stateCode) != strtoupper($validationResponse['State']))
							{
								$result = $db->query('SELECT stid FROM '.DB_PREFIX.'states WHERE coid="1" AND short_name LIKE "'.strtoupper($validationResponse['State']).'"');

								if ($db->moveNext($result, MYSQL_ASSOC))
								{
									$addressToValidate['state'] = $db->col['stid'];
									$addressToValidate['state_id'] = $db->col['stid'];
								}
							}

							if (is_numeric($validationResponse['Plus4'])) $addressToValidate['zip'].='-'.$validationResponse['Plus4'];

							break;
						}
					}
				}
			}
		}

		return true;
	}
}