<?php
/**
 * Class Admin_Controller
 */
class Admin_Controller
{
	/**
	 * @var Framework_Request
	 */
	protected $request;

	/**
	 * @var DB
	 */
	protected $db;

	/**
	 * Class constructor
	 * @param Framework_Request $request
	 * @param DB $db
	 */
	public function __construct(Framework_Request $request, DB $db)
	{
		$this->request = $request;
		$this->db = $db;
	}

	/**
	 * @return Framework_Request
	 */
	public function getRequest()
	{
		return $this->request;
	}

	/**
	 * @return mixed
	 */
	public function getDb()
	{
		return $this->db;
	}

	/**
	 * @return Ddm_AdminView
	 */
	public function getView()
	{
		return Ddm_AdminView::getInstance();
	}

	/**
	 * @return DataAccess_SettingsRepository|null
	 */
	public function getSettings()
	{
		return DataAccess_SettingsRepository::getInstance();
	}

	/**
	 * @param $nonce
	 * @param $action
	 */
	protected function nonceCheck($nonce, $action)
	{
		$nonce_valid = verify_nonce($nonce, $action);

		if (!$nonce_valid)
		{
			invalid_nonce_message();
			exit;
		}
	}

	/**
	 * @param string $page
	 * @param array $params
	 */
	protected function redirect($page = '', $params = array())
	{
		if (trim($page) != '')
		{
			$params['p'] = $page;
		}

		$query = '';
		if (count($params) > 0)
		{
			$query = '?'.http_build_query($params);
		}

		header('Location: admin.php'.$query);

		$this->db->done();

		session_write_close();

		exit();
	}
}