<?php
/**
 * Class Admin_Controller_AdminProfile
 */
class Admin_Controller_AdminProfile extends Admin_Controller
{
	protected $currentAdminId;

	/**
	 * Set current admin id
	 *
	 * @param int $currentAdminId
	 */
	public function setCurrentAdminId($currentAdminId)
	{
		$this->currentAdminId = $currentAdminId;
	}

	/**
	 * Get current admin id
	 *
	 * @return int
	 */
	public function getCurrentAdminId()
	{
		return $this->currentAdminId;
	}

	/**
	 * Create a new admin
	 */
	public function insertAction()
	{
		$adminId = 0;
		$formData = array(
			'name' => '',
			'email' => '',
			'username' => '',
			'email_mode' => 'html',
			'active' => 'Yes',
			'expires' => 0,
			'expiration_date' => date('Y-m-d H:i:s'),
			'rights' => '',
			'password_history' => '',
			'receive_notifications' => array('invoice', 'outofstock', 'signup', 'security', 'product_review', 'pwdchange30'),
			'force_password_change' => 1,
		);

		$ret = $this->getDb()->selectOne('SELECT COUNT(*) AS c FROM '.DB_PREFIX.'admins', MYSQL_ASSOC);
		$adminsCount = $ret['c'];

		$canAccess = AccessManager::checkAccess('MAX_ADMINS') > 1 && $adminsCount < AccessManager::checkAccess('MAX_ADMINS');

		$errorMessages = array();

		$request = $this->getRequest();

		if ($canAccess && $request->isPost())
		{
			$formData['username'] = trim($request->request->get('username', ''));
			$formData['password'] = trim($request->request->get('password', ''));
			$password2 = $request->request->get('password2', '');

			$formData['name'] = trim($request->request->get('name', ''));
			$formData['email'] = trim($request->request->get('email', ''));
			$formData['email_mode'] = trim($request->request->get('email_mode', 'html'));
			$formData['active'] = trim($request->request->get('active', 'No'));
			$formData['force_password_change'] = trim($request->request->get('force_password_change', '0'));

			$rights = $request->request->get('rights', array());
			$formData['rights'] = implode(',', $rights);

			$receive_notification = $request->request->get('receive_notification', array());
			$pwdchange = trim($request->request->get('pwdchange', 'pwdchange30'));
			$receive_notification['password_change'] = $pwdchange;

			$formData['receive_notifications'] = implode(',', $receive_notification);

			$formData['expires'] = intval($request->request->get('expires', 0));
			$expiration_date = $request->request->get('expiration_date', date_parse(date('Y-m-d h:i:s')));
			$exp_year = intval($expiration_date['year']);
			$exp_month = intval($expiration_date['month']);
			$exp_day = intval($expiration_date['day']);
			$exp_hour = intval($expiration_date['hour']);
			$exp_minute = intval($expiration_date['minute']);

			$formData['expiration_date'] = $exp_year.'-'.$exp_month.'-'.$exp_day.' '.$exp_hour.':'.$exp_minute.':00';

			$passwordHash = getPasswordHash($formData['password']);
			$passwordStrength = pwdStrength($formData['password']);
			$passwordHistory = array();

			$this->validate($formData, $password2, $errorMessages, $passwordHash, $passwordStrength, $passwordHistory, $adminId);

			if (count($errorMessages) == 0 && !DEMO_LOCK)
			{
				$db = $this->getDb();
				$db->reset();
				$db->assign('last_access', 'NOW()');
				$db->assign('last_update', 'NOW()');
				$db->assignStr('name', $formData['name']);
				$db->assignStr('email', $formData['email']);
				$db->assignStr('email_mode', $formData['email_mode']);
				$db->assignStr('username', $formData['username']);
				$db->assignStr('password', $passwordHash);
				$db->assignStr('rights', $formData['rights']);
				$db->assignStr('receive_notifications', $formData['receive_notifications']);
				$db->assignStr('password_history', base64_encode(serialize(array(array('p' => $passwordHash, 's' => $passwordStrength)))));
				$db->assignStr('active', $formData['active']);
				$db->assignStr('expires', $formData['expires']);
				$db->assignStr('expiration_date', $formData['expiration_date']);
				$db->assign('force_password_change', $formData['force_password_change'] == 1 ? 1 : 0);
				$adminId = $db->insert(DB_PREFIX.'admins');

				Admin_Flash::setMessage('Administrator account has been successfully created');

				$this->redirect('admin', array('mode' => 'update', 'aid' => $adminId));
			}
		}

		foreach ($errorMessages as $errorMessage)
		{
			Admin_Flash::setMessage($errorMessage, Admin_Flash::TYPE_ERROR);
		}

		$adminView = $this->getView();

		$profileForm = new Admin_Form_ProfileForm();
		$adminView->assign('form', $profileForm->getForm($this->prepareFormData($formData), Admin_Form_ProfileForm::MODE_INSERT, true, false));
		$adminView->assign('nonce', create_nonce('admin_insert'));
		$adminView->assign('mode', 'insert');
		$adminView->assign('admin_id', $adminId);
		$adminView->assign('canAccess', $canAccess);
		$adminView->assign('max_admins', AccessManager::checkAccess('MAX_ADMINS'));

		$adminView->render('pages/admin/admin/edit');
	}

	/**
	 * Update admin
	 */
	public function updateAction()
	{
		$request = $this->getRequest();
		$adminId = intval($request->query->get('aid', 0));

		$formData = false;

		if ($adminId > 0 && $adminId != $this->getCurrentAdminId())
		{
			$formData = $this->getDb()->selectOne('SELECT * FROM '.DB_PREFIX.'admins WHERE aid='.intval($adminId).' AND aid<>'.intval($this->getCurrentAdminId()), MYSQL_ASSOC);
		}

		if ($formData !== false)
		{
			$errorMessages = array();

			$request = $this->getRequest();

			if ($request->isPost())
			{
				$formData['username'] = trim($request->request->get('username'));
				$formData['password'] = trim($request->request->get('password'));
				$password2 = trim($request->request->get('password2'));
				$formData['name'] = trim($request->request->get('name'));;
				$formData['email'] = trim($request->request->get('email'));;
				$formData['email_mode'] = trim($request->request->get('email_mode', 'html'));
				$formData['active'] = trim($request->request->get('active', 'No'));
				$formData['force_password_change'] = trim($request->request->get('force_password_change', '0'));

				$rights = $request->request->get('rights', array());
				$formData['rights'] = implode(',', $rights);

				$receive_notification = $request->request->get('receive_notification', array());
				$pwdchange = trim($request->request->get('pwdchange', 'pwdchange30'));
				$receive_notification['password_change'] = $pwdchange;

				$formData['receive_notifications'] = implode(',', $receive_notification);

				$formData['expires'] = intval($request->request->get('expires', 0));
				$expiration_date = $request->request->get('expiration_date', date_parse(date('Y-m-d h:i:s')));
				$exp_year = intval($expiration_date['year']);
				$exp_month = intval($expiration_date['month']);
				$exp_day = intval($expiration_date['day']);
				$exp_hour = intval($expiration_date['hour']);
				$exp_minute = intval($expiration_date['minute']);

				$formData['expiration_date'] = $exp_year.'-'.$exp_month.'-'.$exp_day.' '.$exp_hour.':'.$exp_minute.':00';

				$passwordHash = getPasswordHash($formData['password']);
				$passwordStrength = pwdStrength($formData['password']);
				$passwordHistory = array();

				$this->validate($formData, $password2, $errorMessages, $passwordHash, $passwordStrength, $passwordHistory, $adminId);

				if (count($errorMessages) == 0)
				{
					if (!DEMO_LOCK)
					{
						$db = $this->getDb();
						$db->reset();
						$db->assignStr('name', $formData['name']);
						$db->assignStr('username', $formData['username']);
						$db->assignStr('email', $formData['email']);
						$db->assignStr('email_mode', $formData['email_mode']);
						$db->assignStr('active', $formData['active']);
						$db->assignStr('rights', $formData['rights']);
						$db->assignStr('receive_notifications', $formData['receive_notifications']);
						$db->assignStr('expires', $formData['expires']);
						$db->assignStr('expiration_date', $formData['expiration_date']);
						$db->assign('force_password_change', $formData['force_password_change'] == 1 ? 1 : 0);

						/**
						 * Update password if it is not empty
						 */
						if ($formData['password'] != '')
						{
							$this->updatePasswordHistory($passwordHistory, $passwordHash, $passwordStrength);

							$db->assignStr('password', $passwordHash);
							$db->assignStr('password_history', base64_encode(serialize($passwordHistory)));
							$db->assign('last_update', 'NOW()');
						}

						$db->update(DB_PREFIX.'admins', 'WHERE aid='.intval($adminId).' AND aid<>'.intval($this->getCurrentAdminId()));
					}

					Admin_Flash::setMessage('admin.admin.administrator_successfully_updated');
				}
			}

			foreach ($errorMessages as $errorMessage)
			{
				Admin_Flash::setMessage($errorMessage, Admin_Flash::TYPE_ERROR);
			}

			$canAccess = AccessManager::checkAccess('MAX_ADMINS') > 1;

			$adminView = $this->getView();

			$profileForm = new Admin_Form_ProfileForm();
			$adminView->assign('form', $profileForm->getForm($this->prepareFormData($formData), Admin_Form_ProfileForm::MODE_UPDATE));
			$adminView->assign('nonce', create_nonce('admin_update'));
			$adminView->assign('admin_delete_nonce', create_nonce('admin_delete'));
			$adminView->assign('mode', 'update');
			$adminView->assign('admin_id', $adminId);
			$adminView->assign('canAccess', $canAccess);
			$adminView->assign('max_admins', AccessManager::checkAccess('MAX_ADMINS'));
			$adminView->render('pages/admin/admin/edit');
		}
		else
		{
			$this->redirect('profile');
		}
	}

	/**
	 * Delete administrator
	 */
	public function deleteAction()
	{
		$request = $this->getRequest();

		$adminId = intval($request->query->get('aid', 0));
		$nonce = $request->query->get('nonce', $request->request->get('nonce', ''));
		$this->nonceCheck($nonce, 'admin_delete');

		if (!DEMO_LOCK)
		{
			$this->getDb()->query('DELETE FROM '.DB_PREFIX.'admins WHERE aid='.intval($adminId).' AND aid<>'.intval($this->getCurrentAdminId()));
		}

		Admin_Flash::setMessage('Administrator account has been successfully deleted.');

		$adminView = $this->getView();
		$adminView->render('pages/admin/admin/delete-success');
	}

	/**
	 * Display admin profile form / save changes
	 */
	public function profileAction()
	{
		$formData = $this->getDb()->selectOne('SELECT * FROM '.DB_PREFIX.'admins WHERE aid='.intval($this->getCurrentAdminId()), MYSQL_ASSOC);

		if ($formData !== false)
		{
			$errorMessages = array();

			$request = $this->getRequest();

			$forceUpdatePassword = isset($_SESSION['admin_force_update_password']);
			$forceUpdateQuestion = isset($_SESSION['admin_force_update_security_question']);

			if ($request->isPost())
			{
				$formData['name'] = trim($request->request->get('name'));
				$formData['email'] = trim($request->request->get('email'));
				$formData['email_mode'] = trim($request->request->get('email_mode', 'html'));
				$formData['password'] = trim($request->request->get('password'));
				$password2 = trim($request->request->get('password2'));
				$receive_notification = $request->request->get('receive_notification', array());
				$pwdchange = trim($request->request->get('pwdchange', 'pwdchange30'));
				$receive_notification['password_change'] = $pwdchange;

				$formData['receive_notifications'] = implode(',', $receive_notification);

				// Intuit federated does not require field
				if (defined('IS_INTUIT_FEDERATED')) $formData['password'] = '';

				$nonce = trim($request->request->get('nonce'));
				$this->nonceCheck($nonce, 'profile_update');

				$passwordHash = getPasswordHash($formData['password']);
				$passwordStrength = pwdStrength($formData['password']);
				$passwordHistory = array();

				$formData['security_question_id'] = $request->request->get('security_question_id');
				$formData['security_question_answer'] = trim($request->request->get('security_question_answer'));

				$this->validate($formData, $password2, $errorMessages, $passwordHash, $passwordStrength, $passwordHistory, $this->getCurrentAdminId(), $forceUpdatePassword);

				if (count($errorMessages) == 0)
				{
					if (!DEMO_LOCK)
					{
						$passwordHistory = array();

						if ($formData['password'] != '')
						{
							$passwordHistory = unserialize(base64_decode($formData['password_history']));
						}

						$db = $this->getDb();

						$db->reset();
						$db->assignStr('name', $formData['name']);
						$db->assignStr('email', $formData['email']);
						$db->assignStr('email_mode', $formData['email_mode']);
						$db->assignStr('receive_notifications', $formData['receive_notifications']);

						// update password if it is not empty
						if ($formData['password'] != '')
						{
							$this->updatePasswordHistory($passwordHistory, $passwordHash, $passwordStrength);

							$db->assignStr('password', $passwordHash);
							$db->assignStr('password_history', base64_encode(serialize($passwordHistory)));
							$db->assign('last_update', 'NOW()');
							$db->assign('force_password_change', 0);
						}

						// update security question
						if ($formData['security_question_id'] != '' && $formData['security_question_answer'] != '')
						{
							$db->assignStr('security_question_id', $formData['security_question_id']);
							$db->assignStr('security_question_answer', getPasswordHash($formData['security_question_answer']));
						}

						$db->update(DB_PREFIX.'admins', 'WHERE aid='.intval($this->getCurrentAdminId()));
					}

					Admin_Flash::setMessage('Administrator profile has been successfully updated');

					if ($forceUpdatePassword || $forceUpdateQuestion)
					{
						$forceUpdatePassword = false;
						$forceUpdateQuestion = false;

						unset($_SESSION['admin_force_update_password']);
						unset($_SESSION['admin_force_update_security_question']);

						if (isset($_SESSION['admin_force_update_profile_goto_homepage']))
						{
							unset($_SESSION['admin_force_update_profile_goto_homepage']);
							$this->redirect('home');
						}
					}
				}
			}

			if ($forceUpdatePassword || $forceUpdateQuestion)
			{
				if ($forceUpdatePassword && !$forceUpdateQuestion)
				{
					Admin_Flash::setMessage('admin.profile.pci_change_password', Admin_Flash::TYPE_ERROR);
				}
				else if (!$forceUpdatePassword && $forceUpdateQuestion)
				{
					Admin_Flash::setMessage('admin.profile.pci_answer_question', Admin_Flash::TYPE_ERROR);
				}
				else
				{
					Admin_Flash::setMessage('admin.profile.pci_change_password_and_answer_question', Admin_Flash::TYPE_ERROR);
				}
			}

			foreach ($errorMessages as $errorMessage)
			{
				Admin_Flash::setMessage($errorMessage, Admin_Flash::TYPE_ERROR);
			}

			$adminView = $this->getView();

			$profileForm = new Admin_Form_ProfileForm();
			$adminView->assign('form', $profileForm->getForm($this->prepareFormData($formData), Admin_Form_ProfileForm::MODE_PROFILE, $forceUpdatePassword, $forceUpdateQuestion));
			$adminView->assign('nonce', create_nonce('profile_update'));
			$adminView->assign('is_intuit_federated', defined('IS_INTUIT_FEDERATED') && IS_INTUIT_FEDERATED);
			$adminView->render('pages/admin/profile/edit');
		}
		else
		{
			$this->redirect('');
		}
	}

	/**
	 * Update password history
	 *
	 * @param $passwordHistory
	 * @param $passwordHash
	 * @param $passwordStrength
	 */
	protected function updatePasswordHistory(&$passwordHistory, $passwordHash, $passwordStrength)
	{
		/**
		 * If current password in array, remove it
		 */
		foreach ($passwordHistory as $key=>$value)
		{
			if ($value['p'] == $passwordHash)
			{
				unset($passwordHistory[$key]);
				break;
			}
		}

		/**
		 * and rearrange array keys (like 0,1,3,4 to 0,1,2,3)
		 * @var array
		 */
		$_password_history = array();
		foreach ($passwordHistory as $ph)
		{
			$_password_history[] = $ph;
		}
		$passwordHistory = $_password_history;

		/**
		 * if history already has 4 or more (why??) password - remove last one
		 */
		if (count($passwordHistory) > 3) array_shift($passwordHistory);

		/**
		 * finally add current password
		 */
		$passwordHistory[] = array(
			'p' => $passwordHash,
			's' => $passwordStrength
		);
	}

	/**
	 * Check are there duplicates
	 *
	 * @param $username
	 * @param $adminId
	 *
	 * @return bool
	 */
	protected function hasDuplicateAdmin($username, $adminId)
	{
		$db = $this->getDb();

		$duplicateAdmin = $db->selectOne('SELECT aid FROM '.DB_PREFIX.'admins WHERE username LIKE "'.$db->escape($username).'"  AND aid<>'.intval($adminId), MYSQL_ASSOC);

		return $duplicateAdmin !== false;
	}

	/**
	 * Check area there duplicate passwords
	 *
	 * @param $password
	 * @param $adminId
	 *
	 * @return bool
	 */
	protected function hasDuplicatePassword($password, $adminId)
	{
		$db = $this->getDb();

		$duplicateAdmin = $db->selectOne('SELECT * FROM '.DB_PREFIX.'admins WHERE password = "'.$db->escape(getPasswordHash($password)).'"  AND aid<>'.intval($adminId), MYSQL_ASSOC);

		return $duplicateAdmin !== false;
	}

	/**
	 * Validate admin's data
	 *
	 * @param $admin
	 * @param $password2
	 * @param $errorMessages
	 * @param $passwordHash
	 * @param $passwordStrength
	 * @param $passwordHistory
	 * @param $adminId
	 * @param $forceChangePassword
	 */
	protected function validate($admin, $password2, &$errorMessages, &$passwordHash, &$passwordStrength, &$passwordHistory, $adminId, $forceChangePassword = false)
	{
		global $settings;

		if ($admin['name'] == '')
		{
			$errorMessages[] = 'Please enter name';
		}

		if ($admin['email'] == '')
		{
			$errorMessages[] = 'Please enter email address';
		}

		if ($admin['password'] != '' || $forceChangePassword)
		{
			if ($admin['password'] == '' && $forceChangePassword)
			{
				$errorMessages[] = 'Please enter new password';
			}

			if ($admin['password_history'] != '')
			{
				$passwordHistory = unserialize(base64_decode($admin['password_history']));
			}

			if ($admin['password'] != $password2)
			{
				$errorMessages[] = 'Password and confirmation are not equal';
			}
			else if ($settings['SecurityForceCompliantAdminPasswords'] == 'YES')
			{
				foreach ($passwordHistory as $history)
				{
					if ($passwordHash == $history['p'])
					{
						$errorMessages[] = 'The password you entered has been used before, please enter another password';
					}
				}

				if (count($errorMessages) == 0 && $passwordStrength < 3)
				{
					$errorMessages[] = 'Password is not strong enough';
				}
			}
			else if (strlen($admin['password']) < 8)
			{
				$errorMessages[] = 'Password is too short';
			}
		}

		if ($this->hasDuplicateAdmin($admin['username'], $adminId))
		{
			$errorMessages[] = 'Backend administrator with username '.gs($admin['username']).' already exists';
		}
		else if ($this->hasDuplicatePassword($admin['password'], $adminId))
		{
			$errorMessages[] = 'The password you entered has been used before, please enter another password';
		}
	}

	/**
	 * Prepare admin's data
	 *
	 * @param $admin
	 */
	protected function prepareFormData($admin)
	{
		if (!is_array($admin['rights']))
		{
			$admin['rights'] = trim($admin['rights']) != '' ? explode(',', $admin['rights']) : array();
		}

		$admin['password_history'] = $admin['password_history'] != '' ? unserialize(base64_decode($admin['password_history'])) : array();

		if (!is_array($admin['receive_notifications']))
		{
			$admin['receive_notifications'] = trim($admin['receive_notifications']) != '' ? explode(',', trim($admin['receive_notifications'])) : array();
		}

		return $admin;
	}
}