<?php

class Admin_Form_ProfileForm
{
	const MODE_INSERT = 1;
	const MODE_UPDATE = 2;
	const MODE_PROFILE = 3;

	public function getForm($formData, $mode, $forcePassword = false, $forceSecurityQuestion = false)
	{
		$translator = Framework_Translator::getInstance();

		$generalFormBuilder = new core_Form_FormBuilder('general'); 

		/**
		 * Basic properties group
		 */
		$basicPropertiesGroup = new core_Form_FormBuilder('basic', array('label' => 'admin.profile.basic_info', 'class'=>'ic-icon', 'first'=>true));
		$generalFormBuilder->add($basicPropertiesGroup);

		$basicPropertiesGroup->add('name', 'text', array('label' => 'admin.profile.name', 'required' => true, 'value' => $formData['name']));
		$basicPropertiesGroup->add('email', 'text', array('label' => 'admin.profile.email', 'required' => true, 'value' => $formData['email']));

		$email_mode_options = array(
			new core_Form_Option('html', 'admin.profile.email_mode_options.html'),
			new core_Form_Option('text', 'admin.profile.email_mode_options.text'),
		);
		$basicPropertiesGroup->add('email_mode', 'choice', array('label'=>'admin.profile.email_mode', 'value' => $formData['email_mode'], 'options' => $email_mode_options));

		$notification_options = array(
			new core_Form_Option('invoice', 'admin.profile.notification_options.invoice'),
			new core_Form_Option('outofstock', 'admin.profile.notification_options.outofstock'),
			new core_Form_Option('security', 'admin.profile.notification_options.security_audit'),
			new core_Form_Option('signup', 'admin.profile.notification_options.signup'),
			new core_Form_Option('notactive', 'admin.profile.notification_options.notactive'),
			new core_Form_Option('product_review', 'admin.profile.notification_options.product_review'),
		);

		$basicPropertiesGroup->add('receive_notification[]', 'choice', array('label'=>'', 'value' => $formData['receive_notifications'], 'multiple' => true, 'expanded' => true, 'options' => $notification_options));

		if (!defined('IS_INTUIT_FEDERATED'))
		{
			$password_change = 'pwdchange30';

			if (in_array('pwdchange60', $formData['receive_notifications']))
			{
				$password_change = 'pwdchange60';
			}
			else if (in_array('pwdchange90', $formData['receive_notifications']))
			{
				$password_change = 'pwdchange90';
			}
			else if (in_array('pwdchangenever', $formData['receive_notifications']))
			{
				$password_change = 'pwdchangenever';
			}

			$password_change_options = array(
				new core_Form_Option('pwdchange30', 'admin.profile.pwdchange_options.pwdchange30'),
				new core_Form_Option('pwdchange60', 'admin.profile.pwdchange_options.pwdchange60'),
				new core_Form_Option('pwdchange90', 'admin.profile.pwdchange_options.pwdchange90'),
				new core_Form_Option('pwdchangenever', 'admin.profile.pwdchange_options.pwdchangenever')
			);
			$basicPropertiesGroup->add('pwdchange', 'choice', array('label'=>'admin.profile.pwdchange', 'value' => $password_change, 'multiple' => false, 'expanded' => true, 'options' => $password_change_options));
			
			$basicPropertiesGroup->add('username', 'text', array('label' => 'admin.profile.username', 'readonly' => $mode != self::MODE_INSERT, 'value' => $formData['username'], 'required' => true));
			$basicPropertiesGroup->add('password', 'password', array('label' => 'admin.profile.password', 'value' => '', 'required' => $mode == self::MODE_INSERT || $forcePassword, 'note' => 'admin.profile.password_hint'));
			$basicPropertiesGroup->add('password2', 'password', array('label' => 'admin.profile.password2', 'value' => '', 'required' => $mode == self::MODE_INSERT || $forcePassword));

			if ($mode == self::MODE_PROFILE)
			{
				$security_question_options = array(
					new core_Form_Option('', 'admin.profile.security_question_not_selected'),
					new core_Form_Option('1', 'admin.profile.security_question_1'),
					new core_Form_Option('2', 'admin.profile.security_question_2'),
					new core_Form_Option('3', 'admin.profile.security_question_3'),
					new core_Form_Option('4', 'admin.profile.security_question_4'),
					new core_Form_Option('5', 'admin.profile.security_question_5'),
					new core_Form_Option('6', 'admin.profile.security_question_6'),
					new core_Form_Option('7', 'admin.profile.security_question_7'),
				);
				$basicPropertiesGroup->add('security_question_id', 'choice', array('label' => 'admin.profile.security_question', 'attr' => array('class' => 'large'), 'required' => $forceSecurityQuestion, 'value' => $formData['security_question_id'], 'multiple' => false, 'expanded' => false, 'options' => $security_question_options));
				$basicPropertiesGroup->add('security_question_answer', 'text', array('label' => 'admin.profile.security_question_answer', 'attr' => array('class' => 'large'), 'required' => $forceSecurityQuestion, 'value' => ''));

				if (_ACCESS_INTUIT_ANYWHERE)
				{
					if ($formData['openid'] != '')
					{
						$qb_link = $translator->trans('admin.profile.qb_account_linked');
					}
					else
					{
						$qb_link = $translator->trans('admin.profile.qb_account_unlinked');
					}

					$basicPropertiesGroup->add('qb_account', 'static', array('label' => 'admin.profile.qb_account', 'value' => $qb_link, 'note' => 'admin.profile.notes.qb_account'));
				}
			}
			else
			{
				$basicPropertiesGroup->add('force_password_change', 'checkbox', array('label' => 'admin.profile.force_password_change', 'value' => 1, 'current_value' => $formData['force_password_change']));
			}
		}

		if ($mode != self::MODE_PROFILE)
		{
			/**
			 * Account lock properties group
			 */
			$accountLockPropertiesGroup = new core_Form_FormBuilder('account_lock_grp', array('label'=>'admin.profile.account_lock_grp', 'class'=>'ic-icon'));
			$generalFormBuilder->add($accountLockPropertiesGroup);

			$accountLockPropertiesGroup->add('active', 'choice', array('label' => 'admin.profile.active', 'options' => array('Yes' => 'admin.profile.active_options.yes', 'No' => 'admin.profile.active_options.no'), 'value' => $formData['active'], 'attr' => array('class' => 'short')));
		}

		/**
		 * Expiration properties group
		 */
		$expirationPropertiesGroup = new core_Form_FormBuilder('expiration', array('label'=>'admin.profile.expiration', 'class'=>'ic-icon'));
		$generalFormBuilder->add($expirationPropertiesGroup);

		if ($mode == self::MODE_PROFILE)
		{
			$expirationPropertiesGroup->add('expires', 'static', array('label' => 'admin.profile.expires', 'value' => $formData['expires'] == '0' ? 'No' : 'Yes'));
			if ($formData['expires'] == '1')
			{
				$expirationPropertiesGroup->add('expires_date', 'static', array('label' => 'admin.profile.expires_date', 'value' => date('F j, Y, g:ia', strtotime($formData["expiration_date"]))));
			}
		}
		else
		{
			$expirationPropertiesGroup->add('expires', 'choice', array('label' => 'admin.profile.expires', 'value' => $formData['expires'], 'options' => array('0' => 'common.no', '1' => 'common.yes'), 'attr' => array('class' => 'short')));

			$expirationPropertiesGroup->add('expiration_date', 'datetime', array('label' => 'admin.profile.expiration_date', 'value' => $formData['expiration_date']));
		}

		$privileges_options = array(
			new core_Form_Option('categories', 'admin.profile.privileges_options.categories'),
			new core_Form_Option('products', 'admin.profile.privileges_options.products'),
			new core_Form_Option('users', 'admin.profile.privileges_options.users'),
			new core_Form_Option('orders', 'admin.profile.privileges_options.orders'),
			new core_Form_Option('emails', 'admin.profile.privileges_options.emails'),
			new core_Form_Option('admins', 'admin.profile.privileges_options.admins'),
			new core_Form_Option('global', 'admin.profile.privileges_options.global'),
			new core_Form_Option('business', 'admin.profile.privileges_options.business'),
			new core_Form_Option('content', 'admin.profile.privileges_options.content'),
			new core_Form_Option('export', 'admin.profile.privileges_options.export'),
			new core_Form_Option('stats', 'admin.profile.privileges_options.stats'),
			new core_Form_Option('marketing', 'admin.profile.privileges_options.marketing'),
			new core_Form_Option('ccs', 'admin.profile.privileges_options.ccs'),
		);

		if ($mode != self::MODE_PROFILE)
		{
			$privileges_options[] = new core_Form_Option('demo', 'admin.profile.privileges_options.demo');
		}

		$privileges_options[] = new core_Form_Option('all', 'admin.profile.privileges_options.all');

		/**
		 * Privileges properties group
		 */
		$privilegesPropertiesGroup = new core_Form_FormBuilder('privileges_group', array('label'=>'admin.profile.privileges_group', 'class'=>'ic-icon'));
		$generalFormBuilder->add($privilegesPropertiesGroup);

		$privilegesPropertiesGroup->add('rights[]', 'choice', array('label' => '', 'value' => $formData['rights'], 'multiple' => true, 'expanded' => true, 'readonly' => $mode == self::MODE_PROFILE, 'options' => $privileges_options));

		return $generalFormBuilder->getForm();
	}
}