<?php
/**
 * Total calculations class
 */
class Calculator_Total
{
	protected static $instance;

	protected $settings;

	public function __construct(&$settings)
	{
		$this->settings = $settings;
	}

	/**
	 * Reset order data
	 * 
	 * @param  ORDER  $order
	 */
	public function reset(ORDER $order)
	{
		$order->setTotalAmount(0);
	}

	/**
	 * Calculate total amount
	 * 
	 * @param  ORDER  $order
	 *
	 * @return float The total amount
	 */
	public function calculate(ORDER $order)
	{
		$subtotal = $order->getSubtotalAmount();
		$taxAmount = $order->getTaxAmount(); // includes subtotal, handling and shipping taxes (if applicable)

		$totalAmount =
			$subtotal
			+ $taxAmount
			+ $order->getShippingAmount() // may contain handling amount
			+ ($order->getHandlingSeparated() ? $order->getHandlingAmount() : 0) // if not separated then included in shipping
			- $order->getDiscountAmount()
			- $order->getPromoDiscountAmount()
		;

		$order->setTotalAmount(PriceHelper::round($totalAmount));

		return $totalAmount;
	}

	/**
	 * Get instance
	 *
	 * @param mixed $settings
	 * @return Calculator_Total
	 */
	// @codeCoverageIgnoreStart
	public static function getInstance(&$settings = false)
	{
		if (is_null(self::$instance))
		{
			if (!$settings)
			{
				global $settings;
			}

			self::$instance = new self($settings);
		}

		return self::$instance;
	}

	/**
	 * Set instance
	 *
	 * @param Calculator_Total $instance
	 */
	public static function setInstance(Calculator_Total $instance)
	{
		self::$instance = $instance;
	}
	// @codeCoverageIgnoreEnd
}