<?php

class Events_Event
{
	/**
	 * @var string event name
	 */
	protected $eventName;

	/**
	 * @var array event data
	 */
	protected $data;

	/**
	 * @var bool propagation stopped
	 */
	protected $stopped = false;

	/**
	 * Class constructor
	 * 
	 * @param string $evenName
	 */
	public function __construct($eventName)
	{
		$this->setEventName($eventName);
	}

	/**
	 * Even name setter
	 * 
	 * @param string $evenName
	 * 
	 * @return Events_Event
	 */
	public function setEventName($eventName)
	{
		$this->eventName = $eventName;
		return $this;
	}

	/**
	 * Even name getter
	 * 
	 * @return string
	 */
	public function getEventName()
	{
		return $this->eventName;
	}

	/**
	 * Set event data
	 * 
	 * @param string $name
	 * @param mixed $value
	 * 
	 * @return Events_Event
	 */
	public function setData($name, $value)
	{
		$this->data[$name] = $value;
		return $this;
	}

	/**
	 * Get even data
	 * 
	 * @param string $name
	 * 
	 * @return mixed OR null
	 */ 
	public function getData($name)
	{
		return isset($this->data[$name]) ? $this->data[$name] : null;
	}

	/**
	 * Stops event propagation
	 */
	public function stopPropagation($stopped)
	{
		$this->stopped = (bool) $stopped;
	}

	/**
	 * Returns propagation status
	 * 
	 * @return boolean
	 */
	public function propagationIsStopped()
	{
		return $this->stopped;
	}
}