<?php

class Events_OrderEvent extends Events_Event
{
	const ON_STATUS_CHANGE = 'on-order-status-change';
	const ON_CREATED = 'on-order-created';
	const ON_PLACED = 'on-order-placed';
	const ON_COMPLETED = 'on-order-completed';
	const ON_DELETED = 'on-order-deleted';
	const ON_ADD_ITEM = 'on-add-item';
	const ON_UPDATE_ITEMS = 'on-update-items';
	const ON_REMOVE_ITEM = 'on-remove-item';

	/**
	 * Set order
	 * 
	 * @param ORDER $order
	 * 
	 * @return Events_OrderEvent
	 */
	public function setOrder(&$order)
	{
		$this->setData('order', $order);
		return $this;
	}

	/**
	 * Returns order
	 * 
	 * @return ORDER
	 */
	public function getOrder()
	{
		return $this->getData('order');
	}

	public static function dispatchStatusChangeEvent($order, $oldOrderStatus, $oldPaymentStatus)
	{
		if ($order->getStatus() != $oldOrderStatus || $order->getPaymentStatus() != $oldPaymentStatus)
		{
			/**
			 * Handle on status change event
			 */
			$event = new Events_OrderEvent(Events_OrderEvent::ON_STATUS_CHANGE);

			$event
				->setOrder($order)
				->setData('oldOrderStatus', $oldPaymentStatus)
				->setData('oldPaymentStatus', $oldPaymentStatus)
				->setData('newOrderStatus', $order->getStatus())
				->setData('newPaymentStatus', $order->getPaymentStatus());

			Events_EventHandler::handle($event);
		}
	}
}
