<?php

class View_OrderLinesViewModel extends View_Model
{
	protected $lineItems = array();

	/**
	 * Prepare data for order line items
	 *
	 * @param ORDER $order
	 */
	public function build(ORDER $order)
	{
		$msg = $this->msg;

		$this->resetLineItems();

		/**
		 * Subtotal
		 */
		$this->addLineItem('subtotal', $msg['opc']['invoice_subtotal'], '', $order->getSubtotalAmount());

		/**
		 * Discount
		 */
		if ($order->getDiscountAmount() > 0)
		{
			$this->addLineItem('discount', $msg['opc']['invoice_discount'], '', 0 - $order->getDiscountAmount());
		}

		/**
		 * Promo discount
		 */
		if ($order->getPromoDiscountAmount() > 0 && $order->getPromoType() != 'Shipping')
		{
			$this->addLineItem('promo', $msg['opc']['invoice_promo_discount'], '', 0 - $order->getPromoDiscountAmount());
		}

		/**
		 * Shipping amount
		 */
		if ($order->getShippingAmount() > 0)
		{
			$this->addLineItem('shipping', $msg['opc']['invoice_shipping'], '', $order->getShippingAmount());
		}

		/**
		 * Shipping promo discount
		 */
		if ($order->getPromoDiscountAmount() > 0 && $order->getPromoType() == 'Shipping')
		{
			$this->addLineItem('promo-shipping', $msg['opc']['invoice_promo_discount'], '', 0 - $order->getPromoDiscountAmount());
		}

		/**
		 * Handling
		 */
		if ($order->getHandlingSeparated() && $order->getHandlingAmount() > 0)
		{
			$this->addLineItem('handling', $msg['opc']['invoice_handling'], '', $order->getHandlingAmount());
		}

		/**
		 * Taxes
		 */
		if ($order->getTaxExempt() != ORDER::TAX_EXEMPT_NO)
		{
			$this->addLineItem(
				'tax', $msg['opc']['invoice_tax'],
				$order->getTaxExempt() == ORDER::TAX_EXEMPT_YES ? $msg['opc']['invoice_tax_exempt'] : $msg['opc']['invoice_tax_exempt_partial'],
				$order->getTaxAmount()
			);
		}
		else if ($order->getTaxAmount() > 0)
		{
			$this->addLineItem('tax', $msg['opc']['invoice_tax'], '', $order->getTaxAmount());
		}

		/**
		 * Total
		 */
		$totalAmount = $order->getTotalAmount();

		$this->addLineItem('total', $msg['opc']['invoice_total'], '', $totalAmount);

		/**
		 * Gift certificate remaining amount to be paid
		 */
		if ($order->getGiftCertificateAmount() > 0)
		{
			$this->addLineItem('gift-certificate', $msg['opc']['invoice_gift_certificate'], '', $order->getGiftCertificateAmount());

			if ($order->getGiftCertificateAmount() < $order->getTotalAmount())
			{
				$this->addLineItem('remaining', $msg['opc']['invoice_remaining_amount'], '', $order->getTotalAmount() - $order->getGiftCertificateAmount());
			}
		}
	}

	/**
	 * Return data as array
	 * @return array
	 */
	public function asArray()
	{
		return array('lineItems' => $this->getLineItems());
	}

	/**
	 * Return order line items
	 *
	 * @return array
	 */
	public function getLineItems()
	{
		return $this->lineItems;
	}

	/**
	 * Return line item by key
	 *
	 * @param string $key
	 *
	 * @return mixed
	 */
	public function getLineItem($key)
	{
		return isset($this->lineItems[$key]) ? $this->lineItems[$key] : null;
	}

	/**
	 * Reset order line items
	 */
	protected function resetLineItems()
	{
		$this->lineItems = array();
	}

	/**
	 * Add order amount line item
	 *
	 * @param string $key
	 * @param string $title
	 * @param string $subtitle
	 * @param float $amount
	 */
	public function addLineItem($key, $title, $subtitle, $amount)
	{
		$this->lineItems[$key] = array(
			'title' => $title,
			'subtitle' => $subtitle,
			'amount' => $this->formatPrice($amount)
		);
	}
}