<?php 

/**
 * @class ShoppingCartCatalog
 */
class ShoppingCartCatalog{
	public $db;
	public $catalog_images_dir = "images/catalog";
	public $tree = array();
	
	/* CONSTRUCTOR
	*/
	function ShoppingCartCatalog(&$db){
		$this->db = $db;
		return false;
	}
	
	/* Get Category data by ID
	*/
	function getCategoryById($id, $get_subcats = true){
		if($id){
			$this->db->query("SELECT *, IF(url_custom='',url_default,url_custom) AS url FROM ".DB_PREFIX."catalog WHERE cid ='".$id."' AND is_visible='Yes'");
			if($this->db->moveNext()){
				$data = $this->db->col;
				$data["url"] = getCatalogUrl($data["url"], $data["cid"]);
				
				$data['show_subcategory_image'] = (intval($data['list_subcats_images']) == 1);
				
				// get subcats
				if($data["list_subcats"] == "Yes")
				{
					$data["subcategories"] = $this->getSubCategoriesById($id);
					foreach ($data['subcategories'] as $sub_cat)
					{
						$this->setImagePaths($sub_cat);
					}
				}
				else{
					$data["subcategories"] = false;
				}
				
				$this->setImagePaths($data);
				return $data;
			}
		}
		return false;
	}

	function setImagePaths(&$data)
	{
		$id = $data['cid'];

		// check image here
		$f = $this->catalog_images_dir."/category".$id.".";
		if(file_exists($f."jpg")){
			$data["img"] = $f."jpg";
		}
		elseif(file_exists($f."png")){
			$data["img"] =  $f."png";
		}
		elseif(file_exists($f."gif")){
			$data["img"] =  $f."gif";
		}
		else{
			$data["img"] = false;
		}
		if ($data["img"])
		{
			$data['thumb'] = $this->catalog_images_dir.'/thumbs/category'.$id.'.jpg';
		}
	}
	
	/* Return direct subcategories for selected category
	*/
	function getSubCategoriesById($id)
	{
		$this->db->query("SELECT cid FROM ".DB_PREFIX."catalog WHERE parent='".$id."' AND is_visible='Yes' ORDER BY priority, name");
		
		$subcats = array();
		
		if ($this->db->numRows() > 0)
		{
			$subcats = $this->db->getRecords();
			for($i = 0; $i < count ($subcats); $i++)
			{
				$subcats[$i] = $this->getCategoryById($subcats[$i]["cid"], false);
			}
		}
		
		return $subcats;
	}

	/**
	 * returns parse categories tree
	 *
	 * @param int $category_id
	 * @return array
	 */
	function getParsedTree($category_id=0, $is_visible = false){
		$this->tree = array();
		$sql = "SELECT cid, parent, level, is_visible, key_name, name, IF(url_custom='',url_default,url_custom) AS url FROM ".DB_PREFIX."catalog";
		if ($is_visible)
		{
			$sql .= " WHERE is_visible = 'Yes'";
		}
		$sql .= " ORDER BY level, priority, name";

		$this->db->query($sql);
		if($this->db->numRows()>0){
			while($this->db->moveNext()){
				
				$cid = $this->db->col["cid"];
				$parent = $this->db->col["parent"];
				
				$this->tree[$cid] =	array(
					"cid"=>$cid,
					"parent"=>$parent,
					"level"=>$this->db->col["level"],
					"is_visible"=>$this->db->col["is_visible"],
					"key_name"=>$this->db->col["key_name"],
					"name"=>$this->db->col["name"],
					"children"=>false,
					"category_url"=>getCatalogUrl($this->db->col["url"], $cid)
				);
				
				if (array_key_exists($parent, $this->tree) && $this->tree[$parent] > 0)
				{
					$this->tree[$parent]["children"][$cid] = &$this->tree[$cid];
				}
			}
			//prepare tree -- levels
			foreach ($this->tree as $id=>$node)
			{
				$level = 0;
				$n = $node;
				$j = 0;
				for ($j = 0; $j<count($this->tree); $j++)
				{
					if ($n["parent"] > 0 && array_key_exists($n["parent"], $this->tree))
					{
						$n = $this->tree[$n["parent"]];
					}
					else
					{
						break;
					}
				}
				$this->tree[$id]["level"] = $j + 1;
				$this->tree[$id]["alevel"] = $this->tree[$id]["level"]>1?array_fill(0, $this->tree[$id]["level"]-1, 1):false;
				$this->tree[$id]["open"] = false;
				
				if ($n['parent'] > 0 && !array_key_exists($n["parent"], $this->tree))
				{
					unset($this->tree[$id]);
					break;
				}
				
				if ($id == $category_id)
				{
					$_id = $id;
					$this->tree[$_id]["open"] = true;
					for ($i=0; $i<count($this->tree); $i++)
					{
						$parent_id = $this->tree[$_id]["parent"];
						if ($parent_id > 0 && array_key_exists($parent_id, $this->tree))
						{
							$_id = $this->tree[$_id]["parent"];
							$this->tree[$_id]["open"] = true;
						}
						else
						{
							break;
						}
					}
				}
			}
			
			return $this->tree;
		}
		else
		{	
			return false;
		}
	}
	
	/**
	 * Scans all nested children and set to parent category
	 *
	 * @param int $category_id
	 */
	function setCategoryNestedChildrenIds($category_id = 0){
		if(array_key_exists($category_id, $this->tree)){
			$this->tree[$category_id]["nested"] = array($category_id);
			$this->setCategoryNestedChildrenIds2($category_id, $category_id);
		}
	}
	
	/**
	 * Real scan function for setCategoryNestedChildrenIds
	 *
	 * @param int $parent_id
	 * @param int $category_id
	 */
	function setCategoryNestedChildrenIds2($parent_id, $category_id){
		if(is_array($this->tree[$parent_id]["children"])){
			foreach($this->tree[$parent_id]["children"] as $child_id=>$child_data){
				$this->tree[$category_id]["nested"][] = $child_id;
				$this->setCategoryNestedChildrenIds2($child_id, $category_id);
			}
		}
	}
	
	/**
	 * get parsed tree path from category_id node to top
	 *
	 * @param int $category_id
	 * @return array
	 */
	function getParsedPath($category_id){
		if(array_key_exists($category_id, $this->tree)){
			$parsed_path = array();
			$_category_id = $category_id;
			for($i=0; $i<count($this->tree); $i++){
				$node = $this->tree[$_category_id];
				$parsed_path[] = array("href"=>$node["category_url"], "caption"=>$node["name"]);
				if($node["parent"] > 0){
					$_category_id = $node["parent"];
				}
				else{
					break;
				}	
			}
			return array_reverse($parsed_path);
		}
		else{
			return false;
		}
	}
}