<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of class
 *
 * @author victor
 */
class DobaMapper
{
	protected $db = null;
	protected $settings = null;
	
	/**
	 *
	 * @param DB $db
	 * @param array $settings
	 * @return DobaMapper 
	 */
	public function __construct(DB &$db, array &$settings)
	{
		$this->db = $db;
		$this->settings = $settings;
		return $this;
	}
	
	/**
	 *
	 * @return type 
	 */
	protected function db()
	{
		return $this->db;
	}

	/**
	 *
	 * @return array 
	 */
	public function getDobaCategories()
	{
		$result = $this->db()->query("
			SELECT 
				c.cid, c.parent, c.level, c.name,
				IF((d.doba_cid IS NULL) OR ((d.categories = '') AND d.primary_cid = 0) , 0, 1) AS mapped,
				IF((d.subcats IS NULL) OR (d.subcats = 0) , 0, 1) AS subcats
			FROM ".DB_PREFIX."catalog AS c 
			LEFT JOIN ".DB_PREFIX."catalog_doba_map AS d ON c.cid = d.doba_cid
			WHERE key_name LIKE 'doba_%' 
			ORDER BY c.level, c.priority, c.name
		");
		
		if (!$this->db()->numRows() > 0) return null;
		
		$categories = array();
		
		while (($category = $this->db()->moveNext($result, MYSQL_ASSOC)) != false)
		{
			$category["children"] = array();
			$categories[$category["cid"]] = $category;
			
			if ($category["parent"] > 0)
			{
				$categories[$category["parent"]]["children"][] = &$categories[$category["cid"]];
			}
		}
		
		return $categories;
	}
	
	/**
	 * Get mapped categories
	 * @param int $dobaCid
	 * @return mixed 
	 */
	public function getDobaCategory($dobaCid)
	{
		$result = $this->db()->query("SELECT * FROM ".DB_PREFIX."catalog_doba_map WHERE doba_cid='".intval($dobaCid)."'");
		
		if (($data = $this->db()->moveNext($result, MYSQLI_ASSOC)) != false)
		{
			$data['categories'] = $data['categories'] != '' ? unserialize($data['categories']) : null;
			$data['categories'] = is_array($data['categories']) && count($data['categories']) ? $data['categories'] : null;
			return $data;
		}
		
		return null;
	}
	
	/**
	 * Map categories
	 * @param int $dobaCid
	 * @param array $categories 
	 * @param bool $subcats
	 */
	public function mapCategories($dobaCid, $primaryCid, $includeSubcats, $categories)
	{
		$c = array();
		if ($categories != null && is_array($categories))
		{
			foreach ($categories as $cid) $c[] = intval($cid);
		}
		
		if ($primaryCid != 0 || count($c) > 0 || $includeSubcats == 1)
		{
			$result = $this->db()->query("SELECT doba_cid FROM ".DB_PREFIX."catalog_doba_map WHERE doba_cid='".intval($dobaCid)."'");

			if ($this->db()->moveNext($result, MYSQL_ASSOC))
			{
				$this->db()->reset();
				$this->db()->assignStr("primary_cid", intval($primaryCid));
				$this->db()->assignStr("categories", count($c) > 0 ? serialize($c) : '');
				$this->db()->assignStr("subcats", $includeSubcats ? 1 : 0);
				$this->db()->update(DB_PREFIX."catalog_doba_map", "WHERE doba_cid='".intval($dobaCid)."'");
			}
			else
			{
				$this->db()->reset();
				$this->db()->assignStr("doba_cid", intval($dobaCid));
				$this->db()->assignStr("primary_cid ", intval($primaryCid));
				$this->db()->assignStr("categories", count($c) ? serialize($c) : '');
				$this->db()->assignStr("subcats", $includeSubcats ? 1 : 0);
				$this->db()->insert(DB_PREFIX."catalog_doba_map");
			}
		}
		else
		{
			$this->db()->query("DELETE FROM ".DB_PREFIX."catalog_doba_map WHERE doba_cid='".intval($dobaCid)."'");
		}
	}
	
	/**
	 *
	 * @param type $dobaCid
	 * @param type $dobaCategories
	 * @param type $array 
	 */
	protected function getSubcategories($dobaCid, &$dobaCategories, &$array)
	{
		if (
			isset($dobaCategories[$dobaCid]['children']) && 
			is_array($dobaCategories[$dobaCid]['children']) && 
			count($dobaCategories[$dobaCid]['children']) > 0
		)
		{
			foreach ($dobaCategories[$dobaCid]['children'] as $child)
			{
				$array[] = $child['cid'];
				$this->getSubcategories($child['cid'], $dobaCategories, $array);
			}
		}
	}
	
	/**
	 * Map rpoducts
	 * @param type $remapLocked
	 * @return type 
	 */
	public function remapProducts($remapLocked = false, $keepDobaCategories = true)
	{
		/**
		 * Get categories
		 */
		$dobaCategories = $this->getDobaCategories();
		
		if ($dobaCategories == null) return false;
		
		/**
		 * Get map
		 */
		$result = $this->db()->query("SELECT * FROM ".DB_PREFIX."catalog_doba_map");
		
		if ($this->db()->numRows($result) == 0) return false;
		
		$map = array();
		
		while (($col =  $this->db()->moveNext($result, MYSQL_ASSOC)) != false)
		{
			$col['categories'] = $col['categories'] != '' ? unserialize($col['categories']) : null;
			$map[] = $col;
		}
		
		/**
		 * Remap primary category (easy)
		 */
		foreach ($map as $route)
		{
			// Find doba categories from where we will move products
			
			// Primary category
			$sourceCategories = array($route["doba_cid"]);
			
			// Secondary categories
			if ($route["subcats"] == 1)
			{
				$this->getSubcategories($route["doba_cid"], $dobaCategories, $sourceCategories);
			}
			
			if ($sourceCategories && is_array($sourceCategories) && count($sourceCategories) > 0)
			{
				// Find all products currently placed in doba categories
				$result = $this->db()->query("
					SELECT pc.pid 
					FROM ".DB_PREFIX."products_categories AS pc
					".($remapLocked ? "" : "	INNER JOIN ".DB_PREFIX."products AS p ON pc.pid = p.pid AND NOT (p.lock_fields LIKE '%category%') ")."
					WHERE pc.cid IN (".implode(",", $sourceCategories).")
					GROUP BY pc.pid 
				");

				$products = $this->db()->getRecords($result);

				// we just got list of products to be moved
				if ($products && is_array($products) && count($products) > 0)
				{	
					// delete current categories
					if (!$keepDobaCategories)
					{
						$this->db()->query("
							DELETE FROM ".DB_PREFIX."products_categories
							WHERE pid IN (".implode(",", $products).")
						");
					}
					
					if ($route["categories"] != '')
					{
						$destinationCategories = unserialize($route["categories"]);
					}
					else
					{
						$destinationCategories = array();
					}
					
					// set primary category in products table
					if ($route["primary_cid"] != 0)
					{
						// remap primary category in products table
						$this->db()->query("
							UPDATE ".DB_PREFIX."products 
							SET cid='".intval($route["primary_cid"])."' 
							WHERE pid IN (".implode(",", $products).")
						");
						$destinationCategories[] = $route["primary_cid"];
						
						if ($keepDobaCategories)
						{
							$this->db()->query("UPDATE ".DB_PREFIX."products_categories SET is_primary=0 WHERE pid IN (".implode(',', $products).")");
						}	
					}

					// destination categories
					if (count($destinationCategories) > 0) 
					{
						$destinationCategories = array_unique($destinationCategories);
						
						$insertValues = array();
						
						foreach ($products as $pid)
						{
							foreach ($destinationCategories as $cid)
							{
								$insertValues[] = '("'.intval($cid).'","'.intval($pid).'","'.(($route["primary_cid"] == $cid)?1:0).'")';
							}
						}
						
						if (count($insertValues) > 0)
						{
							$this->db()->query("
								INSERT INTO ".DB_PREFIX."products_categories (cid,pid,is_primary) 
								VALUES ".implode(',', $insertValues)
							);
						}
					}
				}
			}
			// Find products currently allocated in doba categories
			
		}
	}
}

