<?php 

class InsureShip
{
	public $accountId = "";
	public $ssl_pem_path = "";
	public $error = "";
	public $apiURL = "https://api.insureship.com/api/post_test/";

	/**
	 * Class constructor
	 * @param string $accountId
	 * @return InsureShip
	 */
	public function __construct($accountId = "", $apiURL = false)
	{
		$this->accountId = $accountId;
		if ($apiURL) $this->apiURL = $apiURL;		
		$this->ssl_pem_path = str_replace('classes', 'ssl', dirname(__FILE__));
		return $this;
	}
	
	/**
	 * Connect to InsureShip server and get result back
	 */
	public function call($mode, $body)
	{
		$xml = 
			'<xml_request>'."\n".
				'<account_id>'.$this->accountId.'</account_id>'."\n".
				'<transaction_mode>'.$mode.'</transaction_mode>'."\n".
				$body.
			'</xml_request>';
		$c = curl_init($this->apiURL);
		curl_setopt($c, CURLOPT_POST, 1);
		curl_setopt($c, CURLOPT_POSTFIELDS, $xml);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 0);
		$result = curl_exec($c);
		$ok = curl_error($c) == 0 && strpos($result, '<status>OK</status>') > 0;
		curl_close($c);
		return $ok ? $result : false;
	}
	
	/**
	 * Activate account
	 * @param array $form
	 * @return boolean
	 */
	public function activate($form)
	{
		set_time_limit(10000);
		$xml = 	
			'<?xml version="1.0" encoding="UTF-8"?>'."\n".
			'<account>'."\n".
				'<parent_id>553362</parent_id>'."\n".
				'<email>'.$form["email"].'</email>'."\n".
				'<first_name>'.$form["first_name"].'</first_name>'."\n".
				'<last_name>'.$form["last_name"].'</last_name>'."\n".
				'<company_name>'.$form["company_name"].'</company_name>'."\n".
				'<site_url>'.$form["site_url"].'</site_url>'."\n".
				'<address1>'.$form["address1"].'</address1>'."\n".
				'<address2>'.$form["address2"].'</address2>'."\n".
				'<city>'.$form["city"].'</city>'."\n".
				'<country>'.$form["country"].'</country>'."\n".
				'<state>'.$form["state"].'</state>'."\n".
				'<zip>'.$form["zip"].'</zip>'."\n".
				'<phone>'.$form["phone"].'</phone>'."\n".
				'<accounting_name>'.$form["accounting_name"].'</accounting_name>'."\n".
				'<accounting_email>'.$form["accounting_email"].'</accounting_email>'."\n".
				'<accounting_phone>'.$form["accounting_phone"].'</accounting_phone>'."\n".
				'<tech_name>'.$form["tech_name"].'</tech_name>'."\n".
				'<tech_email>'.$form["tech_email"].'</tech_email>'."\n".
				'<tech_phone>'.$form["tech_phone"].'</tech_phone>'."\n".
			'</account>';
		
		$c = curl_init('https://api.insureship.com/api/register_merchant/');
		curl_setopt($c, CURLOPT_POST, 1);
		curl_setopt($c, CURLOPT_POSTFIELDS, $xml);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 1);
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 1);
		curl_setopt($c, CURLOPT_CAPATH, $this->ssl_pem_path);
		curl_setopt($c, CURLOPT_CAINFO, $this->ssl_pem_path.'/cacert.pem');
		
		$result = curl_exec($c);
		
		if ($result && curl_errno($c) == 0)
		{
			$xml = simplexml_load_string($result);
			
			if ($xml->status == "SUCCESS")
			{
				return $xml->account_id;
			}
			else
			{
				$this->error = "Please enter correct information into all fields";
				return false;
			}
		}
		
		if (curl_errno($c))
		{
			$this->error = curl_error($c);
		}
		else 
		{
			$this->error = "Cannot create account. Please try again";
		}
		return false;
	}
	
	/**
	 * Get rate for current order
	 * @param float $subtotal
	 */
	public function getRate($amount)
	{
		$c = curl_init($u = 'http://rates.insureship.com/api/premium/?aid='.$this->accountId.'&amount='.number_format($amount, 2, ".", ""));
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		$result = curl_exec($c);
		$return = curl_errno($c) > 0 ? 0 : $result;
		curl_close($c);
		return $return;
	}
	
	/**
	 * 
	 * @param $order_id
	 * @param $db
	 */
	public function add($user, $order, $db)
	{
		$billing = $user->data;
		$order->getShippingAddress();
		$shipping = isset($order->shippingAddress) ? $order->shippingAddress : false;
		
		$body = 
			'<record>'."\n".
				"\t".'<order_id>'.utf8_encode($order->order_num).'</order_id>'."\n".
				"\t".'<email>'.utf8_encode(str_replace("&", "&amp;", $billing["email"])).'</email>'."\n".
				"\t".'<phone>'.utf8_encode(str_replace("&", "&amp;", $billing["phone"])).'</phone>'."\n".
				"\t".'<first_name>'.utf8_encode(str_replace("&", "&amp;", $billing["fname"])).'</first_name>'."\n".
				"\t".'<last_name>'.utf8_encode(str_replace("&", "&amp;", $billing["lname"])).'</last_name>'."\n".
				"\t".'<address>'.utf8_encode(str_replace("&", "&amp;", $shipping["address1"])).'</address>'."\n".
				"\t".'<address_2>'.utf8_encode(str_replace("&", "&amp;", $shipping["address2"])).'</address_2>'."\n".
				"\t".'<city>'.utf8_encode(str_replace("&", "&amp;", $shipping["city"])).'</city>'."\n".
				"\t".'<state>'.utf8_encode(str_replace("&", "&amp;", $shipping["state_abbr"])).'</state>'."\n".
				"\t".'<zip>'.utf8_encode(str_replace("&", "&amp;", $shipping["zip"])).'</zip>'."\n".
				"\t".'<country>'.utf8_encode(str_replace("&", "&amp;", $shipping["country_iso_a2"])).'</country>'."\n".
				"\t".'<insured_items>'."\n";
		
		foreach ($order->items as $item)
		{
			$product_id = str_replace("&", "&amp;", $item["product_id"].($item["product_sub_id"] != "" ? (" - ".$item["product_sub_id"]) : ""));
			$body.=
				"\t\t".'<item>'."\n".
 					"\t\t".'<item_name>'.utf8_encode(str_replace("&", "&amp;", $item["title"])." (".$product_id.")").'</item_name>'."\n".
 					"\t\t".'<item_qty>'.utf8_encode($item["quantity"]).'</item_qty>'."\n".
					"\t\t".'<insure_amount>'.utf8_encode(number_format($item["product_price"], 2, ".", "")).'</insure_amount>'."\n".
				"\t\t".'</item>'."\n";
		}
		$body .=	
				"\t".'</insured_items>'."\n".
			'</record>'."\n";
		
		$result = $this->call("ADD", $body);
		
		/**
		 * Update order
		 */
		$db->reset();
		if ($result)
		{
			$s = substr($result, strpos($result, "<uuid>") + 6);
			$insureship_policy_id = substr($s, 0, strpos($s, "</uuid>"));
			$db->assignStr("insureship_policy_id", $insureship_policy_id);
		}
		else
 		{
			$db->assignStr("insureship_active", "0");
			$db->assignStr("insureship_amount", "0");
 		}
		$db->update(DB_PREFIX."orders", "WHERE oid=".$order->oid);
 		
 		return $result;
	}
	
	/**
	 * 
	 * @param $order_id
	 * @param $db
	 */
	public function track($order_id, &$db)
	{
		
	}
	
	/**
	 * 
	 * @param $order_id
	 * @param $db
	 */
	public function cancel($order_id, &$db)
	{
		$body = 
			'<record><order_id>'.$order_id.'</order_id></record>';
		
		$result = $this->call("CANCEL", $body);
		
		if ($result)
 		{
			$db->reset();
			$db->assign("total_amount", "total_amount-insureship_amount");
			$db->assignStr("insureship_active", "0");
			$db->assignStr("insureship_amount", "0");
			$db->update(DB_PREFIX."orders", "WHERE order_num=".$order_id);
 		}
 		return $result;
	}
}

?>