<?php
/**
 * Process one page checkout request
 */

	if (!defined("ENV")) exit();
	
	define('OPC_MSG_ERROR_DATA_MISSED', 1);
	
	/**
	 * Init 
	 */
	$action = isset ($_POST["action"]) ? $_POST["action"] : false;
	$data = isset ($_POST["data"]) ? json_decode($_POST["data"]) : false;
	
	$opc = new OPC_Response();
	$getShippingMethods = false;
	$recheckGiftCertificate = true;
	$processOrderData = true; // should be set to false after successful payment completed

	$bongoActive = false;
	$bongoUsed = false;

	//check min subtotal amount
	$userLevel = ($user->auth_ok) ? ($user->level) : ($userCookie ? $userCookie["l"] : 0);
	$min_subtotal_amount = normalizeNumber($settings["MinOrderSubtotalLevel".$userLevel]);
	
	if ($min_subtotal_amount > $order->subtotalAmount)
	{
		$opc->setError($msg["cart"]["minimal_sustotal_not_reached"]."\n".$msg["cart"]["minimal_sustotal"].": ".getPrice($min_subtotal_amount));
	}
	elseif (!$action || !$data)
	{
		$opc->setError("Error processing form");
	}
	else
	{
		/**
	 	 * Process action
 	 	 */
		switch ($action)
		{
			case "doLogin" :
			{
				$_POST["login"] = $data->username;
				$_POST["password"] = $data->password;
				$opc_login = true;
				unset($_POST["remember_me"]); 
				require_once("content/engine/actions/action.user_login.php");
				if (!$login_successful)
				{
					$opc->setError("Please enter valid username and password");
				}
				break;
			}
			/**
			 * Save billing data
			 */
			case "saveBillingData" : 
			{
				//billing form
				$billingForm = $opc->unserialize($data->billingForm);

				$customFieldsData = isset($billingForm["custom_field"]) ? $billingForm["custom_field"] : array();
				unset($billingForm["custom_field"]);
				unset($billingForm["custom_field_required"]);
				unset($billingForm["custom_field_name"]);
				unset($billingForm["custom_field_place"]);

				// try to validate billing data with USPS
				if ($data->shippingAsBilling && $settings['EndiciaUspsAddressValidation'] == 1)
				{
					if (!isset($billingForm["payment-profile-id"]) || (isset($billingForm["payment-profile-id"]) && $billingForm["payment-profile-id"] == 'billing'))
					{
						// check is address correct, and if it is not, just exit
						if (!AddressValidator::validate($db, $billingForm, $msg))
						{
							$opc->setError(AddressValidator::getLastError());
							break;
						}
						else
						{
							$addressVariants = AddressValidator::getAddressVariants();
							if (!is_null($addressVariants) && $addressVariants)
							{
								$opc->setData("shippingAddressVariants", $addressVariants);
							}
						}
					}
				}

				//check data to convert current express checkout to normal user
				$positiveResult = false;
				if (isset($billingForm["create_account"]) && $billingForm["create_account"])
				{
					//try to create account
					if (!$user->signUp($billingForm, $customFieldsData, $user->id))
					{
						$opc->setError($user->errors);		
					}
					else
					{
						emailNewUserRegistered($db, $user->id, $customFields);
						
						$user->login($billingForm["login"], $billingForm["password"]);
						
						if ($user->auth_ok)
						{
							//set tracking cookie
							$user->setCookie();

							// if http and https domain names are different, set cookie on http site too
							if ($user->isCrossDomainCookieRequired())
							{
								$opc->setData("backgroundURL", $user->getCrossDomainCookieUrl($url_http, false));
							}

							unset($_SESSION["user_started_checkout"]);
							
							$positiveResult = true;
						
							$opc->setData("accountCreated", true);
						}
						else
						{
							$opc->setError("Unknown error happened");	
						}
					}
				}
				else
				{
					//check is payment profile used
					if (isset($billingForm["payment-profile-id"]) && $billingForm["payment-profile-id"] != 'billing')
					{
						$paymentProfileId = $billingForm["payment-profile-id"];
						require_once("content/classes/class.payment_profile.php");
						$paymentProfileManager = new Payment_Profile();
						$paymentProfile = $paymentProfileManager->getPaymentProfileById($db, $user, $paymentProfileId);
						
						if ($paymentProfile)
						{
							$user->updateBillingInfoFromPaymentProfile($paymentProfile);
							$positiveResult = true;
						}
					}
					else
					{
						//try to store billing data
						if (!$user->updateBillingInfo($billingForm, $customFieldsData))
						{
							$opc->setError($user->errors);
						}
						else
						{
							$positiveResult = true;
							$opc->setData("accountCreated", false); 
						}
					}
				}

				if ($positiveResult)
				{
					$user->getUserData();
					if ($settings['TaxAddress'] == 'Billing')
					{
						$order->reupdateItems();
					}

					$billingData = $user->getBillingInfo();
					$opc->setData("billingData", $billingData);

					//update shipping address when needed
					if ($data->shippingAsBilling)
					{
						$billingData['address_type'] = isset($billingData['company']) && trim($billingData['company']) != '' ? 'Business' : 'Residential';
						$order->updateShippingAddress($billingData);

						/**
						 * Set updated data
						 */
						$opc->setData("shippingAddress", $order->getShippingAddress());
						
						$getShippingMethods = true;
					}
				}
				break;
			}
			
			/**
			 * Check promo code
			 */
			case "checkPromoCode" :
			{
				$promoResult = false;

				// Reset promo first
				$order->resetPromo(false);

				// Get form data
				$promoCode = $data->promo;
				
				// Check promo code
				if (trim($promoCode) != '' && validate($promoCode, VALIDATE_STRING))
				{
					$promoResult = $order->checkPromoCode($promoCode);
				}
				else
				{
					$order->resetPromo(false);
					$order->calculateTax();
				}

				// Save to session or return error
				if ($promoResult)
				{
					$_SESSION["order_promo_code"] = $promoCode;
				}
				else
				{
					unset($_SESSION["order_promo_code"]);
					if (trim($promoCode) != '')
					{
						$opc->setError("Entered promo code is expired, cannot be used with your order or is invalid");
					}
				}
				
				// Set response
				$opc->setData("promoResult", $promoResult);
				
				break;
			}
			
			/**
			 * Save shipping data
			 */
			case "saveShippingAddress" : 
			{
				//update shipping address when needed
				$shippingForm = $opc->unserialize($data->shippingForm);
				
				//custom fields
				$customFieldsData = isset($shippingForm["custom_field"]) ? $shippingForm["custom_field"] : array();
				unset($shippingForm["custom_field"]);
				unset($shippingForm["custom_field_required"]);
				unset($shippingForm["custom_field_name"]);
				unset($shippingForm["custom_field_place"]);
				
				$shippingAddressChanged = false;

				/**
				 * Shipping the same as billing
				 */
				if ($shippingForm["shipping-address-id"] == "billing")
				{
					$billingData = $user->getBillingInfo();
					$billingData['address_type'] = isset($billingData['company']) && trim($billingData['company']) != '' ? 'Business' : 'Residential';
					$shippingForm = array_merge($shippingForm, $billingData);
				}
				/**
				 * Shipping address from address book
				 */
				elseif (is_numeric($shippingForm["shipping-address-id"]))
				{
					$shippingForm = array_merge($shippingForm, $user->getShippingAddressById(intval($shippingForm["shipping-address-id"])));
				}
				else
				{
					// we suppose that address is new
				}

				/**
				 * Validate address
				 */
				if (!$user->validateAddress($shippingForm, USER::ADDRESS_SHIPPING))
				{
					$opc->setError($user->errors);
				}

				if (is_array($customFieldsData))
				{
					$order->customFields()->resetErrors();
					$order->customFields()->validateCustomFields("shipping", $customFieldsData);

					if ($order->customFields()->is_error)
					{
						$opc->setError($order->customFields()->errors);
					} 
				}

				if ($opc->errors()) break;

				// try to validate shipping data with USPS
				if ($settings['EndiciaUspsAddressValidation'] == 1)
				{
					// check is address correct, and if it is not, just exit
					if (!AddressValidator::validate($db, $shippingForm, $msg))
					{
						$opc->setError(AddressValidator::getLastError());
						break;
					}
					else
					{
						$addressVariants = AddressValidator::getAddressVariants();
						if (!is_null($addressVariants) && $addressVariants)
						{
							$opc->setData("shippingAddressVariants", $addressVariants);
						}
					}
				}

				$_SESSION["opc-shipping-address-id"] = $shippingForm["shipping-address-id"];

				/**
				 * Save shipping address
				 */
				$order->updateShippingAddress($shippingForm, $customFieldsData);
				
				/**
				 * Set updated data
				 */
				$opc->setData("shippingAddress", $order->getShippingAddress());
				
				/**
				 * Get shipping methos
				 */
				$getShippingMethods = true;
				break;
			}
			
			/**
			 * Get shipping methos
			 */
			case "getShippingMethods" :
			{
				$getShippingMethods = true;
				break;
			}

			/**
			 * Set shipping method
			 */
			case "setShippingMethod" :
			{
				// Check shipping method
				if (trim($data->shippingMethodId) != '')
				{
					$order->updateShippingMethod($data->shippingMethodId);
					$errorMessage = '';
					$order->calculateShipping($errorMessage);

					if ($errorMessage != '')
					{
						$opc->setError($errorMessage);
					}
				}

				$getShippingMethods = true;

				break;
			}
			
			/**
			 * Apply gift certificate
			 */
			case "applyGiftCertificate" :
			{
				// Check and apply gift certificate
				$giftCertificateForm = $opc->unserialize($data->giftCertificateForm);
				
				$giftError = true;
				
				$gcFirstName = trim($giftCertificateForm["gift_cert_first_name"]);
				$gcLastName = trim($giftCertificateForm["gift_cert_last_name"]);
				$gcVoucher = trim($giftCertificateForm["gift_cert_voucher"]);
				
				if ($gcFirstName != '' && $gcLastName != '' && $gcVoucher != '')
				{
					$giftCert = new GiftCertificates($db);
					$giftCertData = $giftCert->GetOrderedGiftCert($gcFirstName, $gcLastName, $gcVoucher);
					
					// Note: we only validating gift certificate without applying
					if ($giftCertData) 
					{
						// gift certificate is correct
						$giftError = false;
						$order->setGiftCertificateAmount($giftCertData["balance"]);

						GiftCertificates::saveToSession($gcFirstName, $gcLastName, $gcVoucher);
					}
					else
					{
						$order->setGiftCertificateAmount(0);

						GiftCertificates::removeFromSession();						
					}
				}

				if ($giftError)
				{
					$opc->setError("Provided gift certificate is not valid");
				}	
			
				$recheckGiftCertificate = false;

				break;
			}
		
			/**
			 * Process payment
			 */
			case "processPayment" :
			{
				// extract data from post
				$paymentMethodsForm = $opc->unserialize($data->paymentMethodsForm);
				$paymentMethodForm = $opc->unserialize($data->paymentMethodForm);

				$order->updatePaymentMethod($paymentMethodsForm['payment-method-id']);

				// Check additional information
				if ($data->additionalInformation)
				{
					$additionalInformation = $opc->unserialize($data->additionalInformation);
					
					//gift message
					if ($settings["GiftCardActive"] == "YES")
					{
						if (isset($additionalInformation["add_gift"]) && $additionalInformation["add_gift"] == "yes")
						{
							$order->storeGiftMessage(isset($additionalInformation["gift_message"]) ? $additionalInformation["gift_message"] : "");
						}
					}
					
					//custom fields
					$customFieldsData = isset($additionalInformation["custom_field"]) ? $additionalInformation["custom_field"] : array();
					
					//store custom fields
					if (is_array($customFieldsData))
					{
						if (!$user->customFields()->validateCustomFields("invoice", $customFieldsData))
						{
							$opc->setError($user->customFields()->errors);
						}
						else
						{
							$customFields->storeCustomFields("invoice", $customFieldsData, $user->id, $order->oid, 0);
						}
					}
				}
				
				// Recheck promo code
				if (isset($_SESSION["order_promo_code"]) && $_SESSION["order_promo_code"] != '')
				{
					$promoResult = $order->checkPromoCode($_SESSION["order_promo_code"]);
					if (!$promoResult)
					{
						$_SESSION["order_promo_code"] = '';
						$opc->setError("Entered promo code is expired, cannot be used with your order or is invalid");
					}
				}
				
				// Recheck and apply gift certificate
				if ($data->giftCertificateForm)
				{
					$giftError = true;
					
					$giftCertificateForm = $opc->unserialize($data->giftCertificateForm);
					
					$gcFirstName = trim($giftCertificateForm["gift_cert_first_name"]);
					$gcLastName = trim($giftCertificateForm["gift_cert_last_name"]);
					$gcVoucher = trim($giftCertificateForm["gift_cert_voucher"]);
				
					if ($gcFirstName != '' && $gcLastName != '' && $gcVoucher != '')
					{
						$giftCert = new GiftCertificates($db);
						$giftCertData = $giftCert->GetOrderedGiftCert($gcFirstName, $gcLastName, $gcVoucher);
					
						if ($giftCertData) 
						{
							// gift certificate is correct
							$giftError = false;
							$giftCert->SetOrderGiftCert($order->oid, $gcFirstName, $gcLastName, $gcVoucher);
						
							if ($order->getTotalAmount() > $giftCertData['balance'])
							{
								$giftCert->UpdateOrderGCAmount($order->getId(), $giftCertData['balance']);
								$order->setGiftCertificateAmount($giftCertData['balance']);
							}
							else
							{
								$giftCert->UpdateOrderGCAmount($order->getId(), $order->getTotalAmount());
								$order->setGiftCertificateAmount($order->getTotalAmount());
								$order->updatePaymentMethod("free");
							}
						}
					}
					
					if ($giftError)
					{
						$opc->setError("Please check gift certificate");
					}
				}
				
				// define the backurl var
				$backurl = getBackUrl();

				// get order data
				$order->getOrderData();
				
				// validate and update payment method
				$paymentMethodFreeError = false;
				
				if ((strtolower($paymentMethodsForm["payment-method-id"]) == "free" || $paymentMethodsForm["payment-method-id"] == 0) && ($order->getTotalAmount() > 0) && ($order->getGiftCertificateAmount() != $order->getTotalAmount()))
				{
					$paymentMethodFreeError = true;
				}
				
				if ($paymentMethodFreeError || !$order->updatePaymentMethod($paymentMethodsForm["payment-method-id"]))
				{
					$opc->setError("Please select payment method");
				}
				
				// Process payment
				if (!$opc->errors())
				{
					if ($data->paymentMethodType != 'ipn')
					{
						foreach ($paymentMethodForm as $key=>$value)
						{
							$_REQUEST[$key] = $value;
							$_POST[$key] = $value;
						}
						if ($data->paymentMethodType != 'custom' && $data->paymentMethodType != 'free')
						{
							require_once("content/engine/actions/action.order_process_payment.php");
							
							if (isset($payment) && get_class($payment) == 'PAYMENT')
							{
								if ($payment->redirect && $payment->redirectURL)
								{
									$opc->setData("backURL", $payment->redirectURL);
									$opc->setData("verification", true);
								}
								else if (isset($is_payment_error) && $is_payment_error)
								{
									if (isset($account_blocked) && $account_blocked)
									{
										$opc->setData("backURL", $backurl);
									}
									$opc->setData("paymentErrors", $payment->error_message);
									$opc->setData("success", false);
								}
								else
								{
									$opc->setData("backURL", $backurl);
									$opc->setData("success", true);
									$processOrderData = false;
								}
							}
						}
						else
						{
							require_once("content/engine/actions/action.order_complete_order.php");
							$opc->setData("backURL", $backurl);
							$opc->setData("success", true);
							$processOrderData = false;
						}	
							
						break;	
					}
				}
				// NOTE: do not break here. Under some conditions next step may be executed
			}
			
			/**
			 * Get payment form
			 */
			case "getPaymentForm" : 
			{
				// Get form data
				$paymentMethodsForm = $opc->unserialize($data->paymentMethodsForm);
				$paymentMethodId = $paymentMethodsForm["payment-method-id"];
				$paymentProfileId = isset($data->paymentProfileId) ? $data->paymentProfileId : 'billing';
				
				$payment = isset($payment) && is_object($payment) && get_class($payment) != "PAYMENT" ? $payment : new PAYMENT($db);
				
				// Update payment method
				$order->updatePaymentMethod($paymentMethodId);
				
				$override_payment_form = false;
				$payment_form_wide = false;
				$payment_form_override = false;
				$payment_in_popup = false;
				$payment_in_popup_width = 800;
				$payment_in_popup_height = 600;
				$payment_in_popup_title = 'Payment';
				
				$paymentForm = "";
				
				$_SESSION["opc-payment-method-id"] = $_SESSION["pc_method_id"] = $order->paymentMethodId;

				// Free order
				if ($paymentMethodId == 0)
				{
					$opc->setData('paymentForm', false);
					$opc->setData('paymentFormValidatorJS', false);
				}
				
				// Render form based on gateway properties / order data
				else if ($payment->isMethod($paymentMethodId))
				{
					// Get payment method
					$paymentMethod = $payment->methods[$order->paymentMethodId];
					
					// PayPalEC
					if ($paymentMethod->id == "paypalec" && !isset($_SESSION["paypal_express_checkout_details"]))
					{
						$paymentMethod->type = "ipn";
						$paymentMethod->steps = 2;
					}
					
					// Get data required to render form 
					$order->getOrderData();
					
					// ..and set template varialbes
					view()->assign("order_num", $order->order_num);
					view()->assign("order", $order);
					view()->assign("payment_method", $paymentMethod);
					
					if ($paymentProfileId != "billing")
					{
						require_once("content/classes/class.payment_profile.php");
						$paymentProfileManager = new Payment_Profile();

						//check are there payment profiles
						$paymentProfile = $paymentProfileManager->getPaymentProfileById($db, $user, $paymentProfileId);
						if ($paymentProfile["pid"] == $order->paymentMethodId)
						{
							view()->assign("paymentProfile", $paymentProfile ? $paymentProfile : false);
						}
					}
					
					if ($order->paymentIsRealtime == "Yes")
					{
						$order->getShippingAddress();
						
						// Create gateway class instance
						$gatewayFile = escapeFileName($paymentMethod->id);
						@require_once("content/engine/payment/payment_".$gatewayFile.".php");
						
						$db->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE id='".$db->escape($paymentMethod->id)."'");
						
						if ($db->moveNext())
						{
							$payment_processor_id = $db->col["id"];
							$payment_processor_class = $db->col["class"];
							$payment_processor_name = $db->col["name"];
						}
						eval("\$paymentGateway = new ".$payment_processor_class."();"); // $payment_processor_class loaded from gateway file
						
						// Init gateway
						$paymentGateway->db_id = $order->paymentMethodId;
						$paymentGateway->enableLog = defined('DEVMODE') && DEVMODE;
						$paymentGateway->getSettingsData($db);
						$paymentGateway->assignUser($user);
						$paymentGateway->assignOrder($order);
						$orderItems = $order->getOrderItems();
						$paymentGateway->assignOrderItems($orderItems);
						$paymentGateway->getCommonData($user, $order);
						
						// Check test/demo/sandbox mode
						$paymentTestMode = $paymentGateway->isTestMode();
						
						// Get payment URL 
						$paymentURL = $paymentGateway->getPostUrl();
						
						// PayPalEC
						if ($payment_processor_id == "paypalec" && !isset($_SESSION["paypal_express_checkout_details"]))
						{
							$paymentGateway->type = "ipn";
							$paymentGateway->steps = 2;
							
							$returnUrl = $url_dynamic.'p=one_page_checkout&oa='.ORDER_PROCESS_PAYMENT;
							$cancelUrl = $url_dynamic.'p=one_page_checkout';
							
							$cartItems = $order->getOrderItemsExtended();

							$expressCheckoutUrl = $paymentGateway->expressCheckout($order, $cartItems, false, $returnUrl, $cancelUrl);

							if ($paymentGateway->is_error)
							{
								$opc->setError($paymentGateway->error_message);
							}
							else
							{
								$paymentURL = $expressCheckoutUrl;
							}
						}
						
						if (trim($paymentURL) == "") $paymentURL = $url_https."p=one_page_checkout"; // Force HTTPS here
						
						$validatorJS = $paymentGateway->getValidatorJS();
						
						$payment_form_wide = $paymentGateway->payment_form_wide;
						$payment_in_popup = $paymentGateway->payment_in_popup;
						$payment_in_popup_width = $paymentGateway->payment_in_popup_width;
						$payment_in_popup_height = $paymentGateway->payment_in_popup_height;
						$payment_in_popup_title = $paymentGateway->payment_in_popup_title;
						
						if (!$paymentGateway->override_payment_form)
						{
							view()->assign("payment_test_mode", $paymentTestMode);
							view()->assign("auth_express", $user->express ? "yes" : "no");
							view()->assign("payment_in_popup", $payment_in_popup);
							view()->assign("payment_in_popup_width", $payment_in_popup_width);
							view()->assign("payment_in_popup_height", $payment_in_popup_height);
							view()->assign("payment_in_popup_title", $payment_in_popup_title);
							view()->assign("payment_form", $paymentGateway->getPaymentForm($db));
							view()->assign("payment_profile_form", $paymentGateway->getPaymentProfileForm($db));
							view()->assign("show_cvv2_info", $paymentGateway->show_cvv2);
							view()->assign("payment_url", $paymentURL);
							$paymentForm = view()->fetch("templates/pages/checkout/opc/opc-payment-form.html");
						}
						else
						{
							$paymentForm = $paymentGateway->getPaymentForm($db);
							$paymentProfileForm = $paymentGateway->getPaymentProfileForm($db);
							$payment_form_override = true;
						}
					}
					else
					{
						// Custom payment method
						$validatorJS = '';
						$paymentURL = $url_https."p=one_page_checkout";
						view()->assign("auth_express", $user->express ? "yes" : "no");
						view()->assign("payment_url", $paymentURL);
						$paymentForm = view()->fetch("templates/pages/checkout/opc/opc-payment-form.html");
					}
					
					// TODO: gift cerificate
					/**
					if ($order->gift_cert_amount > 0 && $order->totalAmount > $order->gift_cert_amount)
					{
						//$objResponse->addAssign("gift_cert_error", "style.display", "none");
						//$objResponse->addAssign("gift_cert_alt_payment", "style.display", "block");
						//$objResponse->addScript("altPaymentGiftCert('$order->totalAmount', '$order->gift_cert_amount')");
					}
					**/
					
					$opc->setData("paymentMethodId", $paymentMethodId);
					$opc->setData("paymentForm", $paymentForm);
					$opc->setData("paymentFormWide", $payment_form_wide);
					$opc->setData("paymentFormPopUp", $payment_in_popup);
					$opc->setData("paymentFormPopUpWidth", $payment_in_popup_width);
					$opc->setData("paymentFormPopUpHeight", $payment_in_popup_height);
					$opc->setData("paymentFormPopUpTitle", $payment_in_popup_title);
					$opc->setData("paymentFormOverride", $payment_form_override);
					$opc->setData("paymentFormUrl", $paymentURL);
					$opc->setData("paymentFormValidatorJS", $validatorJS);
				}
				else
				{
					// Error: cannot find class by ID
					$opc->setError("Cannot find payment gateway by ID");
				}
					
				/**
				 * Set session variables
				 */
				$_SESSION["pc_order_id"] = $order->oid;
				$_SESSION["pc_order_amount"] = $order->totalAmount - $order->shippingAmount - $order->taxAmount - ($order->handlingSeparated ? $order->handlingAmount : 0.00);
				$_SESSION["pc_method_id"] = $order->paymentMethodId;
				$_SESSION["pc_order_quantity"] = $order->itemsCount;
				$_SESSION["pc_order_num"] = $order->order_num;

				break;
			}					
		}
	}

	//recheck gift certificate
	if ($recheckGiftCertificate)
	{
		$giftCertStoredData = GiftCertificates::getFromSession();

		if ($giftCertStoredData)
		{
			$giftCert = new GiftCertificates($db);
			$giftCertData = $giftCert->GetOrderedGiftCert($giftCertStoredData["first_name"], $giftCertStoredData["last_name"], $giftCertStoredData["voucher"]);
						
			// Note: we only validating gift certificate without applying
			if ($giftCertData) 
			{
				// gift certificate is correct
				$giftError = false;
				$order->setGiftCertificateAmount($giftCertData["balance"]);
			}
			else
			{
				$order->setGiftCertificateAmount(0);
				GiftCertificates::removeFromSession();						
			}
		}
	}

	//get shipping methods
	if ($getShippingMethods)
	{
		unset($_SESSION['bongo_landed_costs']);

		// check for bongo shipping
		$bongoActive = $db->selectOne('SELECT * FROM '.DB_PREFIX.'payment_methods WHERE id="bongocheckout" AND active="Yes"') ? true : false;
		$bongoUsed = false;
		$bongoShippingMethod = $settings['bongocheckout_Shipping_Method'];

		// get rates
		$rates = false;
		
		require_once('content/classes/class.currencies.php');
		$currencies = new Currencies($db);
		$defaultCurrency = $currencies->getDefaultCurrency();
		$currencyExchangeRates = $currencies->getExchangeRates();

		$opc->setData('shippingIsFree', $order->shippingIsFree ? "YES" : "NO");

		$shippingQuotesViewModel = new View_ShippingQuotesViewModel($settings, $msg);
		$taxProvider = Tax_ProviderFactory::getTaxProvider();

		if ($order->getShippingIsFree())
		{	
			if ($settings["ShippingShowAlternativeOnFree"] == "YES")
			{
				$ratesAlternative = getShippingQuotes(
					$db, $user, $order, false, false, true, 
					$defaultCurrency['code'], $currencyExchangeRates,
					$bongoActive, $bongoUsed
				);

				$opc->setData("shippingMethodsAlternative", $shippingQuotesViewModel->getShippingQuotesView($order, $taxProvider, $ratesAlternative));
			}
			else
			{
				if ($bongoActive)
				{
					$order->getShippingAddress();
					$bongoUsed = $order->shippingAddress['country_id'] != '1';
				}

				$opc->setData("shippingMethodsAlternative", false);
			}
		}
		else
		{
			$rates = getShippingQuotes(
				$db, $user, $order, false, false, false, 
				$defaultCurrency['code'], $currencyExchangeRates,
				$bongoActive, $bongoUsed
			);

			if ($bongoUsed)
			{
				$rates = Payment_Bongo_BongoUtil::filterShippingMethods($rates, $bongoShippingMethod, $msg);
			}
			
			// show alternative shipping methods for free shipping products
			// TODO: recheck alternative shipping methods availability when Bongo used
			if (!$bongoUsed && $settings["ShippingShowAlternativeOnFree"] == "YES" && $order->shippingFreeProductsCount > 0)
			{
				$ratesAlternative = getShippingQuotes(
					$db, $user, $order, false, false, true, 
					$defaultCurrency['code'], $currencyExchangeRates,
					$bongoActive, $bongoUsed
				);

				$ratesAlternative = filterAlternativeShippingQuotes($rates, $ratesAlternative);
				$opc->setData("shippingMethodsAlternative", $shippingQuotesViewModel->getShippingQuotesView($order, $taxProvider, $ratesAlternative));
			}
		}

		$method_id = 'disabled';

		if (isset($rates) && !empty($rates) && $rates['shipping_methods_count'] > 0)
		{
			if (isset($_COOKIE['opc-shipping-method-id']) && !empty($_COOKIE['opc-shipping-method-id']))
			{
				$method_id = $_COOKIE['opc-shipping-method-id'];
				$found_shipping_method = false;

				foreach ($rates['shipping_methods'] as $ks => $sm)
				{
					if ($sm['ssid'] == $method_id)
					{
						$found_shipping_method = true;
						break;
					}
				}

				if (!$found_shipping_method) $method_id = 'disabled';
			}

			if ($method_id == 'disabled')
			$method_id = $rates['shipping_methods'][0]['ssid'];
		}

		$order->updateShippingMethod($method_id);

		// TODO: use session class, do not access session directly
		$_SESSION['bongo_data'] = array('bongoActive' => $bongoActive, 'bongoUsed' => $bongoUsed);

		if ($bongoActive && $bongoUsed)
		{
			// have to recalculate shipping here
			if ($action != 'setShippingMethod')
			{
				$errorMessage = '';
				$order->calculateShipping($errorMessage);
				if ($errorMessage != '') $opc->setError($errorMessage);
			}

			unset($_SESSION['bongo_landed_costs']);

			// TODO: probably change class name to enable auto load
			require_once('content/classes/Payment/Bongo/BongoConnect.php');
			$bongoConnect = new BongoConnect($settings['bongocheckout_Partner_Key']);
			// TODO: BONGO - check for zero shipping amount
			$bongoResult = $bongoConnect->getLandedCost($order->items, $order->shippingAddress['country_iso_a2'], $order->getShippingAmount());

			if ($bongoResult)
			{
				$_SESSION['bongo_landed_costs'] = $bongoResult;
			}
			else
			{
				$opc->setError($bongoConnect->getLastError());
			}
		}

		$ratesView = $shippingQuotesViewModel->getShippingQuotesView($order, $taxProvider, $rates);

		$opc->setData("shippingMethods", $rates && is_array($rates) && count($rates) > 0 ? $ratesView : false);
	}

	/**
 	 * Check do we have enough data to calculate everything
 	 */
	if ($processOrderData)
	{
		$user->getUserData();

		/**
		 * Check do we have enough data to calculate everything
		 */
		if ($order->getShippingComplete())
		{
			$shipping_error_message = '';
			$order->recalcTotals($shipping_error_message);
		}
		else
		{
			$order->recalcSubtotals();
		}

		$order->getOrderData();
		$order->getOrderItems();

		//tax-related for shipping
		$shippingTaxable = $settings["ShippingTaxable"];
		$shippingTaxRate = 0;
		$shippingTaxDescription = "";

		if ($shippingTaxable)
		{
			$shippingTaxClassId = $settings["ShippingTaxClassId"];

			$taxProvider = Tax_ProviderFactory::getTaxProvider();

			if ($taxProvider->hasTaxRate($shippingTaxClassId))
			{
				$shippingTaxRate = $taxProvider->getTaxRate($shippingTaxClassId, false);
				$shippingTaxDescription = $taxProvider->getTaxRateDescription($shippingTaxClassId);
			}
		}

		$orderData = array(
			"totalAmount" => $order->getTotalAmount(),
			"giftCertificateAmount" => $order->getGiftCertificateAmount(),
			"totalRemainingAmount" => $order->getRemainingAmountToBePaid()
		);
		$opc->setData("orderData", $orderData);

		$orderViewModel = new View_OrderLinesViewModel($settings, $msg);
		$orderViewModel->build($order);

		if (isset($_SESSION['bongo_landed_costs']))
		{
			$bongoResponse = $_SESSION['bongo_landed_costs'];

			/**
			if ($bongoResponse->ddpAvailable == 1)
			{
				$orderViewModel->addLineItem('bongo-duty-tax', 'Duty / Tax', '', $bongoResponse->dutyCost + $bongoResponse->taxCost);
			}
			else
			{
				$orderViewModel->addLineItem('bongo-duty-tax', 'Duty / Tax', '', '<span style="font-size:10px;">pay courier on delivery</span>');
			}
			**/

			$orderViewModel->addLineItem('shipping', $msg["opc"]["invoice_shipping"], '', $bongoResponse->shippingCost);

			if ($bongoResponse->insuranceCost > 0)
			{
				$orderViewModel->addLineItem('bongo-insurance', 'Insurance', '', $bongoResponse->insuranceCost);
			}

			$total = $order->getTotalAmount();
			$total += $bongoResponse->dutyCost /** + $bongoResponse->taxCost **/- $order->getShippingAmount() + $bongoResponse->shippingCost + $bongoResponse->insuranceCost;

			$orderViewModel->addLineItem('total', $msg["opc"]["invoice_total"], '', $total);
		}

		$opc->setData('orderView', $orderViewModel->asArray());

		$giftCertificateViewModel = new View_GiftCertificateViewModel($settings, $msg);
		$giftCertificateViewModel->build($order);
		$opc->setData('giftCertificateView', $giftCertificateViewModel->asArray());
	}

	if (isset($_SESSION['bongo_data']))
	{
		$opc->setData("bongoActive", $_SESSION['bongo_data']['bongoActive']);
		$opc->setData("bongoUsed", $_SESSION['bongo_data']['bongoUsed']);
	}

	echo $opc->getResponse();
	
	_session_write_close();
	$db->done();
	die();
	
