<?php 
/**
 * Order actions definitions
 */
define("ORDER_ADD_ITEM", "AddItem");
define("ORDER_REMOVE_ITEM", "RemoveItem");
define("ORDER_CLEAR_ITEMS", "ClearItems");
define("ORDER_UPDATE_ITEMS", "UpdateItems");
define("ORDER_UNLINK_USER", "UnlinkUser");
define("ORDER_RE_UPDATE_ITEMS", "ReUpdateItems");
define("ORDER_STORE_PAYPAL_EC_INFO", "StorePayPalECInfo");
define("ORDER_PROCESS_PAYMENT", "ProcessPayment");
define("ORDER_PROCESS_PAYMENT_VALIDATION", "ProcessPaymentValidation");
define("ORDER_COMPLETE_ORDER", "CompleteOrder");
define("ORDER_WISHLIST_TO_CART", "OrderWishlistToCart");
define("ORDER_OPC", "OnePageCheckout");
define("ORDER_ADD_GIFT_CERT", "AddGiftCert");
define("ORDER_RESTORE_CART", "RestoreCart");

define("ORDER_ADD_RESULT_NOT_FOUND", 0);
define("ORDER_ADD_RESULT_ADDED", 1);
define("ORDER_ADD_RESULT_STOCK_ISSUE", -1);

/**
 * Order class
 */
class ORDER
{
	const STATUS_NEW = 'New';
	const STATUS_ABANDON = 'Abandon';
	const STATUS_PROCESS = 'Process';
	const STATUS_BACKORDER = 'Backorder';
	const STATUS_CANCELED = 'Canceled';
	const STATUS_COMPLETED = 'Completed';
	const STATUS_FAILED = 'Failed';

	const PAYMENT_STATUS_PENDING = 'Pending';
	const PAYMENT_STATUS_PARTIAL = 'Partial';
	const PAYMENT_STATUS_RECEIVED = 'Received';
	const PAYMENT_STATUS_REFUNDED = 'Refunded';
	const PAYMENT_STATUS_DECLINED = 'Declined';
	const PAYMENT_STATUS_CANCELED = 'Canceled';
	const PAYMENT_STATUS_AUTH_EXPIRED = 'Auth Expired';
	const PAYMENT_STATUS_REVERSED = 'Reversed';

	const TAX_EXEMPT_NO = 'No';
	const TAX_EXEMPT_YES = 'Yes';
	const TAX_EXEMPT_PARTIAL = 'Partial';

	// order properties
	private $_db = null;
	private $_settings = null;
	private $_user = null;
	
	private $_userLevel = 0;
	
	public $oid = 0, $order_num = 0;
	public $errors;

	private $forceRecalc = false;

	public $securityId = "";

	public $status_date;
	public $status_date_formatted;

	//products in cart
	//var $productsCount = 0;
	public $itemsCount = 0;
	
	//subtotal amount
	public $subtotalAmount = 0;
	public $subtotalAmountWithTax = 0;
	public $cartSubtotalAmount = 0;

	//discount
	public $discountAmount = 0;
	public $discountValue = 0;
	public $discountType = "";
	
	//promo discount
	public $promoDiscountAmount = 0;
	public $promoDiscountValue = 0;
	public $promoDiscountType = "";
	public $promoType = "";
	public $promoCampaignId = 0;
	
	//tax
	public $taxCountryId = 0;
	public $taxStateId = 0;
	public $taxAmount = 0;
	public $taxExempt = ORDER::TAX_EXEMPT_NO;

	//handling - common
	public $handlingSeparated = false;
	public $handlingFee = 0.00;
	public $handlingFeeType = "";	
	public $handlingFeeWhenShippingFree = 0.00;
	public $handlingText = "";
	public $handlingAmount = 0.00; // when shipping IS NOT FREE
	//handling - tax-related
	public $handlingTaxable = false; // $settings["ShippingHandlingTaxable"]
	public $handlingTaxClassId = 0; // $settings["ShippingHandlingTaxClassId"]
	public $handlingTaxRate = 0.00;
	public $handlingTaxDescription = "";
	public $handlingTaxAmount = 0.00;
	
	//shipping
	public $shippingIsFree = false;
	public $shippingIgnoreFree = false;
	public $shippingFreeProductsCount = 0;
	public $shippingDigital = 0;
	public $shippingSSID = "0";
	public $shippingCarrierID = "";
	public $shippingMethodID = "";
	public $shippingCMName = "";
	public $shippingAddress = array();
	public $shippingAmount = 0.00;	

	public $shippingTrackingNumber = '';
	public $shippingTrackingNumberType = '';
	public $shippingTracking = array();

	public $shippingTaxable = 0;
	public $shippingTaxClassId = 0;
	public $shippingTaxRate = 0.00;
	public $shippingTaxDescription = "";
	public $shippingTaxAmount = 0.00;

	public $giftMessage = "";

	public $is_mobile = 0;
	public $order_source = '';
	public $offsite_campaign_id = 0;

	//payment
	public $paymentIsRealtime = "";
	public $paymentMethodId = "";
	public $paymentMethodName = "";
	public $paymentMethodDescription = "";
	public $paymentGatewayId = "";

	//insure ship
	public $insureShipActive = false;
	public $insureShipAmount = 0;
	public $insureShipPolicyId = "";

	//order total amount
	public $totalAmount = 0;

	public $status;
	public $payment_status;

	// *** GIFT CERTIFICATE START
	public $gift_cert_amount = 0;
	// *** GIFT CERTIFICATE END

	//custom fields
	private $_customFields = null;

	protected $msg = null;

	public $items;
	/** @var array  */
	public $lineItems;

	/**
	 * Order class constructor
	 * 
	 * @param DB $db
	 * @param array $settings
	 * @param USER $user
	 * @param int $oid
	 * @param string $securityId
	 * 
	 * @return ORDER
	 */
	public function __construct(&$db, &$settings, &$user, $oid = 0, $securityId = '')
	{
		$this->_db = $db;
		$this->_settings = $settings;
		$this->_user = $user;

		// Ensure that items / lineItems are valid arrays
		$this->lineItems = array();
		$this->items = array();

		$this->setId($oid);
		$this->errors = array();
		$this->setSecurityId($securityId);
		
		// *** GIFT CERTIFICATE START
		if ($settings['enable_gift_cert'] == 'Yes' && $this->getId() > 0)
		{
			$gift_cert = new GiftCertificates($this->_db);
			$data = $gift_cert->GetOrderGiftCert($this->getId());
			if ($data['gift_cert_valid'] == '1')
			{
				$this->setGiftCertificateAmount($data['gift_cert_amount']);
			} 
			else
			{
				$this->setGiftCertificateAmount(0);
			}
		}
		// *** GIFT CERTIFICATE END
		
		$this->setUserLevel($user->getLevel());
	}

	/**
	 * Get order data from session
	 * 
	 * @param DB $db
	 * @param array $settings
	 * @param USER $user
	 * @param bool $need_db_record
	 * @param boolean $check_user_change
	 * 
	 * @return ORDER
	 */
	public static function getFromSession(&$db, &$settings, &$user, $need_db_record = false, $check_user_change = true)
	{
		global $msg, $userCookie;

		//////////////////////////
		// set initial params
		$securityId = isset($_COOKIE['_pcod']) ? $_COOKIE['_pcod'] : '';
		$securityId = $securityId == '' ? isset($_POST['_pcod']) ? $_POST['_pcod'] : $securityId : $securityId;
		$securityId = $securityId == '' ? isset($_GET['_pcod']) ? $_GET['_pcod'] : $securityId : $securityId;
		
		///////////////////////////////////////////////////////////
		// Trying to restore order by order id or create new one
		// $update_user_info = false;

		$oid = 0;
		$createNewOrder = true;

		if ($securityId != '')
		{
			$orderData = $db->selectOne('SELECT uid, oid FROM '.DB_PREFIX.'orders WHERE security_id="'.$db->escape($securityId).'" AND (status="Abandon" OR status="New") AND (payment_status = "Pending")', MYSQL_ASSOC);
			if ($orderData)
			{
				//use existing order
				if ($check_user_change && ($orderData['uid'] <> $user->getId()) && ($user->getId() > 0))
				{
					$db->reset();
					$db->assign('uid', $user->getId());
					$db->update(DB_PREFIX.'orders', 'WHERE oid='.intval($orderData['oid']));
				}
				$createNewOrder = false;
				$oid = $orderData['oid'];
			}
		}

		if ($createNewOrder && $need_db_record)
		{
			$securityId = md5(uniqid(rand(), true)).time();
			
			ORDER::setCookie($securityId);

			$db->reset();
			$db->assignStr('security_id', $securityId);
			$db->assign('uid', $user->getId());
			$db->assign('create_date', 'NOW()');
			$db->assign('status_date', 'NOW()');
			$db->assignStr('status', 'New');
			
			if (defined('MOBILE')) $db->assign('is_mobile', MOBILE ? '1' : '0');
			if (isset($_SESSION['osc_source'])) $db->assignStr('order_source', $_SESSION['osc_source']);
			if (isset($_SESSION['osc_campaign_id'])) $db->assignStr('offsite_campaign_id', $_SESSION['osc_campaign_id']);

			$oid = $db->insert(DB_PREFIX.'orders');
		}

		$order = new ORDER($db, $settings, $user, $oid, $securityId);

		if (defined("MOBILE")) $order->setIsMobile(MOBILE ? '1' : '0');
		if (isset($_SESSION["osc_source"])) $order->setOrderSource($_SESSION['osc_source']);
		if (isset($_SESSION["osc_campaign_id"])) $order->setOffsiteCampaignId($_SESSION['osc_campaign_id']);

		$order->setLanguageMessages($msg);
		$order->setUserLevel($user->auth_ok ? $user->getLevel() : ($userCookie ? $userCookie["l"] : 0));

		return $order;
	}

	/**
	 * Get order by ID
	 * 
	 * @param DB $db
	 * @param array $settings
	 * @param USER $user
	 * @param int $oid
	 * 
	 * @return ORDER
	 */
	public static function getById(&$db, &$settings, &$user, $oid)
	{
		global $msg;

		$data = $db->selectOne("SELECT $oid FROM ".DB_PREFIX."orders WHERE oid=".intval($oid)." AND uid=".$user->getId());

		if ($data)
		{
			// User owns order
			$order = new ORDER($db, $settings, $user, $oid);

			$order->setLanguageMessages($msg);

			Tax_ProviderFactory::getTaxProvider()->initTax($order);
			return $order;
		}

		return null;
	}

	/**
	 * Set order's id
	 *
	 * @param int $id
	 */
	public function setId($id)
	{
		$this->oid = $id;
	}

	/**
	 * Returns order id ($oid)
	 *
	 * @return int
	 */
	public function getId()
	{
		return intval($this->oid);
	}

	/**
	 * Returns assigned users's class instance
	 *
	 * @return USER
	 */
	public function getUser()
	{
		return $this->_user;
	}

	/**
	 * Returns user's level
	 *
	 * @return int
	 */
	public function getUserLevel()
	{
		return $this->getUser()->getLevel();
	}

	/**
	 * Sets order security id
	 *
	 * @param $securityId
	 */
	public function setSecurityId($securityId)
	{
		$this->securityId = $securityId;
	}

	/**
	 * Returns order security id
	 *
	 * @return string
	 */
	public function getSecurityId()
	{
		return $this->securityId;
	}

	/**
	 * Setter for order number
	 *
	 * @param string $orderNumber value to set
	 */
	public function setOrderNumber($orderNumber)
	{
		$this->order_num = $orderNumber;
	}

	/**
	 * Getter for order number
	 *
	 * @return string
	 */
	public function getOrderNumber()
	{
		return $this->order_num;
	}

	public function getForceRecalc()
	{
		return $this->forceRecalc;
	}

	/**
	 * Setter for items count
	 *
	 * @param int $itemsCount value to set
	 */
	public function setItemsCount($itemsCount)
	{
		$this->itemsCount = $itemsCount;
	}

	/**
	 * Getter for items count
	 *
	 * @return int
	 */
	public function getItemsCount()
	{
		return $this->itemsCount;
	}

	/**
	 * Set tax country id
	 *
	 * @param int $countryId
	 */
	public function setTaxCountryId($countryId)
	{
		$this->taxCountryId = $countryId;
	}

	/**
	 * Returns tax country id
	 *
	 * @return int
	 */
	public function getTaxCountryId()
	{
		return $this->taxCountryId;
	}

	/**
	 * Set tax state id
	 *
	 * @param int $taxStateId
	 */
	public function setTaxStateId($taxStateId)
	{
		$this->taxStateId = $taxStateId;
	}

	/**
	 * Returns tax state id
	 *
	 * @return int
	 */
	public function getTaxStateId()
	{
		return $this->taxStateId;
	}

	/**
	 * Setter for subtotal amount
	 *
	 * @param float $subtotalAmount value to set
	 */
	public function setSubtotalAmount($subtotalAmount)
	{
		$this->subtotalAmount = $subtotalAmount;
		$this->cartSubtotalAmount = $subtotalAmount;
	}

	/**
	 * Getter for subtotal amount
	 *
	 * @return float
	 */
	public function getSubtotalAmount()
	{
		return $this->subtotalAmount;
	}

	/**
	 * Set subtotal with tax
	 *
	 * @param float $subtotalAmountWithTax
	 */
	public function setSubtotalAmountWithTax($subtotalAmountWithTax)
	{
		$this->subtotalAmountWithTax = $subtotalAmountWithTax;
	}

	/**
	 * Returns subtotal with tax
	 *
	 * @return float
	 */
	public function getSubtotalAmountWithTax()
	{
		return $this->subtotalAmountWithTax;
	}

	/**
	 * Setter for discount amount
	 *
	 * @param float $discountAmount value to set
	 */
	public function setDiscountAmount($discountAmount)
	{
		$this->discountAmount = $discountAmount;
	}

	/**
	 * Getter for discount amount
	 *
	 * @return float
	 */
	public function getDiscountAmount()
	{
		return $this->discountAmount;
	}

	/**
	 * Setter for discount value
	 *
	 * @param float $discountValue value to set
	 */
	public function setDiscountValue($discountValue)
	{
		$this->discountValue = $discountValue;
	}

	/**
	 * Getter for discount value
	 *
	 * @return float
	 */
	public function getDiscountValue()
	{
		return $this->discountValue;
	}

	/**
	 * Setter for discount type
	 *
	 * @param string $discountType value to set
	 */
	public function setDiscountType($discountType)
	{
		$this->discountType = $discountType;
	}

	/**
	 * Getter for discount type
	 *
	 * @return string
	 */
	public function getDiscountType()
	{
		return $this->discountType;
	}

	/**
	 * Calculate discount percentage
	 *
	 * @return float|int
	 */
	public function getDiscountPercentage()
	{
		if ($this->getDiscountAmount() == 0)
		{
			return 0;
		}

		return $this->getDiscountType() == "percent" ? $this->getDiscountValue() : $this->getDiscountValue() / ($this->getSubtotalAmount() / 100);
	}

	/**
	 * Setter for promo discount amount
	 *
	 * @param float $promoDiscountAmount value to set
	 */
	public function setPromoDiscountAmount($promoDiscountAmount)
	{
		$this->promoDiscountAmount = $promoDiscountAmount;
	}

	/**
	 * Getter for promo discount amount
	 *
	 * @return float
	 */
	public function getPromoDiscountAmount()
	{
		return $this->promoDiscountAmount;
	}

	/**
	 * Setter for promo discount type
	 *
	 * @param string $promoDiscountType value to set
	 */
	public function setPromoDiscountType($promoDiscountType)
	{
		$this->promoDiscountType = $promoDiscountType;
	}

	/**
	 * Getter for set promo discount type
	 *
	 * @return string
	 */
	public function getPromoDiscountType()
	{
		return $this->promoDiscountType;
	}

	/**
	 * Get promo discount percentage
	 *
	 * @return float|int
	 */
	public function getPromoDiscountPercentage()
	{
		if ($this->getPromoDiscountAmount() == 0)
		{
			return 0;
		}

		return $this->getPromoDiscountType() == "percent" ? $this->getPromoDiscountValue() : $this->getPromoDiscountValue() / ($this->getSubtotalAmount() / 100);
	}

	/**
	 * Setter for promo discount value
	 *
	 * @param float $promoDiscountValue value to set
	 */
	public function setPromoDiscountValue($promoDiscountValue)
	{
		$this->promoDiscountValue = $promoDiscountValue;
	}

	/**
	 * Getter for promo discount value
	 *
	 * @return float
	 */
	public function getPromoDiscountValue()
	{
		return $this->promoDiscountValue;
	}

	/**
	 * Setter for promo discount campaign id
	 *
	 * @param int $promoCampaignId value to set
	 */
	public function setPromoCampaignId($promoCampaignId)
	{
		$this->promoCampaignId = $promoCampaignId;
	}

	/**
	 * Getter for promo campaign id
	 *
	 * @return int
	 */
	public function getPromoCampaignId()
	{
		return $this->promoCampaignId;
	}

	/**
	 * Setter for promo type
	 *
	 * @param string $promoType value to set
	 */
	public function setPromoType($promoType)
	{
		$this->promoType = $promoType;
	}

	/**
	 * Getter for promo type
	 *
	 * @return string
	 */
	public function getPromoType()
	{
		return $this->promoType;
	}

	/**
	 * Setter for handling separated
	 *
	 * @param bool $handlingSeparated value to set
	 */
	public function setHandlingSeparated($handlingSeparated)
	{
		$this->handlingSeparated = $handlingSeparated;
	}

	/**
	 * Getter for handling separated
	 *
	 * @return bool
	 */
	public function getHandlingSeparated()
	{
		return $this->handlingSeparated;
	}

	/**
	 * Setter for handling fee
	 *
	 * @param float $handlingFee value to set
	 */
	public function setHandlingFee($handlingFee)
	{
		$this->handlingFee = $handlingFee;
	}

	/**
	 * Getter for handling fee
	 *
	 * @return float
	 */
	public function getHandlingFee()
	{
		return $this->handlingFee;
	}

	/**
	 * Setter for handling fee type
	 *
	 * @param float $handlingFeeType value to set
	 */
	public function setHandlingFeeType($handlingFeeType)
	{
		$this->handlingFeeType = $handlingFeeType;
	}

	/**
	 * Getter for handling fee type
	 *
	 * @return string
	 */
	public function getHandlingFeeType()
	{
		return $this->handlingFeeType;
	}

	/**
	 * Setter for handlingFeeWhenShippingFree
	 *
	 * @param float $handlingFeeWhenShippingFree value to set
	 */
	public function setHandlingFeeWhenShippingFree($handlingFeeWhenShippingFree)
	{
		$this->handlingFeeWhenShippingFree = $handlingFeeWhenShippingFree;
	}

	/**
	 * Getter for handlingFeeWhenShippingFree
	 *
	 * @return float
	 */
	public function getHandlingFeeWhenShippingFree()
	{
		return $this->handlingFeeWhenShippingFree;
	}

	/**
	 * Setter for handling text
	 *
	 * @param string $handlingText value to set
	 */
	public function setHandlingText($handlingText)
	{
		$this->handlingText = $handlingText;
	}

	/**
	 * Getter for handling text
	 *
	 * @return string
	 */
	public function getHandlingText()
	{
		return $this->handlingText;
	}

	/**
	 * Setter for handling amount
	 *
	 * @param float $handlingAmount value to set
	 */
	public function setHandlingAmount($handlingAmount)
	{
		$this->handlingAmount = $handlingAmount;
	}

	/**
	 * Getter for handling amount
	 *
	 * @return float
	 */
	public function getHandlingAmount()
	{
		return $this->handlingAmount;
	}

	/**
	 * Setter for handling taxable
	 *
	 * @param bool $handlingTaxable value to set
	 */
	public function setHandlingTaxable($handlingTaxable)
	{
		$this->handlingTaxable = $handlingTaxable;
	}

	/**
	 * Getter for handling taxable
	 *
	 * @return bool
	 */
	public function getHandlingTaxable()
	{
		return $this->handlingTaxable;
	}

	/**
	 * Setter for handlingTaxRate
	 *
	 * @param float $handlingTaxRate value to set
	 */
	public function setHandlingTaxRate($handlingTaxRate)
	{
		$this->handlingTaxRate = $handlingTaxRate;
	}

	/**
	 * Getter for handlingTaxRate
	 *
	 * @return float
	 */
	public function getHandlingTaxRate()
	{
		return $this->handlingTaxRate;
	}

	/**
	 * Setter for handling tax amount
	 *
	 * @param float $handlingTaxAmount value to set
	 */
	public function setHandlingTaxAmount($handlingTaxAmount)
	{
		$this->handlingTaxAmount = $handlingTaxAmount;
	}

	/**
	 * Getter for handling tax amount
	 *
	 * @return float
	 */
	public function getHandlingTaxAmount()
	{
		return $this->handlingTaxAmount;
	}

	/**
	 * Setter for handling tax description
	 *
	 * @param string $handlingTaxDescription value to set
	 */
	public function setHandlingTaxDescription($handlingTaxDescription)
	{
		$this->handlingTaxDescription = $handlingTaxDescription;
	}

	/**
	 * Getter for handling tax description
	 *
	 * @return string
	 */
	public function getHandlingTaxDescription()
	{
		return $this->handlingTaxDescription;
	}

	/**
	 * Setter for handling tax class id
	 *
	 * @param int $handlingTaxClassId value to set
	 */
	public function setHandlingTaxClassId($handlingTaxClassId)
	{
		$this->handlingTaxClassId = $handlingTaxClassId;
	}

	/**
	 * Getter for handling tax class id
	 *
	 * @return int
	 */
	public function getHandlingTaxClassId()
	{
		return $this->handlingTaxClassId;
	}

	/**
	 * Check if we have enough information to calculate totals and order is not just started
	 *
	 * @return bool
	 */
	public function getBillingCompleted()
	{
		$user = $this->getUser();
		return
			$user && !is_null($user) && $user->auth_ok
			&& $user->data['fname'] != '' && $user->data['lname'] != ''
			&& $user->data['address1'] != '' && $user->data['city'] != ''
			&& $user->data['country'] != '0' && $user->data['zip'] != ''
			&& $user->data['email'] != '';
	}

	/**
	 * Check if we have enough information to calc shipping
	 * In this case we my run all calculations,
	 * otherwise only subtotals
	 *
	 * @return bool
	 */
	public function getShippingComplete()
	{
		return
			$this->_settings['ShippingCalcEnabled'] == 'NO' ||
			$this->checkShippingDigital() ||
			(
				(is_array($this->shippingAddress) && $this->shippingAddress['country_id'] > 0) &&
				$this->shippingCarrierID !== '' &&
				$this->shippingSSID !== '' &&
				$this->shippingMethodID !== ''
			);
	}

	/**
	 * Setter for shipping is free
	 *
	 * @param bool $shippingIsFree value to set
	 */
	public function setShippingIsFree($shippingIsFree)
	{
		$this->shippingIsFree = $shippingIsFree;
	}

	/**
	 * Getter for shipping is free
	 *
	 * @return bool
	 */
	public function getShippingIsFree()
	{
		return $this->shippingIsFree;
	}

	/**
	 * Setter for shipping ignore free
	 *
	 * @param bool $shippingIgnoreFree value to set
	 */
	public function setShippingIgnoreFree($shippingIgnoreFree)
	{
		$this->shippingIgnoreFree = $shippingIgnoreFree;
	}

	/**
	 * Getter for shipping ignore free
	 *
	 * @return bool
	 */
	public function getShippingIgnoreFree()
	{
		return $this->shippingIgnoreFree;
	}

	/**
	 * Setter for shipping free products count
	 *
	 * @param int $shippingFreeProductsCount value to set
	 */
	public function setShippingFreeProductsCount($shippingFreeProductsCount)
	{
		$this->shippingFreeProductsCount = $shippingFreeProductsCount;
	}

	/**
	 * Getter for shipping free products count
	 *
	 * @return int
	 */
	public function getShippingFreeProductsCount()
	{
		return $this->shippingFreeProductsCount;
	}

	/**
	 * Setter for shipping digital
	 *
	 * @param bool $shippingDigital value to set
	 */
	public function setShippingDigital($shippingDigital)
	{
		$this->shippingDigital = $shippingDigital;
	}

	/**
	 * Getter for shipping digital
	 *
	 * @return bool
	 */
	public function getShippingDigital()
	{
		return $this->shippingDigital;
	}

	/**
	 * Setter for shippingSSID
	 *
	 * @param int $shippingSSID value to set
	 */
	public function setShippingSSID($shippingSSID)
	{
		$this->shippingSSID = $shippingSSID;
	}

	/**
	 * Getter for shippingSSID
	 *
	 * @return int
	 */
	public function getShippingSSID()
	{
		return $this->shippingSSID;
	}

	/**
	 * Setter for shipping carrier id
	 *
	 * @param string $shippingCarrierId value to set
	 */
	public function setShippingCarrierId($shippingCarrierId)
	{
		$this->shippingCarrierID = $shippingCarrierId;
	}

	/**
	 * Getter for shipping carrier id
	 *
	 * @return string
	 */
	public function getShippingCarrierId()
	{
		return $this->shippingCarrierID;
	}

	/**
	 * Setter for shipping method id
	 *
	 * @param string $shippingMethodId value to set
	 */
	public function setShippingMethodId($shippingMethodId)
	{
		$this->shippingMethodID = $shippingMethodId;
	}

	/**
	 * Getter for shippingMethodID
	 *
	 * @return string
	 */
	public function getShippingMethodId()
	{
		return $this->shippingMethodID;
	}

	/**
	 * Setter for shippingCMName
	 *
	 * @param string $shippingCMName value to set
	 */
	public function setShippingCMName($shippingCMName)
	{
		$this->shippingCMName = $shippingCMName;
	}

	/**
	 * Getter for shippingCMName
	 *
	 * @return string
	 */
	public function getShippingCMName()
	{
		return $this->shippingCMName;
	}

	/**
	 * Setter for shippingAddress
	 *
	 * @param array $shippingAddress value to set
	 */
	public function setShippingAddress($shippingAddress)
	{
		$this->shippingAddress = $shippingAddress;
	}

	/**
	 * Getter for shippingAddress
	 *
	 * @return array
	 */
	public function getShippingAddress()
	{
		if ((!$this->shippingAddress || empty($this->shippingAddress)) && $this->getId() > 0)
		{
			$this->getShippingAddressFromDb();
		}

		return $this->shippingAddress;
	}
		
	/**
	 * Setter for shippingAmount
	 *
	 * @param float $shippingAmount value to set
	 */
	public function setShippingAmount($shippingAmount)
	{
		$this->shippingAmount = $shippingAmount;
	}

	/**
	 * Getter for shippingAmount
	 *
	 * @return float
	 */
	public function getShippingAmount()
	{
		return $this->shippingAmount;
	}

	/**
	 * Setter for shippingTrackingNumber
	 *
	 * @param string $shippingTrackingNumber value to set
	 */
	public function setShippingTrackingNumber($shippingTrackingNumber)
	{
		$this->shippingTrackingNumber = $shippingTrackingNumber;
	}

	/**
	 * Getter for shippingTrackingNumber
	 *
	 * @return string
	 */
	public function getShippingTrackingNumber()
	{
		return $this->shippingTrackingNumber;
	}

	/**
	 * Setter for shippingTrackingNumberType
	 *
	 * @param string $shippingTrackingNumberType value to set
	 */
	public function setShippingTrackingNumberType($shippingTrackingNumberType)
	{
		$this->shippingTrackingNumberType = $shippingTrackingNumberType;
	}

	/**
	 * Getter for shippingTrackingNumberType
	 *
	 * @return string
	 */
	public function getShippingTrackingNumberType()
	{
		return $this->shippingTrackingNumberType;
	}

	/**
	 * Setter for shippingTracking
	 *
	 * @param array $shippingTracking
	 */
	public function setShippingTracking(array $shippingTracking)
	{
		$this->shippingTracking = $shippingTracking;
	}

	/**
	 * Getter for shippingTracking
	 *
	 * @return array
	 */
	public function getShippingTracking()
	{
		return $this->shippingTracking;
	}

	/**
	 * Setter for shippingTaxable
	 *
	 * @param bool $shippingTaxable value to set
	 */
	public function setShippingTaxable($shippingTaxable)
	{
		$this->shippingTaxable = $shippingTaxable;
	}

	/**
	 * Getter for shippingTaxable
	 *
	 * @return bool
	 */
	public function getShippingTaxable()
	{
		return $this->shippingTaxable;
	}

	/**
	 * Setter for shippingTaxClassId
	 *
	 * @param int $shippingTaxClassId value to set
	 */
	public function setShippingTaxClassId($shippingTaxClassId)
	{
		$this->shippingTaxClassId = $shippingTaxClassId;
	}

	/**
	 * Getter for shippingTaxClassId
	 *
	 * @return int
	 */
	public function getShippingTaxClassId()
	{
		return $this->shippingTaxClassId;
	}

	/**
	 * Setter for shippingTaxRate
	 *
	 * @param float $shippingTaxRate value to set
	 */
	public function setShippingTaxRate($shippingTaxRate)
	{
		$this->shippingTaxRate = $shippingTaxRate;
	}

	/**
	 * Getter for shippingTaxRate
	 *
	 * @return float
	 */
	public function getShippingTaxRate()
	{
		return $this->shippingTaxRate;
	}

	/**
	 * Setter for shippingTaxDescription
	 *
	 * @param string $shippingTaxDescription value to set
	 *
	 */
	public function setShippingTaxDescription($shippingTaxDescription)
	{
		$this->shippingTaxDescription = $shippingTaxDescription;
	}

	/**
	 * Getter for shippingTaxDescription
	 *
	 * @return string
	 */
	public function getShippingTaxDescription()
	{
		return $this->shippingTaxDescription;
	}

	/**
	 * Setter for shippingTaxAmount
	 *
	 * @param float $shippingTaxAmount value to set
	 */
	public function setShippingTaxAmount($shippingTaxAmount)
	{
		$this->shippingTaxAmount = $shippingTaxAmount;
	}

	/**
	 * Getter for shippingTaxAmount
	 *
	 * @return float
	 */
	public function getShippingTaxAmount()
	{
		return $this->shippingTaxAmount;
	}

	/**
	 * Check is it a dollar days product
	 *
	 * @param $item
	 *
	 * @return bool
	 */

	// TODO: Really $item should be an object, and these functions would be in the Item class
	public function isDollarDaysProduct(&$item)
	{
		return $item['is_dollar_days'] == 'Yes';
	}

	/**
	 * Check is it a doba product
	 *
	 * @param $item
	 *
	 * @return bool
	 */
	public function isDobaProduct(&$item)
	{
		return $item['is_doba'] == 'Yes';
	}

	/**
	 * Check is this product level shipping - do we have shipping price on product level?
	 *
	 * @param $item
	 *
	 * @return bool
	 */
	public function isProductLevelShipping(&$item)
	{
		return $item["product_is_shipping_price"] == "Yes";
	}

	/**
	 * Check is item shippable
	 *
	 * @param $item
	 *
	 * @return bool
	 */
	public function isItemShippable(&$item)
	{
		if ($item['digital_product'] == 'Yes') return false;

		return ($item['free_shipping'] == 'Yes' && $this->getShippingIgnoreFree()) || $item['free_shipping'] == 'No';
	}

	/**
	 * Setter for gift message
	 *
	 * @param string $giftMessage value to set
	 */
	public function setGiftMessage($giftMessage)
	{
		$this->giftMessage = $giftMessage;
	}

	/**
	 * Getter for gift message
	 *
	 * @return string
	 */
	public function getGiftMessage()
	{
		return $this->giftMessage;
	}

	/**
	 * Setter for tax amount
	 *
	 * @param float $taxAmount value to set
	 */
	public function setTaxAmount($taxAmount)
	{
		$this->taxAmount = $taxAmount;
	}

	/**
	 * Getter for tax amount
	 *
	 * @return float
	 */
	public function getTaxAmount()
	{
		return PriceHelper::round($this->taxAmount);
	}

	/**
	 * Set tax exemption status
	 *
	 * @param $exempt
	 */
	public function setTaxExempt($exempt)
	{
		$this->taxExempt = $exempt;
	}

	/**
	 * Return tax exemption status
	 *
	 * @return string
	 */
	public function getTaxExempt()
	{
		return $this->taxExempt;
	}

	/**
	 * Setter for total amount
	 *
	 * @param float $totalAmount value to set
	 */
	public function setTotalAmount($totalAmount)
	{
		$this->totalAmount = $totalAmount;
	}

	/**
	 * Getter for total amount
	 *
	 * @return float
	 */
	public function getTotalAmount()
	{
		return PriceHelper::round($this->totalAmount);
	}

	/**
	 * Sets gift certificate amount
	 * 
	 * @param float $amount
	 */
	public function setGiftCertificateAmount($amount)
	{
		$this->gift_cert_amount = $amount;
	}

	/**
	 * Return gift certificate amount (total value that remains on voucher's "account")
	 * 
	 * @return float
	 */
	public function getGiftCertificateAmount()
	{
		return $this->gift_cert_amount;
	}

	/**
	 * Getter for amount that should be paid after gift certificate / or partial payment applied
	 *
	 * @return float
	 */
	public function getRemainingAmountToBePaid()
	{
		$totalAmount = $this->getTotalAmount();
		$giftCertUsedAmount = $this->getGiftCertificateAmountUsedBeforePayment();

	    return $totalAmount - $giftCertUsedAmount;
	}
	
	/**
	 * Amount that will be used from gift certificate "voucher account"
	 * 
	 * @return float
	 */
	public function getGiftCertificateAmountUsedBeforePayment()
	{
		$totalAmount = $this->getTotalAmount();
		$giftCertAmount = $this->getGiftCertificateAmount();

		return $totalAmount > $giftCertAmount ? $giftCertAmount : $totalAmount;
	}

	/**
	 * Set order status
	 *
	 * @param string $status
	 */
	public function setStatus($status)
	{
		$this->status = $status;
	}

	/**
	 * Get order status
	 *
	 * @return string
	 */
	public function getStatus()
	{
		return $this->status;
	}

	/**
	 * Set payment status
	 *
	 * @param string $payment_status
	 */
	public function setPaymentStatus($payment_status)
	{
		$this->payment_status = $payment_status;
	}

	/**
	 * Get payment status
	 *
	 * @return string
	 */
	public function getPaymentStatus()
	{
		return $this->payment_status;
	}

	/**
	 * Set is mobile
	 *
	 * @param int $is_mobile
	 */
	public function setIsMobile($is_mobile)
	{
		$this->is_mobile = $is_mobile;
	}

	/**
	 * Get is mobile
	 *
	 * @return int
	 */
	public function getIsMobile()
	{
		return $this->is_mobile;
	}

	/**
	 * Set order source
	 *
	 * @param string $order_source
	 */
	public function setOrderSource($order_source)
	{
		$this->order_source = $order_source;
	}

	/**
	 * Get order source
	 *
	 * @return string
	 */
	public function getOrderSource()
	{
		return $this->order_source;
	}

	/**
	 * Set off-site campaign id
	 *
	 * @param int $offsite_campaign_id
	 */
	public function setOffsiteCampaignId($offsite_campaign_id)
	{
		$this->offsite_campaign_id = $offsite_campaign_id;
	}

	/**
	 * Get offsite campaign id
	 *
	 * @return int
	 */
	public function getOffsiteCampaignId()
	{
		return $this->offsite_campaign_id;
	}

	/**
	 * Unlink user from order
	 */
	public function unlinkUser()
	{
		$this->db()->reset();
		$this->db()->assign("uid", 0);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());

		$this->resetValues(true);

		$this->persist();
	}

	/**
	 * Set messages from current language
	 *
	 * @param array $msg
	 */
	public function setLanguageMessages(&$msg)
	{
		$this->msg = $msg;
	}

	/**
	 * Sets user level from current active user or users cookie
	 */
	public function setUserLevel($userLevel)
	{
		$this->_userLevel = $userLevel;
	}

	/**
	 * Assign custom fields class
	 *
	 * @param bool $customFields
	 *
	 * @return mixed
	 */
	public function customFields($customFields = false)
	{
		if ($customFields) $this->_customFields = $customFields;
		return $this->_customFields = $this->_customFields != null ? $this->_customFields : new CustomFields($this->_db);
	}
	
	/**
	 * Sets order cookie
	 *
	 * @param string $securityId
	 */
	public static function setCookie($securityId)
	{
		global $settings;

		setcookie(
			"_pcod", 
			$securityId, 
			time() + 60*60*24*30*6, 
			"/",
			isSslMode() ? $settings["GlobalHttpsCookieDomain"] : $settings["GlobalHttpCookieDomain"]
		);
	}
				
	/**
	 * Set order error message
	 *
	 * @param string $message
	 */
	public function setError($message)
	{
		$this->errors[] = $message;
		$this->is_error = true;
	}
	
	/**
	 * Database class instance
	 *
	 * @return DB
	 */
	public function db()
	{
		return $this->_db;
	}
	
	/**
	 * Returns class instance
	 *
	 * @return USER
	 */
	public function user()
	{
		return $this->_user;
	}
	
	/**
	 * Create new cart from existing one
	 *
	 * @param $uid
	 */
	public function newCart($uid)
	{
		$this->user()->setId($uid);
			
		$this->db()->reset();
		$this->db()->assign("uid", intval($this->user()->getId()));
		$this->db()->assign("create_date", "NOW()");
		$this->db()->assign("status_date", "NOW()");
		$this->db()->assignStr("status", "New");
		$this->setId($this->db()->insert(DB_PREFIX."orders"));
	}
	
	/**
	 * Set abandon status
	 */
	public function setAbandonStatus()
	{
		$this->db()->reset();
		$this->db()->assignStr("status", "Abandon");
		$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());
	}

	/**
	 * Resets promo code
	 *
	 * @param bool $justPromoDiscountAmount
	 */
	public function resetPromo($justPromoDiscountAmount = true)
	{
		Calculator_PromoDiscount::getInstance($this->_settings)->reset($this, $justPromoDiscountAmount);
	}

	protected $promoCodeRepository;
	protected $productsRepository;
	protected $orderRepository;

	protected function getPromoCodeRepository()
	{
		if (is_null($this->promoCodeRepository))
		{
			$this->promoCodeRepository = new DataAccess_PromoCodeRepository($this->db());
		}

		return $this->promoCodeRepository;
	}

	public function setPromoCodeRepository(DataAccess_PromoCodeRepositoryInterface $promoCodeRepository)
	{
		$this->promoCodeRepository = $promoCodeRepository;
	}

	protected function getProductsRepository()
	{
		if (is_null($this->productsRepository))
		{
			$this->productsRepository = new DataAccess_ProductsRepository($this->db());
		}

		return $this->productsRepository;
	}

	public function setProductsRepository(DataAccess_ProductsRepositoryInterface $productsRepository)
	{
		$this->productsRepository = $productsRepository;
	}

	protected function getOrderRepository()
	{
		if (is_null($this->orderRepository))
		{
			$this->orderRepository = new DataAccess_OrderRepository($this->db());
		}

		return $this->orderRepository;
	}

	public function setOrderRepository(DataAccess_OrderRepositoryInterface $orderRepository)
	{
		$this->orderRepository = $orderRepository;
	}

	/**
	 * Checks is promo code correct.
	 *
	 * @param string $promoCode
	 *
	 * @return boolean
	 */
	public function checkPromoCode($promoCode)
	{
		if (strtolower($this->_settings['DiscountsPromo']) == 'yes' && trim($promoCode) != '')
		{
			$promoDiscountCalculator = Calculator_PromoDiscount::getInstance($this->_settings, $this->getOrderRepository(), $this->getPromoCodeRepository());

			$promoData = $promoDiscountCalculator->checkPromoDiscount($this, $promoCode);
			if (!is_null($promoData))
			{
				// Save promo data and exit
				$this->setPromoType($promoData['promo_type']);
				$this->setPromoDiscountType($promoData['discount_type']);
				$this->setPromoDiscountValue($promoData['discount']);
				$this->setPromoCampaignId($promoData['pid']);
				$this->persist();

				return true;
			}
		}

		// Clear promo code data
		$this->resetPromo(false);
		$this->persist();

		return false;
	}

	/**
	 * Add a product to a cart
	 *
	 * @param string $productId
	 * @param int $quantity
	 * @param mixed $attributes
	 * @return bool
	 */
	public function addItem($productId, $quantity, $attributes)
	{
		if (!$this->canEdit())
		{
			// TODO: Ensure that order can be modified
		}

		$productsRepository = $this->getProductsRepository();
		$orderRepository = $this->getOrderRepository();

		/* @var Model_Product $product */
		$product = $productsRepository->getProductByProductId($productId);

		if (!$product)
		{
			// TODO: Check cart for the productId, and if exists then it should be deleted

			$this->setError($this->msg['cart']['error_wrong_product_id']);
			return false;
		}

		$attributesData = $product->parseAttributes($attributes);

		/* @var Model_LineItem $lineItem */
		$lineItem = $orderRepository->findExistingLineItemForProduct($this, $productId, $attributesData['options']);

		if ($lineItem)
		{
			$isNew = false;
			$quantity += $lineItem->getAdminQuantity();
			$product->calculateProductPricing($this->getUserLevel(), $attributesData, $this->_settings);
		}
		else
		{
			$isNew = true;
			$product->calculateProductPricing($this->getUserLevel(), $attributesData, $this->_settings);

			$lineItem = Model_LineItem::createNewLineItem($this->getId(), $product, $quantity, $attributesData, 'No');

			$inventoryList = $attributesData['inventoryList'];

			if ($inventoryList != '')
			{
				if (($inventory = $productsRepository->getProductsInventory($product->getPid(), $inventoryList)) != false)
				{
					$lineItem->setInventoryId($inventory['pi_id']);
					$lineItem->setProductSubId($inventory['product_subid']);
				}
			}
		}

		$ret = $this->doLineItemUpdate($lineItem, $product, $quantity, $attributesData);

		if ($isNew)
		{
			$this->items[] = $lineItem->toArray();
			$this->lineItems[$lineItem->getId()] = $lineItem;
		}

		$this->resetItemProductPricing();

		$event = new Events_OrderEvent(Events_OrderEvent::ON_ADD_ITEM);
		$event->setOrder($this);
		Events_EventHandler::handle($event);

		$this->checkShippingDigital();
		$this->recalcSubtotals();

		return $ret;
	}

	protected function resetItemProductPricing()
	{
		$lineItemIds = array_keys($this->lineItems);
		$products = $this->getOrderRepository()->getOrderItemsProducts($this->getId(), $lineItemIds);

		/* @var Model_LineItem $item */
		foreach ($this->lineItems as $item)
		{
			/* @var Model_Product $product */
			$product = isset($products[$item->getPid()]) ? $products[$item->getPid()] : null;

			if ($product)
			{
				$attributes = $item->parseOptions();

				$attributesData = $product->parseAttributes($attributes, false);

				$product->calculateProductPricing($this->getUserLevel(), $attributesData, $this->_settings);
				$item->setProduct($product, $this->_settings);
			}
		}
	}

	protected function canEdit()
	{
		return $this->getStatus() == ORDER::STATUS_ABANDON || $this->getStatus() == ORDER::STATUS_ABANDON || ($this->getStatus() == ORDER::STATUS_PROCESS && $this->getPaymentStatus() == ORDER::PAYMENT_STATUS_PENDING);
	}
	/**
	 * Bulk updating of items in the cart
	 *
	 * @param array $quantities
	 *
	 * @return bool
	 */
	public function updateItems($quantities)
	{
		if (!$this->canEdit())
		{
			// TODO: Ensure that order can be modified
		}

		Tax_ProviderFactory::getTaxProvider()->initTax($this);

		$lineItemIds = array_map('intval', array_keys($quantities));

		if (count($lineItemIds) == 0) return false;

		$orderRepository = $this->getOrderRepository();

		$products = $orderRepository->getOrderItemsProducts($this->getId(), $lineItemIds);

		if (is_array($this->lineItems) && count($this->lineItems) > 0)
		{
			$lineItemIdsForDeletion = array();

			/* @var $item Model_LineItem */
			foreach ($this->lineItems as $item)
			{
				$needsDeleted = false;

				/* @var $product Model_Product */
				$product = isset($products[$item->getPid()]) ? $products[$item->getPid()] : null;

				if (!$product)
				{
					$needsDeleted = true;
				}
				else
				{
					if ($product->getProductId() == 'gift_certificate' || $item->getIsGift() == 'Yes') continue;

					$attributes = $item->parseOptions();

					$attributesData = $product->parseAttributes($attributes, false);

					$this->doLineItemUpdate($item, $product, $quantities[$item->getId()], $attributesData);
				}

				if ($needsDeleted)
				{
					// item is not available
					$lineItemIdsForDeletion[] = $item->getId();
				}
			}

			foreach ($lineItemIdsForDeletion as $lineItemId)
			{
				unset($this->lineItems[$lineItemId]);
			}

			$this->items = array();
			/* @var Model_LineItem $lineItem */
			foreach ($this->lineItems as $lineItem)
			{
				$this->items[] = $lineItem->toArray();
			}

			$orderRepository->deleteLineItems($this->getId(), $lineItemIdsForDeletion);
		}

		if (count($this->errors) == 0)
		{
			$event = new Events_OrderEvent(Events_OrderEvent::ON_UPDATE_ITEMS);
			$event->setOrder($this);
			Events_EventHandler::handle($event);
		}

		return count($this->errors) > 0 ? false : true;
	}

	/**
	 * @param $product
	 * @param $quantity
	 * @return bool
	 */
	protected function lineItemCheckMinMaxOrderQuantity(Model_Product $product, $quantity)
	{
		// check min / max order
		if ($product->getMinOrder() > 0 && $quantity < $product->getMinOrder())
		{
			// TODO: Move these to cart group
			$this->setError($this->msg['javascript']['number_of_items_exceeded_min']);
			return false;
		}
		else if ($product->getMaxOrder() > 0 && $quantity > $product->getMaxOrder())
		{
			$this->setError($this->msg['javascript']['number_of_items_exceeded_max']);
			return false;
		}

		return true;
	}

	/**
	 * @param $lineItem
	 * @param $product
	 * @param $quantity
	 * @param $attributesData
	 * @return bool
	 */
	protected function lineItemCheckInventory(Model_LineItem $lineItem, Model_Product $product, $quantity, &$attributesData)
	{
		$stock = INF;

		// TODO: Make this into a select statement (couple of select statements) that do the inventory check in one call,
		// having the checks in php lead to a "race" condition where it might have been in stock, but by the time it
		// renders it will have gone out of stock
		// check current inventory
		switch ($product->getInventoryControl())
		{
			case "Yes" :  // on product level
			{
				$stock = $product->getStock();
				break;
			}

			case "AttrRuleInc" : // on attribute level
			case "AttrRuleExc" :
			{
				$inventoryId = intval($lineItem->getInventoryId());
				if ($inventoryId > 0)
				{
					if ($attributeLevelStock = $this->getProductsRepository()->getInventoryStock($inventoryId))
					{
						$stock = $attributeLevelStock["stock"];
					}
				}
				break;
			}
		}

		if ($quantity > $stock && $this->_settings['AllowOutOfInventoryOrders'] != 'Yes')
		{
			$this->setError(sprintf($this->msg['cart']['error_stock_limited'], $lineItem->getTitle()));
			return false;
		}

		return true;
	}

	protected function doLineItemUpdate(Model_LineItem $lineItem, Model_Product $product, $quantity, &$attributesData)
	{
		if (isset($attributesData['weight']) && $attributesData['weight'] > 0)
		{
			$lineItem->setWeight($attributesData['weight']);
		}

		$product->calculateProductPricing($this->getUserLevel(), $attributesData, $this->_settings);

		$ret = $this->lineItemCheckMinMaxOrderQuantity($product, $quantity);
		if (!$ret)
		{
			return false;
		}

		$ret = $this->lineItemCheckInventory($lineItem, $product, $quantity, $attributesData);
		if (!$ret)
		{
			return false;
		}

		$lineItem->setFinalQuantity($quantity);

		$lineItem->setFinalPrice($product->getFinalPrice());
		$lineItem->setPriceBeforeQuantityDiscount($product->getFinalPrice());
		$lineItem->setPriceWithTax($product->getPriceWithTax());

		$lineItem->setTaxDescription($product->getTaxDescription());
		$lineItem->setTaxRate($product->getTaxRate());
		$lineItem->setIsTaxable($product->getIsTaxable());

		$orderRepository = $this->getOrderRepository();
		$orderRepository->persistLineItem($this, $lineItem, $lineItem->getId() > 0 ? false : true);

		return true;
	}

	/**
	 * Remove item from a cart
	 *
	 * @param int $lineItemId
	 *
	 * @return boolean
	 */
	public function removeItem($lineItemId)
	{
		$foundAt = -1;
		if (is_array($this->items))
		{
			foreach ($this->items as $pos => $item)
			{
				if ($item['ocid'] == $lineItemId)
				{
					$foundAt = $pos;
					break;
				}
			}
		}

		if ($foundAt >= 0)
		{
			unset($this->items[$foundAt]);
			unset($this->lineItems[$lineItemId]);

			$lineItemIds = array(intval($lineItemId));
			$this->getOrderRepository()->deleteLineItems($this->getId(), $lineItemIds);

			$this->resetItemProductPricing();

			$event = new Events_OrderEvent(Events_OrderEvent::ON_REMOVE_ITEM);
			$event->setOrder($this);

			Events_EventHandler::handle($event);

			return true;
		}
		else
		{
			$this->setError("Incorrect line item id");
			return false;
		}
	}

	/**
	 * Remove all items from a cart
	 */
	public function clearItems()
	{
		$this->getOrderRepository()->deleteAllLineItems($this->getId());

		$this->lineItems = array();
		$this->items = array();
	}

	/**
	 * Checkout attempts
	 *
	 * @param int $limit
	 *
	 * @return boolean
	 */
	public function attempt($limit)
	{
		return $this->getOrderRepository()->incrementPaymentAttempt($this->getId(), $limit);
	}

	/**
	 * Sets order number
	 *
	 * @return mixed
	 */
	public function setOrderNum()
	{
		$this->order_num = $this->getOrderRepository()->getNextOrderNumber($this->getId(), $this->_settings["MinOrderNumber"]);

		return $this->order_num != 0;
	}

	/**
	 * Closes order on user side
	 *
	 * @param string $order_status
	 * @param mixed $result payment processing result
	 * @param array $form
	 * @param mixed $customFields
	 * @param null $is_mobile
	 * @param null $source
	 * @param null $osc_campaign_id
	 * @param bool $send_notification
	 *
	 * @return int
	 */
	public function closeOrder($order_status, $result, $form, $customFields, $is_mobile = null, $source = null, $osc_campaign_id = null, $send_notification = true)
	{
		global $_SESSION;
		unset($_SESSION["payment_session_delay_timeout"]);
		unset($_SESSION["order_promo_code"]);
		unset($_SESSION["order_gift_certificate"]);
		unset($_SESSION["facebook_new_account"]);

		/**
		 * Set order number (if unset yet)
		 */
		$this->setOrderNum();

		$this->getShippingAddress();

		/**
		 * Set payment status based on payment gateways result
		 */
		$payment_status = is_bool($result) ? ($result ? 'Received' : 'Pending') : (in_array($result, array("Pending", "Received")) ? $result : 'Received');

		if ($this->totalAmount == 0) $payment_status = "Received";

		/**
		 * GIFT CERTIFICATE HANDLING
		 */
		require_once(dirname(dirname(__FILE__))."/classes/class.gift_certificates.php");

		$giftCertificates = new GiftCertificates($this->_db);

		// check is gift certificate used to place (fund) an order
		$giftCertificateAmountUsed = 0;
		if ($this->totalAmount > 0)
		{
			$giftCertificateOrderData = $giftCertificates->GetOrderGiftCert($this->getId());
			
			if ($giftCertificateOrderData['gift_cert_valid'] == '1')
			{
				$giftCertificateData = $giftCertificates->GetOrderedGiftCert(
					$giftCertificateOrderData['gift_cert_first_name'],
					$giftCertificateOrderData['gift_cert_last_name'],
					$giftCertificateOrderData['gift_cert_voucher']
				);

				if ($giftCertificateData !== false)
				{
					$giftCertificateAmountUsed = $this->totalAmount > $giftCertificateData['balance'] ? $giftCertificateData['balance'] : $this->totalAmount;
					$giftCertificates->UpdateBalance($this->getId(), $giftCertificateData['ocid'], $giftCertificateAmountUsed);

					//when everything was paid by Gift Certificate, update payment method
					if ($giftCertificateAmountUsed >= $this->getTotalAmount())
					{
						$this->db()->reset();
						$this->db()->assignStr("payment_method_name", "Gift Certificate");
						$this->db()->assignStr("payment_method_description", "Gift Certificate");
						$this->db()->assignStr("payment_gateway_id", "");
						$this->db()->update(DB_PREFIX."orders", "WHERE oid=".intval($this->getId()));
						view()->assign("payment_method_name", "Gift Certificate");
						$payment_status = "Received";
					}
				}
			}
		}

		view()->assign("gift_cert_amount", $giftCertificateAmountUsed);

		/**
		 * Set the payment_method_description on the order
		 */
		if (array_key_exists('cc_type', $form) && isset($form['cc_type']) && trim($form['cc_type']) != '')
		{
			$this->paymentMethodDescription = $form['cc_type'];
		}
		else
		{
			$this->paymentMethodDescription = $this->paymentMethodName;
		}

		/**
		 * Store the buyers IP address upon completing order
		 */
		$ipAddress = (isset($_SERVER['REMOTE_ADDR'])) ? $_SERVER['REMOTE_ADDR'] : 'unknown';
		$this->db()->reset();
		$this->db()->assignStr('ipaddress', $ipAddress);
		$this->db()->assignStr("payment_method_description", $this->paymentMethodDescription);
		$this->db()->update(DB_PREFIX.'orders', 'WHERE oid='.$this->getId());

		/**
		 * Set order dates / status
		 */
		$this->db()->reset();
		$this->db()->assignStr("status", $order_status);
		$this->db()->assignStr("payment_status", $payment_status);
		$this->db()->assign("placed_date", "NOW()");
		$this->db()->assign("status_date", "NOW()");
		$this->db()->assignStr("removed", "No");

		if (!is_null($is_mobile)) $this->db()->assignStr("is_mobile", $is_mobile ? "1" : "0");
		if (!is_null($source) && $source) $this->db()->assignStr("order_source", $source);
		if (!is_null($osc_campaign_id) && $osc_campaign_id) $this->db()->assignStr("offsite_campaign_id", $osc_campaign_id);

		$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());
		$_SESSION["LastOrderID"] = $this->getId();

		/**
		 * DOBA
		 */
		if ($this->_settings["doba_autosend_orders"] == "Yes")
		{
			$doba_items = array();
			foreach ($this->lineItems as $item)
			{
				if ($item->getIsDoba() == "Yes") $doba_items[] = array("item_id"=>$item->getProductId(), "quantity"=>$item->getFinalQuantity());
			}
			if (count($doba_items) > 0)
			{
				require_once(dirname(dirname(__FILE__))."/classes/class.doba_order.php");
				$dobaOrder = new DobaOrderAPI($this->_settings, $this->_db);

				$doba_order_response = $dobaOrder->createOrder($this->shippingAddress, $doba_items, $this->order_num);

				if ($doba_order_response)
				{
					$this->db()->query("UPDATE ".DB_PREFIX."orders SET doba_order_sent=1, doba_order_id='".$this->db()->escape($doba_order_response['response']['order_id'])."' WHERE oid=".$this->getId());
				}
			}
		}

		/**
		 * Send notifications
		 */
		if ($send_notification)
		{
			emailOrderReceived($this->_db, $this, $this->_user, $customFields, $payment_status);
		}

		/**
		 * Handle on status change event
		 */
		$event = new Events_OrderEvent(Events_OrderEvent::ON_STATUS_CHANGE);

		$event
			->setOrder($this)
			->setData('oldOrderStatus', ORDER::STATUS_ABANDON)
			->setData('oldPaymentStatus', ORDER::PAYMENT_STATUS_PENDING)
			->setData('newOrderStatus', $order_status)
			->setData('newPaymentStatus', $payment_status);

		Events_EventHandler::handle($event);

		return $this->order_num;
	}

	/**
	 * Checks is shipping is digital
	 *
	 * @return boolean
	 */
	public function checkShippingDigital()
	{
		if (!is_array($this->lineItems))
		{
			$this->getOrderItems();
		}

		$hasNonDigitalShipping = false;
		foreach ($this->lineItems as $item)
		{
			/* @var $item Model_LineItem */
			if ($item->getDigitalProduct() == 'No' && $item->getProductId() != 'gift_certificate')
			{
				$hasNonDigitalShipping = true;
				break;
			}
		}

		$shippingDigital = $hasNonDigitalShipping ? 0 : 1;

		$this->shippingDigital = $hasNonDigitalShipping ? 0 : 1;

		if ($shippingDigital)
		{
			$this->setShippingCarrierID('custom');
			$this->setShippingMethodId('digital');
			$this->setShippingCMName('Digital Download');
			$this->setShippingTaxable(0);
			$this->setShippingTaxRate(0);
			$this->setShippingTaxAmount(0);
			$this->setShippingTaxDescription('');
		}

		return $this->shippingDigital;
	}

	/**
	 * Update shipping methods
	 *
	 * @param $method_id
	 *
	 * @return int
	 */
	public function updateShippingMethod($method_id)
	{
		//check free shipping
		if ($this->_settings["ShippingCalcEnabled"] == "YES")
		{
			/**
			 * Check for alternative shipping method
			 */
			if ($method_id != "0" && $method_id[0] == "a")
			{
				//use alternative method and ignore free shipping products
				$shippingMethodData = $this->db()->selectOne("SELECT * FROM ".DB_PREFIX."shipping_selected WHERE ssid='".$this->db()->escape(substr($method_id, 1))."'", MYSQL_ASSOC);
				if ($shippingMethodData)
				{
					$this->setShippingIgnoreFree(true);
					$this->setShippingSSID($shippingMethodData["ssid"]);
					$this->setShippingCarrierId($shippingMethodData["carrier_id"]);
					$this->setShippingMethodId($shippingMethodData["method_id"]);
					$this->setShippingCMName($shippingMethodData["carrier_name"].($shippingMethodData["carrier_id"]!="custom"?(" - ".$shippingMethodData["method_name"]):""));
				}
				else
				{
					$this->setShippingIgnoreFree(false);
					$this->setShippingSSID("0");
					$this->setShippingCarrierId("custom");
					$this->setShippingMethodId("disabled");
					$this->setShippingCMName("Shipping is disabled");
				}
			}
			/**
			 * Check standard shipping method
			 */
			else if(!$this->isShippingFree())
			{
				//check does method exists
				if ($method_id == 0 && $this->_settings["ShippingWithoutMethod"] == "YES")
				{
					$this->setShippingIgnoreFree(false);
					$this->setShippingSSID("0");
					$this->setShippingCarrierId("custom");
					$this->setShippingMethodId("future_charge");
					$this->setShippingCMName("Undefined");
				}
				else
				{
					$shippingMethodData = $this->db()->selectOne("SELECT * FROM ".DB_PREFIX."shipping_selected WHERE ssid=".intval($method_id), MYSQL_ASSOC);
					if ($shippingMethodData)
					{
						$this->setShippingIgnoreFree(false);
						$this->setShippingSSID($shippingMethodData["ssid"]);
						$this->setShippingCarrierId($shippingMethodData["carrier_id"]);
						$this->setShippingMethodId($shippingMethodData["method_id"]);
						$this->setShippingCMName($shippingMethodData["carrier_name"].($shippingMethodData["carrier_id"]!="custom"?(" - ".$shippingMethodData["method_name"]):""));
					}
					else
					{
						$this->setShippingIgnoreFree(false);
						$this->setShippingSSID("0");
						$this->setShippingCarrierId("custom");
						$this->setShippingMethodId("disabled");
						$this->setShippingCMName("Shipping is disabled");
					}
				}
			}
			/**
			 * Shipping is free
			 */
			else
			{
				$this->setShippingIgnoreFree(false);
				$this->setShippingSSID("0");
				$this->setShippingCarrierId("custom");
				$this->setShippingMethodId("free");
				$this->setShippingCMName("Shipping is free");
			}
		}
		else
		{
			$this->setShippingIgnoreFree(false);
			$this->setShippingSSID("0");
			$this->setShippingCarrierId("custom");
			$this->setShippingMethodId("global_disabled");
			$this->setShippingCMName("Shipping is disabled");
		}

		/**
		 * Update shipping settings 
		 */
		$this->db()->reset();
		$this->db()->assignStr("shipping_amount", "0.00");
		$this->db()->assignStr("shipping_ssid", $this->getShippingSSID());
		$this->db()->assignStr("shipping_carrier_id", $this->getShippingCarrierId());
		$this->db()->assignStr("shipping_method_id", $this->getShippingMethodId());
		$this->db()->assignStr("shipping_cm_name", $this->getShippingCMName());
		$this->db()->assignStr("shipping_ignore_free", $this->getShippingIgnoreFree() ? "1" : "0");
		$this->db()->assignStr("shipping_tax_amount", 0.00);
		$this->db()->assignStr("shipping_tax_rate", 0.00);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());

		return $this->shippingSSID;
	}

	/**
	 * Update shipping address information
	 * @param $address
	 */
	public function updateShippingAddress($address, $customFieldsData = false)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE stid='".intval($address["state"])."' AND coid='".intval($address["country"])."'");
		if ($this->db()->moveNext())
		{
			$address["province"] = $this->db()->col["name"];
		}
		else
		{
			$address["state"] = 0;
		}

		$this->updateShippingMethod(0);
		$this->db()->reset();
		if (isset($address['address_type'])) $this->db()->assignStr("shipping_address_type", $address["address_type"]);
		$this->db()->assignStr("shipping_name", $address["name"]);
		$this->db()->assignStr("shipping_company", $address["company"]);
		$this->db()->assignStr("shipping_address1", $address["address1"]);
		$this->db()->assignStr("shipping_address2", $address["address2"]);
		$this->db()->assignStr("shipping_city", $address["city"]);
		$this->db()->assignStr("shipping_state", intval($address["state"]));
		$this->db()->assignStr("shipping_province", $address["province"]);
		$this->db()->assignStr("shipping_country", intval($address["country"]));
		$this->db()->assignStr("shipping_zip", $address["zip"]);
		$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());

		$this->getShippingAddressFromDb();

		if ($this->_settings["TaxAddress"] == "Shipping")
		{
			$this->reupdateItems();
		}

		if (is_array($customFieldsData))
		{
			$this->customFields()->storeCustomFields("shipping", $customFieldsData, $this->user()->getId(), $this->getId(), 0);
		}
	}

	/**
	 * Update payment method
	 *
	 * @param $payment_method_id
	 *
	 * @return boolean
	 */
	public function updatePaymentMethod($payment_method_id)
	{
		if ($payment_method_id == 0 || $payment_method_id == "free")
		{
			$this->db()->reset();
			$this->db()->assignStr("payment_is_realtime", "No");
			$this->db()->assignStr("payment_method_id", 0);
			$this->db()->assignStr("payment_method_name", "Free");
			$this->db()->assignStr("payment_method_description", "Free");
			$this->db()->assignStr("payment_gateway_id", "Free");
			$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());
			$this->paymentMethodId = 0;
			return true;
		}
		else
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE pid='".intval($payment_method_id)."' AND active='Yes'");
			if ($this->db()->moveNext())
			{
				$this->db()->reset();
				$this->db()->assignStr("payment_is_realtime", $this->db()->col["type"]!="custom"?"Yes":"No");
				$this->db()->assignStr("payment_method_id", $payment_method_id);
				$this->db()->assignStr("payment_method_name", $this->db()->col["name"]);
				$this->db()->assignStr("payment_gateway_id", $this->db()->col["id"]);
				$this->db()->update(DB_PREFIX."orders", "WHERE oid=".$this->getId());
				$this->paymentMethodId = $payment_method_id;
				return true;
			}
			else
			{
				return false;
			}
		}
	}

	/**
	 * Update order gift message
	 *
	 * @param $gift_message
	 */
	public function storeGiftMessage($gift_message)
	{
		$this->giftMessage = $gift_message;
		$this->db()->query("UPDATE ".DB_PREFIX."orders SET gift_message = '".$this->db()->escape($gift_message)."' WHERE oid=".$this->getId());
	}

	/**
	 * Run items update again to reset ordered items info
	 */
	public function reupdateItems()
	{
		$this->db()->query("SELECT `payment_status` FROM `".DB_PREFIX."orders` WHERE `oid`=".$this->getId());

		if ($this->db()->moveNext())
		{
			if ($this->db()->col['payment_status'] != 'Received')
			{
				$this->db()->query("SELECT ocid, admin_quantity as quantity FROM ".DB_PREFIX."orders_content WHERE oid=".$this->getId());
				$q = array();
				while ($this->db()->moveNext())
				{
					$q[$this->db()->col["ocid"]] = $this->db()->col["quantity"];
				}
				$this->updateItems($q);
			}
		}
	}

	/**
	 * Calculate items count
	 */
	public function calculateItemsCount()
	{
		$itemsCount = 0;
		if (is_array($this->lineItems))
		{
			foreach ($this->lineItems as $item)
			{
				/* @var $item Model_LineItem */
				$itemsCount += $item->getAdminQuantity();
			}
		}

		$this->setItemsCount($itemsCount);
	}
	
	/**
	 * Check is shipping is free
	 * 
	 * @return boolean
	 */
	public function isShippingFree()
	{
		if (is_array($this->lineItems))
		{
			foreach ($this->lineItems as $item)
			{
				/* @var $item Model_LineItem */
				if ($item->getFreeShipping() == 'No')
				{
					return false;
				}
			}
		}

		return true;
	}
	
	/**
	 * Get free shipping products count
	 *
	 * @return int
	 */
	public function freeShippingProductsCount()
	{
		$freeShippingCount = 0;
		if (is_array($this->lineItems))
		{
			foreach ($this->lineItems as $item)
			{
				/* @var $item Model_LineItem */
				if ($item->getFreeShipping() == 'Yes')
				{
					$freeShippingCount++;
				}
			}
		}

		return $freeShippingCount;
	}

	/**
	 * Calculate subtotal amount
	 * 
	 * @return float
	 */
	public function calculateSubtotal($persist = true, $excludeGiftCertificates = false)
	{
		return Calculator_Subtotal::getInstance()->calculate($this, $persist, $excludeGiftCertificates);
	}

	/**
	 * Calculate promo discount amount
	 * 
	 * @param boolean $reset
	 * 
	 * @return float
	 */
	protected function calculatePromoDiscount($reset = true)
	{
		return Calculator_PromoDiscount::getInstance($this->_settings)->calculate($this, $reset);
	}

	/**
	 * Calculate discount amount
	 * 
	 * @param boolean $reset
	 * 
	 * @return float
	 */
	protected function calculateDiscount($reset = true)
	{
		return Calculator_Discount::getInstance($this->_settings)->calculate($this, $reset);
	}

	/**
	 * Calculate shipping amount
	 *
	 * @param $error_message
	 *
	 * @return float
	 */
	public function calculateShipping(&$error_message)
	{
		$shippingMethod = Shipping_MethodFactory::getMethod($this->getShippingSSID());
		
		return Calculator_Shipping::getInstance($this->_settings)->calculate(
			$this->db(), $this->_settings, $this, $shippingMethod,
			isset($_SESSION["product_level_method"]) ? $_SESSION["product_level_method"] : null,
			$error_message
		);
	}

	/**
	 * Calculate shipping tax
	 * 
	 * @return float
	 */
	public function calculateShippingTax()
	{
		return Calculator_ShippingTax::getInstance($this->_settings)->calculate($this);
	}

	/**
	 * Calculate handling amount
	 * 
	 * @param boolean $shippingIsFree
	 * @param boolean $addToShipping
	 *
	 * @return float
	 */
	public function calculateHandling($shippingIsFree = false, $addToShippingWhenNeeded = true)
	{
		return Calculator_Handling::getInstance($this->_settings)->calculate($this, $shippingIsFree, $addToShippingWhenNeeded);
	}

	/**
	 * Calculate handling tax
	 * 
	 * @return float
	 */
	public function calculateHandlingTax()
	{
		return Calculator_HandlingTax::getInstance($this->_settings)->calculate($this);
	}

	/**
	 * Calculate taxes
	 * 
	 * @return float
	 */
	public function calculateTax()
	{
		if ($this->getShippingDigital() || ($this->getShippingComplete() && $this->getShippingMethodId() != 'disabled'))
		{
			return Calculator_Tax::getInstance()->calculate($this);
		}

		return 0;
	}

	/**
	 * Calculate total amount
	 * 
	 * @return float
	 */
	public function calculateTotal()
	{
		return Calculator_Total::getInstance()->calculate($this);
	}
	
	/**
	 * Get Shipping Address
	 *
	 * @return mixed
	 */
	public function getShippingAddressFromDb()
	{
		$addressData = $this->getOrderRepository()->getShippingAddress($this->getId());

		$this->setShippingAddress($addressData);

		return $this->getShippingAddress();
	}

	/**
	 * Get Order Items
	 *
	 * @return mixed
	 */
	public function getOrderItems()
	{
		if ($this->getId())
		{
			// Ensure that getProductUrl is defined / loaded
			if (!function_exists('getProductUrl'))
			{
				require_once dirname(__FILE__).'/engine_url_'.($this->_settings['USE_MOD_REWRITE'] == 'YES' ? 'rewrite' : 'default').'.php';
			}

			$a = $this->getOrderRepository()->getLineItems($this->getId());

			if (count($a) > 0)
			{
				$subtotal = 0;
				$subtotalWithTax = 0;

				for ($i=0; $i < count($a); $i++)
				{
					$lineItem = new Model_LineItem($a[$i]);

					$this->lineItems[$a[$i]['ocid']] = $lineItem;

					$a[$i] = $lineItem->toArray();

					$subtotal += $lineItem->getFinalPrice() * $lineItem->getFinalQuantity();
					$subtotalWithTax += $lineItem->getPriceWithTax() * $lineItem->getFinalQuantity();
				}

				$this->items = $a;

				$this->setSubtotalAmountWithTax($subtotalWithTax);
				$this->setSubtotalAmount($subtotal);

				$this->calculateItemsCount();
				$this->setShippingDigital($this->checkShippingDigital());
				$this->setShippingIsFree($this->isShippingFree());
				$this->setShippingFreeProductsCount($this->freeShippingProductsCount());

				return $a;
			}
			else
			{
				$this->calculateItemsCount();
				$this->setShippingIsFree($this->isShippingFree());
				$this->setShippingFreeProductsCount($this->freeShippingProductsCount());
			}
		}

		return false;
	}

	/**
	 * Get order items with additional info
	 *
	 * @return mixed
	 */
	public function getOrderItemsExtended()
	{
		if (!class_exists("ShoppingCartProducts"))
		{
			require_once(dirname(dirname(__FILE__))."/classes/class.products.php");
		}
		$pr = new ShoppingCartProducts($this->_db, $this->_settings);

		if (!class_exists('file_utils'))
		{
			require_once(dirname(dirname(__FILE__)).'/classes/class.file_utils.php');
		}
		$file_utils = file_utils::getInstance();

		return self::_getOrderItemsExtended($this->items, $this->_settings['CatalogThumbSize'], $this->_settings['CatalogThumbType'], $pr, $file_utils);
	}

	/**
	 * Helper function for get order items extended
	 *
	 * @param $orderItems
	 * @param $catalogThumbSize
	 * @param $catalogThumbType
	 * @param $pr
	 * @param $file_utils
	 *
	 * @return mixed
	 */
	public static function _getOrderItemsExtended(&$orderItems, $catalogThumbSize, $catalogThumbType, $pr, $file_utils)
	{
		if ($orderItems)
		{
			foreach ($orderItems as $key => $orderItem)
			{
				$orderItem['product_price'] = $orderItem['price_withtax'];

				$product_id = $orderItem['product_id'];

				if ($orderItem['product_sub_id'] != '')
				{
					$orderItem['product_id'] = $orderItem['product_id'].' / '.$orderItem['product_sub_id'];
				}
				
				$orderItem['image'] = $pr->getProductImage($product_id, $orderItem['image_location'], $orderItem['image_url']);

				if ($orderItem['image'])
				{
					$orderItem['thumb'] = $thumb_path = $pr->getProductThumb($product_id, $orderItem['image_location'], $orderItem['image_url']);
					$orderItem['thumb'] = $orderItem['thumb']?$orderItem['thumb']:$orderItem['image'];
					
					if ($orderItem['image_location'] == 'Web' || $catalogThumbType == 'Square')
					{
						$orderItem['thumb_size'] = ' width="'.$catalogThumbSize.'" height="'.$catalogThumbSize.'" ';
					}
					else
					{
						$imageSize = file_utils::getInstance()->getImageSize($thumb_path);

						if ($imageSize)
						{
							if ($imageSize[0] > $imageSize[1])
							{
								$orderItem['thumb_size'] = ' width="'.$catalogThumbSize.'" ';
							}
							else
							{
								$orderItem['thumb_size'] = ' height="'.$catalogThumbSize.'" ';
							}
						}
						else
						{
							$orderItem['thumb_size'] = ' height="'.$catalogThumbSize.'" ';
						}
					}
				}
				else
				{
					$orderItem['thumb'] = false;
				}

				$orderItems[$key] = $orderItem;
			}
		}

		return $orderItems;
	}

	/**
	 * Reset order values
	 *
	 * @param bool $resetShippingMethod
	 */
	protected function resetValues($resetShippingMethod = false)
	{
		$this->resetSubtotal();
		$this->resetDiscount();
		$this->resetPromo(); // resetting only promo amount
		if ($resetShippingMethod) $this->resetShippingMethod(); // should be called only before we suppose to set shipping method - on subtotals ;
		$this->resetShipping();
		$this->resetHandling();
		$this->resetShippingTax();
		$this->resetHandlingTax();
		$this->resetTax();
		$this->resetTotal();
	}

	/**
	 * Reset subtotal'
	 */
	protected function resetSubtotal()
	{
		Calculator_Subtotal::getInstance()->reset($this);
	}

	/**
	 * Reset discounts
	 */
	protected function resetDiscount()
	{
		Calculator_Discount::getInstance($this->_settings)->reset($this);
	}

	/**
	 * Reset handling
	 */
	protected function resetHandling()
	{
		Calculator_Handling::getInstance($this->_settings)->reset($this);
	}

	/**
	 * Reset handling tax
	 */
	protected function resetHandlingTax()
	{
		Calculator_HandlingTax::getInstance($this->_settings)->reset($this);
	}

	/**
	 * Reset shipping method
	 */
	protected function resetShippingMethod()
	{
		$this->setShippingSSID('');
		$this->setShippingCarrierId('');
		$this->setShippingMethodId('');
		$this->setShippingCMName('');
		$this->setShippingIgnoreFree(false);
	}

	/**
	 * Reset shipping
	 */
	protected function resetShipping()
	{
        Calculator_Shipping::getInstance($this->_settings)->reset($this);
	}

	/**
	 * Reset shipping taxes
	 */
	protected function resetShippingTax()
	{
		Calculator_ShippingTax::getInstance($this->_settings)->reset($this);
	}

	/**
	 * Reset taxes
	 */
	protected function resetTax()
	{
		Calculator_Tax::getInstance()->reset($this);
	}

	/**
	 * Reset totals
	 */
	protected function resetTotal()
	{
		Calculator_Total::getInstance()->reset($this);
	}

	/**
	 * Get order data
	 *
	 * @return mixed
	 */
	public function getOrderData()
	{
		if ($this->getId() > 0)
		{
			$this->hydrate();
		}
	}

	/**
	 * Recalculates order subtotal amount and discounts only
	 * 
	 * @param boolean $resetDiscounts
	 */
	public function recalcSubtotals($resetDiscounts = true)
	{
		/**
		 * Do full reset
		 */
		$this->resetValues(true); // reset everything

		$this->calculateItemsCount();

		/**
		 * Recalculate amount
		 */
		$this->calculateSubtotal(); // calculate subtotal
		$this->calculateDiscount($resetDiscounts); // discount
		$this->calculatePromoDiscount($resetDiscounts); // promo discounts - at this moment we don't care about shipping
		$this->calculateTotal(); // total at this moment will contain running total only without taxes and shipping

		$this->persist();
	}

	/**
	 * Recalculate totals
	 * Suppose to be called when shipping information available
	 *
	 * @param $shipping_error_message
	 * @param bool $resetDiscounts
	 */
	public function recalcTotals(&$shipping_error_message, $resetDiscounts = true)
	{
		/**
		 * Do partial reset first 
		 */
		$this->resetShipping(); // note - we do not reset shipping method here
		$this->resetHandling();
		$this->resetPromo(); // note - resetting it because it may be changed by shipping
		$this->resetShippingTax();
		$this->resetHandlingTax();
		$this->resetTax();
		$this->resetTotal();

		$this->calculateItemsCount();

		/**
		 * Recalculate amount
		 */
		$shipping_error_message = "";
		$this->calculateShipping($shipping_error_message); // calculate shipping
		$this->calculateHandling($this->isShippingFree(), true); // handling
		$this->calculateShippingTax();
		$this->calculatePromoDiscount($resetDiscounts); // promo discounts - at this moment we don't care about shipping
		$this->calculateHandlingTax();
		$this->calculateTax();
		$this->calculateTotal();

		$this->forceRecalc = false;

		$this->persist();
	}

	/**
	 * Recalculate order data
	 *
	 * @param string $shippingErrorMessage
	 * @param bool $resetDiscounts
	 */
	public function recalcEverything(&$shippingErrorMessage = '', $resetDiscounts = true)
	{
		$shippingErrorMessage = '';

		$this->resetValues();

		$this->calculateItemsCount();

		$this->calculateSubtotal();
		$this->calculateDiscount($resetDiscounts);
		$this->calculateShipping($shippingErrorMessage);
		$this->calculateHandling($this->isShippingFree(), true);
		$this->calculatePromoDiscount($resetDiscounts); // PromoDiscounts after shipping because of Shipping promo discounts
		$this->calculateShippingTax();
		$this->calculateHandlingTax();
		$this->calculateTax();
		$this->calculateTotal();

		$this->forceRecalc = false;

		$this->persist();
	}

	
	/**
	 * Save order data into database
	 */
	public function persist()
	{
		$this->getOrderRepository()->persistOrderData($this);

		return $this;
	}

	/**
	 * Hydrate order object from database
	 */
	public function hydrate()
	{
		$data = $this->getOrderRepository()->getOrderData($this->getId());

		if ($data)
		{
			$this->forceRecalc = $data['force_recalc'] == 1;

			//ORDER NUMBER
			$this->setOrderNumber($data["order_num"]);
			$this->setStatus($data['status']);
			$this->setPaymentStatus($data['payment_status']);

			$this->status_date = $data['status_date'];
			$this->status_date_formatted = $data['status_date_formatted'];

			//SUBTOTAL
			$this->setSubtotalAmount($data["subtotal_amount"]);
			
			//DISCOUNT
			$this->setDiscountAmount($data["discount_amount"]);
			$this->setDiscountValue($data["discount_value"]);
			$this->setDiscountType($data["discount_type"]);
			
			//PROMO DISCOUNT
			$this->setPromoDiscountAmount($data["promo_discount_amount"]);
			$this->setPromoDiscountValue($data["promo_discount_value"]);
			$this->setPromoDiscountType($data["promo_discount_type"]);
			$this->setPromoType($data["promo_type"]);
			$this->setPromoCampaignId($data['promo_campaign_id']);
			
			//HANDLING
			$this->setHandlingSeparated($data["handling_separated"]);
			$this->setHandlingFee($data["handling_fee"]);
			$this->setHandlingFeeType($data["handling_fee_type"]);
			$this->setHandlingText($data["handling_text"]);
			$this->setHandlingAmount($data["handling_amount"]);

			$this->setHandlingTaxable($data["handling_taxable"] == "1");
			$this->setHandlingTaxRate($data["handling_tax_rate"]);
			$this->setHandlingTaxAmount($data["handling_tax_amount"]);
			$this->setHandlingTaxDescription($data["handling_tax_description"]);
			
			//SHIPPING

			$this->getShippingAddressFromDb();

			$this->setShippingDigital($data["shipping_digital"]);
			$this->setShippingSSID($data["shipping_ssid"]);
			$this->setShippingCarrierId($data["shipping_carrier_id"]);
			$this->setShippingMethodId($data["shipping_method_id"]);
			$this->setShippingCMName($data["shipping_cm_name"]);
			$this->setShippingAmount($data["shipping_amount"]);
			$this->setShippingTaxable($data["shipping_taxable"]);
			$this->setShippingTaxRate($data["shipping_tax_rate"]);
			$this->setShippingTaxAmount($data["shipping_tax_amount"]);
			$this->setShippingTaxDescription($data["shipping_tax_description"]);
			$this->setShippingIgnoreFree($data["shipping_ignore_free"] == "1");

			$this->setShippingTrackingNumber($data['shipping_tracking_number']);
			$this->setShippingTrackingNumberType($data['shipping_tracking_number_type']);
			$this->setShippingTracking(trim($data['shipping_tracking']) != '' ? unserialize($data['shipping_tracking']) : array());

			//TAXES
			$this->setTaxAmount($data["tax_amount"]);
			$this->setTaxExempt($data["tax_exempt"]);
			
			//PAYMENT
			$this->paymentIsRealtime = $data["payment_is_realtime"];
			$this->paymentMethodId = $data["payment_method_id"];
			$this->paymentMethodName = $data["payment_method_name"];
			$this->paymentGatewayId = $data["payment_gateway_id"];
			$this->paymentMethodDescription = $data["payment_method_description"];
			
			//OTHER STUFF
			$this->setGiftMessage($data["gift_message"]);
			$this->setIsMobile(intval($data['is_mobile']));
			$this->setOffsiteCampaignId(intval($data['offsite_campaign_id']));
			$this->setOrderSource($data['order_source']);

			$this->setTotalAmount($data["total_amount"]);

		}
	}
}
