<?php

class GoogleOrderProvider
{
	protected $db;
	protected $settings;
	protected $orderRepository;

	public function __construct(&$db, &$settings, $orderRepository)
	{
		$this->db = $db;
		$this->settings = $settings;
		$this->orderRepository = $orderRepository;
	}

	public function handleOrderStatusChange($fulfillment_state, $financial_state, $google_order_number, $timestamp)
	{
		//------Update order status according to financial-order-state and fulfillment-order-state 
		$this->db->query("UPDATE ".DB_PREFIX."orders SET status_date=NOW(), custom3='".$fulfillment_state."-".$financial_state."' WHERE custom1='".$google_order_number."'");

		//------Cancel order
		if ($financial_state == "CANCELLED" or $financial_state == "CANCELLED_BY_GOOGLE")
		{
			$order = $this->orderRepository->getOrderByGoogleOrderNumber($google_order_number);
			$old_order_status = $order->getStatus();
			$old_payment_status = $order->getPaymentStatus();

			//Update order payment status
			$this->db->query("UPDATE ".DB_PREFIX."orders SET status='Canceled', payment_status='Canceled', 'status_date' = NOW() WHERE custom1='".$google_order_number."'");				
			$response = "Order Cancelled ".$timestamp;
			$this->orderRepository->createTransaction($google_order_number, $response, 0);

			$order->setStatus('Canceled');
			$order->setPaymentStatus('Canceled');

			Events_OrderEvent::dispatchStatusChangeEvent($order, $old_order_status, $old_payment_status);
		}

		if ($financial_state == 'CHARGEABLE' && $this->settings["googlecheckout_Auth_Type"] != "Sale")
		{
			$order = $this->orderRepository->getOrderByGoogleOrderNumber($google_order_number);
			$old_payment_status = $order->getPaymentStatus();

			Events_OrderEvent::dispatchStatusChangeEvent($order, 'Abandon', $old_payment_status);
		}
	}

	public function handleChargeAmountNotification($google_order_number, $total_charge_amount, $currency, $timestamp)
	{
		$data = "Total charged amount ('".$currency."'): ".$total_charge_amount."\nTimestamp: ".$timestamp;
		// removed "\n Google staus for the order: ".$google_order_status." as order status is not set
		$this->orderRepository->createTransaction($google_order_number, $data, $total_charge_amount);

		// TODO: If is digital, need to set order to completed
		// TODO: If order is completed, would need to send email notifications?

		$order = $this->orderRepository->getOrderByGoogleOrderNumber($google_order_number);
		$old_order_status = $order->getStatus();
		$old_payment_status = $order->getPaymentStatus();

		//Update order payment status
		$this->db->reset();
		$this->db->assignStr("payment_status", "Received");
		$this->db->assign("status_date", "NOW()");
		$this->db->update(DB_PREFIX."orders", "WHERE custom1='".$google_order_number."'");

		$order->setPaymentStatus('Received');

		Events_OrderEvent::dispatchStatusChangeEvent($order, $old_order_status, $old_payment_status);
	}

	public function handleRiskInformationNotification($google_order_number, $risk_info)
	{
		$risk_data = "risk-information-notification \n ";

		foreach ($risk_info as $k => $v)
		{
			if(is_array($v[0])) foreach($v[0] as $k1 => $v1){ $risk_data .= $k1 ."=>".$v1[0]."\n";}
			$risk_data .= $k ."=>".$v[0]."\n";
		}
		$this->orderRepository->createTransaction($google_order_number, $risk_data, 0);
	}

	public function handleRefundAmountNotification($google_order_number, $latest_refund, $total_refund, $currency, $timestamp)
	{
		$data = "Latest Refund amout ".$latest_refund ."\n Total Refund amount ('".$currency."'): ".$total_refund ."\n Notification timestamp: ".$timestamp;

		$this->orderRepository->createTransaction($google_order_number, $data, $latest_refund );
		
		$this->db->query("SELECT total_amount FROM ".DB_PREFIX."orders WHERE custom1='".$google_order_number."'");

		$total_amount = 0;
		if($this->db->moveNext()){ $total_amount =  $this->db->col["total_amount"]; }

		if($total_amount == $total_refund)
		{
			$order = $this->orderRepository->getOrderByGoogleOrderNumber($google_order_number);
			$old_order_status = $order->getStatus();
			$old_payment_status = $order->getPaymentStatus();

			//Update order payment status
			$this->db->reset();
			$this->db->assignStr("payment_status", "Refunded");
			$this->db->assign("status_date", "NOW()");
			$this->db->update(DB_PREFIX."orders", "WHERE custom1='".$google_order_number."'");

			$order->setPaymentStatus('Refunded');

			Events_OrderEvent::dispatchStatusChangeEvent($order, $old_order_status, $old_payment_status);
		}
	}
}