<?php

$payment_processor_id = 'firstdata';
$payment_processor_name = 'FirstData';
$payment_processor_class = 'PAYMENT_FIRSTDATA';
$payment_processor_type = 'cc';

class PAYMENT_FIRSTDATA extends PAYMENT_PROCESSOR
{
	function PAYMENT_FIRSTDATA()
	{
		parent::PAYMENT_PROCESSOR();
		$this->id = 'firstdata';
		$this->name = 'First Data Global Gateway e4';
		$this->class_name = 'PAYMENT_FIRSTDATA';
		$this->type = 'cc';
		$this->description = '';
		$this->steps = 1;
		$this->testMode = false;
		$this->support_ccs = true;
		$this->need_cc_codes = true;
		return $this;
	}

	function getPaymentForm($db)
	{
		$fields = parent::getPaymentForm($db);
		if ($this->isCardinalCommerceEnabled())
		{
			$fields['do_cmpi_lookup'] = array('type' => 'hidden', 'name'=> 'do_cmpi_lookup', 'value' => 1);
		}

		return $fields;
	}

	public function replaceDataKeys()
	{
		return array('password', 'cc_number', 'cc_verification_str2');
	}

	function isTestMode()
	{
		global $settings;

		if (strtolower($settings['firstdata_Test_Request']) == 'true')
		{
			$this->testMode = true;
		}
		else
		{
			$this->testMode = false;
		}
		return $this->testMode;
	}

	protected function getGatewayUrl()
	{
		if ($this->isTestMode())
		{
			return 'https://api.demo.globalgatewaye4.firstdata.com/transaction/v12';
		}

		return 'https://api.globalgatewaye4.firstdata.com/transaction/v12';
	}

	public function process($db, $user, $order, $post_form)
	{
		global $settings;

		$authorizeResult = false;

		if ($this->isCardinalCommerceEnabled())
		{
			$authorizeResult = $this->processCardinalCommerce($db, $user, $order, $post_form);
		}
		else
		{
			$authorizeResult = $this->processPaymentForm($db, $user, $order, $post_form);
		}

		return $this->is_error ? false : $authorizeResult;
	}

	protected function isCardinalCommerceEnabled()
	{
		global $settings;

		return $settings['cardinalcommerce_Enabled'] == 'Yes' && $settings['firstdata_Enable_Cardinal_Centinel'] == 'Yes';
	}

	protected function processCardinalCommerce($db, $user, $order, &$post_form)
	{
		global $settings;

		$authorizeResult = false;

		if (isset($post_form['payment_method_way']) && $post_form['payment_method_way'] == 'profile')
		{
			return $this->processPaymentForm($db, $user, $order, $post_form);
		}

		//include module
		require_once(dirname(dirname(__FILE__)).'/engine_cardinal_commerce.php');
		//create object and set it up
		$cardinal = new CardinalCommerce();
		$cardinal->common_vars = $this->common_vars;

		if (isset($_REQUEST['do_cmpi_lookup']))
		{
			$cardinal_lookup_xml = $cardinal->getLookupXML(
				$post_form['cc_first_name'],
				$post_form['cc_last_name'],
				$post_form['cc_number'],
				$post_form['cc_expiration_month'],
				$post_form['cc_expiration_year'],
				$this->currency['code']
			);

			$cardinal_lookup_result = $cardinal->processRequest($cardinal_lookup_xml);

			if ($cardinal_lookup_result)
			{
				if ($cardinal_lookup_result['CardinalMPI'][0]['Enrolled'][0] == 'Y')
				{
					$this->redirect = true;
					$this->redirectURL = $settings['GlobalHttpsUrl'].'/'.$settings['INDEX'].'?pcsid='.session_id().'&p=payment_validation';

					$_SESSION['CardinalPostForm'] = $post_form;
					$_SESSION['CardinalTransactionId'] = $cardinal_lookup_result['CardinalMPI'][0]['TransactionId'][0];
					$_SESSION['CardinalACSUrl'] = $cardinal_lookup_result['CardinalMPI'][0]['ACSUrl'][0];
					$_SESSION['CardinalPayload'] = $cardinal_lookup_result['CardinalMPI'][0]['Payload'][0];
					//validator will replace ORDER_PROCESS_PAYMENT_VALIDATION with ORDER_PROCESS_PAYMENT
					$_SESSION['CardinalNotifyUrl'] = $settings['GlobalHttpsUrl'].'/'.$settings['INDEX'].'?pcsid='.session_id().'&p='.$this->payment_page.'&oa='.ORDER_PROCESS_PAYMENT_VALIDATION.'&do_cmpi_authenticate=true';
					$_SESSION['CardinalRedirectUrl'] = $settings['GlobalHttpsUrl'].'/'.$settings['INDEX'];

					return false;
				}
				else
				{
					//do not redirect, process simple authorize transaction
					$authorizeResult = $this->processPaymentForm($db, $user, $order, $post_form);
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = $cardinal->error_message;
			}
		}

		if (isset($_REQUEST['do_cmpi_authenticate']) && isset($_SESSION['CardinalPostForm']) && isset($_REQUEST['PaRes']))
		{
			$post_form = $_SESSION['CardinalPostForm'];

			unset($_SESSION['CardinalPostForm']);

			$cardinal_auth_xml = $cardinal->getAuthenticateXML($_REQUEST['PaRes']);
			$cardinal_auth_result = $cardinal->processRequest($cardinal_auth_xml);

			if ($cardinal_auth_result)
			{
				$EciFlag = $cardinal_auth_result['CardinalMPI'][0]['EciFlag'][0];
				$PAResStatus = $cardinal_auth_result['CardinalMPI'][0]['PAResStatus'][0];
				$Cavv = $cardinal_auth_result['CardinalMPI'][0]['Cavv'][0];
				$SignatureVerification = $cardinal_auth_result['CardinalMPI'][0]['SignatureVerification'][0];
				$Xid = $cardinal_auth_result['CardinalMPI'][0]['Xid'][0];

				if ($SignatureVerification == 'Y' && $PAResStatus != 'N')
				{
					switch ($PAResStatus)
					{
						case 'Y' :
						case 'A' : $authorizeResult = $this->processPaymentForm($db, $user, $order, $post_form, $EciFlag, $Cavv, $Xid); break;
						case 'U' : $authorizeResult = $this->processPaymentForm($db, $user, $order, $post_form); break;
					}
				}
				else
				{
					$this->is_error = true;
					$this->error_message = 'Your card can not be validated. Please try other card or other payment method';
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = $cardinal->error_message;
			}
		}

		return $authorizeResult;
	}

	function processPaymentForm($db, $user, $order, $post_form, $EciFlag = false, $Cavv = false, $Xid = false)
	{
		$profile_id = isset($post_form['ext_profile_id']) ? $post_form['ext_profile_id'] : false;
		if (isset($post_form['payment_method_way']) && $post_form['payment_method_way'] != 'profile') $profile_id = false;

		$post_data = array(
			'gateway_id' => $this->settings_vars[$this->id.'_GatewayId']['value'],
			'password' => $this->settings_vars[$this->id.'_Password']['value'],
			'transaction_type' => $this->settings_vars[$this->id.'_Auth_Type']['value'] == 'Auth-Capture' ? '00' : '01',
			'amount' => number_format($this->common_vars['order_total_amount']['value'], 2, '.', ''),
			'cc_verification_str1' => $this->common_vars['billing_address1']['value'].'|'.$this->common_vars['billing_zip']['value'].'|'.
					$this->common_vars['billing_city']['value'].'|'.$this->common_vars['billing_state_abbr']['value'].'|'.$this->common_vars['billing_country_iso_a3']['value'],
			'cvd_presence_ind' => 1,
			'zip_code' => $this->common_vars['billing_zip']['value'],
			'tax1_amount' => number_format($this->common_vars['order_tax_amount']['value'], 2, '.', ''),
			'customer_ref' => $this->common_vars['order_id']['value'],
			'client_ip' => $_SERVER['REMOTE_ADDR'],
			'client_email' => $this->common_vars['billing_email']['value'],
			'currency_code' => $this->settings_vars[$this->id.'_Currency_Code']['value'],
			'ecommerce_flag' => 7,
		);

		if ($profile_id)
		{
			$details = $this->getPaymentProfileDetails($db, $profile_id);
			$post_data['cardholder_name'] = $details['first_name'].' '.$details['last_name'];
			$post_data['transarmor_token'] = $profile_id;
			$post_data['cc_expiry'] = $details['cc_expiration_month'].substr($details['cc_expiration_year'], 2);
			$post_data['credit_card_type'] = $details['credit_card_type'];
		}
		else
		{
			$post_data['cardholder_name'] = $post_form['cc_first_name'].' '.$post_form['cc_last_name'];
			$post_data['cc_number'] = $post_form['cc_number'];
			$post_data['cc_expiry'] = $post_form['cc_expiration_month'].substr($post_form['cc_expiration_year'], 2);
			$post_data['cc_verification_str2'] = $post_form['cc_cvv2'];
		}

		if ($EciFlag !== false)
		{
			$post_data['cavv'] = $Cavv;
			$post_data['xid'] = $Xid;
			$post_data['ecommerce_flag'] = intval($EciFlag);
		}

		$this->log('Request: ', $post_data);

		$buffer = $this->process_curl($post_data);

		if ($buffer)
		{
			$response = @json_decode($buffer);

			if ($response)
			{
				$this->log('Response: ', (array)$response);

				if ($response->transaction_approved)
				{
					$extra = $response->transaction_tag;
					$custom1 = $this->settings_vars[$this->id.'_Auth_Type']['value'] == 'Auth-Capture' ? 'AUTH_CAPTURE' : 'AUTH_ONLY';
					//If Auth only, then store auth'd amount to custom2
					$custom2 = $this->settings_vars[$this->id.'_Auth_Type']['value'] != 'Auth-Capture' ? number_format($this->common_vars['order_total_amount']['value'], 2, '.', '') : '';
					//If Auth Capture, then store captured amount to custom3
					$custom3 = $this->settings_vars[$this->id.'_Auth_Type']['value'] == 'Auth-Capture' ? number_format($this->common_vars['order_total_amount']['value'], 2, '.', '') : 0;

					$security_id = $response->authorization_num;

					$this->createTransaction($db, $user, $order, (array)$response, $extra, $custom1, $custom2, $custom3, $security_id);

					$this->is_error = false;

					$this->error_message = '';

					return $this->settings_vars[$this->id.'_Auth_Type']['value'] == 'Auth-Capture' ? 'Received' : 'Pending';
				}
				else
				{
					$this->storeTransaction($db, $user, $order, (array)$response);

					if (in_array($response->exact_resp_code, array('08', '22', '25', '26', '27', '28', '31', '32', '57', '58', '60', '63', '64', '68', '72', '93')))
					{
						$this->is_error = true;
						$this->error_message = 'We are sorry, but there has been an error processing this transaction.<br>Payment gateway response: '.$response->exact_message;
					}
					else
					{
						$this->is_error = true;
						$this->error_message = 'We are sorry, but there has been an error processing this transaction.<br>Payment gateway response: '.$response->bank_message;
					}
				}
			}
			else
			{
				$this->storeTransaction($db, $user, $order, $response);

				$this->is_error = true;
				$this->error_message = 'We are sorry, but there has been an error processing this transaction.<br>Payment gateway response: '.$response;
			}
		}
		else
		{
			$this->is_error = true;
			$this->error_message = 'We are sorry, but there has been an error processing this transaction.';
		}

		return !$this->is_error;
	}

	function process_curl($post_data)
	{
		global $settings;

		$post_url = $this->getGatewayUrl();

		$c = curl_init($post_url);

		if ($settings['ProxyAvailable'] == 'YES')
		{
			//curl_setopt($c, CURLOPT_VERBOSE, 1);
			if (defined('CURLOPT_PROXYTYPE') && defined('CURLPROXY_HTTP') && defined('CURLPROXY_SOCKS5'))
			{
				curl_setopt($c, CURLOPT_PROXYTYPE, $settings['ProxyType'] == 'HTTP' ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
			}
			curl_setopt($c, CURLOPT_PROXY, $settings['ProxyAddress'].':'.$settings['ProxyPort']);
			if ($settings['ProxyRequiresAuthorization'] == 'YES')
			{
				curl_setopt($c, CURLOPT_PROXYUSERPWD, $settings['ProxyUsername'].':'.$settings['ProxyPassword']);
			}

			curl_setopt($c, CURLOPT_TIMEOUT, 120);
		}


		$keyId = $settings['firstdata_KeyId'];
		$hmacKey = $settings['firstdata_HmacKey'];

		$content = json_encode($post_data);

		$contentType = 'application/json; charset=UTF-8';
		$contentDigest = sha1($content);
		$gge4Date = gmdate('c');
		$uri = parse_url($post_url,  PHP_URL_PATH);

		$hmacContent = "POST\n".$contentType."\n".$contentDigest."\n".$gge4Date."\n".$uri;

		$hmacHash = base64_encode(hash_hmac('sha1', $hmacContent, $hmacKey, true));

		$headers = array(
			'Content-Type: '.$contentType,
			'Accept: application/json',
			'X-GGe4-Content-SHA1: '.$contentDigest,
			'X-GGe4-Date: '.$gge4Date,
			'Authorization: GGE4_API '.$keyId.':'.$hmacHash
		);

		curl_setopt($c, CURLOPT_VERBOSE, 0);
		curl_setopt($c, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($c, CURLOPT_POSTFIELDS, $content);
		curl_setopt($c, CURLOPT_POST, 1);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 1);
		curl_setopt($c, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
		curl_setopt($c, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);

		if (!safe_mode()) set_time_limit(3000);

		$buffer = curl_exec($c);

		if (curl_errno($c) > 0)
		{
			$this->is_error = true;
			$this->error_message = curl_error($c);
			return false;
		}

		return !$buffer ? false : $buffer;
	}

	function supportsCapture($order_data = false)
	{
		if ($order_data)
		{
			if ($order_data['payment_status'] == 'Pending' || $order_data['payment_status'] == 'Partial')
			{
				if ($this->isTotalAmountCaptured($order_data))
					return false;

				return true;
			}
		}
		return false;
	}

	function supportsPartialCapture($order_data = false)
	{
		return false;
	}

	function isTotalAmountCaptured($order_data)
	{
		return $order_data['custom3'] == $order_data['total_amount'];
	}

	function capturableAmount($order_data=false)
	{
		if ($order_data)
		{
			if ($this->supportsCapture($order_data))
			{
				$capturableAmount = $order_data['total_amount'] - $order_data['custom3'];
				if ($capturableAmount < 0)
					$capturableAmount = 0.0;

				return number_format($capturableAmount, 2, '.', '');
			}
		}
		return 0;
	}

	function getTotalCaptured($order_data)
	{
		return 0.00 + $order_data['custom3'];
	}

	function getLastTransactionId($oid)
	{
		global $db;

		$transactionId = $db->selectOne('SELECT extra FROM '.DB_PREFIX.'payment_transactions WHERE oid='.$oid.' AND is_success = 1 ORDER BY tid DESC LIMIT 1');

		if ($transactionId)
		{
			$transactionId = array_shift($transactionId);
		}

		return $transactionId;
	}

	function capture($order_data=false, $capture_amount=false)
	{
		global $db, $order;

		$oldPaymentStatus = $order->getPaymentStatus();
		$user = $order->getUser();

		$this->is_error = false;

		if ($order_data)
		{
			//check is transaction completed
			if ($this->isTotalAmountCaptured($order_data))
			{
				$this->is_error = true;
				$this->error_message = 'Transaction already completed';
				return false;
			}

			//get captured total amount & x_trans_id
			$captured_total_amount = $capture_amount ? $capture_amount : $order_data['custom2'];
			$transaction_id = $this->getLastTransactionId($order->getId());
			$authorization_num = $order_data['security_id'];

			if ($transaction_id == '')
			{
				$this->is_error = true;
				$this->error_message = 'This order does not have a valid transaction id';
				return false;
			}
			else
			{
				if ($captured_total_amount <= 0)
				{
					$this->is_error = true;
					$this->error_message = 'Amount to capture must be greater than 0';
					return false;
				}
				else if ($captured_total_amount > $order_data['total_amount'])
				{
					$this->is_error = true;
					$this->error_message = 'Amount to capture cannot be more than total amount';
					return false;
				}
				else if ($order_data['custom3'] + $captured_total_amount > $order_data['total_amount'])
				{
					$this->is_error = true;
					$this->error_message = 'Amount to capture cannot be more than total amount';
					return false;
				}

				$post_data = array(
					'gateway_id' => $this->settings_vars[$this->id.'_GatewayId']['value'],
					'password' => $this->settings_vars[$this->id.'_Password']['value'],
					'transaction_type' => '32',
					'amount' => number_format($captured_total_amount, 2, '.', ''),
					'transaction_tag' => $transaction_id,
					'authorization_num' => $authorization_num,
				);

				$this->log('Request: ', $post_data);

				$buffer = $this->process_curl($post_data);

				if ($buffer)
				{
					$response = @json_decode($buffer);

					if ($response)
					{
						$this->log('capture Response:', (array)$response);

						if ($response->transaction_approved)
						{
							$totalCaptured = $this->getTotalCaptured($order_data) + $captured_total_amount;
							$custom3 = number_format($totalCaptured, 2, '.', '');

							$db->reset();
							$db->assignStr('custom1', '');
							$db->assignStr('custom2', '');
							$db->assignStr('custom3', $custom3);

							$order->setPaymentStatus(ORDER::PAYMENT_STATUS_RECEIVED);

							if ($order_data['total_amount'] > $totalCaptured)
							{
								$order->setPaymentStatus(ORDER::PAYMENT_STATUS_PARTIAL);
								$extra = $transaction_id;
								$security_id = $authorization_num;
							}
							else
							{
								$extra = $response->transaction_tag;
								$security_id = $response->authorization_num;
							}

							$db->assignStr('payment_status', $order->getPaymentStatus());

							$db->update(DB_PREFIX.'orders', "WHERE oid='".$order_data['oid']."'");

							$order_data['custom1'] = 'PRIOR_AUTH_CAPTURE';
							$order_data['custom2'] = '';
							$order_data['custom3'] = $custom3;
							$order_data['payment_status'] = $order->getPaymentStatus();

							$custom1 = $order_data['custom1'];
							$custom2 = $order_data['custom2'];

							$this->createTransaction($db, $user, $order,
								(array)$response,
								$extra,
								$custom1,
								$custom2,
								$custom3,
								$security_id
							);

							$this->fireOrderEvent($order, $order->getStatus(), $oldPaymentStatus);

							return true;
						}
						else
						{
							$this->is_error = true;
							$this->error_message = $response->exact_message;

							$this->storeTransaction($db, $userTmp, $orderTmp, $response, '', false);

							return false;
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_message = $buffer;

						$this->log('capture Response:'.$buffer);

						$this->storeTransaction($db, $user, $order, $buffer, '', false);

						return false;
					}
				}
				else
				{
					$this->is_error = true;

					return false;
				}
			}
		}
		return !$this->is_error;
	}

	function supportsVoid($order_data = false)
	{
		return $order_data && ($order_data['payment_status'] == 'Received' || $order_data['payment_status'] == 'Pending');
	}

	function void($order_data=false)
	{
		/** @var DB $db */
		/** @var ORDER $order */
		global $db, $order;

		$oldPaymentStatus = $order->getPaymentStatus();
		$oldOrderStatus = $order->getStatus();

		$user = $order->getUser();

		$this->is_error = false;

		if ($order_data)
		{
			$total_amount = $order_data['total_amount'];
			$transaction_id = $this->getLastTransactionId($order->getId());
			$authorization_num = $order_data['security_id'];

			$post_data = array(
				'gateway_id' => $this->settings_vars[$this->id.'_GatewayId']['value'],
				'password' => $this->settings_vars[$this->id.'_Password']['value'],
				'transaction_type' => '33',
				'amount' => number_format($total_amount, 2, '.', ''),
				'transaction_tag' => $transaction_id,
				'authorization_num' => $authorization_num,
			);

			$this->log('Request: ', $post_data);

			$buffer = $this->process_curl($post_data);

			if ($buffer)
			{
				$response = @json_decode($buffer);

				if ($response)
				{
					$this->log('void Response:', (array)$response);

					if ($response->transaction_approved)
					{
						$extra = $response->transaction_tag;
						$security_id = $response->authorization_num;
						$custom1 = '';
						$custom2 = '';
						$custom3 = '';

						$db->reset();
						$db->assignStr('custom1', $custom1);
						$db->assignStr('custom2', $custom2);
						$db->assignStr('custom3', $custom3);

						$order->setPaymentStatus(ORDER::PAYMENT_STATUS_CANCELED);
						$order->setStatus(ORDER::STATUS_CANCELED);

						$db->assignStr('payment_status', $order->getPaymentStatus());
						$db->assignStr('status', $order->getStatus());
						$db->update(DB_PREFIX.'orders', "WHERE oid='".$order_data['oid']."'");

						$custom1 = 'CANCELED';
						$order_data['custom1'] = $custom1;
						$order_data['custom2'] = $custom2;
						$order_data['custom3'] = $custom3;
						$order_data['payment_status'] = $order->getPaymentStatus();
						$order_data['status'] = $order->getStatus();

						$this->createTransaction($db, $user, $order,
							(array)$response,
							$extra,
							$custom1,
							$custom2,
							$custom3,
							$security_id
						);

						$this->fireOrderEvent($order, $oldOrderStatus, $oldPaymentStatus);

						return true;
					}
					else
					{
						$this->is_error = true;
						$this->error_message = $response->exact_message;

						$this->storeTransaction($db, $user, $order, (array)$response, '', false);

						return false;
					}
				}
				else
				{
					$this->is_error = true;
					$this->error_message = $buffer;

					$this->log('void Response:'.$buffer);

					$this->storeTransaction($db, $user, $order, $buffer, '', false);

					return false;
				}
			}
			else
			{
				$this->is_error = true;

				return false;
			}
		}

		return !$this->is_error;
	}

	function supportsRefund($order_data = false)
	{
		if ($order_data)
		{
			if ($order_data['payment_status'] == 'Received' || $order_data['payment_status'] == 'Partial')
			{
				return true;
			}
			else
				if ($order_data['payment_status'] == 'Refunded')
				{
					return $this->_refundableAmount($order_data) > 0.0;
				}
		}
		return false;
	}

	function refundableAmount($order_data=false)
	{
		if ($order_data)
		{
			if ($this->supportsRefund($order_data))
			{
				return $this->_refundableAmount($order_data);
			}
		}
		return 0;
	}

	function _refundableAmount($order_data)
	{
		$refundableAmount = $this->getTotalCaptured($order_data);
		return number_format($refundableAmount, 2, '.', '');
	}

	function refund($order_data=false,$refund_amount=false)
	{
		global $db, $order;

		$user = $order->getUser();

		$oldPaymentStatus = $order->getPaymentStatus();

		$this->is_error = false;

		if ($order_data)
		{
			if (!$this->supportsRefund($order_data))
			{
				$this->is_error = true;
				$this->error_message = 'Order cannot be refunded';
				return false;
			}

			//get captured total amount & x_trans_id
			$refund_total_amount = $refund_amount;

			if (!$refund_total_amount)
			{
				$refund_total_amount = $order_data['custom3'];
			}

			if ($refund_total_amount < 0)
			{
				$this->is_error = true;
				$this->error_message = 'Amount to refund must be greater than 0';
				return false;
			}
			else if ($refund_total_amount > $order_data['custom3'] || $refund_total_amount > $order_data['total_amount'])
			{
				$this->is_error = true;
				$this->error_message = 'Amount to refund cannot be more than total captured amount';
				return false;
			}

			$transaction_id = $this->getLastTransactionId($order->getId());
			$authorization_num = $order_data['security_id'];

			if ($transaction_id == '')
			{
				$this->is_error = true;
				$this->error_message = 'This order does not have a valid transaction id';
				return false;
			}
			else
			{
				$post_data = array(
					'gateway_id' => $this->settings_vars[$this->id.'_GatewayId']['value'],
					'password' => $this->settings_vars[$this->id.'_Password']['value'],
					'transaction_type' => '34',
					'amount' => number_format($refund_total_amount, 2, '.', ''),
					'transaction_tag' => $transaction_id,
					'authorization_num' => $authorization_num,
				);

				$this->log('Refund Request: ', $post_data);

				$buffer = $this->process_curl($post_data);

				if ($buffer)
				{
					$response = @json_decode($buffer);

					if ($response)
					{
						$this->log('Refund Response:', (array)$response);

						$user = $order->getUser();

						if ($response->transaction_approved)
						{
							$totalCaptured = $this->getTotalCaptured($order_data) - $refund_total_amount;
							$custom3 = number_format($totalCaptured, 2, '.', '');

							$db->reset();
							$db->assignStr('custom1', '');
							$db->assignStr('custom2', '');
							$db->assignStr('custom3', $custom3);

							if ($order_data['total_amount'] > $totalCaptured)
							{
								$extra = $transaction_id;
								$security_id = $authorization_num;
							}
							else
							{
								$extra = $response->transaction_tag;
								$security_id = $response->authorization_num;
							}

							$order->setPaymentStatus(ORDER::PAYMENT_STATUS_REFUNDED);

							$db->assignStr('payment_status', $order->getPaymentStatus());
							$db->update(DB_PREFIX.'orders', "WHERE oid='".$order_data['oid']."'");

							$order_data['custom1'] = 'REFUND';
							$order_data['custom2'] = '';
							$order_data['custom3'] = $custom3;
							$order_data['payment_status'] = $order->getPaymentStatus();

							$custom1 = $order_data['custom1'];
							$custom2 = $order_data['custom2'];

							$this->createTransaction($db, $user, $order,
								(array)$response,
								$extra,
								$custom1,
								$custom2,
								$custom3,
								$security_id
							);

							$this->fireOrderEvent($order, $order->getStatus(), $oldPaymentStatus);

							return true;
						}
						else
						{
							$this->is_error = true;
							$this->error_message = $response->exact_message;

							$this->storeTransaction($db, $user, $order, (array)$response, '', false);

							return false;
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_message = $buffer;

						$this->log('refund Response:'.$buffer);

						$this->storeTransaction($db, $user, $order, $buffer, '', false);

						return false;
					}
				}
				else
				{
					$this->is_error = true;

					return false;
				}
			}
		}

		return !$this->is_error;
	}

	public function getTransactionDetails(ORDER $order)
	{
		// ctr response value must be mapped to
		global $db;

		$row = $db->selectOne('SELECT *
FROM '.DB_PREFIX.'payment_transactions
WHERE oid = '.$order->oid.' AND is_success = 1
ORDER BY completed DESC
LIMIT 1');
		$ret = false;

		if ($row)
		{
			$ret = array();

			$response_values = explode("\n", $row['payment_response']);
			foreach ($response_values as  $response_value)
			{
				$kvPair = explode(' = ', $response_value);

				if ($kvPair[0] == 'authorization_num')
				{
					$ret['authorization_code'] = urldecode($kvPair[1]);
				}
				else if ($kvPair[0] == 'credit_card_type')
				{
					$ret['card_type'] = urldecode($kvPair[1]);
				}
				else if ($kvPair[0] == 'cc_number')
				{
					// TODO: This value is fully masked on the response
					//$ret['card_number'] = urldecode($kvPair[1]);
				}
				else if ($kvPair[0] == 'ctr')
				{
					$ret['notes'] = urldecode($kvPair[1]);
				}
			}
		}

		return $ret;
	}

	public function supportsPaymentProfiles()
	{
		global $settings;

		return $settings['firstdata_Payment_Profiles'] == 'Yes';
	}

	protected function generateToken($post_form)
	{
		global $settings;

		$post_data = array(
			'gateway_id' => $settings['firstdata_GatewayId'],
			'password' => $settings['firstdata_Password'],
			'transaction_type' => '05',
			'amount' => 0.00,
			'cc_number' => $post_form['cc_number'],
			'cc_expiry' => $post_form['cc_expiration_month'].substr($post_form['cc_expiration_year'], 2),
			'cardholder_name' => $post_form['first_name'].' '.$post_form['last_name'],
			'zip_code' => $post_form['zip'],
			'client_ip' => $_SERVER['REMOTE_ADDR'],
		);

		$this->log('Token Request: ', $post_data);

		$buffer = $this->process_curl($post_data);

		if ($buffer)
		{
			$response = @json_decode($buffer);

			if ($response)
			{
				$this->log('Token Response:', (array)$response);

				if ($response->transaction_approved)
				{
					if (isset($response->transarmor_token) && trim($response->transarmor_token) != '')
					{
						return $response;
					}
					else
					{
						$this->is_error = true;
						$this->error_message = 'There was an error storing your credit card.';
						$this->log('transarmor token no returned. Merchant account is possibly not configured with transarmor.');
					}
				}
			}
			else
			{
				$this->log('token Response: '.$response);

				$this->is_error = true;
				$this->error_message = $response;
			}
		}
		else
		{
			$this->is_error = true;
			$this->error_message = 'There was an error storing your credit card.';
		}

		return false;
	}

	function getPaymentProfileDetails($db, $ext_profile_id, $active = false)
	{
		$db->selectOne("SELECT * FROM ".DB_PREFIX."users_payment_profiles WHERE ext_profile_id = '".$db->escape($ext_profile_id)."'");

		$data = unserialize($db->col['billing_data']);

		$country = getCountryData($db, false, $data['country']);
		$data['country_id'] = $country['coid'];

		$state = getStateData($db, false, $data['state']);
		$data['state_id'] = $state['stid'];

		$data['name'] = $data['first_name'].' '.$data['last_name'];

		return $data;
	}

	protected function handleModifyPaymentProfile(DB $db, USER $user, $post_form, $do_insert = true, $profile_id = null)
	{
		$tokenResponse = $this->generateToken($post_form);

		if(!$tokenResponse)
		{
			if ($do_insert)
			{
				$user->errors[] = 'There was an error storing your credit card.';
			}
			else
			{
				$user->errors[] = 'There was an error updating your credit card.';
			}
			$user->is_error = true;

			view()->assign('user_errors', $user->errors);

			return false;
		}

		$token = $tokenResponse->transarmor_token;
		$ccType = $tokenResponse->credit_card_type;

		$country = getCountryData($db, $post_form['country']);
		$state = getStateData($db, $post_form['state']);

		$cc_num = str_repeat('X', strlen($post_form['cc_number']) - 4);
		$cc_num .= substr($post_form['cc_number'], -4);

		$billing_data = array(
			'first_name' => $post_form['first_name'],
			'last_name' => $post_form['last_name'],
			'address1' => $post_form['address1'],
			'address2' => $post_form['address2'],
			'city' => $post_form['city'],
			'country' => $country['iso_a2'],
			'state' => $state['short_name'],
			'province' => $post_form['province'],
			'zip' => $post_form['zip'],
			'phone' => $post_form['phone'],
			'cc_number' => $cc_num,
			'cc_expiration_month' => $post_form['cc_expiration_month'],
			'cc_expiration_year' => $post_form['cc_expiration_year'],
			'cc_expiration_date' => $post_form['cc_expiration_month'].'/'.$post_form['cc_expiration_year'],
			'credit_card_type' => $ccType,
		);

		if(isset($post_form['is_primary']))
		{
			$db->query("UPDATE ".DB_PREFIX."users_payment_profiles SET is_primary = 'No' WHERE uid = ".$user->id);
		}

		if (!is_null($profile_id))
		{
			$profileData = $db->selectOne("SELECT * FROM ".DB_PREFIX."users_payment_profiles WHERE usid = ".intval($profile_id));
			if (!$profileData)
			{
				$do_insert = true;
			}
		}

		$db->reset();
		$db->assignStr('uid', $user->id);
		$db->assignStr('pid', $post_form['pid']);
		$db->assignStr('ext_profile_id', $token);
		$db->assignStr('is_primary',isset($post_form['is_primary'])?'Yes':'No');
		$db->assignStr('billing_data',serialize($billing_data));
		if ($do_insert)
		{
			$db->insert(DB_PREFIX.'users_payment_profiles');
		}
		else
		{
			$db->update(DB_PREFIX."users_payment_profiles","WHERE usid = ".intval($profile_id));
		}

		return true;
	}

	function addPaymentProfile($db, $user, $post_form)
	{
		return $this->handleModifyPaymentProfile($db, $user, $post_form, true);
	}

	public function updatePaymentProfile($db, $user, $post_form, $profile_id)
	{
		return $this->handleModifyPaymentProfile($db, $user, $post_form, false, $profile_id);
	}

	function deletePaymentProfile($db, $user, $ext_profile_id)
	{
		$db->query("DELETE FROM ".DB_PREFIX."users_payment_profiles WHERE ext_profile_id = '".$db->escape($ext_profile_id)."'");
	}
}