<?php 
	$payment_processor_id = "securetrading";
	$payment_processor_name = "Secure Trading Payment Pages";
	$payment_processor_class = "PAYMENT_SECURETRADING";
	$payment_processor_type = "cc";

	class PAYMENT_SECURETRADING extends PAYMENT_PROCESSOR
	{
		function PAYMENT_SECURETRADING()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "securetrading";
			$this->name = "Secure Trading Payment Pages";
			$this->class_name = "PAYMENT_SECURETRADING";
			$this->type = "ipn";
			$this->description = "";
			$this->testMode = false;
			$this->need_cc_codes = false;
			$this->support_ccs = false;
			$this->post_url = "https://securetrading.net/authorize/form.cgi";
			$this->steps = 2;
			$this->possible_payment_delay = true;
			$this->possible_payment_delay_timeout = 10; // seconds
			return $this;
		}

		function getPaymentForm($db)
		{
			global $settings, $order;

			$pre_fields = parent::getPaymentForm($db);
			$fields = array();

			$send_customer_email = ($this->settings_vars[$this->id."_customer_email"]["value"] == 'Yes') ? 1 : 0;

			// create signature for cart consists of order id, userid, and session_id
			$signature = md5($this->common_vars["order_id"]["value"].$this->settings_vars[$this->id."_password"]["value"]);

			$fields[] = array("type"=>"hidden", "name"=>"signature", "value"=> $signature);
			$fields[] = array("type"=>"hidden", "name"=>"pcsid", "value" => session_id());
			$fields[] = array("type"=>"hidden", "name"=>"preauth", "value" => 'true');
			$fields[] = array("type"=>"hidden", "name"=>"oa", "value"=>ORDER_PROCESS_PAYMENT);

			// gateway specific variables
			$fields[] = array("type"=>"hidden", "name"=>"merchant", "value"=> $this->settings_vars[$this->id."_merchant_site_reference"]["value"]);
			$fields[] = array("type"=>"hidden", "name"=>"merchantemail", "value"=> $this->settings_vars[$this->id."_merchant_email_address"]["value"]);
			$fields[] = array("type"=>"hidden", "name"=>"customeremail", "value"=> $send_customer_email);
			$fields[] = array("type"=>"hidden", "name"=>"amount", "value"=> $this->formatAmountString($this->common_vars['order_total_amount']['value']));
			$fields[] = array("type"=>"hidden", "name"=>"orderref", "value"=>$this->common_vars["order_id"]["value"]);
			$fields[] = array("type"=>"hidden", "name"=>"currency", "value"=>$_SESSION['current_currency']['code']);

			$fields[] = array("type"=>"hidden", "name"=>"callbackurl", "value"=> $this->settings_vars[$this->id."_callback_url_reference"]["value"]);
			$fields[] = array("type"=>"hidden", "name"=>"failureurl", "value"=> $this->settings_vars[$this->id."_failure_url_reference"]["value"]);
			$fields[] = array("type"=>"hidden", "name"=>"formref", "value"=> $this->settings_vars[$this->id."_form_reference"]["value"]);

			// // the absolute path to any graphics used 
			$fields[] = array("type"=>"hidden", "name"=>"path", "value"=> $this->settings_vars[$this->id."_path_to_images"]["value"]);
			// 
			// // customer personal information
			$fields[] = array("type"=>"hidden", "name"=>"company", "value"=> $this->common_vars['billing_company']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"name", "value"=> $this->common_vars['billing_first_name']['value'].' '.$this->common_vars['billing_last_name']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"email", "value"=> $this->common_vars['billing_email']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"address", "value"=> $this->common_vars['billing_address1']['value'].' '.$this->common_vars['billing_address2']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"town", "value"=> $this->common_vars['billing_city']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"county", "value"=> $this->common_vars['billing_state']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"country", "value"=> $this->common_vars['billing_country_iso_a3']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"url", "value"=> $settings["GlobalHttpsUrl"]);
			$fields[] = array("type"=>"hidden", "name"=>"postcode", "value"=> $this->common_vars['billing_zip']['value']);
			$fields[] = array("type"=>"hidden", "name"=>"telephone", "value"=> $this->common_vars['billing_phone']['value']);

			// misc values to pass (date, etc)
			$fields[] = array("type"=>"hidden", "name"=>"usadate", "value"=> date('m/d/Y'));

			// var_dump($this->payment_page, $fields); exit;

			return $fields;
		}

		function isTestMode()
		{
			static $testMode = null;

			if (!is_null($testMode)) return $testMode;
			$testMode = ($this->settings_vars[$this->id."_test_mode"]["value"] == "On");
			$this->testMode = $testMode;

			return $this->testMode;
		}

		function formatAmountString($amount)
		{
			return str_replace(array(',','.'), '', number_format($amount, 2, '.', ''));
		}

		function process($db, $user, $order, $post_form)
		{
			global $settings;

			if (isset($post_form) && is_array($post_form))
			{
				$this->log("Process:", $post_form);

				if ($post_form['stresult'] == '1')
				{
					// Transaction Valid
					$response = print_r($post_form, 1);

					$charged_amount = $this->common_vars["order_total_amount"]["value"];

					// log the transaction
					$this->createTransaction($db, $user, $order, $response, '', 'preauth', '', $charged_amount, $post_form['streference']);

					// transaction successful, return to not go past this point
					return 'Received';
				}

				$this->is_error = true;
				$this->error_message = PAYMENT_SERVER_ERROR_TEXT;
			}
			else
			{
				$this->is_error = true;
				$this->error_message = "Incorrect form params. Please contact site administrator.";
			}

			if ($this->is_error)
			{
				$this->storeTransaction($db, $user, $order, $post_form, '', false);
			}

			return !$this->is_error;
		}
	}
