<?php

define('DDM_TARGET_VERSION', '3.7.14');

include("../../content/engine/engine_config.php");
include("../../content/engine/engine_functions.php");
include("../../content/engine/engine_mysql.php");

if (!safe_mode()) set_time_limit(300);

// Connect to current database
$db = new DB;
$db->init(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);

include("../../content/engine/engine_settings.php");
include("../../content/classes/class.ddm_view.php");

$settings = settings();

echo '<p><b>Database Connect Results</b><br><li>Connection ID '.$db->link_id.' - connected - OK</li></p>';
if(!defined("DB_PREFIX")) define("DB_PREFIX", "");

// Check current vaersion
echo '<p><b>Check Current App Version</b><br/>';


// check the HTTPS url if applicable
if (array_key_exists('ssl_check_detected', $_POST))
{
	try
	{
		// check to see if they selected an option (required)
		if (!array_key_exists('ssl_check', $_POST)) throw new Exception('You must provide an answer to whether you have an SSL certificate.');
		
		switch ($_POST['ssl_check'])
		{
			// they do have a certificate, so we check to see if they have an HTTPS setting as their GlobalHttpsUrl, if it does not, throw an error
			case 'yes':
				if (!strstr(strtolower($_POST['https_url']), 'https://')) throw new Exception('Please provide a valid HTTPS url and try again.');
				
				$db->query("UPDATE ".DB_PREFIX."settings SET value = '".$db->escape($_POST['https_url'])."' WHERE name = 'GlobalHttpsUrl'");
				break;
			
			// they do not have a certificate, so we check to see if they have an HTTPS setting as their GlobalHttpsUrl, if it does, fill it with GlobalHttpUrl
			case 'no':
				if (strstr(strtolower($settings['GlobalHttpsUrl']), 'https://'))
				{
					// TODO: Update settings table
					$db->query("UPDATE ".DB_PREFIX."settings SET value = '".$db->escape($settings['GlobalHttpUrl'])."' WHERE name = 'GlobalHttpsUrl'");
				}
				break;
			
			default: throw new Exception('Invalid value encountered for SSL check. Expecting yes or no.');
		}
	}
	catch (Exception $e)
	{
		echo '<p style="color: red;"><b>'.$e->getMessage().'</b></p><p>Go back and try again.</p>';
		exit;
	}

}

/**
 * Runs the sql update script
 *
 * @param string $current_version 
 * @return void
 */
function updateCart($current_version, $target_version = null, $overwrite_config = true)
{
	global $db;
	
	if (is_null($target_version)) $target_version = DDM_TARGET_VERSION;
	
	if(file_exists("update-".$current_version.".sql")){
		$f = fopen("update-".$current_version.".sql", "r");
		if($f)
		{
			echo "<span style='color:green'>OK - Version is valid (".$current_version.")</span><br/>";
			
			while($s = fgets($f, 8096)){
				if(trim($s) != ""){
					if(strstr($s,"DELETE FROM ")) 
						$s = str_replace("DELETE FROM ","DELETE FROM ".DB_PREFIX,$s);
					elseif(strstr($s,"DROP TABLE IF EXISTS")) 
						$s = str_replace("EXISTS ","EXISTS ".DB_PREFIX,$s); 
					elseif(strstr($s,"ALTER TABLE "))
						$s = str_replace("TABLE ","TABLE ".DB_PREFIX,$s);
					elseif(strstr($s,"CREATE TABLE IF NOT EXISTS "))
						$s = str_replace("IF NOT EXISTS ","IF NOT EXISTS ".DB_PREFIX,$s);
					elseif(strstr($s,"CREATE TABLE "))
						$s = str_replace("TABLE ","TABLE IF NOT EXISTS ".DB_PREFIX,$s);
					elseif(strstr($s,"INSERT IGNORE INTO "))
						$s = str_replace("INTO ","INTO ".DB_PREFIX,$s);
					elseif(strstr($s,"INSERT INTO "))
						$s = str_replace("INTO ","IGNORE INTO ".DB_PREFIX,$s);
					elseif(strstr($s,"UPDATE ") && strstr($s,"SET "))
						$s = str_replace("UPDATE ","UPDATE ".DB_PREFIX,$s);
					$db->query($s);
				}
			}
		}
		if ($overwrite_config)
		{
			$config_file_path = "../../content/engine/engine_config.php";
			$f = fopen($config_file_path, "w+");
			if ($f)
			{
				fputs($f, "<?php\n");
				fputs($f, "\tdefine('DB_HOST', '".DB_HOST."');\n");
				fputs($f, "\tdefine('DB_USER', '".DB_USER."');\n");
				fputs($f, "\tdefine('DB_PASSWORD', '".DB_PASSWORD."');\n");
				fputs($f, "\tdefine('DB_NAME', '".DB_NAME."');\n");
				fputs($f, "\tdefine('DB_PREFIX', '".DB_PREFIX."');\n");
				fputs($f, "\tdefine('LICENSE_NUMBER', '".LICENSE_NUMBER."');\n");
				fputs($f, "?>\n");
				fclose($f);
			}
			else
			{
				echo '<p><b>Cannot write config file. Please check file privileges</b></p>';
			}
		}
	}
	else
	{
		echo "<span style='color:red'><b>ERROR - Update file for version ".$current_version." does not exist. Upload update files again. Please contact customer support if you need assistance.</b></span><br/>";
		exit;
	}
	echo '<p><b>Database has been successfully updated to '.$target_version.'</b></p>';
}


///// start prep /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// get the current version and revision
$db->query("SELECT value, description FROM ".DB_PREFIX."settings WHERE name='AppVer'");
$db->moveNext();
$current_app_ver = $db->col["value"];
$rev_number = $db->col["description"];


// get the old skin name
$db->query("SELECT value FROM ".DB_PREFIX."settings WHERE name='DesignSkinName'");
$db->moveNext();
$old_skin_name = $db->col['value'];

// store the new skin name the user selected
$new_skin_name = (array_key_exists('skin_name', $_POST)) ? $_POST['skin_name'] : $old_skin_name;

$settings['DesignSkinName'] = $new_skin_name;


///// end prep /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

$msg_not_supported = "<span style='color:red'><b>ERROR - Version ".$current_app_ver." is not supported.  To continue, please update your cart to version 3.6.4. Please contact customer support if you need assistance.</b></span><br/>";

switch ($current_app_ver)
{
	case '3.6.4':
		// include design mode class to create css
		include("../../content/classes/class.ddm_design_mode.php");
		
		// Apply database changes
		updateCart('3.6.4', '3.7.0');
		
		// Attempt to migrate logo
		migrateImages();
		
		// Update the DesignSkinName setting
		$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($new_skin_name)."', description='' WHERE name='DesignSkinName'");
		
		if (!array_key_exists('CatalogCategoryThumbType', $settings))
			$settings['CatalogCategoryThumbType'] = 'Proportionate';
		
		if (!array_key_exists('DesignSkinTheme', $settings))
			$settings['DesignSkinTheme'] = 'theme-default';
		
		if (!array_key_exists('LayoutElements', $settings))
			$settings['LayoutElements'] = 'a:1:{s:5:"pages";a:8:{s:7:"_global";a:1:{s:5:"zones";a:2:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}s:5:"right";a:2:{i:0;a:2:{s:5:"class";s:34:"element_panels_panel-checkout-cart";s:4:"file";s:41:"templates/panels/panel-checkout-cart.html";}i:1;a:2:{s:5:"class";s:35:"element_panels_panel-catalog-search";s:4:"file";s:42:"templates/panels/panel-catalog-search.html";}}}}s:7:"catalog";a:1:{s:5:"zones";a:1:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}}}s:4:"cart";a:1:{s:5:"zones";a:0:{}}s:6:"signup";a:1:{s:5:"zones";a:1:{s:4:"left";a:4:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}}}}s:28:"billing_and_shipping_address";a:1:{s:5:"zones";a:1:{s:4:"left";a:5:{i:0;a:2:{s:5:"class";s:39:"element_panels_panel-catalog-categories";s:4:"file";s:46:"templates/panels/panel-catalog-categories.html";}i:1;a:2:{s:5:"class";s:42:"element_panels_panel-catalog-manufacturers";s:4:"file";s:49:"templates/panels/panel-catalog-manufacturers.html";}i:2;a:2:{s:5:"class";s:42:"element_panels_panel-newsletters-subscribe";s:4:"file";s:49:"templates/panels/panel-newsletters-subscribe.html";}i:3;a:2:{s:5:"class";s:36:"element_panels_panel-account-profile";s:4:"file";s:43:"templates/panels/panel-account-profile.html";}i:4;a:2:{s:5:"class";s:34:"element_panels_panel-checkout-cart";s:4:"file";s:41:"templates/panels/panel-checkout-cart.html";}}}}s:15:"shipping_method";a:1:{s:5:"zones";a:0:{}}s:7:"invoice";a:1:{s:5:"zones";a:0:{}}s:17:"one_page_checkout";a:1:{s:5:"zones";a:0:{}}}}';
	
	case '3.7.0':
		updateCart('3.7.0', '3.7.1');
	
	case '3.7.1':
		updateCart('3.7.1', '3.7.2');
		
		// update revision to latest 3.7.2 R.3095
		$db->query("UPDATE ".DB_PREFIX."settings SET description='R.3095' WHERE name = 'AppVer'");
		
	case '3.7.2':
		updateCart('3.7.2', '3.7.3');
	
	case '3.7.3':
		// migrateThemes();
		updateCart('3.7.3', '3.7.4');
		
	case '3.7.4':
		updateCart('3.7.4', '3.7.5 R.4000');
		$rev_number = 'R.4000';
	
	case '3.7.5':
		switch ($rev_number)
		{
			case 'R.4000':
			{
				updateCart('3.7.5r4000', '3.7.5 R.4100');
				
				upgrade375r4000();
					
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.5', description='R.4100' WHERE name='AppVer'");
			}
			case 'R.4100':
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.5', description='R.4110' WHERE name='AppVer'");
				
				// fixes any secondary images by renaming them to the schema that is introduced in 3.7.5
				// the fix is applied to existing 3.7.5 users in case they upgraded from 3.7.4 previously
				updateSecondaryImages($db);
				
			case 'R.4110':
				updateCart('3.7.5r4110', '3.7.6');
				
				upgrade375r4110();
				
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.6', description='R.220' WHERE name='AppVer'");
		}
	
	case '3.7.6':
		updateCart('3.7.6', '3.7.7');
		
		// Copy ie7.js and rename it as ie7base.js
		if (file_exists('../../content/engine/design/javascript/ie7.js'))
		{
			copy('../../content/engine/design/javascript/ie7.js', '../../content/engine/design/javascript/ie7base.js');
		}
		else if (!file_exists('../../content/engine/design/javascript/ie7base.js'))
		{
			file_put_contents('../../content/engine/design/javascript/ie7base.js', '');
		}
		
		// Perform google checkout retro fix
		googleCheckoutRetroFix();
		
		// Perform column changes
		upgrade376r220();
		
		$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.7', description='R.350' WHERE name='AppVer'");
		
	case '3.7.7':
		updateCart('3.7.7', '3.7.8 R.750');
		
		// perform column changes
		upgrade377r350();
		
		$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.8', description='R.750' WHERE name='AppVer'");
		$rev_number = 'R.750';
	
	case '3.7.8':
		switch ($rev_number)
		{
			case 'R.750':
			{
				updateCart('3.7.8r750', '3.7.8 R.800');
				upgrade378r750();
				
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.8', description='R.800' WHERE name='AppVer'");
			}
			case 'R.800':
			{
				updateCart('3.7.8r800', '3.7.9r1');
				upgrade378r800();
				
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.9', description='R.1' WHERE name='AppVer'");
				$rev_number = 'R.1';
			}
		}
	
	case '3.7.9':
		switch ($rev_number)
		{
			case 'R.1':
			{
				upgrade379r1();
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.9', description='R.2' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.9r2</b></p>';
			}
			case 'R.2':
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.9', description='R.3' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.9r3</b></p>';
			}
			case 'R.3':
			{
				upgrade379r3();
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.10', description='R.1' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.10r1</b></p>';
				$rev_number = 'R.1';
			}
		}
	
	case '3.7.10':
		switch ($rev_number)
		{
			case 'R.1':
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.10', description='R.2' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.10r2</b></p>';
			}
			case 'R.2':
			{
				
				if (DDM_TARGET_VERSION == '3.7.10' && $rev_number == 'R.2')
				{
					$rename_files = array(
						'../../content/classes/admin/categoryprocess.php',
						'../../content/classes/admin/productprocess.php'
					);
					
					foreach ($rename_files as $file)
					{
						if (file_exists($file))
						{
							if (@rename($file, 'backup-'.$file))
							{
								echo "Successfully renamed {$file}<br />";
							}
							else if (@unlink($file))
							{
								echo "Successfully deleted {$file}<br />";
							}
							else
							{
								echo "Could not remove {$file}, please remove or rename this file manually.<br />";
							}
						}
					}
				}
				
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.10', description='R.3' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.10r3</b></p>';
				
				$rev_number = 'R.3';
			}
			case 'R.3':
			case 'R.4':
			{
				upgrade3710r3();
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.11', description='R.1' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.11r1</b></p>';
				$rev_number = 'R.1';
			}
		}
	case '3.7.11':
		switch ($rev_number)
		{
			case 'R.1':
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.11', description='R.2' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.11r2</b></p>';
				$rev_number = 'R.2';
			}
			case 'R.2':
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.11', description='R.3' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.11r3</b></p>';
				$rev_number = 'R.3';
			}
			case 'R.3': // No db / file changes in R.4
			case 'R.4':
			{
				upgrade3711r4();
				echo '<p><b>Database has been successfully updated to 3.7.12-R1</b></p>';
				$rev_number = 'R.1';
			}
		}
	case '3.7.12':
	{
		switch ($rev_number)
		{
			case 'R.1':
			case 'R1':
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.12', description='R.2' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.12-R2</b></p>';
				$rev_number = 'R.2';
			}
			case 'R.2':
			case 'R2':
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.12', description='R.3' WHERE name='AppVer'");
				echo '<p><b>Database has been successfully updated to 3.7.12-R3</b></p>';
				$rev_number = 'R.3';
			}
			case 'R.3':
			case 'R3':
			{
				upgrade3712r3();
				echo '<p><b>Database has been successfully updated to 3.7.13-R1</b></p>';
				$rev_number = 'R.1';
			}
		}
	}
	case '3.7.13':
	{
		switch ($rev_number)
		{
			case 'R.1':
			case 'R1':
			{
				upgrade3713r1();
				echo '<p><b>Database has been successfully updated to 3.7.13-R2</b></p>';
				$rev_number = 'R.2';
			}
			case 'R.2':
			case 'R2':
			{
				upgrade3713r2();
				echo '<p><b>Database has been successfully updated to 3.7.13-R3</b></p>';
				$rev_number = 'R.3';
			}
			case 'R.3':
			case 'R3':
			case 'R.4':
			case 'R4':
			case 'R.5':
			case 'R5':
			{
				upgrade3713r3();
				echo '<p><b>Database has been successfully updated to 3.7.14-R1</b></p>';
				$rev_number = 'R.1';
			}
		}
	}
	case '3.7.14':
	{
		switch ($rev_number)
		{
			case 'R.1':
			case 'R1':
			{
				upgrade3714r1();
				echo '<p><b>Database has been successfully updated to 3.7.14-R2</b></p>';
				$rev_number = 'R.2';
			}
			case 'R.1':
			case 'R1':
			{
				upgrade3714r2();
				echo '<p><b>Database has been successfully updated to 3.7.14-R3</b></p>';
				$rev_number = 'R.3';
			}
		}
		break;
	}
	case DDM_TARGET_VERSION:
		echo "<br/><span style='color:green'><b>Your cart is the current version of this software.<br/>No further update needed.</b></span><br/>";
		break;
	
	default: echo $msg_not_supported; exit;
}

$db->done();
recompileSkin();

echo '<p><a href="../../login.php">Click here to go to admin area.</a></p>';

function columnCheck($tablename)
{
	global $db, $settings;
	static $columns = array();
	
	$result = $db->selectAll("DESC ".DB_PREFIX.$tablename);
	foreach ($result as $row)
	{
		$columns[] = "$tablename.".$row['Field'];
	}
	
	return $columns;
}

function updateSettings($key, $value = null)
{
	global $settings;
	
	if (!is_array($key))
	{
		if (is_null($value)) throw new Exception('You must pass a value for the settings key');
		$key = array($key => $value);
	}
	
	foreach ($key as $index => $setting_value)
	{
		if (!isset($settings[$index]))
		{
			$settings[$index] = $setting_value;
		}
	}
}


function recompileSkin()
{
	global $db, $settings;
	
	$wd = getcwd();
	chdir($settings['GlobalServerPath']);
	
	$db = new DB();
	$db->init(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
	require_once("content/engine/engine_settings.php");
	
	$view = Ddm_View::getInstance();
	
	$view->assemble('content'.DS.'cache'.DS.'skins'.DS.'_mobile'.DS);
	$view->assemble('content'.DS.'cache'.DS.'skins'.DS.escapeFileName($settings['DesignSkinName']).DS);
	
	chdir($wd);
}

function upgrade3714r2()
{
	global $db;

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.14', description='R.3' WHERE name='AppVer'");
}

function upgrade3714r1()
{
	global $db;

	$columns = columnCheck('orders');
	if (!in_array('orders.force_recalc', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN force_recalc tinyint NOT NULL default '0' AFTER gift_cert_amount");

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.14', description='R.2' WHERE name='AppVer'");
}

function upgrade3713r3()
{
	global $db, $settings;

	$columns = columnCheck('admins');
	if (!in_array('admins.security_question_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN security_question_id INT UNSIGNED NOT NULL DEFAULT 0 AFTER expiration_date");
	if (!in_array('admins.security_question_answer', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN security_question_answer VARCHAR(100) NOT NULL default '' AFTER security_question_id");
	if (!in_array('admins.force_password_change', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN force_password_change tinyint not null DEFAULT 0 AFTER openid_provider");
	$db->query("ALTER TABLE ".DB_PREFIX."admins CHANGE receive_notifications receive_notifications set('invoice', 'outofstock', 'signup', 'security', 'product_review', 'notactive', 'pwdchange30', 'pwdchange60', 'pwdchange90', 'pwdchangenever') NOT NULL default ''");

	$columns = columnCheck('catalog');
	if (!in_array('catalog.exactor_euc_code', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."catalog ADD COLUMN exactor_euc_code varchar(255) not null default '' AFTER category_path");
	if (!in_array('catalog.avalara_tax_code', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."catalog ADD COLUMN avalara_tax_code varchar(255) not null default '' AFTER exactor_euc_code");

	$columns = columnCheck('orders');
	if (!in_array('orders.tax_exempt', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN tax_exempt enum('No', 'Yes', 'Partial') NOT NULL default 'No' AFTER tax_amount");
	if (!in_array('orders.force_recalc', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN force_recalc tinyint NOT NULL default '0' AFTER gift_cert_amount");

	$columns = columnCheck('orders_content');
	if (!in_array('orders_content.price_withtax', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN price_withtax decimal(20, 5) unsigned NOT NULL default '0.00000' AFTER price");
	if (!in_array('orders_content.tax_amount', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN tax_amount decimal(20, 5) unsigned NOT NULL default '0.00000' AFTER is_taxable");
	if (!in_array('orders_content.tax_exempt', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN tax_exempt tinyint(1) unsigned NOT NULL default '0' AFTER tax_amount");
	if (!in_array('orders_content.discount_amount', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN discount_amount decimal(20, 5) unsigned NOT NULL default '0.00000' AFTER title");
	if (!in_array('orders_content.promo_discount_amount', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN promo_discount_amount decimal(20, 5) unsigned NOT NULL default '0.00000' AFTER discount_amount");

	$columns = columnCheck('products');
	if (!in_array('products.avalara_tax_code', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN avalara_tax_code varchar(500) not null default '' AFTER nextag_item_condition");
	if (!in_array('products.exactor_euc_code', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN exactor_euc_code varchar(255) not null default '' AFTER avalara_tax_code");

	$columns = columnCheck('tax_zones');
	if (!in_array('tax_zones.display_with_tax', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."tax_zones ADD COLUMN display_with_tax tinyint default '0' AFTER zone_description");

	$columns = columnCheck('users');
	if (!in_array('users.is_tax_exempt', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN is_tax_exempt tinyint(1) unsigned NOT NULL default 0 AFTER `level`");
	if (!in_array('users.avalara_tax_exempt_number', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN avalara_tax_exempt_number varchar(128) not null AFTER facebook_id");
	if (!in_array('users.exactor_tax_exemption_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN exactor_tax_exemption_id varchar(128) not null AFTER avalara_tax_exempt_number");

	$db->query("ALTER TABLE ".DB_PREFIX."users CHANGE COLUMN newsletters newsletters enum('Yes', 'No') NOT NULL default 'No'");
	$db->query("ALTER TABLE ".DB_PREFIX."users CHANGE COLUMN updates updates enum('Yes', 'No') NOT NULL default 'No'");

	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."shipping_scan_forms (scan_form_id int(10) unsigned NOT NULL auto_increment, scan_form_source varchar(32) NOT NULL default '', scan_form_date DATE NOT NULL, scan_form_code varchar(250) NOT NULL default '', scan_form_url TEXT NOT NULL default '', PRIMARY KEY (scan_form_id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."orders_labels (olid int(11) unsigned NOT NULL AUTO_INCREMENT, oid int(11) unsigned NOT NULL DEFAULT '0', unique_id varchar(32) NOT NULL DEFAULT '', type varchar(32) NOT NULL DEFAULT '', date_created datetime NOT NULL DEFAULT '0000-00-00 00:00:00',date_canceled datetime NOT NULL DEFAULT '0000-00-00 00:00:00', status enum('Created','Canceled','Failed') NOT NULL DEFAULT 'Failed', url varchar(4096) NOT NULL DEFAULT '', transaction_id varchar(128) NOT NULL DEFAULT '', tracking_number varchar(128) NOT NULL DEFAULT '', shipping_date date NOT NULL, shipping_weight decimal(10,2) NOT NULL DEFAULT 0, shipping_insurance decimal(10,2) unsigned NOT NULL DEFAULT 0, shipping_service_type varchar(255) NOT NULL default '', shipping_package_type varchar(255) NOT NULL default '', data varchar(4096), PRIMARY KEY (olid), KEY oid (oid))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."orders_avalara_tax (oid integer(11) not null, document_code varchar(64) not null, status enum('uncommitted','committed','void','returned') not null default 'uncommitted', primary key(oid))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."avalara_tax_codes (id integer(11) not null auto_increment, name varchar(255) not null, code varchar(64) not null, primary key(id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."avalara_cache (id int not null AUTO_INCREMENT, oid int not null, request_hash varchar(255) not null, response text not null, request_date datetime not null, INDEX avalara_cache_oid (oid ASC, request_hash ASC), PRIMARY KEY(id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."exactor_invoices (ID int(20) NOT NULL AUTO_INCREMENT, TransactionID varchar(100) DEFAULT NULL, OrderID varchar(100) DEFAULT NULL, DateOfOrder date DEFAULT NULL, PRIMARY KEY (ID))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."exactor_transaction_order (ID int(20) NOT NULL AUTO_INCREMENT, TransactionID varchar(100) DEFAULT NULL, OrderID varchar(100) DEFAULT NULL, UserID varchar(50) DEFAULT NULL, MerchantID varchar(50) DEFAULT NULL, DateOfOrder date DEFAULT NULL, PRIMARY KEY (ID))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."exactor_cache (id int not null AUTO_INCREMENT, oid int not null, request_hash varchar(255) not null, response_xml text not null, request_date datetime not null, transaction_id varchar(255) null, INDEX exactor_cache_oid (oid ASC, request_hash ASC), PRIMARY KEY(id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."exactor_euc_categories (id int unsigned not null AUTO_INCREMENT, parent int unsigned null, level int unsigned null, category_path varchar(500) not null, description varchar(500) not null, primary key(id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."exactor_tax_codes (id integer(11) not null auto_increment, name varchar(255) not null, code varchar(64) not null, primary key(id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."avalara_tax_codes_categories (id int unsigned not null AUTO_INCREMENT, parent int unsigned null, level int unsigned null, tax_code varchar(500) not null, description varchar(500) not null, primary key(id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."shift4_timeout_log (oid int(11) NOT NULL, uid int(11) NOT NULL, request_code varchar(128) NOT NULL, amount decimal(20, 10) NOT NULL, unique_id varchar(128) NOT NULL, timestamp datetime NOT NULL, invoice varchar(50) NOT NULL)");

	// 3.7.13-R.4 upgrade fix
	$columns = columnCheck('shift4_timeout_log');
	if (!in_array('shift4_timeout_log.invoice', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."shift4_timeout_log ADD COLUMN invoice varchar(50) NOT NULL");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods VALUES (null, 'No', 'https://secure.i4go.com/index.cfm', 'ipn', 0, 'shift4i4go', 'PAYMENT_SHIFT4i4Go', 'Shift4.com', 'Shift4.com', 'Credit Card Payment', 0, 'gateway', 'US', '', '<u>Enter Credit Card Information</u>', '<b>Your Order Has Been Sent</b>', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods VALUES (null, 'No', 'https://api.globalgatewaye4.firstdata.com/transaction/v12', 'cc', 0, 'firstdata', 'PAYMENT_FIRSTDATA', 'First Data Global Gateway e4', 'First Data Global Gateway e4', 'Credit Card Payment', 0, 'gateway', 'US', '', '<u>Enter Credit Card Information</u>', '<b>Your Order Has Been Sent</b>', '')");
	$firstDataId = $db->insertId();
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'Yes', 'VISA', 'Visa', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'Yes', 'MC', 'Master Card', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'Yes', 'AMEX', 'American Express', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'DICL', 'Diners Club', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'JCB', 'JCB', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'CARTE', 'Carte Blanche', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'ABC', 'Australian BankCard', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'DINO', 'Discover/Novus', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'SW', 'Maestro', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'SO', 'Solo', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'ERT', 'enRoute', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$firstDataId.", 'No', 'UKE', 'Visa Electron', 'No')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingOriginName', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingOriginPhone', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingOriginProvince', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExRateRequestType', 'LIST', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExFreightClass', 'CLASS_050', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExFreightPackaging', 'PALLET', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExFreightPackagingWeight', '0', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Tax Settings', 0, 'TaxExemptionsActive', '0', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'search_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'search_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'search_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'testimonials_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'testimonials_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'testimonials_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'login_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'login_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'login_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'register_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'register_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'register_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'sitemap_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'sitemap_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'sitemap_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'notfound_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'notfound_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'notfound_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'cart_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'cart_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'cart_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'subscribe_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'subscribe_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'subscribe_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'unsubscribe_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'unsubscribe_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'unsubscribe_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'reset_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'reset_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'reset_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'international_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'international_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'international_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'giftcert_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'giftcert_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'giftcert_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'shippingquote_page_title', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'shippingquote_meta_description', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'shippingquote_meta_keywords', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Security Settings', 0, 'SecurityLastAuditEmailTimestamp', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_shift4i4go', 1, 'shift4i4go_Merchant_Id', '', '', '', '', '', 'Please enter your Merchant Id')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_shift4i4go', 2, 'shift4i4go_Account_Id', '', '', '', '', '', 'Please enter your Account Id')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_shift4i4go', 3, 'shift4i4go_Site_Id', '', '', '', '', '', 'Please enter your Site Id')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_shift4i4go', 4, 'shift4i4go_Login', '', '', '', '', '', 'User name as registered in $$$ ON THE NET system.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'password', 0, 'payment_shift4i4go', 5, 'shift4i4go_Password', '', '', '', '', '', 'Valid password for Merchant Login ID')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_shift4i4go', 6, 'shift4i4go_SN', '', '', '', '', '', 'Please enter Serial Number as assigned to this $$$ ON THE NET account')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_shift4i4go', 2, 'shift4i4go_Auth_Type', 'Auth-Capture', 'Auth-Capture, Auth-Only', 'Auth-Capture', '', '', 'Auth-Capture is the normal transaction method; a transaction is sent to the system for approval, the transaction is approved, the merchant is notified of the approval, and the transaction automatically settles at the end of the business day without any further action by the merchant. Auth-Only stands for Authorization-Only and means obtaining an authorization for a certain amount on a customer\'s credit card without actually charging the card.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 1, 'payment_shift4i4go', 2, 'shift4i4go_UTG_Server', '', '', '', '', '', 'Shift4 UTG Service endpoint')");

	$enableBML = 'Yes';
	$paypalEc = $db->selectOne("SELECT COUNT(*) AS c FROM ".DB_PREFIX."payment_methods WHERE active='Yes' AND id = 'paypalec'");
	if ($paypalEc['c'] > 0)
	{
		$enableBML = 'No';
	}
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_paypalec', 2, 'paypalec_Offer_PayPal_BML', '".$enableBML."', 'Yes, No', 'Yes', '', 'Offer PayPal Bill Me Later?', 'Would you like to offer PayPal Bill Me Later&reg;?')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, '', 0, 'paypal_publisher_id', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, '', 0, 'paypal_publisher_email', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_firstdata', 1, 'firstdata_GatewayId', '', '', '', '', 'Exact Id / Gateway Id', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'password', 0, 'payment_firstdata', 2, 'firstdata_Password', '', '', '', '', 'Password', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_firstdata', 3, 'firstdata_KeyId', '', '', '', '', 'Key Id', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'password', 0, 'payment_firstdata', 4, 'firstdata_HmacKey', '', '', '', '', 'Hmac Key', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_firstdata', 1, 'firstdata_Test_Request', 'FALSE', 'TRUE, FALSE', 'FALSE', '', '', 'Processing mode (TRUE for test mode and FALSE for real processing)')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_firstdata', 2, 'firstdata_Auth_Type', 'Auth-Capture', 'Auth-Capture, Auth-Only', 'Auth-Capture', '', '', 'Auth-Capture is the normal transaction method; a transaction is sent to the system for approval, the transaction is approved, the merchant is notified of the approval, and the transaction automatically settles at the end of the business day without any further action by the merchant. Auth-Only stands for Authorization-Only and means obtaining an authorization for a certain amount on a customer\'s credit card without actually charging the card. If the money is not captured within 30 days, the transaction will expire.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_firstdata', 3, 'firstdata_Currency_Code', 'USD', 'USD,CAD,DEM,CHF,GBP,JPY,AFA,ALL,DZD,ADF,ADP,AON,ARS,AWG,AUD,ATS,BSD,BHD,BDT,BBD,BEF,BZD,BMD,BTN,BOB,BWP,BRL,BND,BGL,BIF,XOF,XAF,KHR,CVE,KYD,CLP,CNY,COP,KMF,CRC,HRK,CYP,CSK,DKK,DJF,DOP,NLG,XEU,ECS,EGP,SVC,EEK,ETB,EUR,FKP,FJD,FIM,FRF,GMD,GHC,GIP,XAU,GRD,GTQ,GNF,GYD,HTG,HNL,HKD,HUF,ISK,INR,IDR,IEP,ILS,ITL,JMD,JOD,KZT,KES,KWD,LAK,LVL,LSL,LRD,LTL,LUF,MOP,MGF,MWK,MYR,MVR,MTL,MRO,MUR,MXN,MNT,MAD,MZM,MMK,ANG,NAD,NPR,NZD,NIO,NGN,NOK,OMR,PKR,XPD,PAB,PGK,PYG,PEN,PHP,XPT,PLN,PTE,QAR,ROL,RUB,WST,STD,SAR,SCR,SLL,XAG,SGD,SKK,SIT,SBD,ZAR,KRW,ESP,LKR,SHP,SRG,SZL,SEK,TWD,TZS,THB,TOP,TTD,TND,TRL,UGS,UAH,UYP,AED,VUV,VEB,VND,YUN,ZMK', 'USD', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No', 'select', 1, 'payment_firstdata', 4, 'firstdata_Payment_Profiles', 'No', 'Yes, No', 'No', '', '', 'This allows customers to store the credit cards for later use on the checkout page in a secure manner.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No', 'select', 1, 'payment_firstdata', 5, 'firstdata_Enable_Cardinal_Centinel', 'No', 'Yes, No', 'No', '', '', '<table height=\"22\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tr><td width=\"20\"><img src=\"images/icons/ic_centinel.gif\"></td><td><a href=\"admin.php?p=payment_cardinal\" style=\"font-size:11px\"><b>Click here to setup Cardinal Centinel Module</b></a></td></tr></table>')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','text',0,'Exactor',1,'exactor_plugin_version','','','','','','Exactor Plugin Version')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','text',0,'Exactor',1,'exactor_merchant_id','','','','','','Merchant ID')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','text',0,'Exactor',1,'exactor_user_id','','','','','','User ID')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','text',0,'Exactor',1,'exactor_password','','','','','','Exactor Account Password')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','text',0,'Exactor',1,'exactor_full_name','','','','','','Exactor Merchant Name')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','text',0,'Exactor',1,'exactor_currency_code','USD','','','','','Currency Code')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','combo',0,'Exactor',1,'exactor_commit_taxes','1','Yes,No','','','','Commit Taxes')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes','combo',0,'Exactor',1,'exactor_sku_source','SKU','','','','','Exactor Source Of SKU')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No','select',1,'Exactor',1,'ExactorEnableTax','No','Yes,No','No','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Avalara', 0, 'AvalaraAccountNumber', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Avalara', 0, 'AvalaraLicenseKey', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Avalara', 0, 'AvalaraCompanyCode', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Avalara', 0, 'AvalaraURL', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Avalara', 0, 'AvalaraEnableTax', 'No', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Avalara', 0, 'AvalaraCurrencyCode', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No', 'hidden', 1, 'payment_bongocheckout', 4, 'bongocheckout_Show_Instructions', '1', '1', '1', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_ExtendJS','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'',0,'bongocheckout_Shipping_Method','auto','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'StampscomUsername', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'StampscomPassword', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'StampscomApiUrl', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'StampscomActive', 'No', '', 'No', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'StampscomTestMode', 'No', '', 'No', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'StampscomSCANFormEnable', 'No', '', 'No', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."layout_elements (type, name, title, is_active) VALUES ('panel', 'element_panels_panel-paypalbml-180x150', 'PayPal Banner 180x150', 0)");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."layout_elements (type, name, title, is_active) VALUES ('panel', 'element_panels_panel-paypalbml-120x90', 'PayPal Banner 120x90', 0)");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."layout_elements (type, name, title, is_active) VALUES ('panel', 'element_panels_panel-paypalbml-250x250', 'PayPal Banner 250x250', 0)");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."layout_elements (type, name, title, is_active) VALUES ('panel', 'element_panels_panel-paypalbml-300x250', 'PayPal Banner 300x250', 0)");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."layout_elements (type, name, title, is_active) VALUES ('panel', 'element_panels_panel-paypalbml-160x600', 'PayPal Banner 160x600', 0)");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."layout_elements (type, name, title, is_active) VALUES ('panel', 'element_panels_panel-paypalbml-728x90', 'PayPal Banner 728x90', 0)");

	$db->query("UPDATE ".DB_PREFIX."settings SET default_value = 'YES' WHERE name = 'SecurityForceCompliantAdminPasswords'");
	$db->query("UPDATE ".DB_PREFIX."settings SET default_value = 'GBP' WHERE name = 'paypalhsuk_Currency_Code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET options = 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD' WHERE name = 'paypalipn_Currency_Code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET options = 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD' WHERE name = 'paypalwps_Currency_Code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET options = 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD' WHERE name = 'paypalhsuk_Currency_Code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET options = 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD' WHERE name = 'paypalpro_Currency_Code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET options = 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD' WHERE name = 'paypalec_Currency_Code'");

	$db->query("UPDATE ".DB_PREFIX."settings SET caption = 'Set Shipping Address to Billing Address by default?' WHERE name = 'OrderShippingAsBillingDefaultOption'");
	$db->query("UPDATE ".DB_PREFIX."settings SET description = 'Require your customers to agree with your site terms and conditions in order to checkout?' WHERE name = 'DisplayTermsAndConditionsCheckbox'");
	$db->query("UPDATE ".DB_PREFIX."settings SET description = 'Require registered customers to agree with your site terms and conditions on <span style=\"text-decoration:underline;\">every</span> checkout?' WHERE name = 'AlwaysDisplayTermsAndConditionsCheckbox'");

	$db->query("UPDATE ".DB_PREFIX."orders SET force_recalc = 1 WHERE ".DB_PREFIX."orders.status IN ('Abandon', 'New')");

	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'OrderShippingAsBillingControlType'");

	// Remove "duplicate"
	$shift4 = $db->selectOne("SELECT COUNT(*) AS c FROM ".DB_PREFIX."payment_methods WHERE active='No' AND id = 'shift4'");
	if ($shift4['c'] > 0)
	{
		$db->query('DELETE FROM '.DB_PREFIX."payment_methods WHERE active='No' AND id = 'shift4'");
	}

	if ($settings['DisplayPricesWithTax'] == 'YES')
	{
		$db->query("UPDATE ".DB_PREFIX."tax_zones SET display_with_tax = 1");
	}

	if (!defined('PASSWORD_SALT'))
	{
		$passwordUtil = new PasswordUtil();
		$salt = $passwordUtil->generatePasswordSalt();

		$engineConfig = file_get_contents($settings['GlobalServerPath'].'/content/engine/engine_config.php');
		if (strpos($engineConfig, "?>") !== false)
		{
			$engineConfig = str_replace("?>", "", $engineConfig);
		}

		$engineConfig .= "\n\tdefine('PASSWORD_SALT', '".$salt."');\n";

		@file_put_contents($settings['GlobalServerPath'].'/content/engine/engine_config.php', $engineConfig);
	}

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.14', description='R.1' WHERE name='AppVer'");
}

function upgrade3713r2()
{
	global $db, $settings;

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalec', 8, 'paypalec_Send_Line_Item_Details', 'Yes', 'Yes, No', 'Yes', '', '', 'Send line item details to be displayed on PayPal checkout process')");
	$db->query("UPDATE ".DB_PREFIX."settings SET name='authorizenet_cp_1_0_Duplicate_Window' WHERE name ='authorizene_cp_1_0_Duplicate_Window'");

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.13', description='R.3' WHERE name='AppVer'");
}

function upgrade3713r1()
{
	global $db, $settings;

	$db->query("UPDATE ".DB_PREFIX."settings SET visible = 'No' WHERE group_name = 'payment_american_express'");

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.13', description='R.2' WHERE name='AppVer'");
}

function upgrade3712r3()
{
	global $db, $settings;

	$columns = columnCheck('users_payment_profiles');
	if (!in_array('users_payment_profiles.billing_data', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users_payment_profiles ADD COLUMN `billing_data` text NOT NULL default '' AFTER `is_primary`");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingOriginAddressLine1', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingOriginAddressLine2', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExRateRule', 'max', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogCategoryImageResizeRule', 'Resize', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogCategoryThumbResizeRule', 'Resize', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogThumbResizeRule', 'Resize', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogThumb2ResizeRule', 'Resize', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogManufacturerImageResizeRule', 'Resize', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'EndiciaLabelServer', 0, 'EndiciaShippingLabelsActive', 'No', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'EndiciaLabelServer', 0, 'ndiciaShippingLabelsTestMode', 'No', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'EndiciaLabelServer', 0, 'EndiciaShippingLabelsAccountID', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'EndiciaLabelServer', 0, 'EndiciaShippingLabelsPassword', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'EndiciaLabelServer', 0, 'EndiciaUspsAddressValidation', '0', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'select', 4, 'Order/Cart Settings', 16, 'OrderShippingAsBillingControlType', 'Radio', 'Radio,Drop-down', 'Radio', '', 'Selection method  for \"Shipping same as billing\"', 'This will manage how \"Ship to this address\" will be displayed during checkout')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'select', 4, 'Order/Cart Settings', 17, 'OrderShippingAsBillingDefaultOption', 'No', 'Yes,No', 'No', '', '\"Shipping is the same as billing\" is defaulted to active?', 'A selection of \"yes\" will default \"Shipping same as billing\"  to active. May assist in speeding up checkout.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'Social Networks', 0, 'facebookPageAccount', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'Social Networks', 0, 'pinItButtonProduct', 'No', 'Yes,No', 'No', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'Social Networks', 0, 'pinterestUsername', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'Social Networks', 0, 'disallowPinterestPinning', 'No', 'Yes,No', 'No', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 2, 'payment_paypalipn', 1, 'paypalipn_Cert_ID', '', '', '', '', 'Cert ID', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 2, 'payment_paypalhsuk', 1, 'paypalhsuk_Cert_ID', '', '', '', '', 'Cert ID', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', '', 'AdminShowTrialForm', '1', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods (active,url_to_gateway,`type`,priority,id,class,gateway_name,name,title,enable_ccs,solution_type,countries_include,countries_exclude,message_payment,message_thankyou,description) VALUES ('No','https://secure.na.tnspayments.com','cc','0','american_express','PAYMENT_AMERICAN_EXPRESS','American Express','American Express','Credit Card Payment','0','instant','US','','<u>Enter Credit Card Information</u>','<b>Your Order Has Been Sent</b>','')");
	$amexId = $db->insertId();
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc (pid,enable,code,name,enable_cvv2) VALUES (".$amexId.",'Yes','AMEX','American Express','No')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No','text',0,'payment_american_express',1,'american_express_Merchant_ID','','','','','','Your Merchand ID from American Express')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No','text',0,'payment_american_express',2,'american_express_API_Password','','','','','','Your API password from American Express')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No','select',1,'payment_american_express',1,'american_express_Auth_Type','Auth-Capture','Auth-Capture, Auth-Only','Auth-Capture','','','Auth-Capture is the normal transaction method; a transaction is sent to the system for approval, the transaction is approved, the merchant is notified of the approval, and the transaction automatically settles at the end of the business day without any further action by the merchant. Auth-Only stands for Authorization-Only and means obtaining an authorization for a certain amount on a customer\'s credit card without actually charging the card. If the money is not captured within 30 days, the transaction will expire.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No','select',1,'payment_american_express',3,'american_express_Payment_Profiles','No','Yes, No','No','','','This allows customers to store the credit cards for later use on the checkout page in a secure manner.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings values ('No','select',1,'payment_american_express',4,'american_express_Token_Generation','American Express','American Express, Local','American Express','','','This settings determines how tokens are generated.  Tokens are used to save a customer\'s credit card.  It may be ether generated off server by American Express, or locally by your server.  Note:  This setting must correspond to the token generation settings on your American Express merchant account.')");

	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."endicia_recreditrequests (`rrid` int(11) NOT NULL AUTO_INCREMENT, `request_id` varchar(20) NOT NULL, `serial_number` int(11) NULL, `postage_balance` decimal(20, 5) NOT NULL, `ascending_balance` decimal(20, 5) NOT NULL, `device_id` varchar(12) NULL, `request_status` int(11) NOT NULL, `request_timestamp` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP, PRIMARY KEY (`rrid` ))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."endicia_shippinglabels (`slid` int(11) NOT NULL AUTO_INCREMENT, `tracking_number` varchar(50) NOT NULL, `pic_number` varchar(50) NOT NULL, `label_amount` decimal(20,5) NOT NULL, `label_url` varchar(99) NOT NULL, `order_id` varchar(45) NOT NULL, PRIMARY KEY (`slid`))");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Global Site Settings', 0, 'FooterAnchorText', 'Ecommerce Software', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods VALUES (null, 'No', '', 'ipn', 0, 'bongocheckout', 'PAYMENT_BONGOCHECKOUT', 'Bongo Checkout', 'Bongo Checkout', 'Credit Card Payment', 0, 'all-in-one', 'US', '', 'To complete this transaction, it is necessary to send you to payment processor website. You will be returned back to our website after transaction completed.', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'payment_bongocheckout',1,'bongocheckout_Partner_Key','','','','', '','Required Key to access the functions of Connect API')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_Company_Name','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_Attention_To','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_Address1','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_Address2','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_City','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_State','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_Zip_Code','','','','', '','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','text',0,'',0,'bongocheckout_Phone','','','','', '','')");

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.13', description='R1' WHERE name='AppVer'");
}

function upgrade3711r4()
{
	global $db, $settings;

	$columns = columnCheck('orders');
	if (!in_array('orders.shipping_address_type', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN `shipping_address_type` enum('Business', 'Residential') NOT NULL default 'Residential' AFTER `comments`");

	$columns = columnCheck('payment_methods');
	if (in_array('payment_methods.proto', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_methods DROP COLUMN `proto`");
	if (in_array('payment_methods.version', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_methods DROP COLUMN `version`");
	if (!in_array('payment_methods.solution_type', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_methods ADD COLUMN `solution_type` enum('all-in-one', 'gateway', 'instant', 'custom') not null default 'gateway' AFTER `enable_ccs`");
	if (!in_array('payment_methods.countries_include', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_methods ADD COLUMN `countries_include` text AFTER `solution_type`");
	if (!in_array('payment_methods.countries_exclude', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_methods ADD COLUMN `countries_exclude` text AFTER `countries_include`");
	if (!in_array('payment_methods.description', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_methods ADD COLUMN `description` text AFTER `message_thankyou`");
	
	$columns = columnCheck('payment_transactions');
	if (!in_array('payment_transactions.is_success', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."payment_transactions ADD COLUMN `is_success` tinyint NOT NULL default '1' AFTER tax_amount");

	$columns = columnCheck('users_shipping', $columns);
	if (!in_array('users_shipping.address_type', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users_shipping ADD COLUMN `address_type` enum('Business', 'Residential') NOT NULL default 'Residential' AFTER `is_primary`");
	
	$columns = columnCheck('users');
	if (!in_array('users.removed', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN `removed` enum('Yes', 'No') NOT NULL default 'No' AFTER block_date");

	$db->query("ALTER TABLE ".DB_PREFIX."products CHANGE COLUMN `lock_fields` `lock_fields` varchar(255) NOT NULL DEFAULT ''");
	$db->query("ALTER TABLE ".DB_PREFIX."orders CHANGE COLUMN `payment_status` `payment_status` enum('Pending','Partial','Received','Refunded','Declined','Canceled','Error','Auth Expired','Reversed') NOT NULL DEFAULT 'Pending'");

	$db->query("SELECT * FROM ".DB_PREFIX."design_elements WHERE group_id = 'image' AND name = 'image-logo-mobile'");
	if (!$db->moveNext())
		$db->query("INSERT INTO ".DB_PREFIX."design_elements VALUES (null, 'image', 'Yes', 'image-logo-mobile', 'image-logo-mobile', '', '', 'image', '', null)");

	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = '2Checkout.com', `name` = '2Checkout.com', `title` = 'Credit Card Payment',  `solution_type` = 'instant', `countries_include` = 'US' WHERE `id` = '2checkout'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Authorize.Net AIM 3.0', `name` = 'Authorize.Net AIM 3.0', `title` = 'Credit Card Payment',  `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'authorizenet'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'First Data / LinkPoint.com', `name` = 'First Data / LinkPoint.com', `title` = 'Credit Card Payment',  `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'linkpoint'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'WorldPay.com', `name` = 'WorldPay.com', `title` = 'Credit Card Payment',  `solution_type` = 'instant', `countries_include` = 'global' WHERE `id` = 'worldpay'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'PayPal Payments Standard', `name` = 'PayPal Payments Standard', `title` = 'Pay with PayPal',  `solution_type` = 'all-in-one', `countries_include` = 'US', description='PayPal Payments Standard:  Add a PayPal payment button to your site to accept Visa, MasterCard®, American Express, Discover and PayPal payments securely. When your customers check out, they are redirected to PayPal to pay, then return to your site after they’re finished.' WHERE `id` = 'paypalipn'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods VALUES (null, 'No', 'https://www.paypal.com/cgi-bin/webscr', 'ipn', 0, 'paypalwps', 'PAYMENT_PAYPALWPS', 'PayPal Website Payments Standard', 'PayPal Website Payments Standard', 'Pay with PayPal', 0, 'all-in-one', 'GB,CA,AU,global', '', '<p><b>You can pay for this purchase using PayPal, credit card or debit card.</b></p>\r\n<p>To complete this transaction, it is necessary to send you to PayPal.com.</p>\r\n<p>You do not need to have a PayPal account.  After the transaction is complete, you will be returned to our site.</p>', '', 'PayPal Payments Standard:  Add a PayPal payment button to your site to accept Visa, MasterCard, American Express, Discover and PayPal payments securely. When your customers check out, they are redirected to PayPal to pay, then return to your site after they''re finished.')");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `id` = 'paypalpfpro', `class` = 'PAYMENT_PAYPALPFPRO', `gateway_name` = 'PayPal Payflow Pro', `name` = 'PayPal Payflow Pro', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US,CA,AU', `message_payment` = '<u>Enter Credit Card Information</u>', `message_thankyou` = '<b>Your Order Has Been Sent</b>', `description` = 'PayPal Payflow Pro:  Accept Payments with a Flexible Gateway Connect a PCI-compliant payment gateway with your Internet merchant account to accept Visa, MasterCard, American Express, Discover and PayPal payments directly on your site. Payflow Pro offers you complete control of your customersвЂ™ payment experience and helps make sure every transaction is secure.' WHERE `id` = 'verisignpfpro'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'PayLeaf (NTPNow.Com)', `name` = 'PayLeaf (NTPNow.Com)', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'ntpnowcom'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Merchant Anywhere (Transaction Central)', `name` = 'Merchant Anywhere (Transaction Central)', `title` = 'Credit Card Payment', `solution_type` = 'instant', `countries_include` = 'US' WHERE `id` = 'transactioncentral'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'FrontStream Payments (FastTransact.com)', `name` = 'FrontStream Payments (FastTransact.com)', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'fasttransact'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Echo-Inc.com', `name` = 'Echo-Inc.com', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'echo'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'HSBC-CPI', `name` = 'HSBC-CPI', `title` = 'Credit Card Payment', `solution_type` = 'instant', `countries_include` = 'GB' WHERE `id` = 'hsbccpi'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Shift4.com', `name` = 'Shift4.com', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'shift4'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'iTransact.com', `name` = 'iTransact.com', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'itransact'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `id` = 'paypalpflink', `class` = 'PAYMENT_PAYPALPFLINK', `gateway_name` = 'PayPal Payflow Link', `name` = 'PayPal Payflow Link', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US,CA,AU', `message_payment` = '<p><b>You can pay for this purchase using PayPal, credit card or debit card.</b></p><p>To complete this transaction, it is necessary to send you to PayPal.com.</p><p>You do not need to have a PayPal account.  After the transaction is complete, you will be returned to our site.</p>', `description` = 'PayPal Payflow Link:  Accept Payments with Your Merchant Account.  Accept Visa, MasterCard®, American Express, Discover and PayPal payments on your site by connecting a PayPal-hosted checkout page with your own internet merchant account. Customize the page to look like the rest of your site and make sure customer transactions are secure.' WHERE `id` = 'verisignpflink'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'PayPal Payments Pro', `name` = 'PayPal Payments Pro', `title` = 'Credit Card Payment', `solution_type` = 'all-in-one', `countries_include` = 'US', `message_payment` = '<u>Enter Credit Card Information</u>', `description` = 'PayPal Payments Pro:  Your Most Customizable Payments Solution Fully customize your website checkout with this complete, PCI-compliant payment solution. Customers can pay by credit card or PayPal without ever leaving your site, and you can tap into more than 100 million active PayPal users.' WHERE `id` = 'paypalpro'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'PayPal Express Checkout', `name` = 'PayPal Express Checkout', `title` = '<img src=\"https://www.paypal.com/en_US/i/logo/PayPal_mark_37x23.gif\" align=\"left\" style=\"margin-right:7px;\"> <a target=\"blank\" class=\"paypal_help\" href=\"https://www.paypal.com/cgi-bin/webscr?cmd=xpt/Marketing/popup/OLCWhatIsPayPal-outside\">What is PayPal?', `solution_type` = 'instant', `countries_include` = 'US,GB,CA,AU,global', `message_payment` = '<u>Click button below to complete your payment</u>', `description` = 'PayPal Express Checkout:  Add PayPal as an additional payment method to your checkout page.' WHERE `id` = 'paypalec'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'NetBilling.com', `name` = 'NetBilling.com', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US' WHERE `id` = 'netbilling'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'PsiGate.com', `name` = 'PsiGate.com', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'psigate'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Protx.com', `name` = 'Protx.com', `title` = 'Credit Card Payment', `solution_type` = 'instant', `countries_include` = 'GB'  WHERE `id` = 'vspform'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'DataCash (Elavon - Europe)', `name` = 'DataCash (Elavon - Europe)', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'GB'  WHERE `id` = 'datacash'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Google Checkout', `name` = 'Google Checkout', `title` = 'Payment with Google Checkout', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'googlecheckout'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Authorize.Net AIM 3.1', `name` = 'Authorize.Net AIM 3.1', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'authorizenet_aim_3_1'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Authorize.Net CP 1.0', `name` = 'Authorize.Net CP 1.0', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'authorizenet_cp_1_0'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Authorize.Net eCheck.Net', `name` = 'Authorize.Net eCheck.Net', `title` = 'Check Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'authorizenet_echeck'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Moneris.com', `name` = 'Moneris.com', `title` = 'Check Payment', `solution_type` = 'gateway', `countries_include` = 'CA'  WHERE `id` = 'moneris'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'USAepay', `name` = 'USAepay', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'epay'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Credit Card Storage', `name` = 'Credit Card Storage', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'other'  WHERE `id` = 'cart_ccs'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Payjunction', `name` = 'Payjunction', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'payjunction'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Intuit Payment Services', `name` = 'Intuit Payment Services', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'qbms'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Secure Hosting', `name` = 'Secure Hosting Gateway', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'GB'  WHERE `id` = 'securehosting'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'EC Suite', `name` = 'EC Suite Gateway', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'ecsuite'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Secure Trading Payment Pages', `name` = 'Secure Trading Payment Pages', `title` = 'Credit Card Payment', `solution_type` = 'instant', `countries_include` = 'GB'  WHERE `id` = 'securetrading'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Amazon Simple Pay', `name` = 'Amazon Simple Pay', `title` = 'Pay Now with Amazon', `solution_type` = 'instant', `countries_include` = 'US'  WHERE `id` = 'amazon_sp'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Skrill (MoneyBookers Quick Checkout)', `name` = 'Skrill (MoneyBookers Quick Checkout)', `title` = 'Pay securely by credit / debit card', `solution_type` = 'instant', `countries_include` = 'global'  WHERE `id` = 'moneybookers'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'DirectOne (SecurePay)', `name` = 'DirectOne (SecurePay)', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'AU'  WHERE `id` = 'securepay_securexml'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'DirectOne', `name` = 'DirectOne', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'AU'  WHERE `id` = 'directone'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Virtual Merchant', `name` = 'Virtual Merchant', `title` = 'Credit Card Payment', `solution_type` = 'instant', `countries_include` = 'US'  WHERE `id` = 'virtual_merchant'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'PayLeap', `name` = 'PayLeap', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'payleap'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Chase Paymentech Orbital', `name` = 'Chase Paymentech Orbital', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'orbital'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'SecureNet', `name` = 'SecureNet', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'US'  WHERE `id` = 'secure_net'");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET `gateway_name` = 'Beanstream', `name` = 'Beanstream', `title` = 'Credit Card Payment', `solution_type` = 'gateway', `countries_include` = 'CA'  WHERE `id` = 'beanstream'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods VALUES (null, 'No', 'https://payflowlink.paypal.com', 'ipn', 0, 'paypaladv', 'PAYMENT_PAYPALADV', 'PayPal Payments Advanced', 'PayPal Payments Advanced', 'Credit Card Payment', 0, 'all-in-one', 'US', '', '<p><b>You can pay for this purchase using PayPal, credit card or debit card.</b></p>\r\n<p>To complete this transaction, it is necessary to send you to PayPal.com.</p>\r\n<p>You do not need to have a PayPal account.  After the transaction is complete, you will be returned to our site.</p>', '', 'PayPal Payments Advanced:  Accept Payments Directly on Your Site. Accept Visa, MasterCard, American Express, Discover, and PayPal payments. Customers stay on your site for the entire checkout process with this all-in-one option. It simplifies PCI compliance and opens the door to more than 100 million active PayPal customers. Requires credit approval (2-3 business days).')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods VALUES (null, 'No', 'https://securepayments.paypal.com/cgi-bin/acquiringweb', 'ipn', 0, 'paypalhsuk', 'PAYMENT_PAYPALHSUK', 'PayPal Website Payments Pro Hosted Solution', 'PayPal Website Payments Pro Hosted Solution', 'Credit Card Payment', 0, 'all-in-one', 'GB,AU', '', '<p><b>You can pay for this purchase using PayPal, credit card or debit card.</b></p>\r\n<p>To complete this transaction, it is necessary to send you to PayPal.com.</p>\r\n<p>You do not need to have a PayPal account.  After the transaction is complete, you will be returned to our site.</p>', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Engine', 0, 'ReferralPartnerId', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Engine', 0, 'ShowReferralMessage', '1', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'select', 4, 'Order/Cart Settings', 15, 'AllowOutOfInventoryOrders', 'No', 'Yes,No', 'No', '', '', 'If \"Yes\", products already added to carts will not be rechecked for inventory levels')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Security Settings', 0, 'SecurityCCSActive', '0', '', '', '', '', '') ON DUPLICATE KEY UPDATE group_id=0");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Security Settings', 0, 'SecurityCCSStoredFor', '1', '', '', '', '', '') ON DUPLICATE KEY UPDATE group_id=0");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Security Settings', 0, 'SecurityCCSCertificate', '', '', '', '', '', '') ON DUPLICATE KEY UPDATE group_id=0");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Security Settings', 0, 'SecurityCCSPrivateKey', '', '', '', '', '', '') ON DUPLICATE KEY UPDATE group_id=0");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'authorizenet_Test_Request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'authorizenet_Auth_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'authorizenet_Enable_Cardinal_Centinel'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'authorizenet_Duplicate_Window'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'authorizenet_aim_3_1_Test_Request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'authorizenet_aim_3_1_Auth_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'authorizenet_aim_3_1_Enable_Cardinal_Centinel'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'authorizenet_aim_3_1_Duplicate_Window'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'authorizenet_cp_1_0_Test_Request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'authorizenet_cp_1_0_Auth_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'authorizenet_cp_1_0_Enable_Cardinal_Centinel'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'authorizene_cp_1_0_Duplicate_Window'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'authorizenet_echeck_Test_Request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'authorizenet_echeck_Secure_Source_eCheck'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'linkpoint_Mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'linkpoint_order_type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'worldpay_instId'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'worldpay_testMode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'worldpay_currency'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'echo_DebugMode'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'hsbccpi_Mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'hsbccpi_Transaction_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'hsbccpi_Currency'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'hsbccpi_Server_Os'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1', `caption` = 'Email address for PayPal', `description` = 'PayPal Account ID' WHERE `name` = 'paypalipn_business'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1', `name` = 'paypalipn_Mode', `description` = 'PayPal Mode' WHERE `name` = 'paypalipn_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2', `name` = 'paypalipn_Transaction_Type', `description` = 'When your buyer approves an authorization, the buyer\'s balance can be placed on hold for a 29-day period to ensure the availability of the authorization amount for capture. You can reauthorize a transaction only once, up to 115% of the originally authorized amount (not to exceed an increase of $75 USD).After a successful reauthorization, PayPal honors 100% of authorized funds for three days.You can settle without a reauthorization from day 4 to day 29 of the authorization period, but PayPal cannot ensure that 100% of the funds will be available after the three-day honor period. However, PayPal will not allow you to capture funds if the buyer\'s account is restricted, locked, or a fraudulent case occurs, or if your account has a high restriction level. You can use Authorization &amp; Capture only when your account has a low restriction level.' WHERE `name` = 'paypalipn_Payment_Action'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3', `name` = 'paypalipn_Currency_Code', `description` = 'PayPal-Supported Currencies and Max Amount || \r\n AUD- Australian Dollar - 12,500 AUD || \r\n CAD- Canadian Dollar-12,500 CAD ||\r\n EUR- Euro - 8,000 EUR || \r\n GBP- Pound Sterling - 5,500 GBP || \r\nJPY- Japanese Yen - 1,000,000 JPY || \r\nUSD- U.S. Dollar - 10,000 USD\r\n' WHERE `name` = 'paypalipn_currency_code'");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypalwps', 1, 'paypalwps_business', '', '', '', '', 'Email address for PayPal', 'PayPal Account ID')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalwps', 1, 'paypalwps_Mode', 'Live', 'Live, Test', 'Live', '', '', 'PayPal Mode')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalwps', 2, 'paypalwps_Transaction_Type', 'Sale', 'Sale, Authorization', 'Sale', '', '', 'When your buyer approves an authorization, the buyer\'s balance can be placed on hold for a 29-day period to ensure the availability of the authorization amount for capture. You can reauthorize a transaction only once, up to 115% of the originally authorized amount (not to exceed an increase of $75 USD).After a successful reauthorization, PayPal honors 100% of authorized funds for three days.You can settle without a reauthorization from day 4 to day 29 of the authorization period, but PayPal cannot ensure that 100% of the funds will be available after the three-day honor period. However, PayPal will not allow you to capture funds if the buyer\'s account is restricted, locked, or a fraudulent case occurs, or if your account has a high restriction level. You can use Authorization &amp; Capture only when your account has a low restriction level.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalwps', 3, 'paypalwps_Currency_Code', 'USD', 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MXN, MYR, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD', 'USD', '', '', 'PayPal-Supported Currencies and Max Amount || \r\nAUD- Australian Dollar - 12, 500 AUD || \r\nCAD- Canadian Dollar-12, 500 CAD ||\r\nEUR- Euro - 8, 000 EUR ||\r\nGBP- Pound Sterling - 5, 500 GBP ||\r\nJPY- Japanese Yen - 1, 000, 000 JPY||\r\nUSD- U.S. Dollar - 10, 000 USD\r\n')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypalhsuk', 1, 'paypalhsuk_business', '', '', '', '', '', 'Secure Merchant ID (found on the profile page) or the email address associated with your PayPal account.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_paypalhsuk', 2, 'paypalhsuk_Mode', 'Live', 'Live, Test', 'Live', '', '', 'PayPal Mode')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_paypalhsuk', 3, 'paypalhsuk_Currency_Code', 'GBP', 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MXN, MYR, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD', 'USD', '', '', 'PayPal-Supported Currencies and Max Amount || \r\nAUD- Australian Dollar - 12, 500 AUD || \r\nCAD- Canadian Dollar-12, 500 CAD ||\r\nEUR- Euro - 8, 000 EUR ||\r\nGBP- Pound Sterling - 5, 500 GBP ||\r\nJPY- Japanese Yen - 1, 000, 000 JPY||\r\nUSD- U.S. Dollar - 10, 000 USD\r\n')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_paypalhsuk', 4, 'paypalhsuk_Transaction_Type', 'Sale', 'Sale, Authorization', 'Sale', '', '', 'When your buyer approves an authorization, the buyer\'s balance can be placed on hold for a 29-day period to ensure the availability of the authorization amount for capture. You can reauthorize a transaction only once, up to 115% of the originally authorized amount (not to exceed an increase of $75 USD).After a successful reauthorization, PayPal honors 100% of authorized funds for three days.You can settle without a reauthorization from day 4 to day 29 of the authorization period, but PayPal cannot ensure that 100% of the funds will be available after the three-day honor period. However, PayPal will not allow you to capture funds if the buyer\'s account is restricted, locked, or a fraudulent case occurs, or if your account has a high restriction level. You can use Authorization &amp; Capture only when your account has a low restriction level.')");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '1', `name` = 'paypalpfpro_Partner', `caption` = 'Partner Name', `description` = 'Partner name used when you log into your Payflow account' WHERE `name` = 'verisignpfpro_Partner'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '2', `name` = 'paypalpfpro_Vendor', `caption` = 'Merchant Login', `description` = 'Login name associated to your Payflow account' WHERE `name` = 'verisignpfpro_Vendor'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '3', `name` = 'paypalpfpro_User', `caption` = 'User (optional)', `description` = 'PayPal recommends you set up an additional User on your account at <a href=\"https://manager.paypal.com\" target=\"_blank\">PayPal Manager</a>, instead of entering your admin username and password here. This will enhance your security and prevent service interruption should you change your Merchant Login password.' WHERE `name` = 'verisignpfpro_UserName'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '4', `name` = 'paypalpfpro_Password', `caption` = 'Password', `description` = 'Password you use to log in to your Payflow account, or password you created for API calls' WHERE `name` = 'verisignpfpro_Password'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `group_name` = 'payment_paypalpfpro', `priority` = '1', `name` = 'paypalpfpro_Mode', `description` = 'Transaction Mode (Live or Test)' WHERE `name` = 'verisignpfpro_ProcessingMode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `group_name` = 'payment_paypalpfpro', `priority` = '2', `name` = 'paypalpfpro_Transaction_Type', `description` = 'Transaction Type (Sale or Authorization only)' WHERE `name` = 'verisignpfpro_Transaction_Type'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalpfpro', 3, 'paypalpfpro_Currency_Code', 'USD', 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD', 'USD', '', 'Currency Code', 'PayPal-Supported Currencies')");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE `name` = 'verisignpfpro_ServerOs'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '1', `name` = 'paypalpfpro_Partner', `caption` = 'Partner Name', `description` = 'Partner name used when you log into your Payflow account' WHERE `name` = 'verisignpfpro_Partner'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '2', `name` = 'paypalpfpro_Vendor', `caption` = 'Merchant Login', `description` = 'Login name associated to your Payflow account' WHERE `name` = 'verisignpfpro_Vendor'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpfpro', `priority` = '3', `name` = 'paypalpfpro_User', `caption` = 'User (optional)', `description` = 'PayPal recommends you set up an additional User on your account at <a href=\"https://manager.paypal.com\" target=\"_blank\">PayPal Manager</a>, instead of entering your admin username and password here. This will enhance your security and prevent service interruption should you change your Merchant Login password.' WHERE `name` = 'verisignpfpro_UserName'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpflink', `priority` = '1', `name` = 'paypalpflink_Partner', `caption` = 'Partner Name', `description` = 'Partner name used when you log into your Payflow account' WHERE `name` = 'verisignpflink_Partner'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `group_name` = 'payment_paypalpflink', `priority` = '2', `name` = 'paypalpflink_Vendor', `caption` = 'Merchant Login', `description` = 'Login name associated to your Payflow account' WHERE `name` = 'verisignpflink_Login'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypalpflink', 3, 'paypalpflink_User', '', '', '', '', 'User (optional)', 'PayPal recommends you set up an additional User on your account at <a href=\"https://manager.paypal.com\" target=\"_blank\">PayPal Manager</a>, instead of entering your admin username and password here. This will enhance your security and prevent service interruption should you change your Merchant Login password.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'password', 0, 'payment_paypalpflink', 4, 'paypalpflink_Password', '', '', '', '', '', 'Password you use to log in to your Payflow account, or password you created for API calls')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalpflink', 1, 'paypalpflink_Mode', 'Live', 'Live, Test', 'Live', '', '', 'Transaction Mode (Live or Test)')");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `group_name` = 'payment_paypalpflink', `priority` = '2', `name` = 'paypalpflink_Transaction_Type', `description` = 'Transaction Type (Sale or Authorization only)' WHERE `name` = 'verisignpflink_Transaction_Type'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalpflink', 3, 'paypalpflink_Currency_Code', 'USD', 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD', 'USD', '', 'Currency Code', 'PayPal-Supported Currencies')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 1, 'payment_paypalpflink', 4, 'paypalpflink_Show_Instructions', '1', '1', '1', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalpflink', 1, 'paypalpflink_Color_Header_Text', '', '', '', '', 'Header text color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalpflink', 2, 'paypalpflink_Color_Border', '', '', '', '', 'Border color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalpflink', 3, 'paypalpflink_Color_Field_Label', '', '', '', '', 'Field label color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalpflink', 4, 'paypalpflink_Color_Button_Text', '', '', '', '', 'Button text color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalpflink', 5, 'paypalpflink_Color_Button_Background', '', '', '', '', 'Button background color', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypaladv', 1, 'paypaladv_Partner', 'PayPal', '', '', '', 'Partner Name', 'Your Partner name is PayPal')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypaladv', 2, 'paypaladv_Vendor', '', '', '', '', 'Merchant Login', 'Login name you created for your PayPal Payments Advanced account')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypaladv', 3, 'paypaladv_User', '', '', '', '', 'User (optional)', 'PayPal recommends you set up an additional User on your account at <a href=\"https://manager.paypal.com\" target=\"_blank\">PayPal Manager</a>, instead of entering your admin username and password here. This will enhance your security and prevent service interruption should you change your Merchant Login password.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'password', 0, 'payment_paypaladv', 4, 'paypaladv_Password', '', '', '', '', 'Password', 'Password you use to log in to your PayPal Manager account, or password you created for API calls')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypaladv', 1, 'paypaladv_Mode', 'Live', 'Live, Test', 'Live', '', '', 'Transaction Mode (Live or Test)')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypaladv', 2, 'paypaladv_Transaction_Type', 'Sale', 'Sale, Authorization', '', '', '', 'Transaction Type (Sale or Authorization only)')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypaladv', 3, 'paypaladv_Currency_Code', 'USD', 'AUD, BRL, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, ILS, JPY, MYR, MXN, NOK, NZD, PHP, PLN, SEK, SGD, THB, TRY, TWD, USD', 'USD', '', 'Currency Code', 'PayPal-Supported Currencies')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 1, 'payment_paypaladv', 4, 'paypaladv_Show_Instructions', '1', '1', '1', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypaladv', 1, 'paypaladv_Color_Header_Text', '', '', '', '', 'Header text color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypaladv', 2, 'paypaladv_Color_Border', '', '', '', '', 'Border color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypaladv', 3, 'paypaladv_Color_Field_Label', '', '', '', '', 'Field label color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypaladv', 4, 'paypaladv_Color_Button_Text', '', '', '', '', 'Button text color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypaladv', 5, 'paypaladv_Color_Button_Background', '', '', '', '', 'Button background color', '')");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1', `caption` = 'API Username', `description` = 'PayPal Payments Pro API Username' WHERE `name` = 'paypalpro_Username'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2', `caption` = 'API Password', `description` = 'PayPal Payments Pro API Password' WHERE `name` = 'paypalpro_Password'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_paypalpro', 3, 'paypalpro_Signature', '', '', '', '', 'API Signature', 'PayPal Payments Pro API Signature')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_paypalpro', 1, 'paypalpro_Mode', 'Live', 'Live, Test', 'Live', '', '', 'Select Payments Pro Checkout mode')");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2', `name` = 'paypalpro_Transaction_Type', `description` = 'When your buyer approves an authorization, the buyer\'s balance can be placed on hold for a 29-day period to ensure the availability of the authorization amount for capture. You can reauthorize a transaction only once, up to 115% of the originally authorized amount (not to exceed an increase of $75 USD).After a successful reauthorization, PayPal honors 100% of authorized funds for three days.You can settle without a reauthorization from day 4 to day 29 of the authorization period, but PayPal cannot ensure that 100% of the funds will be available after the three-day honor period. However, PayPal will not allow you to capture funds if the buyer\'s account is restricted, locked, or a fraudulent case occurs, or if your account has a high restriction level. You can use Authorization &amp; Capture only when your account has a low restriction level.' WHERE `name` = 'paypalpro_Payment_Action'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3', `name` = 'paypalpro_Currency_Code', `caption` = 'Currency Code', `description` = 'PayPal-Supported Currencies' WHERE `name` = 'paypalpro_code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4', `description` = '<table height=\"22\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tr><td width=\"20\"><img src=\"images/icons/ic_centinel.gif\"></td><td><a href=\"admin.php?p=payment_cardinal\" style=\"font-size:11px\"><b>Click here to setup Cardinal Centinel Module</b></a></td></tr></table>' WHERE `name` = 'paypalpro_Enable_Cardinal_Centinel'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1', `caption` = 'Email address for PayPal', `description` = 'Please enter the email address where you want to receive PayPal payments. You can set up your PayPal Business Account now or wait until you receive your first payment. <a href=\"https://www.paypal.com/cgi-bin/webscr?cmd=_simple-referral-flow&partner_id=PG8AZARNXLGBU&product_id=ECA\" target=\"_blank\">Open an account now</a>.' WHERE `name` = 'paypalec_Email'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1', `caption` = 'API Username', `description` = 'PayPal Express Checkout Username' WHERE `name` = 'paypalec_Username'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2', `caption` = 'API Password', `description` = 'PayPal Express Checkout Password' WHERE `name` = 'paypalec_Password'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'password', 1, 'payment_paypalec', 3, 'paypalec_Signature', '', '', '', '', 'API Signature', 'PayPal Express Checkout Signature')");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4', `description` = 'Select PayPal Express Checkout mode' WHERE `name` = 'paypalec_Mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '5', `name` = 'paypalec_Transaction_Type', `options` = 'Sale,Authorization,Order', `description` = 'When your buyer approves an authorization, the buyer\'s balance can be placed on hold for a 29-day period to ensure the availability of the authorization amount for capture. You can reauthorize a transaction only once, up to 115% of the originally authorized amount (not to exceed an increase of $75 USD).	An order period is 29-days long. Within this period, you can request an authorization to ensure the availability of funds. You can request an authorization on an order up to 10 times, up to 115% of the originally authorized amount (not to exceed an increase of $75 USD). Every successful authorization includes an honor period of 3 days.After a successful reauthorization, PayPal honors 100% of authorized funds for three days. You can settle without a reauthorization from day 4 to day 29 of the authorization period, but PayPal cannot ensure that 100% of the funds will be available after the three-day honor period. However, PayPal will not allow you to capture funds if the buyer\'s account is restricted, locked, or a fraudulent case occurs, or if your account has a high restriction level. You can use Authorization &amp; Capture only when your account has a low restriction level.' WHERE `name` = 'paypalec_Payment_Action'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '6', `name` = 'paypalec_Currency_Code', `options` = 'AUD, CAD, CHF, CZK, DKK, EUR, GBP, HKD, HUF, JPY, NOK, NZD, PLN, SEK, SGD, USD', `description` = 'PayPal-Supported Currencies' WHERE `name` = 'paypalec_currency'");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE `name` = 'paypalec_Exclusive'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalec', 1, 'paypalec_Border', '', '', '', '', 'Header Border color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'color', 2, 'payment_paypalec', 2, 'paypalec_Background', '', '', '', '', 'Header Background color', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'image', 2, 'payment_paypalec', 3, 'paypalec_Logo', '', '', '', '', 'Logo to be displayed on PayPal checkout', 'Maximum size for your business/logo graphic is 750 pixels wide by 90 pixels high<br/>Business/logo graphic format can be a gif, jpg or png')");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'psigate_Mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'psigate_Test_Result'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'vspform_Transaction_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'vspform_Currency'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'vspform_ApplyAVSCV2'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'datacash_method'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'datacash_currency'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'googlecheckout_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'googlecheckout_Auth_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'googlecheckout_Shipping_Price_On_Error'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'googlecheckout_request_buyer_phone_number'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '5' WHERE `name` = 'googlecheckout_google_checkout_button_background'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '6' WHERE `name` = 'googlecheckout_google_checkout_button_size'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '7' WHERE `name` = 'googlecheckout_your_location'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '8' WHERE `name` = 'googlecheckout_continue_shopping_link'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '9' WHERE `name` = 'googlecheckout_Callback_URL'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'moneris_Transaction_Mode'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'epay_pin'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'epay_Test_Request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'epay_Auth_Type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'qbms_test_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'qbms_transaction_type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'securehosting_gateway_configuration'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'securehosting_sh_reference'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '3' WHERE `name` = 'securehosting_checkcode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '4' WHERE `name` = 'securehosting_password'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'securehosting_test_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'securehosting_transaction_type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'ecsuite_test_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'ecsuite_transaction_type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'ecsuite_currency'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'securetrading_merchant_site_reference'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'securetrading_password'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '3' WHERE `name` = 'securetrading_merchant_email_address'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '4' WHERE `name` = 'securetrading_customer_email'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'securetrading_test_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'securetrading_form_reference'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'securetrading_callback_url_reference'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'securetrading_failure_url_reference'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '5' WHERE `name` = 'securetrading_path_to_images'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'payleap_Test_Mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'payleap_Transaction_Type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'amazon_checkout_access_key'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'amazon_checkout_merchant_id'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '3' WHERE `name` = 'amazon_checkout_secret_key'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'amazon_checkout_test_mode'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'amazon_sp_access_key'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'amazon_sp_secret_key'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'amazon_sp_test_mode'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'securepay_securexml_merchant_id'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'securepay_securexml_merchant_pw'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'securepay_securexml_test_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'securepay_securexml_transaction_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'securepay_securexml_fraudguard'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'securepay_securexml_order_prefix'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'directone_test'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '1' WHERE `name` = 'moneybookers_firstname'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '0', `priority` = '2' WHERE `name` = 'moneybookers_lastname'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'moneybookers_currency_code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'moneybookers_language'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'moneybookers_status_url2'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'moneybookers_shop_url'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'virtual_merchant_test_mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'virtual_merchant_site_form'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'virtual_merchant_avs'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'orbital_Test_Mode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'orbital_Message_Type'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '3' WHERE `name` = 'orbital_Currency_Code'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '4' WHERE `name` = 'orbital_Cust_Profiles'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'secure_net_Test_Request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'secure_net_Auth_Type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '1' WHERE `name` = 'payjunction_dc_test_request'");
	$db->query("UPDATE ".DB_PREFIX."settings SET `group_id` = '1', `priority` = '2' WHERE `name` = 'payjunction_dc_transaction_type'");

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.7.12', description='R1' WHERE name='AppVer'");
}

function upgrade3710r3()
{
	global $db, $settings;

	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."catalog_doba_map (doba_cid int(10) unsigned NOT NULL default '0', primary_cid int(10) unsigned NOT NULL default '0', categories text, subcats TINYINT(1) UNSIGNED NOT NULL DEFAULT 0, KEY doba_cid(doba_cid))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."offsite_campaigns (campaign_id int(10) unsigned NOT NULL auto_increment, name varchar(100) NOT NULL default '', description varchar(250) NOT NULL default '', type varchar(24) not null default '', created datetime NOT NULL default '0000-00-00 00:00:00', PRIMARY KEY (campaign_id))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."admin_oauth (uid int not null, oauth_token varchar(100) not null, oauth_token_secret varchar(100) not null, realm_id varchar(100), data_source varchar(50))");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."intuit_sync (id varchar(50) not null, type varchar(10) not null, intuit_id varchar(50) not null, last_sync_date datetime null )");

	/**
	 * admins table schema changes
	 */
	$columns = columnCheck('admins');
	if (!in_array('admins.expires', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN expires TINYINT(1) unsigned NOT NULL default '0' AFTER last_update");
	if (!in_array('admins.expiration_date', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN expiration_date datetime NOT NULL default '0000-00-00 00:00:00' AFTER expires");
	if (!in_array('admins.preferences', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN preferences TEXT NOT NULL default '' AFTER display_quickstart");
	if (!in_array('admins.openid', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN openid VARCHAR(250) NOT NULL default '' AFTER preferences");
	if (!in_array('admins.openid_provider', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."admins ADD COLUMN openid_provider VARCHAR(250) NOT NULL default '' AFTER openid");

	$db->query("UPDATE ".DB_PREFIX."countries SET iso_a2 = '' WHERE name = 'Madeira'");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."countries VALUES (null, 'Yes', 'No', 'No', 'No', '0.00000', '0', 'ME', 'MNE', '499', 'Republic of Montenegro')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."countries VALUES (null, 'Yes', 'No', 'No', 'No', '0.00000', '0', 'RS', 'SRB', '688', 'Republic of Serbia');");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."design_elements VALUES (null, 'image', 'Yes', 'image-logo-mobile', 'image-logo-mobile', '', '', 'image', 'content/cache/skins/".$settings['DesignSkinName']."/images/image-logo-mobile.png', null)");

	$db->query("ALTER TABLE ".DB_PREFIX."hosts CHANGE COLUMN `ip` `ip` int(10) UNSIGNED NOT NULL DEFAULT '0'");

	/**
	 * shipping_selected table schema changes
	 */
	$columns = columnCheck('shipping_selected');
	if (!in_array('shipping_selected.method_calc_mode', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."shipping_selected ADD COLUMN method_calc_mode ENUM('before_discount', 'after_discount') NOT NULL default 'before_discount' AFTER method_id");

	/**
	 * users table schema changes
	 */
	$columns = columnCheck('users');
	if (!in_array('users.facebook_user', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN facebook_user tinyint(1) unsigned NOT NULL default 0 AFTER last_update");
	if (!in_array('users.facebook_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN facebook_id varchar(128) NOT NULL default '' AFTER facebook_user");

	/**
	 * orders table schema changes
	 */
	$columns = columnCheck('orders');
	if (!in_array('orders.order_source', $columns))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN order_source varchar(20) NOT NULL DEFAULT 0 AFTER is_mobile");
		$db->query("UPDATE ".DB_PREFIX."orders SET order_source = 'qr_codes' WHERE is_qr <> 0");
	}
	if (!in_array('orders.offsite_campaign_id', $columns))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN offsite_campaign_id int(10) UNSIGNED NOT NULL DEFAULT 0 AFTER order_source");
		$db->query("UPDATE ".DB_PREFIX."orders SET offsite_campaign_id = qr_campaign_id WHERE qr_campaign_id <> 0");
	}
	if (in_array('orders.is_qr', $columns))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders DROP COLUMN is_qr");
	}
	if (in_array('orders.qr_campaign_id', $columns))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders DROP COLUMN qr_campaign_id");
	}

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."offsite_campaigns (campaign_id, name, description, created)
SELECT campaign_id, name, description, created
FROM ".DB_PREFIX."qr_campaigns");

	/**
	 * Securenet payment gateway
	 */
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods (active, proto, url_to_gateway, type, priority, id, class, version, gateway_name, name, title, enable_ccs, message_payment, message_thankyou) VALUES ('No','https','','cc','1','secure_net','PAYMENT_SECURE_NET','4.1','SecureNet','SecureNet','Payment with SecureNet',0,'','')");

	/**
	 * Beanstream payment gateway
	 */
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods (active, proto, url_to_gateway, type, priority, id, class, version, gateway_name, name, title, enable_ccs, message_payment, message_thankyou) VALUES ('No', 'https', 'https://www.beanstream.com/scripts/process_transaction.asp', 'cc', 2, 'beanstream', 'PAYMENT_BEANSTREAM', '5.7', 'Beanstream', 'Beanstream', 'Credit Card Payment with BeanStream', 0, '<u>Enter Credit Card Information</u>', '<b>Your Order Has Been Sent</b>')");
	$beanstreamId = $db->insertId();
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'Yes', 'VISA', 'Visa', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'Yes', 'MC', 'Master Card', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'Yes', 'AMEX', 'American Express', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'DICL', 'Diners Club', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'JCB', 'JCB', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'CARTE', 'Carte Blanche', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'ABC', 'Australian BankCard', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'DINO', 'Discover/Novus', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'SW', 'Maestro', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'SO', 'Solo', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'ERT', 'enRoute', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$beanstreamId.", 'No', 'UKE', 'Visa Electron', 'No')");

	/**
	 * statistics table schema changes
	 */
	$db->query("ALTER TABLE ".DB_PREFIX."statistics CHANGE COLUMN `page` `page` enum('product', 'catalog', 'qr', 'wg', 'other') NOT NULL default 'other'");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogManufacturerImageDisplay', 'YES', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExBillingAccountNumber', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExFreightAccountNumber', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExApiKey', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingFedExApiPassword', '', '', '', '', '', '')");

	if (isset($settings['ShippingCPUrlRating']) && $settings['ShippingCPUrlRating'] == 'http://206.191.4.228:30000/')
		$db->query("UPDATE ".DB_PREFIX."settings SET value = 'http://sellonline.canadapost.ca:30000/' WHERE name = 'ShippingCPUrlRating'");
	
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'ShippingCPUrlTracking'");

	if (isset($settings['LocalizationCurrencySeparatingSymbol']) && $settings['LocalizationCurrencySeparatingSymbol'] == ', ')
		$db->query("UPDATE ".DB_PREFIX."settings SET value = ',' WHERE name = 'LocalizationCurrencySeparatingSymbol' AND value = ', '");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'text', 3, 'Global Site Settings', 6, 'GlobalHttpCookieDomain', '', '', '', '', '', 'Advanced: keep it empty OR set your cookie domain for HTTP site like .mycart.com')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'text', 3, 'Global Site Settings', 7, 'GlobalHttpsCookieDomain', '', '', '', '', '', 'Advanced: keep it empty OR set your cookie domain for HTTP site like .mycart.sharedssl.com')");

	$db->query("UPDATE ".DB_PREFIX."settings set priority = 8 WHERE name = 'GlobalServerPath'");
	$db->query("UPDATE ".DB_PREFIX."settings set priority = 9, description = 'The permission modification to set when writing to files. Value must be a valid octal (Default is 0755).' where name = 'FilePermissionMode'");
	$db->query("UPDATE ".DB_PREFIX."settings set priority = 10, description = 'The permission modification to set when creating directories. Value must be a valid octal (Default is 0755).' where name = 'DirectoryPermissionMode'");

	if (!isset($settings['session_save_path']))
		$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'text', 3, 'Global Site Settings', 11, 'session_save_path', '', '', '', '', 'Session Save Path', 'Enter the full path of the session save path<br/>Note: This should be left blank to use the default.')");

	$db->query("UPDATE ".DB_PREFIX."settings SET description = 'Valid range 0 to 3000000000' WHERE name = 'MinOrderNumber'");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'Social Networks', 0, 'facebookAppSecret', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'Social Networks', 0, 'facebookLogin', 'No', 'Yes,No', 'No', '', '', '')");

	$newMobileThemeValue = 'Black';
	switch ($settings['MobileSiteTheme'])
	{
		case 'A':
			$newMobileThemeValue = 'Black';
			break;
		case 'B':
			$newMobileThemeValue = 'Blue';
			break;
		case 'C':
			$newMobileThemeValue = 'Light Gray';
			break;
		case 'D':
			$newMobileThemeValue = 'Dark Gray';
			break;
		case 'E':
			$newMobileThemeValue = 'Yellow';
			break;
	}
	$db->query("UPDATE ".DB_PREFIX."settings SET value = '".$newMobileThemeValue."', options = 'Black,Blue,Light Gray,Dark Gray,Yellow', default_value = 'Black' WHERE name = 'MobileSiteTheme'");

	$db->query("UPDATE ".DB_PREFIX."settings SET default_value = 'NO' WHERE name = 'CatalogBestsellersAvailable'");
	$db->query("UPDATE ".DB_PREFIX."settings SET default_value = 'NO' WHERE name = 'CustomerAlsoBoughtAvailable'");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_authorizenet', 7, 'authorizenet_Duplicate_Window', '120', '0,30,60,120,300', '120', '', '', 'Any transaction submitted with the same card number within specified period of time (in seconds) will be considered a duplicate transaction. Set to 0 to disable.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_authorizenet_aim_3_1', 6, 'authorizenet_aim_3_1_Duplicate_Window', '120', '0,30,60,120,300', '120', '', '', 'Any transaction submitted with the same card number within specified period of time (in seconds) will be considered a duplicate transaction. Set to 0 to disable.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 0, 'payment_authorizenet_cp_1_0', 6, 'authorizene_cp_1_0_Duplicate_Window', '120', '0,30,60,120,300', '120', '', '', 'Any transaction submitted with the same card number within specified period of time (in seconds) will be considered a duplicate transaction. Set to 0 to disable.')");

	$db->query("UPDATE ".DB_PREFIX."settings SET input_type = 'password' WHERE name = 'payleap_Transaction_Key'");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_active','No','Yes,No','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_type','product','product,cart','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_image_type','Product','Product,Thumb,None','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_style','1','1,2,3','','','','')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_anywhere_enabled','','Yes,No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_anywhere_cron_enabled','No','Yes,No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_anywhere_connected','No','Yes,No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_certificate','No','Yes,No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_private_key','No','Yes,No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_anywhere_custom1','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit Anywhere','intuit_anywhere_custom2','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_expense_account','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_expense_account_domain','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_income_account','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_income_account_domain','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_force_clean_sync','No','Yes,No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_nontaxable_tax_code','Non','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_taxable_tax_code','Tax','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_shipping_product_id','shipping','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_shipping_product_id_domain','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_handling_product_id','handling','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_handling_product_id_domain','handling','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_discount_product_id','discount','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_discount_product_id_domain','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_promo_code_product_id','promo_code','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_promo_code_product_id_domain','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_zero_tax_rate_name','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','Intuit IDS Settings','intuit_default_tax_rate_name','','')");

	if (defined('IS_INTUIT_FEDERATED') && IS_INTUIT_FEDERATED)
	{
		$db->query("UPDATE ".DB_PREFIX."settings SET value = 'Yes' WHERE name = 'intuit_force_clean_sync'");
		$db->query("UPDATE ".DB_PREFIX."settings SET value = '' WHERE name = 'intuit_sync_date'");
	}

	$db->query("UPDATE ".DB_PREFIX."settings SET input_type = 'password' WHERE name = 'amazon_sp_secret_key'");
	$db->query("UPDATE ".DB_PREFIX."settings SET input_type = 'password' WHERE name = 'securepay_securexml_merchant_pw'");
	$db->query("UPDATE ".DB_PREFIX."settings SET input_type = 'password' WHERE name = 'moneybookers_secret_word'");
	$db->query("UPDATE ".DB_PREFIX."settings SET input_type = 'password' WHERE name = 'virtual_merchant_pin'");
	$db->query("UPDATE ".DB_PREFIX."settings SET input_type = 'password' WHERE name = 'orbital_Connection_Password'");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', '0', 'payment_secure_net', '1', 'secure_net_Group_Id', '', '', '', '', 'Group ID', 'SecureNet Group ID. Not applicable to all merchants, should be provided by SecureNet. Please contact SecureNet for more information')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', '0', 'payment_secure_net', '2', 'secure_net_SecureNet_Id', '', '', '', '', 'SecureNet ID', '7-digit unique SecureNet ID provided by SecureNet')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'password', '0', 'payment_secure_net', '3', 'secure_net_SecureNet_Key', '', '', '', '', 'SecureNet Key', 'Case-sensitive unique SecureKey is associated with your ID')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'select', 0, 'payment_secure_net', '4', 'secure_net_Test_Request', 'True', 'True, False', 'False', '', '', 'Processing mode (TRUE for test mode and FALSE for real processing)')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'select', 0, 'payment_secure_net', '5', 'secure_net_Auth_Type', 'Auth-Capture', 'Auth-Capture, Auth-Only', 'Auth-Capture', '', '', 'Auth-Capture is the normal transaction method; a transaction is sent to the system for approval, the transaction is approved, the merchant is notified of the approval, and the transaction automatically settles at the end of the business day without any further action by the merchant. Auth-Only stands for Authorization-Only and means obtaining an authorization for a certain amount on a customer\'s credit card without actually charging the card. If the money is not captured within 30 days, the transaction will expire.')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, 'payment_beanstream', '1', 'beanstream_Merchant_Id', '', '', '', '', 'Merchant ID', 'BeanStream Merchant ID')");

	//TODO: How to map fedex values?
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_express_us', 'FedEx Express U.S. Shipping', 'FIRST_OVERNIGHT', 'FedEx First Overnight', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_express_us', 'FedEx Express U.S. Shipping', 'PRIORITY_OVERNIGHT', 'FedEx Priority Overnight', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_express_us', 'FedEx Express U.S. Shipping', 'STANDARD_OVERNIGHT', 'FedEx Standard Overnight', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_express_us', 'FedEx Express U.S. Shipping', 'FEDEX_2_DAY', 'FedEx 2Day', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_express_us', 'FedEx Express U.S. Shipping', 'FEDEX_2_DAY_AM', 'FedEx 2Day A.M.', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_express_us', 'FedEx Express U.S. Shipping', 'FEDEX_EXPRESS_SAVER', 'FedEx Express Saver', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_freight_us', 'FedEx Freight U.S. Shipping', 'FEDEX_FIRST_FREIGHT', 'FedEx First Overnight Freight', '1', 0, 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_freight_us', 'FedEx Freight U.S. Shipping', 'FEDEX_FREIGHT_PRIORITY', 'FedEx Freight Priority', '1', 0, 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_freight_us', 'FedEx Freight U.S. Shipping', 'FEDEX_FREIGHT_ECONOMY', 'FedEx Freight Economy', '1', 0, 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_freight_us', 'FedEx Freight U.S. Shipping', 'FEDEX_1_DAY_FREIGHT', 'FedEx 1Day Freight', '1', 0, 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_freight_us', 'FedEx Freight U.S. Shipping', 'FEDEX_2_DAY_FREIGHT', 'FedEx 2Day Freight', '1', 0, 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_freight_us', 'FedEx Freight U.S. Shipping', 'FEDEX_3_DAY_FREIGHT', 'FedEx 3Day Freight', '1', 0, 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_ground_us', 'FedEx Ground U.S. Shipping', 'FEDEX_GROUND', 'FedEx Ground', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_ground_us', 'FedEx Ground U.S. Shipping', 'GROUND_HOME_DELIVERY', 'FedEx Ground Home Delivery', '1', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr', 'FedEx Express International Shipping', 'EUROPE_FIRST_INTERNATIONAL_PRIORITY', 'FedEx Europe First', '13,21,55,56,70,71,77,80,93,98,100,117,119,142,154,163,165,182,183,186,202,203,226', 0, 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr', 'FedEx Express International Shipping', 'INTERNATIONAL_ECONOMY', 'FedEx International Economy', 0, '1', 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr', 'FedEx Express International Shipping', 'INTERNATIONAL_INTERNATIONAL_ECONOMY_DISTRIBUTION', 'FedEx International Economy Distribution', 0, '1', 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr', 'FedEx Express International Shipping', 'INTERNATIONAL_FIRST', 'FedEx International First', 0, '1', 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr', 'FedEx Express International Shipping', 'INTERNATIONAL_PRIORITY', 'FedEx International Priority', 0, '1', 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr', 'FedEx Express International Shipping', 'INTERNATIONAL_PRIORITY_DISTRIBUTION', 'FedEx International Priority Distribution', 0, '1', 0, '0.01', '150.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr_freight', 'FedEx Freight International Shipping', 'INTERNATIONAL_ECONOMY_FREIGHT', 'FedEx International Freight Priority', 0, '1', 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr_freight', 'FedEx Freight International Shipping', 'INTERNATIONAL_PRIORITY_FREIGHT', 'FedEx International Freight Economy', 0, '1', 0, '150.00', '2200.00', 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default VALUES (null, 'fedex_api_intr_freight', 'FedEx Freight International Shipping', 'INTERNATIONAL_DISTRIBUTION_FREIGHT', 'FedEx International Distribution Freight', 0, '1', 0, '150.00', '2200.00', 'lbs')");

	/**
	 * Remove viaKlix payment gateway
	 */
	$db->query("SELECT COUNT(*) FROM ".DB_PREFIX."payment_methods WHERE pid = 12 AND id = 'viaklix' AND active = 'No'");
	$row = $db->moveNext();
	if ($row[0] == 1)
	{
		$db->query("DELETE FROM ".DB_PREFIX."payment_accepted_cc WHERE pid = 12");
		$db->query("DELETE FROM ".DB_PREFIX."payment_methods WHERE pid = 12");
		$db->query("DELETE FROM ".DB_PREFIX."settings WHERE group_name in ('payment_viaklix')");
	}

	$db->query("SELECT COUNT(*) FROM ".DB_PREFIX."states WHERE short_name = 'PR' and coid = 1");
	$row = $db->moveNext();
	if ($row[0] == 1)
	{
		$db->query("INSERT INTO ".DB_PREFIX."states VALUES (NULL, 1, 'No', 'No', '0.00000', 'Puerto Rico', 'PR')");
	}
	
	// WholesaleCentral Settings
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'WholesaleCentral', 0, 'WholesaleCentralUsername', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'WholesaleCentral', 0, 'WholesaleCentralPassword', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'WholesaleCentral', 0, 'WholesaleCentralEnableCron', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'WholesaleCentral', 0, 'WholesaleCentralUseOverviewDescription', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'WholesaleCentral', 0, 'WholesaleCentralLastUpload', '', '', '', '', '', '')");
	
	// Social Network Settings changess
	$db->query("UPDATE `".DB_PREFIX."settings` SET `visible`='No', `priority`=0, `group_id`=0, `caption`='', `description`='' WHERE group_id=100");
}

function upgrade379r3()
{
	global $db, $settings;
	
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."qr_campaigns (campaign_id int(10) unsigned NOT NULL auto_increment, name varchar(100) NOT NULL default '', description varchar(250) NOT NULL default '', created datetime NOT NULL default '0000-00-00 00:00:00', PRIMARY KEY (campaign_id))");
	
	/**
	 * catalog table schema changes
	 */
	$columns = columnCheck('catalog');
	if (!in_array('catalog.list_subcats_images', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."catalog ADD COLUMN list_subcats_images tinyint(1) default 0 AFTER list_subcats");
	
	/**
	 * orders table schema changes
	 */
	$columns = columnCheck('orders');
	if (!in_array('orders.handling_fee_when_free_shipping', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN handling_fee_when_free_shipping decimal(20, 5) unsigned NOT NULL default '0.00000' AFTER handling_fee_type");
	if (!in_array('orders.insureship_policy_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN insureship_policy_id varchar(32) NOT NULL default '' AFTER insureship_amount");
	if (!in_array('orders.is_mobile', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN is_mobile tinyint(1) unsigned NOT NULL default 0 AFTER total_amount");
	if (!in_array('orders.is_qr', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN is_qr tinyint(1) unsigned NOT NULL default 0 AFTER is_mobile");
	if (!in_array('orders.qr_campaign_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN qr_campaign_id int(10) UNSIGNED NOT NULL default 0 AFTER is_qr");
	
	/**
	 * products table schema changes
	 */
	$columns = columnCheck('products');
	if (!in_array('products.google_online_only', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_online_only varchar(255) NOT NULL default '' AFTER google_product_type");
	if (!in_array('products.google_availability', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_availability varchar(255) NOT NULL default '' AFTER google_online_only");
	if (!in_array('products.google_gender', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_gender varchar(255) NOT NULL default '' AFTER google_availability");
	if (!in_array('products.google_color', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_color varchar(255) NOT NULL default '' AFTER google_gender");
	if (!in_array('products.google_material', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_material varchar(255) NOT NULL default '' AFTER google_color");
	if (!in_array('products.google_pattern', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_pattern varchar(255) NOT NULL default '' AFTER google_material");
	if (!in_array('products.google_size', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_size varchar(255) NOT NULL default '' AFTER google_pattern");
	if (!in_array('products.google_age_group', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_age_group varchar(255) NOT NULL default '' AFTER google_size");
	
	/**
	 * statistics table schema changes
	 */
	$db->query("ALTER TABLE ".DB_PREFIX."statistics CHANGE COLUMN `page` `page` enum('product', 'catalog', 'qr', 'other') NOT NULL default 'other'");
	
	/**
	 * New settings/records
	 */
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."design_elements VALUES (null, 'button', 'Yes', 'button-apply-promo', 'button-apply-promo', '', '', 'image', '', null)");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogCategoryImageSize', '300', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogCategoryImageType', 'Proportionate', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogManufacturerImageSize', '300', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogManufacturerImageType', 'Proportionate', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogCategoryThumbAuto', 'YES', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogDisplaySort', 'YES', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogDisplayView', 'YES', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Catalog Settings', 0, 'CatalogDisplayItems', 'YES', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingHandlingFeeWhenFreeShipping', '0.00', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingAllowSeparateAddress', '1', '', '1', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Shipping', 0, 'InsureShipApiURL', 'https://api.insureship.com/api/post_test/', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'Search Engines Settings', 0, 'SearchHtaccessOverrides', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, '', 0, 'LocalizationUseSystemTimezone', 'Yes', 'Yes,No', 'Yes', '', '','If set to Yes, all dates will be stored in the database using the system\'s timezone')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, '', 0, 'LocalizationDefaultTimezone', '', '', '', 'string', '', 'Timezone used when storing dates in the database. This setting is not retroactive.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'select', 4, 'Order/Cart Settings', 11, 'AlwaysDisplayTermsAndConditionsCheckbox', 'No', 'Yes,No', 'No', '', 'Always display Terms &amp; Conditions Checkbox?', 'Would you like your customers to be agree with your site terms and conditions on every checkout?')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'select', 100, 'Social Networks', 35, 'googlePlusButtonProduct', 'No', 'Yes,No', 'No', '', 'Post to Google Plus from product page', 'Help people share stuff from your website in Google Plus.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'qr_codes',0,'qr_active','No','Yes,No','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'qr_codes',0,'qr_media','1','0,1,2,3,4,5,6','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'qr_codes',0,'qr_error_check','L','L,M,Q,H','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'qr_codes',0,'qr_default_size','4','4,5,6,7,8,9,10,11,12','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'qr_codes',0,'qr_image_type','3','3,2,1','','','','')");
	
	/**
	 * Update existing settings/records
	 */
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET gateway_name = 'Intuit Payment Services', name = 'Intuit Payment Services' WHERE id = 'qbms' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET title = 'Credit Card Payments with Intuit Payment Services' WHERE id = 'qbms' AND title = 'Credit Card Payments with QuickBooks Merchant Services' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."settings SET description = 'Please enter the email address where you want to receive PayPal payments. You can set up your PayPal Business Account now or wait until you receive your first payment. <a href=\"https://www.paypal.com/cgi-bin/webscr?cmd=_simple-referral-flow&partner_id=PG8AZARNXLGBU&product_id=ECA\" target=\"_blank\">Open an account now</a>.<br/><span style=\"color:red;\">Note: Only use the email field when you are not using the api method (username / password).  When email is populated, the system will only process orders as Sale.</span>' WHERE name = 'paypalec_Email' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."settings SET value = 'pinnaclecartapp.pinnaclecart.com' WHERE name = 'qbms_app_login' AND value = '' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."settings SET value = '213404560' WHERE name = 'qbms_app_id' AND value = '' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."settings SET validation = 'ordernum' WHERE name = 'MinOrderNumber' LIMIT 1");
	
	
	$result = $db->query("DESC ".DB_PREFIX."products product_id");
	if ($product_id_col = $db->moveNext())
	{
		if ($product_id_col['Key'] == '')
		{
			$db->query("ALTER TABLE ".DB_PREFIX."products ADD KEY product_id (product_id)");
		}
	}
	
	if ($settings['CatalogCategoryThumbType'] == 'Manual')
	{
		$db->query("UPDATE ".DB_PREFIX."settings SET value = 'NO' WHERE name = 'CatalogCategoryThumbAuto' LIMIT 1");
	}
}

function upgrade379r1()
{
	global $db, $settings;
	
	$filePermissionMode = $settings['FilePermissionMode'];
	$directoryPermissionMode = $settings['DirectoryPermissionMode'];
	
	if (file_exists('../../content/cache/license/license.key'))
	{
		$filePermissionMode = getChmodMode('../../content/cache/license/license.key', true);
	}
	
	if (is_dir('../../content/cache'))
	{
		$directoryPermissionMode = getChmodMode('../../content/cache', true);
	}
	
	$db->query("UPDATE ".DB_PREFIX."settings SET value='".$filePermissionMode."' WHERE name='FilePermissionMode'");
	$db->query("UPDATE ".DB_PREFIX."settings SET value='".$directoryPermissionMode."' WHERE name='DirectoryPermissionMode'");
}

function upgrade378r800()
{
	global $db, $settings;
	
	$columns = array();
	
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 3, 'Global Site Settings', 6, 'FilePermissionMode', '0777', '', '', '', '', 'The permission modification to set when writing to files. Value must be a valid octal (Default is 0666).')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 3, 'Global Site Settings', 6, 'DirectoryPermissionMode', '0777', '', '', '', '', 'The permission modification to set when creating directories. Value must be a valid octal (Default is 0777).')");
	
	updateSettings(array(
		'FilePermissionMode' => '0777',
		'DirectoryPermissionMode' => '0777'
	));
	
	/**
	 * PRODUCTS TABLE FIELD CHANGES
	 */
	$columns = columnCheck('products');
	
	if (!in_array('products.google_product_type', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_product_type varchar(255) NOT NULL DEFAULT '' after google_item_condition");
}

function upgrade378r750()
{
	global $db, $settings;
	
	$columns = array();
	
	$db->query("ALTER TABLE ".DB_PREFIX."admins CHANGE display_quickstart display_quickstart int(1)  NOT NULL DEFAULT '0' after login_errors");
	$db->query("UPDATE ".DB_PREFIX."pages SET url_hash = '9d7f564aa5e716dbae961dd1f0c23d30', url_default = 'about-us'  WHERE url_hash = '' AND url_default = '' AND name = 'about_us' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."pages SET url_hash = 'd7ad2fdac3cb7a66c4cdafba168bbd44', url_default = 'terms-and-conditions'  WHERE url_hash = '' AND url_default = '' AND name = 'terms_and_conditions' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."pages SET url_hash = '03f2197d47a602c679c5f667e3482855', url_default = 'contact-us'  WHERE url_hash = '' AND url_default = '' AND name = 'contact_us' LIMIT 1");
	$db->query("UPDATE ".DB_PREFIX."payment_methods SET gateway_name = 'First Data / LinkPoint.com' WHERE id = 'linkpoint' LIMIT 1");
	
	/**
	 * MANUFACTURERS TABLE FIELD CHANGES
	 */
	$columns = columnCheck('manufacturers');
	
	if (!in_array('manufacturers.url_hash', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."manufacturers ADD COLUMN url_hash varchar(32) NOT NULL DEFAULT '' after is_visible");
	
	if (!in_array('manufacturers.url_default', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."manufacturers ADD COLUMN url_default varchar(128) NOT NULL DEFAULT '' after url_hash");
	
	if (!in_array('manufacturers.url_custom', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."manufacturers ADD COLUMN url_custom varchar(128) NOT NULL DEFAULT '' after url_default");
	
	
	/**
	 * PRODUCTS_LOCATIONS TABLE FIELD CHANGES
	 */
	$columns = columnCheck('products_locations');
	
	if (!in_array('products_locations.email_type', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products_locations ADD COLUMN email_type enum('html','text') NOT NULL DEFAULT 'html' after notify_email");

	
	/**
	 * ORDERS_CONTENT TABLE FIELD CHANGES
	 */
	$columns = columnCheck('orders_content');
	
	if (!in_array('orders_content.price_before_quantity_discount', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN price_before_quantity_discount decimal(20,5) unsigned NOT NULL DEFAULT '0.00000' after cost");
		
	if (!in_array('orders_content.product_gtin', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN product_gtin varchar(64) NOT NULL DEFAULT '' after product_upc");
		
	if (!in_array('orders_content.product_mpn', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN product_mpn varchar(64) NOT NULL DEFAULT '' after product_gtin");


	/**
	 * PRODUCTS TABLE FIELD CHANGES
	 */
	$columns = columnCheck('products');
	
	if (!in_array('products.estimated_shipping_price', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN estimated_shipping_price decimal(20,5) NOT NULL DEFAULT '-1.00000' after shipping_price");
	
	if (!in_array('products.product_gtin', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN product_gtin varchar(64) NOT NULL DEFAULT '' after product_upc");
		
	if (!in_array('products.product_mpn', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN product_mpn varchar(64) NOT NULL DEFAULT '' after product_gtin");

	$db->query("UPDATE ".DB_PREFIX."settings SET value = '<script type=\"text/javascript\">\r\n  var _gaq = _gaq || [];\r\n  _gaq.push([''_setAccount'', ''UA-XXXXX-X'']);\r\n  _gaq.push([''_trackPageview'']);\r\n  (function() {\r\n    var ga = document.createElement(''script''); ga.type = ''text/javascript''; ga.async = true;\r\n    ga.src = (''https:'' == document.location.protocol ? ''https://ssl'' : ''http://www'') + ''.google-analytics.com/ga.js'';\r\n    var s = document.getElementsByTagName(''script'')[0]; s.parentNode.insertBefore(ga, s);\r\n  })();\r\n</script>\r\n' WHERE name = 'GoogleAnalytics' AND value = '<script type=\"text/javascript\">\r\nvar gaJsHost = ((\"https:\" == document.location.protocol) ? \"https://ssl.\" : \"http://www.\");\r\ndocument.write(unescape(\"%3Cscript src=''\" + gaJsHost + \"google-analytics.com/ga.js'' type=''text/javascript''%3E%3C/script%3E\"));\r\n</script>\r\n<script type=\"text/javascript\">\r\ntry {\r\nvar pageTracker = _gat._getTracker(\"UA-xxxxx-x\");\r\npageTracker._trackPageview();\r\n} catch(err) {}</script>' LIMIT 1");

	/**
	 * USERS TABLE FIELD CHANGES
	 */
	$columns = columnCheck('users');
	
	if (!in_array('users.last_update', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN last_update datetime NOT NULL DEFAULT '0000-00-00 00:00:00' after phone");
}

function upgrade377r350()
{
	global $db, $settings;
	
	$columns = array();
	
	/**
	 * USERS TABLE FIELD CHANGES
	 */
	$columns = columnCheck('users');
	if (!in_array('users.created_date', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN created_date datetime NOT NULL default '0000-00-00 00:00:00' AFTER uid");
	
	
	// update the users table and assign a creation date based on initial order date
	$temp_table = DB_PREFIX.'tmp_update_'.md5(mktime().rand(1,5000));
	$db->query("CREATE TEMPORARY TABLE {$temp_table} SELECT uid, MIN(create_date) as create_date FROM ".DB_PREFIX."orders GROUP BY uid");
	$db->query("ALTER TABLE {$temp_table} ADD INDEX(`uid`);");
	$db->query("UPDATE ".DB_PREFIX."users u LEFT JOIN {$temp_table} t ON t.uid = u.uid SET u.created_date = t.create_date");
	$db->query("UPDATE ".DB_PREFIX."users SET created_date = NOW() WHERE created_date = '0000-00-00 00:00:00'");
	$db->query("DROP TABLE {$temp_table}");
	
	// delete the unused settings
	$db->query("DELETE FROM ".DB_PREFIX."payment_methods WHERE gateway_name = 'PinnProcessing' AND class = 'PAYMENT_PINNACLECART' AND id = 'pinnaclecart' LIMIT 1");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'pinnaclecart_store_name' LIMIT 1");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'pinnaclecart_notify_email' LIMIT 1");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'pinnaclecart_color_bg' LIMIT 1");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'pinnaclecart_color_text' LIMIT 1");
}

function upgrade376r220()
{
	global $db, $settings;
	
	$columns = array();
	
	$result = $db->selectAll("DESC ".DB_PREFIX."products");
	foreach ($result as $row)
	{
		$columns[] = 'products.'.$row['Field'];
	}
	
	/**
	 * PRODUCTS TABLE FIELD CHANGES
	 */
	if (!in_array('products.lock_fields', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN lock_fields set('category', 'name', 'price', 'sale-price', 'description', 'overview', 'meta-title', 'meta-keywords', 'meta-description', 'images') NOT NULL default '' AFTER zoom_option");
	
	
	$result = $db->selectAll("DESC ".DB_PREFIX."orders");
	foreach ($result as $row)
	{
		$columns[] = 'orders.'.$row['Field'];
	}
	
	/**
	 * ORDERS TABLE FIELD CHANGES
	 */
	if (!in_array('orders.shipping_ignore_free', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN shipping_ignore_free tinyint(1) unsigned NOT NULL default 0 AFTER shipping_tax_description");
	if (!in_array('orders.shipping_tracking', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN shipping_tracking text NOT NULL AFTER shipping_tracking_number_type");
	if (!in_array('orders.insureship_active', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN insureship_active tinyint(1) unsigned not null default 0 AFTER buysafe_bond_amount");
	if (!in_array('orders.insureship_amount', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN insureship_amount decimal(20, 5) unsigned NOT NULL default '0.00000' AFTER insureship_active");
	if (!in_array('orders.doba_order_funded', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN doba_order_funded tinyint(1) unsigned not null default '0' AFTER doba_order_sent");
	
	if (!isset($settings["facebookAppId"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'text', 100, 'Social Networks', 1, 'facebookAppId', '', '', '', '', 'Facebook application ID', 'To use some Facebook plug-ins, your site must have an application ID. You can get an application ID by registering your site here: http://developers.facebook.com/setup.')");
	if (!isset($settings["facebookLikeButtonProduct"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 100, 'Social Networks', 2, 'facebookLikeButtonProduct', 'No', 'Yes,No', 'No', '', 'Facebook Like Button on product page', 'The Like button lets users share product pages from your site back to their Facebook profile with one click.')");
	if (!isset($settings["facebookCommentsProduct"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 100, 'Social Networks', 3, 'facebookCommentsProduct', 'No', 'Yes,No', 'No', '', 'Facebook Comments on product page', 'The Comments plugin lets users comment on products on your site. Requires Facebook application ID')");
	if (!isset($settings["twitterAccount"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'text', 100, 'Social Networks', 11, 'twitterAccount', '', '', '', '', 'Twitter account', 'Twitter account for users to follow after they share content from your website')");
	if (!isset($settings["twitterTweetButtonProduct"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 100, 'Social Networks', 12, 'twitterTweetButtonProduct', 'No', 'Yes,No', 'No', '', 'Tweet button on product page', 'Add this button to your website to let people share content on Twitter without having to leave the page. Promote strategic Twitter accounts at the same time while driving traffic to your website')");
	if (!isset($settings["diggButtonProduct"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 100, 'Social Networks', 21, 'diggButtonProduct', 'No', 'Yes,No', 'No', '', 'Digg button on product page', 'The Digg button, also known as the ìDigg Thisî button, is for website and blog publishers that want to encourage their audience to submit content to Digg or Digg their content.')");
	if (!isset($settings["googleBuzzButtonProduct"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 100, 'Social Networks', 31, 'googleBuzzButtonProduct', 'No', 'Yes,No', 'No', '', 'Post to Google Buzz from product page', 'Help people share stuff from your website in Google Buzz.')");
	if (!isset($settings["deliciousButtonProduct"])) $db->query("INSERT INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 100, 'Social Networks', 41, 'deliciousButtonProduct', 'No', 'Yes,No', 'No', '', 'Bookmark with Delicious on product page', 'With a \"Bookmark this on Delicious\" link, you can provide your site visitors an easy way to save it to Delicious.')");
	
	// delete the unused settings
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name = 'ShippingAtProductLevel' LIMIT 1;");
}

function upgrade375r4110()
{
	global $db, $settings;
	
	$columns = array();
	
	$result = $db->selectAll("DESC ".DB_PREFIX."products");
	foreach ($result as $row)
	{
		$columns[] = 'products.'.$row['Field'];
	}
	
	// check reviews rating, reviews_count fields
	if (!in_array('products.rating', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN rating decimal(10,3) unsigned NOT NULL default '0' AFTER products_location_id");
		
	if (!in_array('products.reviews_count', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN reviews_count int unsigned NOT NULL default 0 AFTER rating");
		
	// begin varchar 300 check: checking for the following in case they were not inserted due to varchar 300 statement(s)
	if (!in_array('products.pricegrabber_category', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN pricegrabber_category varchar(255) NOT NULL default '' AFTER pricegrabber_part_number");
	if (!in_array('products.nextag_category', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN nextag_category varchar(255) NOT NULL default '' AFTER nextag_part_number");
	// end varchar 300 check
	
	
	// check the dollar days setting to see if they have been installed
	if (!array_key_exists('plugin_dollar_days_import_categories', $settings))
		$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_categories','','')");
		
	if (!array_key_exists('plugin_dollar_days_import_price_option', $settings))
		$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_price_option','inc','inc,dec')");
		
	if (!array_key_exists('plugin_dollar_days_import_price_value', $settings))
		$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_price_value','0.00','')");
		
	if (!array_key_exists('plugin_dollar_days_import_last_update', $settings))
		$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_last_update','','')");
	
	
	// update reCAPTCHA credentials
	if (array_key_exists('testimonials_Captcha_Public_Key', $settings))
		$db->query("UPDATE ".DB_PREFIX."settings SET value='".$settings['testimonials_Captcha_Public_Key']."' WHERE name='captchaReCaptchaPublicKey'");
		
	if (array_key_exists('testimonials_Captcha_Private_Key', $settings))
	$db->query("UPDATE ".DB_PREFIX."settings SET value='".$settings['testimonials_Captcha_Private_Key']."' WHERE name='captchaReCaptchaPrivateKey'");
	
	// delete the unused settings
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name='testimonials_Captcha_Public_Key' LIMIT 1");
	$db->query("DELETE FROM ".DB_PREFIX."settings WHERE name='testimonials_Captcha_Private_Key' LIMIT 1");
	
	
	
	$result = $db->selectAll("DESC ".DB_PREFIX."orders");
	foreach ($result as $row)
	{
		$columns[] = 'orders.'.$row['Field'];
	}
	
	
	// check for the new placed_date
	if (!in_array('orders.placed_date', $columns))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN placed_date datetime NOT NULL default '0000-00-00 00:00:00' AFTER create_date");
		$db->query("UPDATE ".DB_PREFIX."orders SET placed_date = create_date");
	}
	
	
	// check for the new key_name field in tax_classes table
	$result = $db->selectAll("DESC ".DB_PREFIX."tax_classes");
	foreach ($result as $row)
	{
		$columns[] = 'tax_classes.'.$row['Field'];
	}
	
	if (!in_array('tax_classes.key_name', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."tax_classes ADD COLUMN key_name varchar(255) NOT NULL default ''");
	
}

function upgrade375r4000()
{
	global $db;
	
	$columns = array();
	$result = $db->selectAll("DESC ".DB_PREFIX."products");
	foreach ($result as $row)
	{
		$columns[] = 'products.'.$row['Field'];
	}
	
	$result = $db->selectAll("DESC ".DB_PREFIX."products_images");
	foreach ($result as $row)
	{
		$columns[] = 'products_images.'.$row['Field'];
	}
	
	if (!in_array('products.ebay_cat_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN ebay_cat_id int(8) unsigned NOT NULL default '0'");
	
	if (!in_array('products.amazon_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN amazon_id varchar(64) NOT NULL default ''");
	
	if (!in_array('products.amazon_id_type', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN amazon_id_type int(3) unsigned NOT NULL default '0'");
		
	if (!in_array('products.amazon_item_condition', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN amazon_item_condition int(3) unsigned NOT NULL default '11'");
	
	if (!in_array('products.google_item_condition', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN google_item_condition varchar(50) NOT NULL default 'new'");
	
	if (!in_array('products.yahoo_path', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN yahoo_path varchar(255) NOT NULL default ''");
	
	if (!in_array('products.pricegrabber_item_condition', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN pricegrabber_item_condition varchar(50) NOT NULL default 'New'");
	
	if (!in_array('products.pricegrabber_part_number', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN pricegrabber_part_number varchar(50) NOT NULL default ''");
	
	if (!in_array('products.pricegrabber_category', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN pricegrabber_category varchar(255) NOT NULL default ''");
	
	if (!in_array('products.nextag_part_number', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN nextag_part_number varchar(50) NOT NULL default ''");
	
	if (!in_array('products.nextag_category', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN nextag_category varchar(255) NOT NULL default ''");
	
	if (!in_array('products.nextag_item_condition', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN nextag_item_condition varchar(50) NOT NULL default 'New'");
	
	if (!in_array('products_images.filename', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products_images ADD COLUMN filename varchar(255) NOT NULL default ''");
}


function getChmodMode($filepath, $refresh = false)
{
	static $_stats = array();
	
	if (!isset($_stats[$filepath]) || $refresh)
	{
		if ($refresh) clearstatcache();
		
		$_stats[$filepath] = stat($filepath);
	}
	
	return sprintf('0%o', 0777 & $_stats[$filepath]['mode']);
}

/**
 * Migrates the themes into the new folder
 *
 * @return void
 */
function migrateThemes()
{
	global $old_skin_name, $db;
	
	try
	{
		$skin_dir = '../../content/skins/';
		
		// read the skins directory and make the themes 3.7.4 compatible
		if ($skin_dir_handle = opendir($skin_dir))
		{
			while (false !== ($skin = readdir($skin_dir_handle)))
			{
				if (!in_array($skin, array('.', '..', '_custom')))
				{
					// create the themes directory
					if (!file_exists($skin_dir.$skin.'/styles/themes'))
					{
						mkdir($skin_dir.$skin.'/styles/themes', getDirectoryPermissionMode(), true);
						File_Utility::setDirectoryPermissions($skin_dir.$skin.'/styles/themes', getDirectoryPermissionMode());
					}
						
					
					// loop over the themes directory
					$old_themes_dir = $skin_dir.$skin.'/themes/';
					$new_themes_dir = $skin_dir.$skin.'/styles/themes';
					
					// copy the themes directory
					copy_dir($old_themes_dir, $new_themes_dir, array(
							'overwrite' => true
						));
					
					$files = traverse($new_themes_dir, true);
					
					foreach ($files as $filename)
					{
						chmod($filename, getFilePermissionMode());
					}
					
					// loop through the themes directory
					if ($theme_dir_handle = @opendir($new_themes_dir))
					{
						while (false !== ($theme = readdir($theme_dir_handle)))
						{
							if (!in_array($theme, array('.', '..', '_custom')))
							{
								$file_info = pathinfo($new_themes_dir.'/'.$theme);
								
								// check for actual file
								if (array_key_exists('extension', $file_info))
								{
									$theme_css = @file_get_contents($file_info['dirname'].'/'.$file_info['basename']);
									$theme_css = str_replace('('.$file_info['filename'], '(theme-'.$file_info['filename'], $theme_css);
									file_put_contents($file_info['dirname'].'/'.$file_info['basename'], $theme_css);
								}
								
								rename($file_info['dirname'].'/'.$file_info['basename'], $file_info['dirname'].'/theme-'.$file_info['basename']);
							}
						}
						
						closedir($theme_dir_handle);
					}
				}
			}
			
			closedir($skin_dir_handle);
		}
		else
		{
			throw new Exception('Cannot open the skins directory. Please ensure this update is being ran in the install/update directory.');
		}
		
		// get the current theme name
		$db->query("SELECT value FROM ".DB_PREFIX."settings WHERE name='DesignSkinTheme'");
		$db->moveNext();
		$current_theme_name = $db->col['value'];
		$new_theme_name = 'theme-'.str_replace('.css', '', $current_theme_name);
		
		
		$db->reset();
		$db->assignStr('value', $new_theme_name);
		$db->update(DB_PREFIX.'settings', "WHERE name = 'DesignSkinTheme'");
		
		$settings = settings();
		$settings['DesignSkinTheme'] = $new_theme_name;
	}
	catch(Exception $e)
	{
		die($e->getMessage());
	}
	
}


/**
 * Migrates logo/slogan and custom images (hot deal, print, etc)
 */
function migrateImages()
{
	// First before we make any database changes, we need to know the current language code they are using
	global $db, $new_skin_name, $settings;
	
	
	// create skin images folder in case it doesn't exist
	if (!file_exists('../../content/skins/_custom/skin/images'))
	{
		mkdir('../../content/skins/_custom/skin/images', getDirectoryPermissionMode(), true);
		File_Utility::setDirectoryPermissions('../../content/skins/_custom/skin/images', getDirectoryPermissionMode());
	}


	// get the current language for migrating buttons and images
	$db->query("SELECT code FROM ".DB_PREFIX."languages WHERE is_default = 'Yes' LIMIT 1");
	$db->moveNext();
	$current_language = $db->col['code'];


	// get all current design elements
	$db->query("SELECT * FROM ".DB_PREFIX."design_elements");
	$current_design_elements = $db->getObjects();
	
	
	foreach ($current_design_elements as $de)
	{
		
		$img = false;
		$extension = false;
		// loop through each design element image
		if(is_file("../../images/".$de->group_id."s/".$current_language."/".$de->element_id.".gif")){
			$img = "../../images/".$de->group_id."s/".$current_language."/".$de->element_id.".gif";
			$extension = '.gif';
		}
		elseif(is_file("../../images/".$de->group_id."s/".$current_language."/".$de->element_id.".jpg")){
			$img = "../../images/".$de->group_id."s/".$current_language."/".$de->element_id.".jpg";
			$extension = '.jpg';
		}
		elseif(is_file("../../images/".$de->group_id."s/".$current_language."/".$de->element_id.".png")){
			$img = "../../images/".$de->group_id."s/".$current_language."/".$de->element_id.".png";
			$extension = '.png';
		}
		
		if (false !== $img)
		{
			$element = false;
			switch ($de->element_id)
			{
				case 'imageLogo': $element = 'image-logo'; break;
				case 'imageLogoMobile': $element = 'image-logo-mobile'; break;
				case 'imageSlogan': $element = 'image-slogan'; break;
				case 'imageHome': $element = 'image-home'; break;
				case 'imageHotDeal': $element = 'image-hot-deal'; break;
				case 'imageNoImageSmall': $element = 'image-no-image-small'; break;
				case 'imageNoImageBig': $element = 'image-no-image-big'; break;
				case 'imageDeleteItem': $element = 'image-delete-item'; break;
				case 'imageCVV2': $element = 'image-cvv2'; break;
				case 'imageClickToView': $element = 'image-click-to-view'; break;
				case 'imagePrintVersion': $element = 'image-print-version'; break;
				case 'imageCart': $element = 'image-cart'; break;
				case 'imageGiftCertificate': $element = 'image-gift-certificate'; break;
			}
			
			if (false !== $element)
			{
				$filename = $element.$extension;
				
				$db_destination = 'content/cache/skins/'.escapeFileName($new_skin_name).'/images/'.$filename;
				$file_destination = '../../content/skins/_custom/skin/images/'.$filename;

				$db->reset();
				$db->assignStr('group_id', 'image');
				$db->assignStr('removable', 'Yes');
				$db->assignStr('removable', 'Yes');
				$db->assignStr('element_id', $element);
				$db->assignStr('name', $element);
				$db->assignStr('description', '');
				$db->assignStr('caption', '');
				$db->assignStr('type', 'image');
				$db->assignStr('content', $db_destination);
				$db->insert(DB_PREFIX.'design_elements');
				
				copy($img, $file_destination);
			}
		}
	}
}


function migrateCss()
{
	global $db, $old_skin_name, $new_skin_name;
	
	$db->query("SELECT value FROM ".DB_PREFIX."settings WHERE name = 'DesignSkinColorScheme' LIMIT 1");
	$db->moveNext();
	
	$conf_file = '../../content/skins/'.$old_skin_name.'/schemes/'.escapeFileName($db->col['value']).'.conf';
	
	if (!file_exists($conf_file)) return;
	
	$colors = parse_ini_file($conf_file);

	$css = array();

	foreach ($colors as $key => $value)
	{
		switch ($key)
		{
			case 'ColorBodyBackground': $css['.page']['background-color'] = $value; break;
			case 'ColorSiteBackground': $css['.page .container']['background-color'] = $value; break;
			case 'ColorSiteBorder': $css['.page .container']['border-color'] = $value; break;
			case 'ColorSiteTextColor': $css['.page .container']['color'] = $value; break;
			case 'ColorSiteLink': $css['.page .container a:link']['color'] = $value; break;
			case 'ColorSiteLinkHover': $css['.page .container a:hover']['color'] = $value; break;
			case 'ColorBarsTopBottomBackground':
				$css['.page .layout-zone-navigation']['background-color'] = $value;
				$css['.page .layout-zone-footer']['background-color'] = $value;
				break;
				
			case 'ColorBarsTopBottomText':
				$css['.page .layout-zone-navigation']['color'] = $value;
				$css['.page .layout-zone-footer']['color'] = $value;
				break;
				
			// panels border
			case 'ColorBoxFrame':
				$css['div.panel']['border-color'] = $value;
				$css['div.panel']['border-width'] = '1px';
				$css['div.panel']['border-style'] = 'solid';
				break;
			
			case 'ColorBoxHeaderBackground': $css['div.panel h4']['background-color'] = $value; break;
			case 'ColorBoxHeaderText': $css['div.panel h4']['color'] = $value; break;
			case 'ColorBoxBackground': $css['div.panel']['background-color'] = $value; break;
			case 'ColorBoxText': $css['div.panel .content']['color'] = $value; break;
			case 'ColorBoxLink': $css['div.panel .content a:link']['color'] = $value; break;
			case 'ColorBoxLinkHover': $css['div.panel .content a:hover']['color'] = $value; break;
			case 'ColorPageHeaderBackground': $css['.page .layout-zone-content h1']['background-color'] = $value; break;
			case 'ColorPageHeaderText': $css['.page .layout-zone-content h1']['color'] = $value; break;
			
			case 'ColorPageNavigationBackground': $css['.page-catalog .catalog-bread-crumbs']['background-color'] = $value; break;
			case 'ColorPageNavigationText': $css['.page-catalog .catalog-bread-crumbs']['color'] = $value; break;
			case 'ColorCatalogNavigationBackground': $css['.page-catalog .catalog-navigation']['background-color'] = $value; break;
			case 'ColorCatalogNavigationText': $css['.page-catalog .catalog-navigation']['color'] = $value; break;
			case 'ColorCatalogItemDarkBackground': $css['tr.even']['background-color'] = $value; break;
			case 'ColorCatalogItemDarkText': $css['tr.even']['color'] = $value; break;
			case 'ColorCatalogItemLightBackground': $css['tr.odd']['background-color'] = $value; break;
			case 'ColorCatalogItemLightText': $css['tr.odd']['background-color'] = $value; break;
			case 'ColorCatalogItemBorder': $css['.catalog-product']['border-color'] = $value; break;
			
			case 'ColorCatalogPrice': $css['.price']['color'] = $value; break;
			case 'ColorCatalogHotdeal': $css['.catalog-product-hot-deal span']['color'] = $value; break;
			case 'ColorFormHeaderBackground': $css['#content form h3']['background-color'] = $value; break;
			case 'ColorFormHeaderText': $css['#content form h3']['color'] = $value; break;
			case 'ColorFormItemsText': $css['#content form']['color'] = $value; break;
			case 'ColorFormCommentsText': $css['#content form .msg'][''] = $value; break;
			case 'ColorFormInput': $css['#content form input']['background-color'] = $value; break;
			case 'ColorFormSeparatorBackground': $css['#content form']['background-color'] = $value; break;
			case 'ColorFormSeparatorText': $css['#content form']['color'] = $value; break;
			case 'ColorFormBorder': $css['#content form']['border-color'] = $value; break;
		}
	}

	$css_string = '';

	foreach ($css as $selector => $properties)
	{

		$css_string .= "{$selector} {";
		foreach ($properties as $propname => $propvalue)
		{
			$css_string .= " {$propname}: {$propvalue};";
		}
		$css_string .= " }\n";
	}
	
	// Open the skin.css file for writing
	if (false !== ($fh = fopen('../../content/skins/'.escapeFileName($new_skin_name).'/default/default.css', 'a')))
	{
		if (false === @fwrite($fh, $css_string))
		{
			echo 'Please make sure ../../content/skins/'.escapeFileName($new_skin_name).'/styles/default.css is writable';
		}
		fclose($fh);
	}
}


function updateSecondaryImages(&$db)
{

	function __imageFileName($product_id)
	{
		$s = strtolower(trim($product_id));
		for($i=0; $i<strlen($s); $i++){
			if(in_array($s[$i], array("\\", "/", "*", ":", "?", "<", ">", "|", '"', "'")))$s[$i] = "_";
		}
		return $s;
	}
	//get images info from database
	$db->query("
		SELECT i.iid, i.type, p.product_id FROM ".DB_PREFIX."products_images AS i
		INNER JOIN ".DB_PREFIX."products AS p ON p.pid=i.pid
		ORDER BY p.product_id, i.iid
	");
	
	//build images array
	$images = array();
	while ($db->moveNext()) $images[] = $db->col;
	
	$lastProductId = "";
	$imageNum = 0;
	
	//rename files and update number file_name in db
	foreach ($images as $key=>$image)
	{
		//rename products secondary images to prod-id-1.jpg, prod-id-2.jpg, etc
		if ($lastProductId != $image["product_id"])
		{
			$lastProductId = $image["product_id"];
			$imageNum = 1;
		}
		else
		{
			$imageNum = $imageNum + 1;
		}
		
		//check does image file exist
		$imageFile = "../../images/products/secondary/".$image["iid"].".".$image["type"];
		if (is_file($imageFile))
		{
			$newImageFile = "../../images/products/secondary/".__imageFileName($image["product_id"])."-".$imageNum.".".$image["type"];
			@rename($imageFile, $newImageFile);
			$db->query("UPDATE ".DB_PREFIX."products_images SET filename='".addslashes(__imageFileName($image["product_id"])."-".$imageNum)."' WHERE iid='".$image["iid"]."'");
		}
	}
}


function googleCheckoutRetroFix()
{
	global $db;
	
	$db->query('SELECT * FROM '.DB_PREFIX.'payment_transactions WHERE oid IN (SELECT oid FROM '.DB_PREFIX.'orders WHERE uid = 0 AND payment_gateway_id = "googlecheckout" AND status = "Process") AND payment_response LIKE "risk-information-notification%"');

	$emails = array();
	$names = array();

	while ($db->moveNext())
	{
		$response = preg_split("/\\n/", $db->col['payment_response']);
		foreach ($response as $line)
		{
			if (strstr($line, '=>'))
			{
				list($key, $value) = preg_split("/=>/", $line);

				if ($key == 'email') $emails[$db->col['oid']] = trim($value);
				if ($key == 'contact-name') $names[$db->col['oid']] = trim($value);
			}
		}
	}

	$db->query('SELECT * FROM '.DB_PREFIX.'orders WHERE uid = 0 AND payment_gateway_id = "googlecheckout" AND status = "Process"');

	$orders = $db->getRecords();

	foreach ($orders as $order)
	{
		$name = preg_split('/ /', trim($names[$order['oid']]));

		$db->reset();
		$db->assignStr("fname", $name[0]);
		$db->assignStr("lname", (count($name) > 1) ? $name[1] : '');
		$db->assignStr("company", $order['shipping_company']);
		$db->assignStr("email", $emails[$order['oid']]);
		$db->assignStr("address1", $order['shipping_address1']);
		$db->assignStr("address2", $order['shipping_address2']);
		$db->assignStr("city", $order['shipping_city']);
		$db->assignStr("zip", $order['shipping_zip']);
		$db->assignStr("province", $order['shipping_province']);
		$db->assignStr("state", $order['shipping_state']);
		$db->assignStr("country", $order['shipping_country']);


		// create a new user with the info from google checkout
		$db->assignStr("login", "GoogleCheckoutUser");
		$u_id = $db->insert(DB_PREFIX.'users');

		print "Patched Order ".$order['order_num']." (oid: ".$order['oid'].") and merged billing information for user id {$u_id}: \n".print_r($db->fields, 1)."\n\n";

		$db->reset();

		// update the orders table with the newly created user id
		$db->assign('uid', $u_id);
		$db->update(DB_PREFIX.'orders', 'WHERE oid = "'.$order['oid'].'" LIMIT 1');
	}
}
